/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef BASICAPP_H
#define BASICAPP_H

#include "basiclog.h"
#include <string>
#ifndef __windows__
#include <sys/utsname.h>
#endif

/**
@defgroup singleusage Single inited classes
@brief Classes that can only be used once at a time.

Some of netrinjo's classes must or should not be used in more than one initialized
variable.\n
If you use these classes, it is recommended to initialize them once, best at the
launch of the app, and delete them directly before the application is closed.\n
All of these classes have a static member function, called \a getThe(), which
returns a pointer to the initialized variable of this class, or NULL if it has not
been inited yet. If your app should only run on Linux, you can also use the static
pointer \a the .
*/
/**
@mainpage
<h2>Welcome to the documentation of the Netrinjo game engine!</h2>
@section about About Netrinjo
@subsection summary Summary
Netrinjo is a game engine, designed to minimize the programming effort, concerning
the basic application development. One should be able to start implementing the
game without the need to deal a lot with things that are required in every game:
menus, loading data, ....

Furthermore there are classes for networking and the GUI (sound, input, graphics).

Netrinjo's homepage is at http://netrinjo.sourceforge.net/ .

@subsection overview Overview of abilities
@li 2D and 3D graphics (using framebuffer or OpenGL)
@li mouse and keyboard input
@li 2D graphical user interface
@li sound (wav)
@li music (ogg)
@li network: TCP, UDP
@li Compiles on Linux and (after some adjustments) on Windows and some other

@subsection depends-on Dependencies
Netrinjo depends on the following libraries:
@li SDL
@li SDL-ttf
@li SDL-image
@li SDL-gfx
@li SDL-mixer
@li SDL-net
@li SDL-websrv
@li GL and GLU
@li TinyXml (included)

@subsection license License
Netrinjo may be used and distributed under the terms of the GNU Lesser General
Public License (version 2.1 or later)

@section howto How-To
There are two examples of "main.cpp"-files included in this documentation.
One uses a class "GameApp" (must be derived from BasicApp) for basic application
handling, see @ref gameapp-main.
The other one uses the class XmlApp to manage the basic things (using this method
is recommended to minimize programming effort), see @ref xmlapp-main.

@section general General information
@subsection env Environment variables
The following environment strings are set by the application:
- DATAPATH (contains the path to the application's data directory,
           see BasicApp::getDataPath(), but does not end with '/')
- USERPATH (contains the path to the user's data directory,
           see BasicApp::getUserdataPath(), but does not end with '/')
- EXEPATH (the absolute path to the folder, in which the executable is,
           see BasicApp::getExePath(), but does not end with '/')
.
When using the method makeEnvPath(), it is simple to specify filenames relative
to these data-paths.
Example: \code $(DATAPATH)/subpath/filename.txt \endcode
Avoid using paths relative to the current working directory, because it can
change sometimes.
*/
/**
@page gameapp-main An example of a main.cpp-file with a class "GameApp"
Note: This method requires much work and detailed knowledge of this engine.
Especially beginners should use the XmlApp example @ref xmlapp-main.

In every Netrinjo-program, a variable of a class derived from BasicApp
has to be initialized. Otherwise Netrinjo-classes cannot be used.

Here an example (GameApp is here a class, derived from BasicApp):
@code
int the_main( int argc, char *argv[] );

int main( int argc, char *argv[] )
{
	return the_main( argc, argv );
}

#include "SDL/SDL.h"// "SDL.h" cannot be included before main, if compiling with mingw
#include "gameapp.h"// header file which declares GameApp

int the_main( int argc, char *argv[] )
{
	BasicApp::chDirToUser();
	if( !BasicApp::setWorkingDir( "./.game-dir" ) )
		return -1;
	GameApp app( argc, argv );

	if( SDL_Init( 0 ) < 0 )	{
		app.logLine("ERROR: could not init SDL");
		return -2;
	}
	atexit( SDL_Quit );

	app.init();// ignore fail-status (so i.e. audio need not be captured)

	return app.mainLoop();
}
@endcode
In GameApp::init you can perform all operations that you need to initialize variables...

*/
/**
@ingroup basic
@file basicapp.h
@brief Definition of the class BasicApp.
*/

// Prepends the data path to a string
//#define MAKE_DATA_PATH(fn)	((string(BasicApp::the->getDataPath()) + fn).c_str())

/**
 * Declares a BasicLog::Logger, named "log", using the BasicApp::getThe() for
 * logging.
 * \param s the string for the log-level's beginning
 */
#define BASICAPP_LOGGER(s)	BasicLog::Logger log( BasicApp::getThe(), s )

/**
 * @ingroup basic
 * @class BasicApp
 * @brief Basic functions to handle the application
 *
 * @ingroup singleusage
 * Do not use two or more variables of this class at a time.
 * But to use the netrinjo-engine, it has to be contructed once.
 *
 * BasicApp is used to handle the main process and getting important paths.
 * Only use it in a derived class and inherit mainLoop().
 * For better understanding of this class have a look at an example application.
**/
class DECLSPEC BasicApp : public BasicLog
{
public:
	/**
	 * Constructor
	 * @param argc number of arguments given to the app
	 * @param argv content of the arguments given to the app
	 * @param datapath the path to the data-containing directory, if NULL, the
	 * executable's directory is taken
	 * @param debug the filename of the log-file, if NULL, logging is not possible
	 *
	 * nInitFailStatus is set to zero by default.
	**/
	DECLSPEC BasicApp( int argc, char *argv[], const char *datapath = NULL, const char *debug = NULL );
	/**
	 * Constructor, using XML data
	 * @param argc number of arguments given to the app
	 * @param argv content of the arguments given to the app
	 * @param el the XML data, the value of the Attribute "DataPath" is read and is
	 * then returned by the method getDataPath()
	 *
	 * nInitFailStatus is set to zero by default.
	**/
	DECLSPEC BasicApp( int argc, char *argv[], TiXmlElement *el );
	virtual DECLSPEC ~BasicApp();

	/**
	 * May be implemented in derived classes.
	 * @param el the XML data, used for initialisation (not useful, if that was already
	 * specified on creation)
	 * @returns nInitFailStatus: if important initial routins fail, it should not return zero
	 *
	 * recommended usage: call init() in the "main(..)" function and
	 * exit, if init() does not return zero
	 * In the derived class, inherit onInit() and perform all initialisation routines
	 * there.
	**/
	DECLSPEC int init(TiXmlElement *el = NULL);

	/**
	 * by calling destroy() the application finishes the log-file and exits with "0"
	**/
	virtual DECLSPEC void destroy();

	/**
	 * only for derived classes @see init()
	 *
	 * recommended usage: call mainLoop() at the end of the "main(..)" function and
	 * return with it's result (like: "return app.mainLoop();")
	 * In the derived class, inherit mainLoop() and store all operations you need
	 * for running your app, probably a loop that continues, until the program shoul
	 * be quitted.
	**/
	virtual long mainLoop()	{ return 0; }

	/**
	 * @returns the path of the application's executable;
	 * the last character is a '/' (slash)
	**/
	DECLSPEC const char * getExePath();
	
	/**
	 * @returns the path to the data-containing directory; if not set, this is equal
	 * to getExePath();
	 * the last character is a '/' (slash)
	**/
	DECLSPEC const char * getDataPath();
	/**
	 * Makes an absolute path out of a path, relative to the data containing folder.
	 * At first the method makeEnvPath() is called to replace environment expressions.
	 * Then it is checked, whether it is absolute or not. If the path is absolute, the result
	 * equals the one of makeEnvPath(), otherwise the data path is prepended.
	 * @returns the absolute path to the file/folder in the data-containing directory
	 **/
	DECLSPEC std::string makeDataPath(const char *path);

	/**
	 * @returns the path to the directory, where user specific data is loaded
	 * from; (usually a subdirectory of the user's home directory);
	 * if the folder has not been specified before, the user's home is returned.
	 * The last character is a '/' (slash)
	**/
	const char *getUserdataPath()	{return chUserdataPath;}
	/**
	 * Makes an absolute path out of a path, relative to the user's data containing folder.
	 * At first the method makeEnvPath() is called to replace environment expressions.
	 * Then it is checked, whether it is absolute or not. If the path is absolute, the result
	 * equals the one of makeEnvPath(), otherwise the user's data path is prepended.
	 * @returns the absolute path to the file/folder in the user-specific directory
	 **/
	DECLSPEC std::string makeUserPath(const char *path);

	/**
	 * @returns the number of arguments given to the application
	**/
	int getArgC()	{ return argc; }

	/**
	 * @returns the content of the n-th argument given to the application
	**/
	const char * getArgV( int n )	{ return argv[n]; }
	
	/**
	 * prints an error-message with "str" to the standard-output
	 * and also writes it to the logfile
	 * then it exits by calling the "BasicApp::destroy()"-function
	 * @param str the message to print and write to the logfile
	**/
	DECLSPEC void exitWithMessage( const char *str );

	/**
	 * sets the current working directory to "dn".
	 * if "dn" does not exist, it trys to create it
	 * @param dn the working directory to be set; may be a relative path
	 *
	 * @returns true, if the working-dir could be changed, otherwise false
	**/
	DECLSPEC static bool setWorkingDir( const char *dn );

	/**
	 * changes the current working directory to the user's home-directory
	 * Linux usually: /home/username
	 * Windows usually: C:/Documents and Settings/username
	 * @returns true, if successful, false if not
	**/
	DECLSPEC static bool chDirToUser();
	/**
	 * @returns the name of the operating system, where Netrinjo has been
	 * compiled on (e.g.: Linux, Windows)
	**/
	DECLSPEC static const char *getOSName();
	/**
	 * @returns the name of the CPU architecture (e.g.: x86_64, i386)
	 **/
	DECLSPEC static const char *getCPUArch();
	/**
	 * @returns the name of the running operating system, independent of
	 * the compilation-OS of Netrinjo
	 **/
	DECLSPEC static const char *getRunningOSName();
	/**
	 * @returns the version of the running operating system
	 **/
	DECLSPEC static const char *getRunningOSVersion();
	/**
	 * @returns the architecture, the running operating system is compiled for
	 **/
	DECLSPEC static const char *getRunningOSArch();
	
	/**
	not accessible in Windows, equals the result of getThe()
	*/
	static BasicApp *the;
	/**
	@returns a pointer to the BasicApp variable, or NULL if not initialized.
	(There is only one BasicApp variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static
	function.
	@see singleusage
	*/
	DECLSPEC static BasicApp *getThe();
protected:
	/**
	Sets the path to the data containing directory. The environment variable
	@a DATAPATH is also set/updated to the new value.
	@param path the path to set
	@see getDataPath()
	*/
	DECLSPEC void setDataPath(const char *path);
	/**
	Sets the path to the user specific, data containing directory. The environment
	variable @a USERPATH is also set/updated to the new value.
	@param path the path to set, if it starts with "~/", it is taken relative
	            to the user's home directory
	@see getUserdataPath()
	*/
	DECLSPEC void setUserdataPath(const char *path);
	DECLSPEC void onInit();
	/// set to a number different from zero, if initialisation failed; use this
	/// in a derived class to set the return value of init()
	int nInitFailStatus;
private:
	char *chExePath, *chDataPath, *chUserdataPath;
	int argc;
	char **argv;
#ifndef __windows__
	static utsname osData;
#endif
};

#endif
