/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef BASICLOG_H
#define BASICLOG_H

/**
 * @defgroup basic Basic classes
 * @version 0.8
 * @todo have to think about it...
 *
 * @author Tobias Nadler (nadlerto at gmx dot de)
 *
 * The Basic* files support classes for handling very basic things like
 * logging and comfortable handling of the application and threads
*/

#include <fstream>
#include <iostream>
#include "basicobject.h"

/**
\ingroup basic
\file basiclog.h
\brief Definition of the class BasicLog.
*/

/**
 * @ingroup basic
 * @class BasicLog
 * @brief Basic functions for debugging, using a log file
 *
 * BasicLog is a class for logging debug-information to a file
 * Usually used as an inherited class
**/
class DECLSPEC BasicLog : public BasicObject
{
public:
	/**
	 * This struct simplifies logging in some cases. Use it in functions like
	 * in the following example:
	 * \code
	 * Logger log( BasicApp::getThe(), "Entered method ..." );
	 * [...]
	 * log.line("Method progressed.");
	 * [...]
	 * if( failed )
	 * 	return;
	 * [...]
	 * log.succeede();
	 * return;
	 * \endcode
	 */
	struct Logger
	{
		/**
		 * \param target the object used for logging
		 * \param strStart the string to write at the beginning
		 */
		Logger( BasicLog *target, const char *strStart )
		{
			pTarget = target;
			pTarget->logBegin( strStart );
			bSuccess = false;
		}
		/// Closes the log-level.
		~Logger()
		{
			pTarget->logEnd( bSuccess ? "done." : "failed." );
		}
		/**
		 * Logs a line, using BasicLog::logLevelLine().
		 */
		void line( const char * str )
		{
			pTarget->logLevelLine( str );
		}
		/**
		 * Logs a string into the current line, using BasicLog::logString().
		 */
		void append( const char * str )
		{
			pTarget->logString( str );
		}
		/**
		 * After a call to this method, it will write "done." on destruction.
		 * If bSuccess is false on destruction, a "failed." is written.
		 */
		void succeede()
		{
			bSuccess = true;
		}
	private:
		BasicLog *pTarget;
		bool bSuccess;
	};
	/**
	 * Create a BasicLog-variable, specifying the filename of the log-file
	 * @param filename the path to the file to write to, or NULL, if
	 * you need no loggings or want to init it later with "initLogFile"
	 * @see initLogFile
	**/
	DECLSPEC BasicLog( const char *name, BasicObject *parent, const char *filename );
	/**
	 * Create a BasicLog-variable, specifying the stream for logging
	 * @param strm the stream, used for logging; if no param specified,
	 * the standard output (std::cout) is used
	 * @see initLogStream
	**/
	DECLSPEC BasicLog( const char *name, BasicObject *parent, std::ostream *strm = &std::cout );
	/**
	 * Constructor, used with XML data.
	 * @param el the XML data, if the attribute "Logfile" is specified,
	 * the concerning file will be used for log data; if "Logstream" is
	 * specified, a stream is used for logging ("out" or "err");
	 * if LogTimes is specified and set "false", no times are logged
	 * @param p the parent element, or NULL to take GuiManager::the as parent
	*/
	DECLSPEC BasicLog( TiXmlElement *el, BasicObject *p );
	virtual DECLSPEC ~BasicLog();

	/**
	 * writes a string in a new line to the log-file
	 * @param ln the string to log
	**/
	DECLSPEC void logLine( const char *ln );

	/**
	 * simply writes a string to the log-file
	**/
	DECLSPEC void logString( const char *str );
	/**
	 * writes the time in msecs since SDL initialisation as decimal
	 * string to the logfile
	**/
	DECLSPEC void logTime();

	/**
	 * writes a string structured to the log-file,
	 * incrementing the number of forward-spaces
	 * @param ln string to log
	**/
	DECLSPEC void logBegin( const char *ln );

	/**
	 * writes a string structured to the log-file
	 * @param str string to log
	**/
	DECLSPEC void logLevel( const char *str );

	/**
	 * writes a string structured in a new line to the log-file
	 * @param ln string to log
	**/
	DECLSPEC void logLevelLine( const char *ln );

	/**
	 * writes a string structured to the log-file,
	 * incrementing the number of forward-spaces
	 * @param ln string to log
	**/
	DECLSPEC void logEnd( const char *ln );

	/**
	 * enable/disable logging;
	 * if no logfile was specified before, "debug.log" will be opened
	 * @param dodebug true to enable / false to disable logging
	 * @returns true, if logging is now enabled, otherwise false
	 * (if no log-file could be opened, then false will be returned)
	**/
	DECLSPEC bool setDebug( bool dodebug );

	/**
	 * @returns the current debug-status (bDebug)
	**/
	DECLSPEC bool getDebug()	{ return bDebug; }

	/**
	 * Prints a string to the standard output and calls flush() to do
	 * that immediatelly (even, if the app crashes directly after that)
	 * @param s the string
	**/
	static DECLSPEC void logStdOut(const char *s);

protected:
	/**
	 * opens a file for logging.
	 * If the file exists, it's content will be cleared,
	 * otherwise a new file will be created
	 * On success, debugging is turned on
	 * @param fn path to the log file
	 * @returns false, if the file cannot be used, otherwise true
	**/
	DECLSPEC bool initLogFile( const char *fn );
	/**
	 * Uses the specified stream for logging.
	 * Logging is always turned on after calling this function.
	 * @param strm the stream to use, if NULL
	**/
	DECLSPEC void initLogStream( std::ostream *strm );
	virtual DECLSPEC void onInit();
	
	/**
	 * if bAutoLogTime is true then there will be inserted the current
	 * time (msecs since SDL_Init) at every new line
	**/
	DECLSPEC bool bAutoLogTime;

private:
	bool bDebug;
	std::ofstream osLogFile;
	std::ostream *pLog;
	int nLogSpaces;
};

#endif
