
#ifndef BASICOBJECT_H
#define BASICOBJECT_H

#include "SDL/SDL.h" // just for DECLSPEC
#include <list>

/**
\ingroup basic
\file basicobject.h
\brief Definition of the classes BasicObject and BasicSlot.
*/

#include "tinyxml.h"

/**
\ingroup basic
Put this into your function, to be listed in the global function list.
\param name the function's name
\param classname the class
\param func the function to call

Here an example:

BasicSlot::getFunction("func","MyFoo") will return a pointer to the
function MyFoo::foofunc.

Then the header file looks like this:
\code
...
class MyFoo : public BasicObject
{
public:
	static void foofunc(void *a, BasicObject *b, void *c)	{
		... implementation ...
	}
...
};
\endcode
And the source file contains a line like this:
\code DECLARE_SLOT(func,MyFoo,MyFoo::foofunc) \endcode
*/
#define DECLARE_SLOT(name,classname,func)\
 static BasicSlot classname ## __ ## name(#name,#classname,func);

class BasicObject;

/**
\ingroup basic
\class BasicSlot
\brief Stores an event's slot (function pointer and name).

This class is used to get a pointer to a function when knowing its name.
Slots can be assigned to a classname.
*/
class BasicSlot
{
public:
	/**
	 * Definition for callback functions. The params might be used different as
	 * described here.
	 * \param sender a pointer to the object that called the function
	 * \param reciever a pointer to the object of which the function is called
	 * \param data a pointer to anywhere, depends on how it is used
	 */
	typedef void (*Callback)(void *sender, BasicObject *reciever, void *data);
	/**
	 * Constructor.
	 * \param fname the name of the function (must not be NULL, neither contain
	 * a '#' nor a dot '.')
	 * \param classname the name of the class (may be NULL for global methods)
	 * \param func the function pointer
	 * \param next pointer to the next slot in the list
	 */
	DECLSPEC BasicSlot(const char *fname, const char *classname, Callback func);
	/// Destructor. Frees the strName.
	virtual DECLSPEC ~BasicSlot();
	/// \returns the function pointer of this slot
	Callback getFunction()	{return pFunc;}
	/// \returns the name of this slot
	const char *getName()	{return strName;}
	/**
	 * Checks, whether the slot's name is equal to the given parameter.
	 * \param name the name to check for equivalence
	 * \returns true if equal, false if not
	*/
	bool matchName(const char *name)	{return strcmp(strName, name) == 0;}
	/// Pointer to the next slot in the global event list
	BasicSlot *beNext;
	/**
	 * \returns a pointer to the first slot (the one, that was added last) in
	 * the global event list
	 */
	static BasicSlot *getFirst()	{return beFirst;}
	/**
	 * \param name the function's name (must not be NULL), has the form:
	 * 	\code [<class>.]<function> \endcode if class is given here, the 2nd
	 * 	param "classname" is ignored
	 * \param classname the class's name or NULL, if the function does not
	 *                  belong to any class
	 * \returns a pointer to the function, that matches the given names
	 */
	static Callback getFunction(const char *name, const char *classname = NULL);
private:
	Callback pFunc;
	char *strName, *strClassName;
	// pointer to the first event in the global event list
	static BasicSlot *beFirst;
};

/**
\class BasicObject
\brief Base class for most Netrinjo-classes.

This class contains event-slots and/or other containers. Every slot can be looked
up by a string, giving the path to the object and its function. The object tree
is parsed similar to the directory tree in Unix: to specify an object either give
the absolute or relative path, absolute looks like "my.wanted.object", relative
looks like "..wanted.object" (every prefixed dot says "go to the parent object").
To get a function, simply append a sharp '#' and the function name, like
"my.wanted.object#foo". To access slots of a base class of an object (e.g:
ToggleVisibility (from GuiElement) of a GuiFrameCounter-object) you have to
specify the class that declares the slot before the function's name -
separated by a dot '.' (like: ...framecounter#GuiElement.ToggleVisibility).

Another special ability of BasicObject is the usage of XML data. When specifying
a XML element on creation, the data can later be used for initialising the object.
When the root's onInit() is called, all children's, grandchildren's,... onInit()
are called subsequently. Each sibling's onInit() is called in the same order as
the siblings were created (if the list-order was not modified after the children's
creation).
*/
class BasicObject
{
public:
	/// Definition for callback functions
	typedef BasicSlot::Callback Callback;
	/**
	\struct FunctionData
	Simple struct, containing a function-pointer and a pointer to a BasicObject.
	Both members are set to NULL after construction.
	*/
	struct FunctionData
	{
		/// pointer to a slot
		Callback func;
		/// pointer to an object
		BasicObject *obj;
		FunctionData(): func(NULL), obj(NULL)
		{
		}
		FunctionData(Callback f, BasicObject *p = NULL): func(f), obj(p)
		{
		}
	};
	/**
	Constructor. There must only be one BasicObject created without parent, and
	that is, if you need UI, the UiManager, for a server app you can take another
	object.
	\param name the name of this object (may be NULL, if parent is NULL); to
	            name the object by the next unused number, name hast to be ""
	\param parent the parent object, where this event is registered
	*/
	DECLSPEC BasicObject(const char *name, BasicObject *parent);
	/**
	Constructor, used to create an object from XML data. When using this method
	for creation, the method getXmlData() returns a valid pointer (not NULL).
	\param el the XML data
	\param p the parent element, or NULL to take GuiManager::the as parent
	*/
	DECLSPEC BasicObject( TiXmlElement *el, BasicObject *p );
	virtual DECLSPEC ~BasicObject();
	/**
	\returns the XML data, used for initializing this object. If the object was
	created without specifying an TiXmlElement, NULL is returned.
	*/
	TiXmlElement *getXmlData()	{ return xmlData; }
	
	/**
	This function should be overriden in every sub-class of BasicObject!
	\returns the name of the class.
	*/
	virtual const char *getClassName()	{ return "BasicObject"; }
	/**
	Checks, whether the name of this object is equal to the first part of the specified name
	(first part means everything before the first '.' or '\0')
	\param name the name to check for equivalence
	\returns true, if equal, otherwise false
	*/
	DECLSPEC bool matchName(const char *name);
	/**
	\param name the function's name to look for (i.e. "GuiManager.myobject#funcname")
	\returns a pointer to the function and to the corresponding object, or NULLs, if
	         there is none (if the object was found, but not the function, only func is 
	         set NULL)
	*/
	DECLSPEC FunctionData getFunctionByName(const char *name);
	/**
	 * \param name the function's name to look for
	 * \returns the function pointer of the event with the matching name, or
	 *          NULL if there is none
	 */
	DECLSPEC Callback getFuncByName( const char *name );
	/**
	 * \param name the object's name-path to look for (letters after a '#' will
	 * be ignored; if there is no dot at the beginning, the result is equal to
	 * the result of the static method getObject())
	 * \returns a pointer to the object with the matching name, or NULL if
	 * there is none
	 */
	DECLSPEC BasicObject *getObjectByName( const char *name );
	/**
	 * \param name the name (no path!) of the child-object (only 1st
	 * generation); characters after a '.' or a '#' are ignored
	 * \returns the child object with the specified name; NULL if there is none
	 */
	DECLSPEC BasicObject *getChildWithName( const char *name );
	/// \returns the name of the object, as specified on creation.
	const char *getName() const	{ return strName; }
	/// \returns a pointer to the parent object
	BasicObject *getParent() const	{ return parent; }
	/// \returns the number of children this object has
	unsigned int numChildren()	{ return children.size(); }
	/// \returns whether this is derived from UiObject
	virtual bool isUiObject() const	{ return false; }
	/// \returns whether this is derived from NetSocket.
	virtual bool isNetSocket() const	{ return false; }
	
	/// Initilises the object by calling onInit().
	void init()	{ onInit(); }
	/**
	 * Uninitilises the object by calling onDestroy() (memory is not freed).
	 * This should usually be called before deleting an object.
	 */
	void destroy()	{onDestroy();}
	
	static BasicObject *getObject(const char *name)	{ return pBase? pBase->getObjectByName(name) : NULL; }
	static Callback getFunc(const char *name)	{ return pBase? pBase->getFuncByName(name) : NULL; }
	DECLSPEC static FunctionData getFunction(const char *name);

protected:
	typedef std::list<BasicObject*> child_list;
	typedef child_list::iterator child_iter;
	typedef child_list::reverse_iterator child_reverse_iter;
	typedef std::list<BasicSlot *>::iterator EventIter;
	
	/**
	Initializes the object's variables by reading data from the XML element.
	If there were no XML data used for creation, nothing is done.
	This function calls onInit() of its children recursively (after it has
	inited itself).
	It is highly recommended to call BasicObject::onInit(), usually at the
	end of the onInit() method, when overwriting it in a derived class.
	*/
	virtual DECLSPEC void onInit();
	/**
	May be called to reinitialise this object or stop an initialised module.
	Calls all children's onDestroy(). Most classes do not use this method.
	But sometimes it is useful to reset the functionality of an object.
	*/
	virtual DECLSPEC void onDestroy();

	/**
	The data used to create this element (NULL, if it was not created using XML)
	A valid pointer may be assigned later after creation as well.
	*/
	TiXmlElement *xmlData;
	// the object's children
	child_list children;
	// item of this object in the parent's children list
	child_iter parentitem;
private:
	// adds a child, name must not be NULL
	child_iter addChild( BasicObject *obj, const char *name, bool top = true );
	// removes a child and frees its name
	void removeChild( BasicObject *obj );
	
	// pointer to the parent object
	BasicObject *parent;
	// allocated/freed by parent
	char *strName;
	
	// pointer to the last object, created without parent
	static BasicObject *pBase;
	friend class UiManager;
	friend class UiObject;
};

class TiXmlElement;

/**
\class SignalMultiplier
\brief Forwards an event to many slots.

This class is designed to be able to send an event to more than just
one slot. To do so, create a SignalMultiplier object and connect its
"event" slot with a signal. Furthermore you have to add slots that
should recieve the event with addSlot().

\test whole class
*/
class SignalMultiplier : public BasicObject
{
public:
	DECLSPEC SignalMultiplier(const char *name, BasicObject *parent);
	/**
	Construction, using XML-data.
	The XML element must have children, called "Slot", with each having
	an attribute "Aim".
	*/
	DECLSPEC SignalMultiplier(TiXmlElement *el, BasicObject *parent);
	DECLSPEC ~SignalMultiplier();
	DECLSPEC void addSlot(Callback func, BasicObject *aim);
	DECLSPEC void removeSlot(Callback func, BasicObject *aim);
	
	static void event(void *sender, BasicObject *aim, void *data)
	{
		((SignalMultiplier*)aim)->onevent(sender, data);
	}
protected:
	DECLSPEC void onInit();
	
private:
	DECLSPEC void onevent(void *sender, void *data);
	typedef std::list<BasicObject::FunctionData> cblist;
	cblist aims;
};

#endif
