/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef BASICTHREAD_H
#define BASICTHREAD_H

#include "SDL/SDL.h"
#include "SDL/SDL_thread.h"

#include "basiclog.h"

/**
 * \ingroup basic
 * \file basicthread.h
 * \brief Definition of the class BasicThread.
 */

/**
 * \ingroup basic
 * \class BasicThread
 * \brief Basic functions to handle threads
 *
 * Required header file:
 * \code #include <basicthread.h> \endcode
 *
 * BasicThread is used to handle threads.
 * Only use it in a derived class and inherit onRun() and / or onCycle().
**/
class DECLSPEC BasicThread : public BasicLog
{
public:
	/**
	 * Constructor. Does not start the thread!
	 * \param fndebug the filename of the log file to use for debugging
	 * can be NULL, if logs are not required
	 */
	DECLSPEC BasicThread( TiXmlElement *el, BasicObject *p );
	/**
	 * Constructor, used without XML-data.
	 * \param fndebug the filename of the logfile to use or NULL if you do not
	 * want to log messages.
	 */
	DECLSPEC BasicThread( const char *name, BasicObject *p, const char * fndebug );
	/**
	 * Constructor, used without XML-data, for logging to a stream.
	 * \param strm the stream where the logs are written to, standard output by
	 * default.
	 */
	DECLSPEC BasicThread( const char *name, BasicObject *p, std::ostream *strm = &std::cout );
	virtual DECLSPEC ~BasicThread();

	virtual const char *getClassName()	{ return "BasicThread"; }
	
	/**
	 * Activates the thread, which then calls onRun()
	 * \param paused if this is true then onCycle() will not be called
	 * until the thread resumes (\see setPause)
	 */
	DECLSPEC bool start( bool paused = false );
	
	/**
	 * Stopps the thread immediately
	 * \warning try not to use this, if you are not informed about the consequences,
	 * prefer stop(10), if unavoidible or, best finish the inherited onRun()
	 */
	DECLSPEC void kill();
	
	/**
	 * Stopps the thread, and starts it again, try to avoid this as kill()
	 */
	DECLSPEC void restart( Uint32 wait = 20 );
	
	/**
	 * \returns true, if the thread is active otherwise false
	 */
	bool running()	{ return pThread != 0; }

	/**
	 * lock is used for locking the mutex, this function is only valid
	 * while the thread is running (otherwise it returns false)
	 * \param wait time to wait (in msecs) if locking is not possible immediatelly
	 * \returns false, if locking was not successful or the thread is not running,
	 * true otherwise
	 * \todo verify functionality
	 */
	DECLSPEC bool lock( Uint32 wait = 0 );
	/**
	 * unlock is used for unlocking the mutex, this function is only valid
	 * while the thread is running (otherwise it returns false)
	 * \returns false, if unlocking was not successful or the thread is not running,
	 * true otherwise
	 * \todo verify functionality
	 */
	DECLSPEC bool unlock();

	/**
	 * Pauses or continues the thread, so that onCycle will no more or again be called
	 * \param pause if true thread will be paused, if false it will be continued
	 * \returns true, if the thread is paused, false if it is stopping or not running
	 */
	DECLSPEC bool setPause( bool pause );
	
	/**
	 * Tries to finish the loop by setting the stopping-status to true, finally kills it,
	 * if still running.
	 * \param wait time to wait (in msecs) until killing, if it's still running
	 * \warning use it with care
	 */
	DECLSPEC void stop( Uint32 wait = 20 );
	
	/**
	 * \returns the duration in msecs of the last cycle or INFINITE, if no time availible
	 */
	virtual unsigned int getTimePerCycle()	{ return tPerCycle; }
	
	/**
	 * \returns the average time of one cycle since last thread-start or INFINITE, if no values availible
	 */
	virtual DECLSPEC unsigned int getAvgTimePerCycle();
	/**
	 * \returns the number of cycles that were done, since last thread-start,
	 * and zero, if it has not been started yet
	 */
	unsigned int numCycles()	{ return nCycles; }

protected:
	/**
	 * inherit onRun to have a function, called once (done in the new thread),
	 * when the thread is started
	 */
	virtual int onRun();
	
	/**
	 * inherit onCycle to get a function, called once per loop-cycle after starting the thread
	 * when using onCycle, the cycle-functions return a usable result
	 * note that this only happens, if onRun is not inherited or if it is called by a inherited function
	 */
	virtual void onCycle()	{}

	/**
	 * \returns the thread's identifyer
	 */
	Uint32 ulThreadId;
	/**
	 * just a mutex (used by lock and unlock) for any use
	 * \see lock, unlock
	 */
	SDL_mutex *mutex;

private:
	bool bStopping, bPaused;
	unsigned int tPerCycle, tLastCycle, nCycles, tStart, tPaused;

	SDL_Thread *pThread;
	static int startThread( void * pvData );
};

#endif
