/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef TIMER_H
#define TIMER_H

#include <list>
#include "SDL/SDL.h"
#include "basicobject.h"

/**
\ingroup ui
\file frametimer.h
\brief Defines the class FrameTimer.
*/

using namespace std;

/**
@ingroup ui
@class FrameTimer
@brief Handles timers, their intervals and activity

\ingroup singleusage
This class can only be used in one variable at a time. It is recommended
not to initialize it directly, because it will automatically be constructed
with UiManager.

This class supports timer events that occur at most once per frame.
After this class was initialized once, you can use it by accessing the
static member function getThe().
*/
class DECLSPEC FrameTimer
{
public:
	/**
	 * This type is used for functions, that are called by the timer
	*/
	typedef BasicObject::Callback Callback;
	/**
	 * This struct contains all data, that is needed for a timer-event
	*/
	struct CbInfo
	{
		/**
		 * You do not need init, it is only used locally by FrameTimer.
		 * @see FrameTimer::addCallback
		*/
		void init( Callback c, BasicObject * d, Uint32 iv, Uint32 n )
		{
			cb = c;
			pData = d;
			tInterval = iv;
			tNext = n;
		}
		/**
		 * Pointer to the function to be called when the time elapsed
		*/
		Callback cb;
		/**
		 * Pointer to any data that will be given to the callback-function
		 * as second param
		*/
		BasicObject * pData;
		/**
		 * The time in msecs between two timer-events
		 * @warning Do not change this value directly, use FrameTimer::setInterval instead
		*/
		Uint32 tInterval; // read-only!
		/**
		 * The time of the next timer-event
		 * @warning Do not change this value directly, use FrameTimer::setInterval instead
		*/
		Uint32 tNext; // read-only!
//		friend class FrameTimer;
	};
	/**
	 * Constructor \n
	 * You never need it, if you use UiManager
	*/
	DECLSPEC FrameTimer();
	virtual DECLSPEC ~FrameTimer();
	/**
	 * Changes the interval of an initialized timer
	 * @param id a pointer to the initialized structure that should be modified
	 * @param iv the time in msecs to wait between two timer-events, the next event will
	 *  be called iv msecs from now on.
	 * @returns false if id is NULL, otherwise true
	*/
	DECLSPEC bool setInterval( CbInfo * id, Uint32 iv );
	/**
	 * Initializes a new timer
	 * @param c the function to call, when the time elapsed
	 * @param d the value of the second param in the callback
	 * @param iv the interval in msecs between two timer events, the next event occurs
	 *  iv msecs from now on.
	 * @returns a pointer to the structure with the initialized data; it is required to
	 *  clean it again with removeCallback(). Do not change its properties directly, but
	 *  by using setInterval().
	*/
	DECLSPEC CbInfo * addCallback( Callback c, BasicObject * d, Uint32 iv );
	/**
	 * Cleans an initialized timer.
	 * @param pCb pointer to the timer's data, must be the result of addCallback()
	*/
	DECLSPEC void removeCallback( CbInfo * pCb );

	/**
	 * only accessible in Linux, and equals the result of getThe()
	 * @see getThe()
	**/
	static FrameTimer * the;
	/**
	 * @returns a pointer to the FrameTimer-variable, or NULL if not initialized.
	 * (There is only one FrameTimer-variable allowed in an application)
	 * 
	 * Every netrinjo-class that can only exist once at a time, has this static function.
	**/
	DECLSPEC static FrameTimer *getThe();
private:
	typedef list<CbInfo*>::iterator CbInfoIter;
	list<CbInfo*> cbs, cbsInactive;
	void doTimer();

	friend class UiManager;
};

#endif
