/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#ifndef DOXYGEN
#include "SDL/SDL.h"
#include "SDL/SDL_net.h"
#include <string>
#endif
/**
@ingroup noclass
@file functions.h
@brief Contains mainly string conversation methods.

Required header file:
@code #include <functions.h> @endcode

With including this file you get all important string conversation methods, that are
used by the Netrinjo engine. You can convert numbers, boolean expressions, IP-addresses
and colors in both ways to a string and from a string.

Furthermore the function getUserDirectory is declared here.

Used Abbreviation:\n
NBO: Network Byte Order
*/

/**
Converts an integer value to a string.
@warning This is not thread-safe, use intToStr( int i, char *buf, int base ) instead,
if you need thread safity.
@param i The number to convert
@param base The base to use, if it's less than 2, 2 will be used; if it's above 36,
 36 will be used.
@returns a pointer to the string with the converted number;
the result points to a static variable, so you have to use the result before calling
this function again.
*/
DECLSPEC const char *intToStr( int i, int base = 10 );// not thread-safe!

/**
Converts an integer value to a string. This function is thread safe.
@warning intToStr does mostly not return the param buf!
@param i The number to convert
@param buf A pointer to reserved space of at least 33 characters
@param base The base to use, if it's less than 2, 2 will be used; if it's above 36,
 36 will be used.
@returns a pointer to the string with the converted number, usually not equal to buf.
*/
DECLSPEC char *intToStr( int i, char *buf, int base = 10 );
/**
Converts an unsigned integer value to a string. This function is thread safe.
See intToStr( int i, char *buf, int base ) for more information.
*/
DECLSPEC char *uintToStr( unsigned int i, char *buf, unsigned int base = 10 );// as intToStr(x,y,z), but for unsigned numbers

/**
Converts a string to an unsigned long value, the base is usually 10,
but it changes, if the number-string has a special prefix:
@code
prefix => base
0q, q => 4
0, 0o, o => 8
0x, x, #, h, $ => 16
0b, b => 2
0d, d => 10
@endcode
@param str The string to convert.
@param err is assigned true, if the string could not be converted, otherwise false is
 assigned; if it is NULL, it is ignored
@returns The number, that str represents.
*/
DECLSPEC Uint32 xStrToUL( const char *str, bool *err = NULL );

/**
Converts a string with decimal numbers to a SDL_Rect.
The numbers have to be separated by a ',' or a ';', spaces/tabs are ignored.
@param str The string to convert; it has to contain 4 numbers in the following sequence:
 X-pos, Y-pos, width, height
@returns The converted data.
*/
DECLSPEC SDL_Rect strToRect( const char *str );

/**
Converts a string to a boolean value.\n
@param s The string to read (case insensitive)
@returns true if s is: "+", "1", "true", "yes" or "y", false otherwise
*/
DECLSPEC bool strToBool( const char *s );

/**
Converts a string to an IPv4-value (in NBO),
@param str can either be a single decimal number or consist of 4 decimal byte-numbers,
 separated by a '.' (no spaces/tabs allowed)
@returns the IP address in network-byte-order
*/
DECLSPEC Uint32 strToIP( const char *str );

/**
Converts a string to an IPv4-address-value (in NBO) like strToIP(..)
if there is a ':' in the string, the number behind the ':' is read as the port.
@param str The string to convert.
@returns The IP address, represented by str.
*/
DECLSPEC IPaddress strToIPaddress( const char *str );

/**
Converts a string to a SDLKey value.
@param str The string to convert.
@returns The SDL_Key, represented by str.
*/
DECLSPEC SDLKey strToKey( const char *str );

/**
Converts a string to a key modifier (may be a combination, separated by '+').
@param str The string to convert.
@returns The key modifier, represented by str.
*/
DECLSPEC SDLMod strToMod( const char *str );

/**
Converts a byte to a string.
@param b The number to convert
@param buf Pointer to reserved space, where the number will be stored in form of a string
@returns the number of affected chars in buf
*/
DECLSPEC int byteToStr( Uint8 b, char *buf );

/**
Converts an IP address (in NBO) without the portnumber to a string in the following format:
@code <a>.<b>.<c>.<d> @endcode
@param ip The IP address to convert
@param buf Pointer to reserved space, where the IP address will be stored
@returns buf
*/
DECLSPEC const char * IPToStr( Uint32 ip, char *buf );
/**
Converts an IP address (in NBO) with the portnumber to a string in the following format:
@code <a>.<b>.<c>.<d>:<Port> @endcode
@param ip The IP address to convert
@param buf Pointer to reserved space, where the IP address will be stored
@returns buf
*/
DECLSPEC const char * IPToStr( const IPaddress &ip, char *buf );

/**
 * Converts an unsigned integer value to a SDL_Color-value.
 * @param v The number to convert: the blue value is the little endian byte,
 * green is the middle value, red value is 2nd-highest (bits 16..23), the big
 * endian byte is ignored
 * @returns The value in form of a color
 * @note Only use this when reading user-data (from xml-files...), but not when
 * getting a surface's pixel-value, because it does not check the surface's
 * RGBA-mask which may differ between environments.
 * @see SdlManager::convertColor()
 */
DECLSPEC SDL_Color ULToColor( Uint32 v );

/**
 * @returns the directory of the user, who executes the app
 * (e.g.: "/home/username")
 */
DECLSPEC const char * getUserDirectory();

/**
Replaces the ocurrencies of "$(...)" with the content of the specified environment
variable. E.g.: "$(USERNAME)" will be replaced by the user's name.
@param str the string to parse
@returns the string with the replaced data
*/
DECLSPEC std::string makeEnvStr(const char *str);

/**
Does the same as makeEnvStr(), but replaces a leading "~/" by the user's home
directory.
E.g: "~/.netrinjo" will become "/home/username/.netrinjo", if the user's home is
at "/home/username".
@param str the string to parse
@returns the string with the replaced data
*/
DECLSPEC std::string makeEnvPath(const char *str);

/**
 * Case-sensitive pattern match
 * \param pattern the pattern
 * \param string the string, that is checked to match the pattern
 * \returns true if string matches the pattern, false if not
 * \author member "Mik" at http://www.linuxquestions.org/
 */
DECLSPEC bool patMatch (const char* pattern, const char* string);
/**
 * Case-insensitive pattern match
 * \param pattern the pattern
 * \param string the string, that is checked to match the pattern
 * \returns true if string matches the pattern, false if not
 * \author member "Mik" at http://www.linuxquestions.org/
 */
DECLSPEC bool patiMatch (const char* pattern, const char* string);

#ifndef __windows__
/**
 * Gets the local IP-address (only for UNIX-based platforms).
 * \param bIPv4 if true, IPv4-addresses are prefered; otherwise IPv6-addresses
 * this does not guarantee, that the result is of that kind.
 * \returns a NUL-terminated string, containing the local IP-address.
 * \note the result will never be "127.0.0.1". If the computer is connected to
 * more than one network, the result is one of the assigned addresses.
 * \author code taken from http://www.hungry.com/~alves/local-ip-in-C.html, but
 * modified a little.
 */
char *getLocalIP( bool bIPv4 = true );
#endif

#endif
