/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler und Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIBACKGROUND_H
#define GUIBACKGROUND_H

#include <list>
#include "guielement.h"
#include "definitions.h"

class GuiRadioBtn;
class SdlImageContainer;
class SdlImage;
class SdlSurfaceContainer;
class SdlSurface;

using namespace std;

/**
\ingroup gui
\file guibackground.h
\brief Defines the classes GuiBackground, GuiFrame, GuiBorder, GuiRadioContainer, GuiPictureBg, GuiFlowWindow and GuiPageControl
*/

/**
\ingroup gui
\class GuiBackground
\brief A class for filling a rect with a color

Use this class to either clear a rect, where graphics will be drawn on, or to
draw a simple, filled rect.
*/
class DECLSPEC GuiBackground : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param name the element's name
	\param c the color to use for filling; if CLR_STANDARD, then the theme's
	default color for this kind of element is used; if CLR_INVALID, then it
	is like a GuiElement
	*/
	DECLSPEC GuiBackground(const SDL_Rect &r, GuiElement *p, const char *name, ARGB_COLOR c = CLR_STANDARD);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation: if the attribute "BgColor"
	          is specified, the value is assigned to the element's bgcolor
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiBackground(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiBackground();

	virtual const char *getClassName()	{ return "GuiBackground"; }
	
	/// the color to use for filling
	ARGB_COLOR bgcolor;
	
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiBackground((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
	/**
	Fills the clip rect with the color bgcolor. Especially useful, when deriving
	this class and you want to draw something onto the background: call this
	method at the beginning of the onFrame() event, then draw your things and
	finally call GuiElement::onFrame(dst)
	\param dst the surface to paint on
	*/
	virtual DECLSPEC void paintBg( SdlSurface *dst );
	virtual DECLSPEC void paintBgGL();

private:
	Uint32 changeTheme( UiTheme *t, const char *classname );
	bool bThemeColor;
};

/**
\ingroup gui
\class GuiFrame
\brief A simple class, that appears different, if the cursor is above it

This is a simple class, that fills its rect with one color, if the
cursor is above it and with another color, if it is not above it.
*/
class DECLSPEC GuiFrame : public GuiBackground
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param name the element's name
	\param c_out the color to use for filling, if the cursor is not above it
	\param c_over the color to use for filling, if the cursor is above it
	*/
	DECLSPEC GuiFrame(const SDL_Rect &r, GuiElement *p, const char *name, Uint32 c_out = CLR_INVALID, Uint32 c_over = CLR_INVALID);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiFrame(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFrame();

	virtual const char *getClassName()	{ return "GuiFrame"; }

	/// the color to use for filling, if the cursor is not above it
	Uint32 color1;
	/// the color to use for filling, if the cursor is above it
	Uint32 color2;
	
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFrame((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onMouseOver();
	virtual DECLSPEC Uint32 onMouseOut();
	virtual DECLSPEC void onInit();
private:
};

/**
\ingroup gui
\class GuiBorder
\brief Draws a border at its rect boundaries

This class is used to draw a border to the rect's boundary. The borders
at the top, bottom, left and right can appear different or be hidden.
Corners are displayed using other bitmap-sources.

The appearance can be switched between three modes: raised, flat or sunken, by
setting the variables neutral and invert accordingly.
*/
class GuiBorder : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param bmps the source for the look of the borders; if flat (neutral) mode
	should be possible, at least 18 surfaces have to be in the container:
	always three surfaces belong together, the first should be light, the second
	dark, the third is shown in flat mode; the first three are used to show
	horizontal borders, the next for vertical borders, then the corners follow:
	top-left, top-right, bottom-left, bottom-right (6 * 3 = 18).
	If flat mode is not needed, then 6 * 2 = 12 surfaces are needed, because
	there are always two in a group (the neutral is not needed).
	If this param is NULL, the theme's default surfaces are taken.
	\param themestyle if true, the variables neutral and invert are set as the
	theme wants them to be, but changing them has still the same effect as if
	the param is false.
	*/
	DECLSPEC GuiBorder( const SDL_Rect &r, GuiElement *p, const char *name, SdlImageContainer * bmps = 0, bool themestyle = true );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: InvertBorder, BorderEdges, HideBorder, BorderBmps,
	          NeutralBorder, BorderTheme
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiBorder(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiBorder();
	
	virtual const char *getClassName()	{ return "GuiBorder"; }
	/**
	Changes the source surfaces for the border's look
	\param sc the surfaces that are used; if less than 18 are in the container,
	flat (neutral) mode is switched off, otherwise it is switched on
	*/
	DECLSPEC void setBmps( SdlImageContainer * ic );
	/// \returns the surfaces, that are used for the appearance
	SdlImageContainer * getBmps()	{ return bmps; }
	/**
	if neutral is true, invert has no effect; otherwise, if invert, the usually
	dark borders become light and the other way round
	*/
	bool invert, neutral;
	/// determine the visibility of the borders
	bool left, right, top, bottom;

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiBorder((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}

protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

	/// if true, flat mode is possible
	bool bCanNeutral,
	/// if true, border is shown, if false, it is hidden
	     bShowBorder;

	/// size of the border's corner
	int nCornerWidth, nCornerHeight,
	/// width of horizontal border surface
		nBmpWidth,
	/// height of vertical border surface
		nBmpHeight;

private:
	Uint32 changeTheme( UiTheme *t, const char *classname );
	
	int brd_height, brd_width;
	SdlImageContainer *bmps;
	bool bThemeBmps, bThemeStyle;
};

/**
\ingroup gui
\class GuiRadioContainer
\brief A container class for radio buttons
*/
class DECLSPEC GuiRadioContainer : public GuiBackground
{
public:
	DECLSPEC GuiRadioContainer(const SDL_Rect &r, GuiElement *p, const char *name, Uint32 c = CLR_INVALID);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: OnCheck (event), AllowNoCheck (bool)
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiRadioContainer(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiRadioContainer();

	virtual const char *getClassName()	{ return "GuiRadioContainer"; }

	/// select the item with the specified index i
	DECLSPEC void setIndex(int i);
	/// \returns the currently selected item-index, -1 if none is selected
	DECLSPEC int getIndex();
	/// \returns the currently selected item
	DECLSPEC GuiRadioBtn * getSelectedItem();
	/// \returns the button, that belongs to the specified index i
	DECLSPEC GuiRadioBtn * getItem(int i);
	/// \returns the button with the highest index, or NULL, if there are none
	DECLSPEC GuiRadioBtn * getLast()	{ return buttons.size() != 0 ? buttons.back() : NULL; }
	/**
	 * Sets the callback, that is called, when another radiobutton is selected.
	 * \param d the callback data
	 */
	DECLSPEC void setCallback(FunctionData d)	{cb = d;}
	/// specifies whether it is allowed to check no item
	DECLSPEC void setAllowNoCheck( bool allow );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiRadioContainer((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC void onInit();
	/**
	 * Called when the selection changes.
	 * \param i the index of the selected item
	 */
	virtual DECLSPEC void onSelect( int i );
	
private:
	typedef list<GuiRadioBtn*> button_list;
	typedef button_list::iterator button_iter;
	button_iter addButton(GuiRadioBtn *b);
	void removeButton(button_iter iter);
	void onCheck(button_iter iter);

	int index;
	bool allownocheck;
	FunctionData cb;

	button_iter checked;
	button_list buttons;
	// just to avoid multiple onSelect()-calls on one check
	bool bChanging;
	
	friend class GuiRadioBtn;
};

/**
\ingroup gui
\class GuiPictureBg
\brief An object with a picture as background
*/
class DECLSPEC GuiPictureBg: public GuiBackground
{
public:
	DECLSPEC GuiPictureBg(const SDL_Rect &r, GuiElement * el, const char *name, Uint32 color = CLR_STANDARD, SdlImage * backbmp = NULL/*, bool dofastblt = false*/);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: BgBmp (name of surface), BgTheme (bool)
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiPictureBg(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiPictureBg();

	virtual const char *getClassName()	{ return "GuiPictureBg"; }
	
	DECLSPEC void setBmp(SdlImage * backbmp);
//	DECLSPEC void setFastBlt(bool dofast);

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiPictureBg((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC void paintBg( SdlSurface *dst );
	virtual DECLSPEC void paintBgGL();
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	Uint32 changeTheme( UiTheme *t, const char *classname );

	// bool fastblt;
	SdlImage * bmp;
	int nBmpWidth, nBmpHeight;

	bool bThemeBmp;
};

/**
\ingroup gui
\class GuiFlowWindow
\brief A window that can move from a position to another
*/
class DECLSPEC GuiFlowWindow : public GuiPictureBg
{
public:
	enum FlyStyles{sinus, cosinus, constant, powsin, sin_2, theme_std} eFlyStyle;

	DECLSPEC GuiFlowWindow( const SDL_Rect &r, GuiElement *el, const char *name, Uint32 color = CLR_STANDARD, SdlImage * backbmp = 0, FlyStyles style = theme_std );
	/**
	 * Constructor. Uses XML data.
	 * \param el the XML data for initialisation, the following attributes are
	 * read: DestRect (rect), Time (int), FlyStyle; after a call to flyTo(),
	 * the value specified by Time is overwritten by the 2nd param of flyTo().
	 * \param p parent element (or NULL, to take GuiManager::the as parent)
	 * \see GuiPictureBg(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiFlowWindow(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFlowWindow();

	virtual const char *getClassName()	{ return "GuiFlowWindow"; }
	
	/**
	 * Starts flying to the specified rect taking the specified time.
	 * \param r the destination rect
	 * \param t the time (in ms) the flight should take; if zero, the default
	 * is used
	 */
	DECLSPEC void flyTo( const SDL_Rect &r, Uint32 t );
	/// same as flyTo(), using rDestination and nTime
	DECLSPEC void start();
	/// stops the flight immediatelly, the window will be placed to rDestination
	DECLSPEC void stop();
	/// swaps the source and destination rects, so it can fly back
	void swapSrcDest()	{SDL_Rect r=src;src=rDestination;rDestination=r;}
	/**
	Sets the callback data, for the onPosReached()-event (called after a flight
	finished, but not if stop() is called).
	*/
	void setPosReachedCallback(Callback c, BasicObject *aim)	{cb.func = c; cb.obj = aim;}

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFlowWindow((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
	static void startflight(void *sender, BasicObject *aim, void *time)
	{
		((GuiFlowWindow*)aim)->start();
	}
	/// the rect where the current flight should finish
	SDL_Rect rDestination;
	/// duration of a flight
	Uint32 nTime;

protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	/**
	This event is called, when the "flight" is done. So the window's position is
	equal to the destination-rect, as specified in the 1st param of flyTo() and
	the time as specified in the 2nd param has expired.
	\returns anything, currently not used, but should return zero.
	*/
	virtual DECLSPEC Uint32 onPosReached();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	Uint32 changeTheme( UiTheme *t, const char *classname );
	// called on each frame
	void calcCurrentPos();
	
	static SDL_Rect keyFrame( const SDL_Rect &s, const SDL_Rect &d, float t );
	
	// true if currently flying
	bool bFly;
	// the rect where it started flying from
	SDL_Rect src;
	// elapsed time of the current flight
	Uint32 elapsed;

	bool bThemeStyle;
	BasicObject::FunctionData cb;
};

/**
\ingroup gui
\class GuiPageControl
\brief A class for page controls
*/
class DECLSPEC GuiPageControl : public GuiPictureBg
{
public:
	typedef list<GuiElement*> page_list;
	typedef page_list::iterator page_iter;
	enum SIDE{top, bottom, left, right};

	DECLSPEC GuiPageControl( const SDL_Rect &r, GuiElement *el, const char *name, Uint32 c = CLR_INVALID, SdlImageContainer *res = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: ButtonBmps
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiPageControl(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiPageControl();

	virtual const char *getClassName()	{ return "GuiPageControl"; }
	
	DECLSPEC page_iter addPage( GuiElement *pg, const char *t = 0, SdlImageContainer *res = 0, Uint32 w = 0 );
	DECLSPEC void setButton( Uint32 w, Uint32 h, SdlImageContainer *res = 0 );
	DECLSPEC void setSide( SIDE s );

	DECLSPEC bool removePage( page_iter i );
	DECLSPEC bool removePage( GuiElement *el );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiPageControl((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onChange( GuiRadioContainer *sender );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	Uint32 changeTheme( UiTheme *t, const char *classname );
	void arrange();

	SIDE side;
	int nBtnWidth, nBtnHeight;
	page_list pages;
	GuiRadioContainer *radios;
	SdlImageContainer *btnBmps;
	SDL_Rect pagerect;
	int count;

	bool bUseTheme;

	static inline void onRadio( void *sender, BasicObject * data, void * )
	{
		((GuiPageControl*)data)->onChange( (GuiRadioContainer *)sender );
	}
};

#endif
