/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUICOMBINATION_H
#define GUICOMBINATION_H

#include "guitext.h"
#include "guidynamicbutton.h"
#include "guibackground.h"
#include "guiscroller.h"

/**
\ingroup gui
\file guicombination.h
\brief Defines the classes GuiCheck, GuiRadio and GuiNumScroller
*/

/**
\ingroup gui
\class GuiCheck
\brief A checkbox with a caption

*/
class DECLSPEC GuiCheck : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param t the text, that appears next to the checkbox; may be NULL, if no text needed
	*/
	DECLSPEC GuiCheck( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = NULL );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiCheck(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiCheck();

	virtual const char *getClassName()	{ return "GuiCheck"; }

	/// the checkbox
	GuiCheckableBtn *check;
	/// the text
	GuiText *text;

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiCheck((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
private:
	void arrange();
};

/**
\ingroup gui
\class GuiRadio
\brief A radio button with text

*/
class DECLSPEC GuiRadio : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param t the text, that appears next to the radio button; may be NULL, if no text needed
	*/
	DECLSPEC GuiRadio(const SDL_Rect &r, GuiRadioContainer *p, const char *name, const char *t = NULL);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p the container element
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiRadio(TiXmlElement *el, GuiRadioContainer *p);
	virtual DECLSPEC ~GuiRadio();
	
	virtual const char *getClassName()	{ return "GuiRadio"; }
	/**
	Uninitializes the element by deleting the radio button and text elements.
	Has to be called, if it's not deleted by the parent.
	*/
	DECLSPEC void destroy();

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r;
		if(strcmp(o->getClassName(), "GuiRadioContainer") == 0)
			r = new GuiRadio((TiXmlElement*)xmldata, (GuiRadioContainer*)o);
		else	{
			BasicApp::the->logLine("Error: GuiRadio::newguiobject(): parent element's class must be GuiRadioContainer!");
			r = NULL;
		}
		*(GuiElement**)presult = r;
	}

protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
private:
	void arrange();
	GuiRadioBtn *radio;
	GuiText *text;
};

/**
\ingroup gui
\class GuiNumScroller
\brief A scroller with a text, containing the scroller's value

*/
class DECLSPEC GuiNumScroller : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	*/
	DECLSPEC GuiNumScroller(const SDL_Rect &r, GuiElement *p, const char *name);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiNumScroller(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiNumScroller();

	virtual const char *getClassName()	{ return "GuiNumScroller"; }
	/// the scroller
	GuiScroller *scroller;
	/// the text, which contains the scroller's decimal value
	GuiText *text;

	/// refreshes the number of the text
	DECLSPEC void updatetext();

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiNumScroller((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
private:
	void arrange();	
	void onChange();

	static inline void onChange(void*, BasicObject * data, void*)
	{
		((GuiNumScroller*)data)->onChange();
	}
};

#endif
