/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIDROPDOWN_H
#define GUIDROPDOWN_H

#include "guilist.h"

/**
\ingroup gui
\file guidropdown.h
\brief Defines the class GuiDropDown.
*/

class GuiList;
class GuiText;
class GuiTextInput;
class GuiClickableBtn;
class UiTheme;

/**
\ingroup gui
\class GuiDropDown
\brief A drop down list

*/
class DECLSPEC GuiDropDown : public GuiElement
{
public:
	/**
	Constructor.
	\param r the rect of the list's and text's appearance, relative to the
	parent's top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param editable if true, the selected value can be edited; if false not
	*/
	DECLSPEC GuiDropDown( const SDL_Rect &r, GuiElement *p, const char *name, bool editable = false );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiDropDown(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiDropDown();

	virtual const char *getClassName()	{ return "GuiDropDown"; }
	/**
	Deletes the list. Call this, if the parent is used after deleting this class.
	*/
	DECLSPEC void destroy();
	/**
	Sets the callback function, which is called, when selecting a list item
	\param c the function to call
	\param d the pointer that will be used as second argument in the callback event
	*/
	void setCallback( Callback c, BasicObject *d )	{ cb.func = c; cb.obj = d; }

	/**
	see GuiList::insertStr(str, pos)
	\param str the string
	\param pos the index of the item
	*/
	int insertStr( const char* str, int pos = -1 )	{ return gList->insertStr( str, pos ); }
	/**
	see GuiList::clearStr(pos)
	\param pos the index of the item to erase
	\returns true, if pos was valid
	*/
	bool clearStr( int pos = -1 )	{ return gList->clearStr( pos ); }
	/**
	see GuiList::clearAll()
	*/
	void clearAll()	{ gList->clearAll(); }
	/**
	see GuiList::changeStr(newstr, pos)
	\param newstr the new string to set
	\param pos the index of the item to change
	\returns true, if pos was valid
	*/
	bool changeStr( const char *newstr, int pos )	{ return gList->changeStr( newstr, pos ); }
	/**
	\param pos index of the item to return
	\returns the content of the selected item
	*/
	const char *getStr( int pos ) const	{ return gList->getStr( pos ); }
	/**
	\returns the selected string from the list
	*/
	const char *getSelStr() const	{ return gList->getSelStr(); }
	/**
	\returns the string, that appears in the textbox
	*/
	const char *getString() const;
	/**
	\returns the index of the selected item
	*/
	int getSelId() const	{ gList->getSelId(); }
	/**
	\returns the number of items in the list
	*/
	int getCount() const	{ gList->getCount(); }
	
	/**
	Selects an item.
	\param i the item's index to select
	*/
	void setSelected(int i)	{ gList->setSelected( i );}

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiDropDown((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	void onButton();
	void onList();

	GuiList *gList;
	GuiText *gText;
	GuiTextInput *gtInput;
	GuiCheckableBtn *gcButton;
	GuiPictureBg *gpBg;

	FunctionData cb;

	static void onButton( void *sender, BasicObject *data, void* )
	{
		((GuiDropDown*)data)->onButton();
	}
	static void onList( void *sender, BasicObject *data, void* )
	{
		((GuiDropDown*)data)->onList();
	}
};

#endif
