/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIDYNAMICBUTTON_H
#define GUIDYNAMICBUTTON_H

#include "guibackground.h"
#include "basicapp.h"
#include "sdlaudio.h"

/**
\ingroup gui
\file guidynamicbutton.h
\brief Defines the classes GuiDynamicButton, GuiClickableBtn, GuiCheckableBtn, GuiRadioBtn and GuiMoveableBtn
*/

class SdlImageContainer;
class SdlSurface;
class SdlFont;
class GuiDynamicButton;
class GuiClickableBtn;
class GuiCheckableBtn;

/**
\ingroup gui
\class GuiDynamicButton
\brief A base class for buttons.
*/
class DECLSPEC GuiDynamicButton : public GuiBorder
{
public:
	enum DynamicState{ds_none, /*ds_dynamic,*/ ds_fast} eDynamic;
	enum StateType {normal = 0,pressed = 1,mouseover = 2};

	DECLSPEC GuiDynamicButton( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, SdlImageContainer *b = 0, SdlAudio *u = 0, SdlAudio *d = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, attributes: Caption (string), ButtonBmps (name of surface),
	          ButtonSoundUp (name of sound), ButtonSoundDown (name of sound)
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiDynamicButton(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiDynamicButton();
	
	virtual const char *getClassName()	{ return "GuiDynamicButton"; }

	StateType getState() const	{ return eBtnState; }
	DECLSPEC void setBorderVisible( bool show );
	DECLSPEC void setSounds( SdlAudio *u, SdlAudio *d );
	DECLSPEC void setButtonBmps( SdlImageContainer *b = 0 );
	DECLSPEC void setText( const char *t = 0 );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiDynamicButton((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}

protected:
	DECLSPEC virtual Uint32 onFrame( SdlSurface *dst );
	DECLSPEC virtual Uint32 onGLFrame();
	DECLSPEC virtual Uint32 onButtonPress( Sint32 x, Sint32 y );
	DECLSPEC virtual Uint32 onButtonRelease( bool bInClient );
	DECLSPEC virtual Uint32 onThemeChanged( UiTheme *t );
	virtual DECLSPEC void onInit();
	DECLSPEC virtual Uint32 changeTheme( UiTheme *t, const char *classname );
	/**
	Changes the button state and sets the redraw-state (only if t is not yet the state)
	\param t the state to assign
	*/
	void setBtnState(StateType t)	{if(t!=eBtnState) {eBtnState=t; setRedraw();}}
	/// \returns the button's state
	StateType getBtnState()	{return eBtnState;}
private:
	StateType eBtnState;
	SdlImageContainer *bmps;
	SdlAudio *up;
	SdlAudio *down;
	SdlFont *fNormal, *fHover;
	char *text;

	bool bThemeBmps, bThemeAudios, bThemeFonts;
};

/**
\class GuiClickableBtn
\brief A class for buttons that respond on clicks
*/
class DECLSPEC GuiClickableBtn : public GuiDynamicButton
{
public:
	typedef void (*CallBack)( GuiClickableBtn*, void* );

	DECLSPEC GuiClickableBtn( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = 0, SdlImageContainer *b = 0, SdlAudio *u = 0, SdlAudio *d = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the attribute OnClick (event) can be specified
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiClickableBtn(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiClickableBtn();
	
	virtual const char *getClassName()	{ return "GuiClickableBtn"; }

	void setClickCallback( Callback c )	{ cbClicked.func = c; }
	void setCallbackData( BasicObject* d )	{ cbClicked.obj = d; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiClickableBtn((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onMouseOver();
	virtual DECLSPEC Uint32 onMouseOut();
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onMouseUp( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
	virtual DECLSPEC void onEnabledChanged( bool enabled );

	FunctionData cbClicked;
private:
};

/**
\ingroup gui
\class GuiCheckableBtn
\brief A class for buttons that can be checked
*/
class DECLSPEC GuiCheckableBtn : public GuiDynamicButton
{
public:
	DECLSPEC GuiCheckableBtn( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = 0, SdlImageContainer *b = 0, SdlAudio *u = 0, SdlAudio *d = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the attribute OnCheck (event) can be specified
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiCheckableBtn(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiCheckableBtn();

	virtual const char *getClassName()	{ return "GuiCheckableBtn"; }

	DECLSPEC void setCheck( bool b );
	bool getCheck() const	{ return checked; }
	void setCheckCallback( Callback c )	{ cbChecked.func = c; }
	void setCallbackData( BasicObject * d )	{ cbChecked.obj = d; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiCheckableBtn((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onMouseUp( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
	/**
	 * Called after the checked-state changed.
	 * \param checked the new state (true if checked, false if unchecked)
	 */
	virtual DECLSPEC void onCheck( bool checked );

private:
	FunctionData cbChecked;
	bool checked;
};


/**
\ingroup gui
\class GuiRadioBtn
\brief A class for radio buttons
*/
class DECLSPEC GuiRadioBtn : public GuiCheckableBtn
{
public:
	DECLSPEC GuiRadioBtn(const SDL_Rect &r, GuiRadioContainer *p, const char *name,const char *t = 0, SdlImageContainer *b = 0, SdlAudio *u = 0, SdlAudio *d = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p the element's container
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiRadioBtn(TiXmlElement *el, GuiRadioContainer *p);
	virtual DECLSPEC ~GuiRadioBtn();
	
	virtual const char *getClassName()	{ return "GuiRadioBtn"; }
	
	GuiRadioContainer::button_iter getiter()	{return iter;}
	
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r;
		if(strcmp(o->getClassName(), "GuiRadioContainer") == 0)
			r = new GuiRadioBtn((TiXmlElement*)xmldata, (GuiRadioContainer*)o);
		else	{
			BasicApp::the->logLine("Error: GuiRadioBtn::newguiobject(): parent element's class must be GuiRadioContainer!");
			r = NULL;
		}
		*(GuiElement**)presult = r;
	}

protected:
	virtual DECLSPEC void onCheck( bool checked );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	GuiRadioContainer::button_iter iter;
};

/**
\ingroup gui
\class GuiMovableBtn
\brief A class for buttons that can be moved by the user
*/
class DECLSPEC GuiMoveableBtn : public GuiClickableBtn
{
public:
	enum direction {horizontal, vertikal};

	DECLSPEC GuiMoveableBtn( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = 0, SdlImageContainer *b = 0, SdlAudio *u = 0, SdlAudio *d = 0 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the attribute OnPosChange (event) can be specified
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiMoveableBtn(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiMoveableBtn();

	virtual const char *getClassName()	{ return "GuiMoveableBtn"; }

	DECLSPEC bool setRange( Sint32 xStart, Sint32 yStart, Sint32 xEnd, Sint32 yEnd );
	DECLSPEC void setDirection( direction d );
	void setPosChangeCallback( Callback c, BasicObject *o )	{ cbPoschanged.func = c; cbPoschanged.obj = o; }
	DECLSPEC void setPosToRange();

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiMoveableBtn((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC Uint32 onMouseMove( Uint32 x, Uint32 y );
	virtual DECLSPEC Uint32 onMouseUp( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onButtonPress( Sint32 x, Sint32 y );
	virtual DECLSPEC Uint32 onButtonRelease( bool bInClient );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

	direction dir;
	Sint32 xStart, yStart;
	Sint32 xEnd, yEnd;
	bool tracking;
	Sint32 xTrackingPoint, yTrackingPoint;

private:
	FunctionData cbPoschanged;
};

#endif
