/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999-2006 by Tobias Nadler and Georg Schnattinger             */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIELEMENT_H
#define GUIELEMENT_H

/**
\defgroup gui 2D graphical user interface
\version 0.8

\author Tobias Nadler (nadlerto at gmx dot de) and Georg Schnattinger

The classes of this group are all derived from UiObject. So, as the user interface
group, they act as user interface. These classes are all used to display something
on the screen.

\see ui
*/

#include "SDL/SDL.h"
#include "uiobject.h"

/**
\ingroup gui
\file guielement.h
\brief Defines the class GuiElement.
*/

class TiXmlElement;

/**
\ingroup gui
\class GuiElement
\brief Base class for all graphical user interface objects

All classes, derived from GuiElement, can be used to display 2D-graphics on the screen.
Every graphical element, but one (the top level element), is placed on another one,
called its "parent", whose "child" it is. If an element becomes invisible, all child
elements (and their children, ...) are hidden, too. The same is with the enabled state.
If an element is invisible, the onFrame-event is not called.
If an element is disabled, the mouse-events are not called.
Also an element cannot (or better should not) be drawn beyond the parent's borders.
The method getClip() is used to arrange the rect, in which blitting... should be done.

When deriving a class from GuiElement and implementing an event (a on...() method), do
not forget to call finally the GuiElement::on...() method. Otherwise some features
will not be availible.\n
In order to be able to create elements from an XML-tree, it is required to have a class
constructor with the params TiXmlElement* and GuiElement*. Furthermore it must be
registered in GuiElement::create().

For some more information, have a look at UiObject.

\see GuiManager, UiObject, BasicObject
*/
class DECLSPEC GuiElement : public UiObject
{
public:
	/**
	Constructor.
	There must be UiManager initialized before creating a GuiElement!
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element (a GuiElement cannot have a child derived from UiObject
	         or BasicObject and not from GuiElement), if NULL, GuiManager::the is its
	         parent.
	\param name the element's name
	\see UiObject()
	*/
	DECLSPEC GuiElement( const SDL_Rect &r, GuiElement *p, const char *name );
	/**
	 * Constructor, used to create an element from XML data.
	 * There must be UiManager initialized before creating a GuiElement!
	 * \param el the XML data, the attribute "Rect" is required, optional are
	 * "Visible" (bool) and "Enabled" (bool)
	 * \param p the parent element, or NULL to take GuiManager::the as parent
	 */
	DECLSPEC GuiElement( TiXmlElement *el, GuiElement *p );
	virtual DECLSPEC ~GuiElement();

	virtual const char *getClassName()	{ return "GuiElement"; }
	/// \returns always true
	virtual bool isGuiElement() const	{ return true; }

	/**
	Resizes the rect of appearance of the element. The top-left corner will
	not be changed, but the right and bottom borders. After changing the
	rect, the onResize() event is called.
	\param w the new width
	\param h the new height
	*/
	DECLSPEC void resizeTo( Uint32 w, Uint32 h );
	/**
	Resizes the rect of appearance of the element. The top-left corner will
	not be changed, but the right and bottom borders. After changing the
	rect, the onResize() event is called.
	\param dw the value to add to the width
	\param dh the value to add to the height
	*/
	DECLSPEC void resizeBy( Sint32 dw, Sint32 dh );
	/**
	Moves the rect of appearance of the element. The width and height will
	not be affected, but the relative position. After changing the rect,
	the onResize() event is called.
	\param x the new x coordinate of the rect's left border
	\param y the new y coordinate of the rect's top border
	*/
	DECLSPEC void moveTo( Sint32 x, Sint32 y );
	/**
	Moves the rect of appearance of the element. The width and height will
	not be affected, but the relative position. After changing the rect,
	the onResize() event is called.
	\param x the new x coordinate of the rect's center
	\param y the new y coordinate of the rect's center
	*/
	DECLSPEC void moveRelCenterTo( Sint32 x, Sint32 y );
	/**
	Moves the rect of appearance of the element. The width and height will
	not be affected, but the relative position. After changing the rect,
	the onResize() event is called.
	\param x the value to add to the relative x position
	\param y the value to add to the relative y position
	*/
	DECLSPEC void moveBy( Sint32 x, Sint32 y );
	/**
	Gets the size of the rect of apperance.
	\param w the width will be stored there
	\param h the height will be stored there
	*/
	DECLSPEC void getWndSize( Uint32 &w, Uint32 &h ) const;
	/**
	Gets the relative (to the parent's top-left) rect of apperance.
	\returns the window's relative rect
	*/
	DECLSPEC SDL_Rect getRelWindow() const;
	/**
	Sets the relative (to the parent's top-left) rect of apperance.
	\param r the window's relative rect
	*/
	DECLSPEC void setRelWindow( const SDL_Rect &r );
	/**
	Gets the absolute rect of apperance (distance from the screen's top-left)
	\returns the window's absolute rect
	*/
	DECLSPEC SDL_Rect getAbsWindow();
	/**
	Sets the absolute rect of apperance (distance from the screen's top-left)
	\param r the window's new, absolute rect
	*/
	DECLSPEC void setAbsWindow( const SDL_Rect &r );
	/**
	Get the rect of apperance, relative to the window itself, so the x and y
	coordinates are zero. So the result contains the same information as
	getWndSize(w,h)
	\returns the rect at position 0, 0
	*/
	DECLSPEC SDL_Rect getClientWindow() const;
	/**
	Get the rect of apperance on the surface on which to draw (not limited
	to the surface's borders).
	\returns the rect relative to the surface on which to draw
	*/
	DECLSPEC SDL_Rect getDrawWindow();
	/**
	Get the relative center of apperance.
	\param x the x coordinate will be stored there
	\param y the y coordinate will be stored there
	*/
	DECLSPEC void getRelCenterPoint( Sint32 &x, Sint32 &y );
	/**
	Get the absolute center of apperance.
	\param x the x coordinate will be stored there
	\param y the y coordinate will be stored there
	*/
	DECLSPEC void getAbsCenterPoint( Sint32 &x, Sint32 &y );
	/**
	Get the center of apperance, relative to the rect itself.
	The assigned values equal the half values of getWndSize(x,y).
	\param x the x coordinate will be stored there
	\param y the y coordinate will be stored there
	*/
	DECLSPEC void getClientCenterPoint( Sint32 &x, Sint32 &y );
	/**
	Get the absolute, visible rect of appearance. You should not draw anything
	beyond its borders.
	\returns the clip rect
	*/
	DECLSPEC SDL_Rect getClip();
	/*
	Gets the rect of apperance on the surface where it is drawed.
	\returns the window's blitting rect
	* /
	DECLSPEC SDL_Rect getDrawWindow();*/
	/**
	 * \returns the parent element, or NULL, if there is not parent
	 */
	DECLSPEC GuiElement *getParent() const;
	/**
	 * \returns the top level element (the element, that has no parent)
	 */
	DECLSPEC GuiElement *getToplevel();
	/**
	Changes the visible-state. (All children will become invisible, if this
	element becomes invisible)
	\param b true, to show it; false, to hide it
	*/
	DECLSPEC void setVisible( bool b );
	/**
	 * \returns the visible-state (true, if shown on parent, so if parent
	 * is invisible and so this is invisible, it is also true;
	 * false, if hidden on parent)
	 */
	DECLSPEC bool isVisible() const;
	/**
	Changes the enabled state. (All children will become disabled, if this
	element is disabled)
	\param b true, to enable it; false to disable it
	*/
	DECLSPEC void setEnabled( bool b );
	/// \returns false, if this or a parent is disabled; true, if enabled
	DECLSPEC bool isEnabled() const;
	/**
	 * Sets the redraw-state, so that the element (and it's children) will be
	 * redrawn in the next frame.
	 */
	DECLSPEC void setRedraw();
	/// \returns true, if the element will be redrawn in the next onFrame-event
	bool getRedraw()	{return bRedraw;}
	/**
	 * \returns true, if the element has its own surface to draw on, so that
	 * children need not refresh, if this element has to draw itself to its
	 * parent
	 */
	virtual bool hasSurface() const	{ return false; }

	/**
	Creates a new GuiElement object (or from a derived class) with all information taken
	from the XML element. If the XML element has children, they are recursively created as
	well. The XML code looks like:
	\code
	<TYPE name="name of the object" rect="X,Y,W,H" [ parent="parent name"] [...]>[...]</TYPE>
	\endcode
	("TYPE" is the class name ie. "GuiElement", children can be put into the element,
	further arguments may be required - it depends on the class)
	\param el the XML element with all necessary data
	\param parent the parent object (only used, if not specified in the XML data)
	\returns a pointer to the new element or, if the class is not availible, NULL
	*/
	static GuiElement *create(TiXmlElement *el, GuiElement *parent = NULL);
	/// Behaves as create(), but does not create children elements.
	static GuiElement *createWithoutChildren(TiXmlElement *el, GuiElement *parent = NULL);
	
	// just for usage via BasicSlot
	static void newelement(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiElement((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
	/**
	 * Toggles the element's visibility.
	 * Name of slot: "ToggleVisibility".
	 * Param is ignored.
	 */
	static void togglevisibility(void *sender, BasicObject *aim, void *)
	{
		((GuiElement *)aim)->setVisible( !((GuiElement *)aim)->isVisible() );
	}

protected:
	/**
	This event is called, when the mouse cursor just moved into the object's rect.
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual DECLSPEC Uint32 onMouseOver();
	/**
	This event is called, when the mouse cursor just moved out of the object's rect.
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual DECLSPEC Uint32 onMouseOut();
	/**
	This event is called, when the mouse cursor is moved over the object.
	\param x the absolute x coordinate of the cursor (distance from left)
	\param y the absolute y coordinate of the cursor (distance from top)
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onMouseMove(Uint32 x, Uint32 y)	{ return 0; }
	/**
	This event is called, when the element changed its place or size.
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onResize()		{ return 0; }
	/**
	This event is called, when a frame is drawn. If the object should be
	visible, is has to draw it's look to a valid surface.
	This method calls the children's function, if visible.
	\returns zero, if no error occured; if not zero, drawing is stopped
	\note if you derive the function, then call GuiElement::onFrame(SdlSurface*)
	\b after your class's drawing was done in the onFrame-event, otherwise you
	paint over the children elements
	*/
	virtual DECLSPEC Uint32 onFrame(SdlSurface * dest);
	/**
	 * Like onFrame(SdlSurface *), but for drawing OpenGL-elements.
	 * \note If the viewport or a matrix is changed, they have to be
	 * restored again before returning, i.e. by calling
	 * GuiManager::getThe()->resetGLstates()!
	 */
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC void onDestroy();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
	/**
	 * Called, after the object became enabled or disabled. (Only when
	 * the result of isEnabled() changed.)
	 * \param enabled is true if the object became enabled
	 */
	virtual DECLSPEC void onEnabledChanged( bool enabled );

	/**
	Increments the reference counters of parent GuiSurfaces, to disable their
	off-screen surface blitting. So the onFrame-Method will always be called
	when drawing a frame.
	\see GuiSurface::incUnuseRef()
	*/
	virtual void incRequireOnFrame()	{if(geParent) geParent->incRequireOnFrame();}
	/// Does the opposite of incRequireOnFrame().
	virtual void decRequireOnFrame()	{if(geParent) geParent->decRequireOnFrame();}
	/**
	 * Sets the draw rect dirty, so that it is recalculated at the next call to
	 * getDrawRect(). This method is required by the GuiSurface class.
	 */
	void setDrawRectDirty();

private:
	void setWndDirty();
	GuiElement *elementFromPos( Sint32 x, Sint32 y );

	// the relative window rect
	SDL_Rect wnd;
	bool bVisible, bEnabled;
// Optimization
	bool bClipdirty, bAbsdirty, bDrawdirty;
	SDL_Rect clip;
	SDL_Rect abswnd;
	SDL_Rect drawwnd;

	GuiElement *geParent;

	// whether repainting is necessary, see setRedraw()
	bool bRedraw;

	friend class GuiManager;
};

#endif
