/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIFADER_H
#define GUIFADER_H

#include "uiobject.h"
#include "tinyxml.h"

/**
\ingroup gui
\file guifader.h
\brief Defines the class GuiFader, GAMMA_WHITE, GAMMA_BLACK, GAMMA_INVERT and GAMMA_NORMAL
*/
/// the gamma value to use, if the whole screen should be white
#define GAMMA_WHITE		0x1FFFF
/// the gamma value to use, if the whole screen should be black
#define GAMMA_BLACK		0x0000
/// the standard gamma value (all colors appere without manipulation)
#define GAMMA_NORMAL	0x10000
/// making bitwise or of an intensity and this value, will invert all colors
#define GAMMA_INVERT	0x80000000
//#define FADEFUNC_END	(~0)

/**
\ingroup gui
\class GuiFader
\brief The class to use for manipulating color appearance

Required header file:
\code #include <guifader.h> \endcode

Use this class for manipulating the colors, visible on the display.
Flashes, fadings, ... are easily possible.

\ingroup singleusage
There is only one variable of this type allowed at a time. Do not initialize it
more than once.
*/
class DECLSPEC GuiFader : public UiObject
{
public:
	/** use this function prototype to make individual fadings, the fade will
	endure until false is returned. The intensity value has to be assigned to
	the params r, g and b or can be left unassigned, if the normal value should
	be used.
	\param time the time in msecs since the fade started
	\param r the intensity of the red part of the colors
	\param g the intensity of the green part of the colors
	\param b the intensity of the blue part of the colors
	\return false, if the fade ends; true, if it continues
	*/
	typedef bool (*FadeFunction)( Uint32 time, Uint32 *r, Uint32 *g, Uint32 *b );

	/**
	Constructor. After creating this class once, the onFrame() event is called
	once per frame from GuiManager.
	\param normal the gamma value to use usually (i.e. after fading in)
	*/
	DECLSPEC GuiFader(BasicObject *parent, const char *name, Uint32 normal = 0xFFFF );
	/**
	Constructor. Used for initalisation by XML data. The XML element has to
	contain an attribute "Name" and an attribute "Normal" (with an integer value)
	\param el the XML data
	\param parent pointer to the parent element
	*/
	DECLSPEC GuiFader(TiXmlElement *el, BasicObject *parent);
	virtual DECLSPEC ~GuiFader();

	virtual const char *getClassName()	{ return "GuiFrame"; }
	/**
	Sets the gamma value to the normal value.
	*/
	void setToNormal()	{ setGamma( nNormal ); }
	/**
	Sets the gamma intensity of all colors (red, green and blue) to the given
	value.
	\param intensity the new gamma value, if the most significant bit is set,
	then setGammaInverted is called with the value (see GAMMA_INVERT)
	\returns true, if successful; false if not
	*/
	DECLSPEC bool setGamma( Uint32 intensity );
	/**
	Sets the gamma intensity individually for the colors red, green and blue.
	\param r the new gamma value for the red part of a color
	\param g the new gamma value for the green part of a color
	\param b the new gamma value for the blue part of a color
	\returns true, if successful; false if not
	
	\todo enable inverted colors
	*/
	DECLSPEC bool setGamma( Uint32 r, Uint32 g, Uint32 b );
	/**
	Sets the gamma intensity of all colors (red, green and blue) to the given
	value.
	\param intensity the new gamma value; GAMMA_INVERT makes no sense here
	\returns true, if successful; false if not
	*/
	DECLSPEC bool setGammaInverted( Uint32 intensity );

	/**
	Turns the colors darker until all is black (after ms msecs).
	\param ms the duration of the fade in msecs
	*/
	DECLSPEC void fadeOut( Uint32 ms );
	/**
	Turns the colors to normal until the intensity is normal (after ms msecs).
	\param ms the duration of the fade in msecs
	*/
	DECLSPEC void fadeIn( Uint32 ms );
	/**
	Turns the colors to "end" until the intensity equals "end" (after ms msecs).
	\param ms the duration of the fade in msecs
	\param end the intensity at the end of the fade
	*/
	DECLSPEC void fade( Uint32 ms, Uint32 end );
	/**
	Sets the gamma values as set by the fade function.
	\param func the function, that gives the values
	*/
	DECLSPEC void fadeByFunction( FadeFunction func );

	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static GuiFader *the;
	/**
	\returns a pointer to the GuiFader-variable, or NULL if not initialized.
	(There is only one GuiFader-variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	DECLSPEC static GuiFader *getThe();

	/// the normal gamma value
	Uint32 nNormal;
protected:
	virtual DECLSPEC Uint32 onFrame();

private:
	FadeFunction pFunction;

	bool bFading;
	Uint32 tDuration, tEnd;
	Uint32 nEnd, nStart;
	Uint32 nNow;

	Uint16 r[256],
		g[256],
		b[256];
		
	friend class GuiManager;
};

#endif
