/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2006 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIFLOWMENU_H
#define GUIFLOWMENU_H

#include <map>
#include "guibackground.h"

/**
 * \ingroup gui
 * \file guiflowmenu.h
 * \brief Defines the class GuiFlowMenu.
 */

class GuiFlowMenuItem;

/**
 * \class GuiFlowMenu
 * \ingroup gui
 * \brief A class for showing a menu that (dis-)appears by flying out of/into the screen.
 * 
 * \todo Add constructor without xml-usage.
 * \note This class may cause a crash, after showItem() failed
 */
class GuiFlowMenu: public GuiFlowWindow
{
public:
	/**
	 * Constructor. Uses XML data. If the attribute XmlFile is specified, the
	 * menu-items are created as specified in that XML-file: The root element
	 * must be "GuiFlowMenu", the children (1st generation) must be "MenuItem"s
	 * with an attribute "ID" (the ID of the item, see addMenuItem()) and the
	 * children 2nd generation must be usual "Gui*"-elements, but only one
	 * child per menu-item is observed (the 1st one).
	 * \param el the XML data for initialisation, the following attributes are
	 * read: XmlFile (string, filename), StartMenuID (int), ShowRect (rect),
	 * DisappearRect (rect); Events: OnChooseNext, OnNoNext, OnFlewIn,
	 * OnDisappeared, OnBack.
	 * \param p parent element
	 * \see GuiFlowWindow(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiFlowMenu(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFlowMenu();
	
	virtual const char *getClassName()	{ return "GuiFlowMenu"; }
	/// \returns the ID of the currently active item
	long getActiveItemID()	{return nID;}
	/// \returns the currently active GuiElement
	GuiFlowMenuItem *getActiveItem()	{return mapMenues[nID];}
	/**
	 * Sets the duration of the menu's (dis)appearing.
	 * \param t the time in millisecs
	 */
	void setFlyTime(Uint32 t)	{nTime = t;}
	/**
	 * Sets the rect to which the window flies, when it appeares.
	 * That is the position, where it stays when being visible to the user.
	 * \param r the rect
	 */
	void setShowRect(const SDL_Rect &r)	{rShow = r;}
	/**
	 * Sets the rect to which the window flies, when disappear() is called.
	 * This is the same rect, where it starts flying from, when appear() is called.
	 * \param r the rect
	 */
	void setDisappearRect(const SDL_Rect &r)	{rDisappear = r;}
	/// Lets the window appear
	DECLSPEC void appear();
	/// Lets the window disappear
	DECLSPEC void disappear();
	/**
	 * Sets the callback function that is called, after the window flew out of
	 * the screen.
	 * \param fd the callback data
	 * \note the 3rd param of the callback-function points to an integer value
	 * equal to the result of getActiveItemID() (that is the ID of the window
	 * that just flew out).
	 */
	void setChooseNextCallback(const BasicObject::FunctionData &fd)	{cbChooseNext = fd;}
	/**
	 * Sets the callback function that is called, when there is no valid
	 * next-ID. So it does not have a menu to display. You can use this
	 * callback i.e. to start the game.
	 * \param fd the callback data
	 * \note the 3rd param of the callback-function points to an integer value
	 * equal to the result of getActiveItemID() (that is the ID of the window
	 * that just flew out), if it is changed to ID_INVALID in the callback,
	 * then the window will fly back with the last view, otherwise not.
	 */
	void setNoNextCallback(const BasicObject::FunctionData &fd)	{cbNoNext = fd;}
	/**
	 * Sets the callback function that is called, after the window flew into
	 * the screen.
	 * \param fd the callback data
	 * \note the 3rd param of the callback-function points to an integer value
	 * equal to the result of getActiveItemID().
	 */
	void setFlewInCallback(const BasicObject::FunctionData &fd)	{cbFlewIn = fd;}
	/**
	 * Sets the callback function that is called, after the window flew back
	 * into the screen.
	 * \param fd the callback data
	 */
	void setBackCallback(const BasicObject::FunctionData &fd)	{cbBack = fd;}
	//BEGIN declaration of slots
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFlowMenu((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
	static void flyout_back(void *sender, BasicObject *aim, void *)
	{
		((GuiFlowMenu*)aim)->fly('b');
	}
	static void flyout_next(void *sender, BasicObject *aim, void *)
	{
		((GuiFlowMenu*)aim)->fly('n', sender);
	}
	static void flyout_disappear(void *sender, BasicObject *aim, void *)
	{
		((GuiFlowMenu*)aim)->fly('d');
	}
	static void flyin(void *sender, BasicObject *aim, void *)
	{
		((GuiFlowMenu*)aim)->fly('s');
	}
	//END declaration of slots
	/// ID of invalid
	static const long ID_INVALID;
		
protected:
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onPosReached();
	
private:
	/**
	 * Adds an element to the menu. If another entry with the same id already exists, it will remain
	 * and the specified element is not added.
	 * \param id the identifier, the menu should be associated with
	 * \param el the element to show/hide, when (not) required (should be a child of the menu object)
	 * \returns true on success, false if the id is already in use
	 */
	DECLSPEC bool registerItem(long id, GuiFlowMenuItem * el);
	/**
	 * Removes the menu-item with the specified id.
	 * \param id the identifier of the item to remove
	 * \returns true on success, false if there is no item with the specified identifier
	 */
	DECLSPEC void unregisterItem(long id);
	
	// param is not always required (requirement depends on where_to)
	DECLSPEC void fly(char where_to, void *param = NULL);
	void createItemsByFile();
	bool showItem(long id, bool savehistory);
	long getIDbySender( BasicObject *sender );
	
	char *strXmlFilename;
	TiXmlDocument xmlFile;
	BasicObject::FunctionData cbFlewIn, cbDisappear, cbBack,
		cbChooseNext,// called, when the next menu must be chosen (3rd param: &next_ID)
		cbNoNext;// called, when there is no valid next-ID (i.e. you can start the game from here)
	
	SDL_Rect rShow, rDisappear;
	
	std::map<long, GuiFlowMenuItem*> mapMenues;
	long nID, nIDnext;// currently active item-ID
	GuiFlowMenuItem *pCurr;
	
	std::list<long> lLastItemIDs;
	
	enum MenuState{sNone = '-', sShow = 's', sBack = 'b', sNext = 'n', sDisappeared = 'd'} state;
	
	friend class GuiFlowMenuItem;
};

/**
 * \class GuiFlowMenuItem
 * \brief A single menu item for a GuiFlowMenu
 */
class GuiFlowMenuItem : public GuiElement
{
	public:
		/**
		 * Constructor. Automatically registers itself in the parent element.
		 * \param el the xml data with the following attributes:
		 * ID (int, required), Activator (string), NextRect and BackRect (Rect)
		 * \param parent the GuiFlowMenu on which this object is placed
		 */
		GuiFlowMenuItem( TiXmlElement *el, GuiFlowMenu *parent );
		/**
		 * Constructor. Automatically registers itself in the parent element.
		 * \param r the geometry
		 * \param parent the GuiFlowMenu on which this object is placed
		 * \param id the ID of this object
		 */
		GuiFlowMenuItem( const SDL_Rect &r, GuiFlowMenu *parent, const char *name, long id );
		virtual ~GuiFlowMenuItem();
		/// \returns the ID of the object
		long getID()	{return nID;}
		/// \param obj the object to compare
		/// \returns true, if obj is the object, where strActivator's path
		/// points at.
		bool activatorCompare( BasicObject *obj );
		/**
		 * Sets the rect the window comes from after a "back"-call, and
		 * to which the window flies, when receiving a "next"-signal.
		 * \param r the rect
		 */
		void setBackRect(const SDL_Rect &r)	{rBack = r;}
		/**
		 * Sets the rect the window comes from after a "next"-call, and
		 * to which the window flies, when receiving a "back"-signal.
		 * \param r the rect
		 */
		void setNextRect(const SDL_Rect &r)	{rNext = r;}
		/**
		 * \returns the rect where the menu comes from, when clicking "next"
		 */
		const SDL_Rect &getRectNext()	{return rNext;}
		/**
		 * \returns the rect where the menu comes from, when clicking "next"
		 */
		const SDL_Rect &getRectBack()	{return rBack;}
		
		// the slot for creating
		static void newguiobject(void *presult, BasicObject *o, void *xmldata)
		{
			GuiElement *r = new GuiFlowMenuItem((TiXmlElement*)xmldata, (GuiFlowMenu*)o);
			*(GuiElement**)presult = r;
		}
		
	protected:
		virtual void onInit();
		
	private:
		long nID;
		bool bFailed;
		char *strActivator;
		SDL_Rect rNext, rBack;
};


#endif
