/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIMANAGER_H
#define GUIMANAGER_H

#include <list>
#include "SDL/SDL.h"
#include "GL/gl.h"
#include "uiobject.h"
#include "tinyxml.h"

/**
 * \ingroup gui
 * \file guimanager.h
 * \brief Defines the class GuiManager.
 */

class GuiElement;
class UiManager;

/**
 * \ingroup gui
 * \class GuiManager
 * \brief Manages 2D graphics functions and user actions
 * 
 * \see UiManager, GuiElement
 */
class DECLSPEC GuiManager : public UiObject
{
public:
	/**
	 * Constructor. Registers itself at the UiManager.
	 * There must be UiManager initialized before creating a GuiElement!
	 * \param parent pointer to the UiManager
	 * \param name the object's name ("GuiManager" by default)
	 */
	DECLSPEC GuiManager( UiManager *parent, const char *name = "GuiManager" );
	virtual DECLSPEC ~GuiManager();
	
	virtual const char *getClassName()	{ return "GuiManager"; }
	/**
	 * Initializes this class and, if el is specified creates elements.
	 * The XML data can be structured as follows (for a screen with the size
	 * 1024x768):
	 * \code
	 * <GuiManager GLClearColor="#000000" GLClearDepth="false">
	 * 	<Front>
	 * 		<GuiElement rect="0;0;1024;768" name="front">
	 * 			[front elements like cursor, hint...]
	 * 		</GuiElement>
	 * 	</Front>
	 * 	<Main>
	 * 		<GuiElement rect="0;0;1024;768" name="main">
	 * 			[window elements...]
	 * 		</GuiElement>
	 * 	</Main>
	 * 	<GuiFader name="fader" normal="0xFFFF"/>
	 * </GuiManager>
	 * \endcode
	 * \param el XML data for the elements to create
	 * \note onInit() is not called in this method, this is not necessary.
	 */
	DECLSPEC bool init(TiXmlElement *el = NULL);
	/**
	 * Sets the elements that recieve onFrame-events from the GuiManager.
	 * \param front the front element (for cursor, hints...)
	 * \param main the main element (for windows...)
	 */
	DECLSPEC void setBaseElements(GuiElement *front, GuiElement *main);
	/// \returns a pointer to the top front element (that contains cursor, hint...)
	GuiElement *getFront()	{return geFront;}
	/// \returns a pointer to the top main element (that contains windows...)
	GuiElement *getMain()	{return geMain;}
	/**
	 * Sets the mouse focus to the specified element. If an element has the focus, its
	 * onMouseMove() function is called, when the mouse is moved, but the other element's
	 * onMouseMove() function is not called. (If no element has the focus, onMouseMove()
	 * of the element is called, over which the cursor is - after the move)
	 * \param el the element to focus
	 */
	DECLSPEC void focus(GuiElement *el);
	/**
	 * Sets the mouse focus to NULL, if the specified element had the focus.
	 * \param el the element
	 * \returns true if the element had the focus
	 */
	DECLSPEC bool release(GuiElement *el);
	/// \returns the element, currently having the mouse focus
	GuiElement *getFocus()	{return geMouseFocus;}
	/**
	 * Sets the mouse cursor. The cursor recieves all mouse actions.
	 * \param cur the cursor
	 */
	DECLSPEC void setCursor(GuiElement *cur);
	/// \returns the currently used mouse cursor
	GuiElement *getCursor()	{ return geCursor; }
	/**
	 * Cleans up all associations to the specified element. This is done, when
	 * an element is deleted. This method shall not be called from somewhere
	 * else but ~GuiElement().
	 * \param ge the element to clean up
	 */
	DECLSPEC void cleanupElement(GuiElement *ge);
	
	/**
	 * Sets the OpenGL-states (viewport and projection-matrix) to the default
	 * values to be used for 2D-GUI-drawing.
	 * This method implicitly calls the static method with the screen's size
	 * as parameters.
	 */
	DECLSPEC void resetGLstates();
	/**
	 * Sets the OpenGL-states (viewport and projection-matrix) for drawing 2D
	 * objects with parameters as screen-pixel-values.
	 * \param w the width and right boundary of the viewport
	 * \param h the height and top boundary of the viewport
	 */
	DECLSPEC static void resetGLstates( Uint16 w, Uint16 h );
	
	/**
	 * Not accessible in Windows (dynamic linking), equals the result of getThe()
	 */
	static GuiManager *the;
	/**
	 * \returns a pointer to the GuiManager-variable, or NULL if not initialized.
	 * (There is only one GuiManager-variable allowed in an application)
	 * 
	 * Every netrinjo-class that can only exist once at a time, has this static function.
	 * \see singleusage
	 */
	DECLSPEC static GuiManager *getThe();
protected:
	virtual Uint32 onKeyDown( SDL_KeyboardEvent *ev )	{return 0;}
	virtual Uint32 onKeyUp( SDL_KeyboardEvent *ev )	{return 0;}
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onMouseUp( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onMouseMotion(SDL_MouseMotionEvent *ev);
	virtual DECLSPEC Uint32 onFocus(FocusTypes gained);
	virtual DECLSPEC Uint32 onRelease(FocusTypes lost);
	virtual DECLSPEC Uint32 onFrame();
	virtual Uint32 onThemeChanged( UiTheme *t )	{return 0;}
	virtual DECLSPEC void onDestroy();

private:
	Uint32 glFrame();
	Uint32 surfFrame(SdlSurface *dest);
	
	Uint32 nDisplayWidth;
	Uint32 nDisplayHeight;
	// true if OpenGL is used, otherwise 2D-blitting is done
	bool bOGL;

	// base element for the windows, buttons,...
	GuiElement *geMain;
	// base element for objects that appear in front of the windows: cursor, hints,...
	GuiElement *geFront;
	// the element, that currently has the mouse focus (changes when focus() or release() is called)
	GuiElement *geMouseFocus;
	// the element, over which the curser is
	GuiElement *geMouseOver;
	// the element that shows the mouse cursor
	GuiElement *geCursor;
	// currenly owning focustypes of UiManager
	FocusTypes ftCaptured;
	
	// for OpenGL only:
	// buffers to clear after each frame
	GLbitfield bfClearBuffers;
};

#endif
