/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIMENU_H
#define GUIMENU_H

#include "guidynamicbutton.h"

/**
 * \ingroup gui
 * \file guimenu.h
 * \brief Defines the classes GuiMenuBar and GuiMenuItem.
 */

class GuiMenuItem;
class GuiMenuBar;

/**
 * \class GuiMenuBar
 * \brief A container class for GuiMenuItems, to show menus.
 */
class DECLSPEC GuiMenuBar : public GuiElement
{
public:
	/**
	 * Constructor.
	 * \param horizontal if true, the menu-items are shown in a horizontal line
	 */
	DECLSPEC GuiMenuBar( const SDL_Rect &r, GuiElement *parent, const char *name, bool horizontal = true );
	virtual DECLSPEC ~GuiMenuBar();
	
	virtual const char * getClassName()	{ return "GuiMenuBar"; }
	
	/// Hides all it's submenus
	DECLSPEC void hideSubmenus();
	/**
	 * Shows the bar next to the specified rect.
	 * \param r the rect
	 * \param parent_horizontal true if the parent's alignment is horizontal
	 * (it tries not to overlap with the parent)
	 */
	DECLSPEC void showAt( const SDL_Rect &r, bool parent_horizontal );
	/**
	 * Sets the function to be called when an item is selected/clicked on.
	 * The sender (1st param) of the callback will be the clicked item (not
	 * this menu-bar). The 3rd param is the item's data, as you get it with
	 * GuiMenuItem::getItemData().
	 * \param fd the function data
	 */
	DECLSPEC void setCallback( const FunctionData &fd )	{ fdSelect = fd; }
	/// \returns true if its items are aligned in a row; false if in a column
	bool isHorizontalAligned()	{ return bHorizontal; }
	
protected:
	/**
	 * Called when an item has been selected/clicked. But not if the item
	 * openes a submenu.
	 */
	virtual DECLSPEC void onSelect( GuiMenuItem *selected );
	
private:
	// refreshes the positions of it's items and it's visibility
	void refresh();
	// sets the positoion and adds it to lItems
	void add( GuiMenuItem *item );
	// true if the items are listed in a row; false if in a column
	bool bHorizontal;
	std::list<GuiMenuItem*> lItems;
	// pointer to an item that makes this menu appear, NULL if there is none
	GuiMenuItem *pActivator;
	
	FunctionData fdSelect;
	
	friend class GuiMenuItem;
};


/**
 * \class GuiMenuItem
 * \brief Used to show items of menus.
 */
class DECLSPEC GuiMenuItem: public GuiElement
{
public:
	enum TYPE {tCheckable, tClickable, tSubmenu};
	/**
	 * Constructor.
	 * \param checkable the kind of item this should be, checkable or clickable
	 * Other params see GuiDynamicButton::GuiDynamicButton()
	 */
	DECLSPEC GuiMenuItem( const SDL_Rect &r, GuiMenuBar *parent, const char *name, bool checkable = false,
						  const char *txt=0, SdlImageContainer *b=0, SdlAudio *u=0, SdlAudio *d=0 );
	/**
	 * Constructor for submenu-items.
	 * \param submenu a pointer to the submenu to be associated with this
	 * object or NULL if a new menu-bar should be created (that will be
	 * destroyed with this object).
	 * If a new menu-bar is created, it will be placed onto the top-level
	 * element of the GuiManager's main element-tree.
	 * 
	 * Other params see GuiDynamicButton::GuiDynamicButton()
	 */
	DECLSPEC GuiMenuItem( const SDL_Rect &r, GuiMenuBar *parent, const char *name, GuiMenuBar *submenu,
						  const char *txt=NULL, SdlImageContainer *b=0, SdlAudio *u=0, SdlAudio *d=0 );
	virtual DECLSPEC ~GuiMenuItem();
	
	virtual const char * getClassName()	{ return "GuiMenuItem"; }
	/// \returns the type of the item.
	TYPE getType()	{ return type; }
	/**
	 * \returns the item's data-pointer (just a pointer assigned to this object)
	 */
	void * getItemData()	{ return pData; }
	/**
	 * Sets the item's data-pointer (that is just something you might need).
	 * When selected, this value is the 3rd param of the callback, as specified
	 * in GuiMenuBar::setCallback()
	 * \param data the new ID
	 */
	void setItemData( void *data )	{ pData = data; }
	/**
	 * Sets whether or not the user can access the item. (The item will be
	 * invisible if not).
	 * \param cansel the new value
	 */
	DECLSPEC void setSelectable( bool cansel );
	/**
	 * Checks or unchecks the item. This is only possible, if the item was
	 * created with checkable=true, otherwise nothing will happen.
	 */
	DECLSPEC void setChecked( bool checked );
	/**
	 * \returns the associated 
	 */
	GuiMenuBar *getBar()	{ return pSubmenu ? pSubmenu : gmbSubmenu; }
	/**
	 * Changes the image of the item's button.
	 * \param b the new images for the different button-states
	 * \see GuiDynamicButton::setButtonBmps()
	 */
	DECLSPEC void setButtonBmps( SdlImageContainer *b );
	
protected:
	virtual void onDestroy();
	virtual Uint32 onThemeChanged( UiTheme *t );

private:
	static void OnClicked( void *sender, BasicObject *aim, void * )
	{
		((GuiMenuItem*)aim)->onClicked();
	}
	void onClicked();
	// called only if type==tSubmenu and when the submenu is closed (hidden)
	void onSubClose( GuiMenuItem *selected );
	// hides other menus (necessary when the submenu appears)
	void hideSiblingMenus();
	
	GuiDynamicButton *gdbMain;
	// pointer to independent menu
	GuiMenuBar *pSubmenu;
	// pointer to self-created menu (will be destroyed with this)
	GuiMenuBar *gmbSubmenu;
	//GuiText *gtDescription;
	TYPE type;
	// pointer to some data
	void *pData;
	// true while setCheck is processed, to avoid callback
	bool bCheckChanging;
	
	friend class GuiMenuBar;
};


#endif
