/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999-2006 by Tobias Nadler and Georg Schnattinger             */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUISURFACE_H
#define GUISURFACE_H

#include "guielement.h"

/**
 * \ingroup gui
 * \file guisurface.h
 * \brief Defines the class GuiSurface.
 */

class TiXmlElement;

/**
 * \ingroup gui
 * \class GuiSurface
 * \brief A simple class for reducing the number of blits and gl-operations
 * respectivly (per frame).
 * 
 * By creating an element of this class, in framebuffer-mode, the children
 * elements draw onto an off-screen surface and thus, only need to draw, if
 * this off-screen surface changes. In OpenGL-mode the gl-operations made by
 * the child elements are compiled into a list, which is called on each frame.
 * 
 * If you have some elements that do not change their look very often, it is
 * efficient to put them onto a GuiSurface.
 * The usage of the off-screen surface can also be temporarily disabled.
 * 
 * Experiance with using SdlSurface:\n
 * In framebuffer-mode, using SdlSurface-objects can improve the performance
 * significantly, even when having just a few elements.
 * In OpenGL-mode the performance increases hardly when only drawing the
 * usual netrinjo-GUI-elements (this could be improved by doing
 * render-to-texture instead of list-compiling).
 * 
 * Deriving a class from this does probably not make much sense.
 */
class DECLSPEC GuiSurface : public GuiElement
{
public:
	/**
	 * Constructor.
	 * \param r the rect of the element's appearance, relative to the parent's
	 * top-left position
	 * \param p the parent element (a GuiElement cannot have a child derived from UiObject
	 *          or BasicObject and not from GuiElement), if NULL, GuiManager::the is its
	 *          parent.
	 * \param name the element's name
	 */
	DECLSPEC GuiSurface( const SDL_Rect &r, GuiElement *p, const char *name );
	/**
	 * Constructor, used to create an element from XML data.
	 * \param el the XML data
	 * \param p the parent element, or NULL to take GuiManager::the as parent
	 */
	DECLSPEC GuiSurface( TiXmlElement *el, GuiElement *p );
	virtual DECLSPEC ~GuiSurface();

	virtual const char *getClassName()	{ return "GuiSurface"; }

	/**
	 * \returns true, if the off-screen surface is used
	 */
	virtual bool hasSurface() const	{ return nUnuseCounter == 0; }
	/**
	 * Increments the reference counter, indicating how many children currently
	 * require the onFrame() method (this disables the functionality of this
	 * object). If the reference counter is zero, the off-screen surface is used.
	 */
	DECLSPEC void incUnuseRef()	{ if( nUnuseCounter++ == 0 ) setDrawRectDirty(); }
	/// Does the opposite of incUnuseRef().
	DECLSPEC void decUnuseRef()	{ if( nUnuseCounter && --nUnuseCounter == 0 ) setDrawRectDirty(); }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiSurface((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	/**
	 * \returns the surface, where the child elements draw onto (only valid in
	 * framebuffer-mode)
	 */
	virtual SdlSurface * getDestSurface()	{return static_data.surfDest;}
	/**
	 * \returns the ID of the OpenGL-list, that is constructed out of the child
	 * elements' gl-operations (only valid in OpenGL-mode)
	 */
	virtual GLuint getGLListID()	{return static_data.nListID;}
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onFrame(SdlSurface *dest);
	virtual DECLSPEC Uint32 onGLFrame();
	virtual void incRequireOnFrame()	{GuiElement::incRequireOnFrame(); incUnuseRef();}
	virtual void decRequireOnFrame()	{GuiElement::decRequireOnFrame(); decUnuseRef();}

private:
	// reference counter
	unsigned int nUnuseCounter;
	union
	{
		// the surface to draw on - in framebuffer-mode
		SdlSurface *surfDest;
		// the list to compile - in OpenGL-mode
		GLuint nListID;
	} static_data;
};

#endif
