/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUITEXT_H
#define GUITEXT_H

#include "guibackground.h"
#include "sdlfont.h"
#include "frametimer.h"

/**
\ingroup gui
\file guitext.h
\brief Defines the classes GuiText, GuiTextInput, GuiKeyControl and GuiFrameCounter
*/

class GuiText;
class GuiTextInput;

/**
\ingroup gui
\class GuiText
\brief Displays a text

*/
class DECLSPEC GuiText : public GuiElement
{
public:
	/// see SdlFont::VAlign
	typedef SdlFont::VAlign VAlign;
	/// see SdlFont::HAlign
	typedef SdlFont::HAlign HAlign;
	/**
	Font sizes. \n
	\a fs_theme the theme default font \n
	\a fs_standard the standard font should be used \n
	\a fs_text the font for texts should be used (usually a small font) \n
	\a fs_button the font for buttons should be used (usually a font of average size) \n
	\a fs_heading the font for headings should be used (usually a font of big size)
	*/
	enum FontSize{fs_theme = -1, fs_standard = 0, fs_text, fs_button, fs_heading};

	/**
	Constructor.
	\param r the rect of the element's appearance, relative to the parent's
	top-left position
	\param p the parent element; or NULL, if it will be used as top level
	\param t the text to display; NULL, if no text should be displayed
	\param font the font to use
	\param light specifies whether to use the light font or the dark one
	\param stdAlign if true, the theme's default align for this class is used
	*/
	DECLSPEC GuiText( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = 0, FontSize font = fs_theme, bool light = false, bool stdAlign = true );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: ThemeAlign (bool), FontSize, LightFont (bool), Text (string),
	          VAlign, HAlign
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiText(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiText();

	virtual const char *getClassName()	{ return "GuiText"; }
	/**
	Changes the displayed text.
	\param t the new text string (maximum possible length is 65535 signs)
	*/
	DECLSPEC void setText( const char *t );
	/**
	Adds text at the end of the current text.
	\param t the text string to concatenate
	*/
	DECLSPEC void addText( const char *t );
	/**
	Adds text in a new line at the end of the current text.
	\param t the text string to add
	*/
	DECLSPEC void addLine( const char *t );
	/**
	Clears a line from the text.
	\param l the line's index (starting with 0 for the first line, last line is numLines()-1)
	\returns true, if the line could be removed
	*/
	DECLSPEC bool clearLine( int l = 0 );
	/// \returns the number of lines, the text has
	DECLSPEC int numLines();
	/**
	Changes the alignments of the text.
	\param v the vertical alignment
	\param h the horizontal alignment
	*/
	void setAlign( VAlign v, HAlign h )	{eVAlign = v; eHAlign = h;}
	/// \returns the current text
	const char *getText() const	{ return pText ? pText : ""; }
	/**
	Changes the font.
	\param size the font's size category
	\param light whether it should be light or dark
	*/
	DECLSPEC void chooseFont( FontSize size, bool light );
	/**
	Changes the font by specifying the font directly.
	\param f the font to use
	*/
	DECLSPEC void setFont( SdlFont *f );
	/**
	Specifies, whether or not to start a new line, when a line does not fit
	into the element's rect.
	\param active if true, word wrapping is enabled; if false it is not
	*/
	virtual void setWordWrap( bool active )	{ bWordWrap = active; }
	
	/**
	Converts a string to a kind of font size. Possible strings are:
	standard; text; button; heading; theme.
	\param s the string to convert
	\returns the corresponding value or fs_standard, if the expression is unknown
	*/
	static DECLSPEC FontSize strToSize( const char *s );
	/**
	Fetches the font, that is used in the theme, has the size and is light or
	dark (as specified).
	\param s the font's size category
	\param l whether it must be light (true) or dark (false)
	\param t the theme, in which the font must be deklared
	\returns the font
	*/
	static DECLSPEC SdlFont *fontSizeToFont( FontSize s, bool l, UiTheme *t );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiText((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
	/// Used to set the text directly by callbacks that support a string as
	/// last param.
	static void settext(void *sender, BasicObject *aim, void *text)
	{
		((GuiText*)aim)->setText( (const char*)text );
	}

protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

	/// the size of one font letter
	int nCharWidth, nCharHeight;
	/// the position of the cursor, or -1 if invisible
	int curpos;
	/// whether to show the cursor or not
	bool showcursor;
	/// whether to use word wrapping or not
	bool bWordWrap;

private:
	VAlign eVAlign;
	HAlign eHAlign;
	char *chBuffer, *pText;
	int nBufferSize;
	bool bLightFont;
	FontSize eFontSize;
	SdlFont *font;

	bool bThemeAlign, bThemeFont;
};

/**
\ingroup gui
\class GuiTextInput
\brief A field, where the user can write text into

*/
class DECLSPEC GuiTextInput : public GuiText
{
public:
	DECLSPEC GuiTextInput( const SDL_Rect &r, GuiElement *p, const char *name, const char *t = NULL, bool singleline = true);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: MaxLen (int), SingleLine (bool), OnReturn (event)
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiTextInput(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiTextInput();

	virtual const char *getClassName()	{ return "GuiTextInput"; }
	
	void setCallback( Callback c )	{ cb.func = c; }
	void setCbData( BasicObject* d )	{ cb.obj = d; }
	/**
	Sets the maximum number of letters, the box may contain.
	\param l the maximum length of the text, -1 for infinite
	*/
	DECLSPEC void setMaxlen( int l );

	static Uint32 nBlinkIv;
	static char alpha[], ALPHA[], altAlpha[];
	static const unsigned int nKeys, nModKeys;
	static char MOD_LETTER;

	int bwActive, bwInactive;// if & 0x80000000 then
	Sint32 cActive, cInactive;// set by theme

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiTextInput((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onKeyDown( SDL_KeyboardEvent *ev );
	virtual DECLSPEC Uint32 onKeyUp( SDL_KeyboardEvent *ev );
	virtual DECLSPEC Uint32 onFocus( FocusTypes gained );
	virtual DECLSPEC Uint32 onRelease(FocusTypes lost);
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	static void refresh(void *, BasicObject *aim, void*)	{
		((GuiTextInput*)aim)->showcursor = !((GuiTextInput*)aim)->showcursor;
		((GuiTextInput*)aim)->setRedraw();
	}
	FunctionData cb;
	FrameTimer::CbInfo *blinkTimer;

	bool hasfocus;
	bool bSingleLine;
	int maxlen;
	static void setInfo();
	friend class guiinfo;
};

/**
\ingroup gui
\class GuiKeyControl
\brief Displays a key that can be modified by the user

*/
class DECLSPEC GuiKeyControl : public GuiText 
{
public:
	DECLSPEC GuiKeyControl(const SDL_Rect &r, GuiElement *p, const char *name);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiKeyControl(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiKeyControl();

	virtual const char *getClassName()	{ return "GuiKeyControl"; }

	SDL_keysym getKey() const	{return key;}
	DECLSPEC void setKey( const SDL_keysym &k );
	
	static DECLSPEC const char *alpha[];
	static DECLSPEC const SDLKey code[];
	static DECLSPEC const unsigned int count;

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiKeyControl((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
	
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onKeyDown( SDL_KeyboardEvent *ev );
	virtual DECLSPEC Uint32 onFocus();
	virtual DECLSPEC Uint32 onRelease();
	virtual DECLSPEC void onInit();
	
private:
	void update();
	bool hasfocus;
	SDL_keysym key;
};

/**
\ingroup gui
\class GuiFrameCounter
\brief Prints the current framerate onto the screen.

The number is the average amount of frames per second during the last 30 frames.
*/
class DECLSPEC GuiFrameCounter : public GuiText
{
public:
	DECLSPEC GuiFrameCounter(const SDL_Rect &r, GuiElement *p, const char *name);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiFrameCounter(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFrameCounter();

	virtual const char *getClassName()	{ return "GuiFrameCounter"; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFrameCounter((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	
private:
	long frame;
	Uint32 time;
};

#endif
