/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETMANAGER_H
#define NETMANAGER_H

#include "SDL/SDL_net.h"
#include <list>
#include "basicobject.h"

/**
\ingroup net
\file netmanager.h
\brief Defines the classes NetManager and NetSimpleManager, MAX_PEERS and NETWORK_PORT
*/
/// Defines the number of connections allowed
#define MAX_PEERS			127
/// Defines the game client's default port to listen for incomming connection requests
#define NETWORK_PORT		8457
/**
Defines the default port to use for UDP transmission
(the UDP-port must be the one after the TCP-port when using UDP with NetSockets)
*/
#define NETWORK_UDPPORT		(NETWORK_PORT+1)
/// Defines the game server's default port to listen for incomming connection requests
#define NETWORK_SERVERPORT	8455

using namespace std;

class NetSocket;
struct NetBasicData;

/**
\defgroup net Networking
\todo testing, looking for errors

\author Tobias Nadler (nadlerto at gmx dot de)
\brief This group contains classes and structs for network transfer

At first some important notices:

\b Abbreviations: \n
\a NBO indicates the values being in Network-Byte-Order \n
\a OSBO indicates Operating System dependant Byte-Order

\b Common \b Net-rules: \n
A variable of IPaddress always contains it's data in Network-Byte-Order,
while the single occurance of a Uint32 host or a int port indicates an OS-dependent Byte-Order.\n
If a port is an "Uint16"-value, then it should be in NBO, if it is an "int"-value, then it should be in OSBO.
If this is not the case, there must be a comment that makes the usage clear.

To use the netrinjo network classes, you have to initialize one class (not more classes)
derived from NetBasicManager (NetSimpleManager, NetPeerManager or a class of your own).
After that was done you can use functions based on the TCP/IP protocol and make your app to a client, server
or both (i.e. needed for peer to peer connections).
*/

/**
\class NetBasicManager
\ingroup net
\brief Basic class for managing network transfers

To use network, initialize a derived class from this, because NetBasicManager
contains virtual prototype functions.

Every network managing class works closely together with a socket class, which
has to be derived from NetSocket. All Sockets have to be children of a class
derived from NetBasicManager and all children of a NetBasicManager have to be
NetSocket objects.

Similar to the creation of GuiElements via XML-file, it is possible to create
NetSockets by XML-data, if the concerning NetSocket-derived class has a valid
slot registered for creating it. In comparison to the Gui-objects, the sockets
have to be direct children of the Net*Manager and they have to be compatible to
the Net*Manager (i.e. NetStandardSocket must not be a child of NetPeerManager)

\b UDP \n
The NetBasicManager will always (try to) initialize UDP. But if UDP cannot be
initialized, the init() method will return true though (provided that TCP could
be set up). The reason for this is, that UDP is a protocol, that cannot guarantee
that packet reach the destination host. So UDP not initialized is equal to loosing
all packets.

\ingroup singleusage
This class must not be used in two variables at a time.
\see singleusage
*/
class DECLSPEC NetBasicManager: public BasicObject
{
public:
	DECLSPEC NetBasicManager(const char *name, BasicObject *p);
	DECLSPEC NetBasicManager(TiXmlElement *el, BasicObject *p);
	virtual DECLSPEC ~NetBasicManager();

	/**
	Initializes the network manager. If you do not use the constructor
	NetBasicManager(TiXmlElement *, BasicObject *) calling this is
	neccessary before using further network functions.
	\param port port number on which to listen for connections or zero,
	if it should only work as client and does not need server abilities \n
	If you do not specify a param, the default netrinjo network port will
	be used, as defined in NETWORK_PORT.
	\param udpport the port number on which to send/receive UDP packets.
	This must be <port>+1 when using UDP with NetSockets! If zero, the port
	number is chosen by the system.
	\param udpmaxsize the maximum size (in bytes), an incomming UDP packet
	must not exceed.
	\returns true, if everything (except UDP) could be initialized correctly
	and false, if an error occured
	*/
	DECLSPEC bool init( unsigned int port = NETWORK_PORT, unsigned int udpport = NETWORK_UDPPORT,
						int udpmaxsize = 1024 );
	/**
	Uninitializes the network manager. Call this before closing the app.
	*/
	void destroy()	{onDestroy();}

	/**
	Sends data to all connected hosts by calling sendData(data,size) of all
	sockets.
	\param data a pointer to the data to be sent
	\param size the size (in bytes) of the data to be sent
	\returns the number of hosts to whom the message was sent successfully
	*/
	DECLSPEC unsigned int send( char *data, int size );
	/**
	Sends data via UDP to the specified host.
	\param data the data to send
	\param size the size of the data (in bytes)
	\param ip the destination IP-address
	\returns true on success, false on error
	*/
	DECLSPEC bool sendUDP( char *data, int size, const IPaddress &ip );
	/**
	Checks all connections, whether there was an incomming message and,
	if so, the callback function will be called on every message.\n
	This function should be called regularly to be able to act on the
	messages.
	\see setPeerCallBack
	\returns true, if there was at least one incomming message and
	false, if either no message came, or an error occured
	*/
	virtual DECLSPEC bool checkPeers();
	/**
	Checks on incomming connection requests.
	If someone wants to connect, the callback function will be called.
	There is only one new connection possible per checkServer-call.\n
	This function should be called regularly to be able to act on
	connection requests.
	\see setConnectCallBack
	\returns true, if a socket was added to the manager's socket list,
	but however it is possible, that the socket is disconnected; \n
	false, if no socket was added to the list, so either no request came,
	an error occured or the number of connections exeeds MAX_PEERS
	*/
	virtual DECLSPEC bool checkServer();

	/**
	\param ip the address of the socket to return (its port can be set to
	          zero in order to ignore it)
	\returns the socket belonging to the given ip address, or NULL if
	         no socket matches the address
	*/
	DECLSPEC NetSocket *getSocket( const IPaddress &ip );
	/**
	Deletes all disconnected sockets and removes them from the manager's
	socket list.
	*/
	DECLSPEC void deleteDisconnected();

	/**
	\returns the number of connections, that are currently valid
	*/
	int numConnections()	{ return nPeersConnected; }
	/**
	Sets the callback function, that is called, when a connection request is comming in
	\param func the function to call
	\param pData a pointer to data, that should be sent to the callback as 2nd param
	*/
	void setConnectCallback( BasicObject::Callback func, BasicObject *pData )	{ cbServer.func = func; cbServer.obj = pData; }
	/**
	Sets the default callback function, that is called, when a socket receives data.
	\param func the function to call
	\param pData a pointer to data, that should be sent to the callback as 2nd param
	*/
	void setPeerCallback( BasicObject::Callback func, BasicObject *pData )	{ cbPeer.func = func; cbPeer.obj = pData; }
	/**
	Sets the default callback function, that is called, when the UDP socket receives data.
	\param func the function to call
	\param pData a pointer to data, that should be sent to the callback as 2nd param
	 */
	void setUDPCallback( BasicObject::Callback func, BasicObject *pData )	{ cbUDP.func = func; cbUDP.obj = pData; }

	/**
	\returns the port on which the network manager is listening, if it works as server;
	zero, if it does not use server functions
	\see init
	*/
	int localPort()	{ return nListenPort; }
	/**
	\returns the local IP address as string in the format \code xxx.xxx.xxx.xxx \endcode
	\warning on linux-OSes it might fail, because it just reads the IP address written in
	the /etc/hosts file. If there is not the correct IP address or the file's format is
	not compatible to this function, the result will be garbage.
	*/
	DECLSPEC const char *localIP();

	/**
	Initialises SDL-networking, by calling SDLNet_Init() - if not done already.
	If initialisation succedes, or if it was already done, the reference counter is
	increased.
	\returns true, on success or if it has already been initialized.
	*/
	DECLSPEC static bool initSDL();
	/**
	Decreases the reference counter and uninitialises SDL-networking, if it becomes zero
	by calling SDLNet_Quit().
	*/
	DECLSPEC static void quitSDL();
	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static NetBasicManager *the;
protected:
	/**
	This function is protected, so use the getThe function of the derived class that was inited.
	\returns a pointer to the NetBasicManager-variable, or NULL if not initialized.
	(There is only one NetBasicManager-variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage, NetSimpleManager::getThe(), NetPeerManager::getThe()
	*/
	DECLSPEC static NetBasicManager *getThe();
	
	virtual DECLSPEC void onInit();
	virtual DECLSPEC void onDestroy();
	/**
	Called after a UDP packet has been received.\n
	NetBasicManager::onReceiveUDP() looks for a NetSocket, that uses the same address
	(IP + port) as the packet's source is and calls its onReceiveUDP() method.
	\param udp the received packet (the content will be overwritten when receiving the
	next packet)
	*/
	virtual DECLSPEC void onReceiveUDP(UDPpacket *udp);

	/**
	Creates a class, derived from NetSocket, that is compatible to the managing class.
	\param socket the SDL-TCP socket which is used by the new NetSocket
	\returns a pointer to the created class, converted to NetSocket*
	*/
	virtual NetSocket *createSocket( TCPsocket socket ) = 0;
	/**
	Signals, whether the manager has been inited.
	*/
	bool bInited;
	/**
	Contains the TCP port number on which the manager listens for connection requests.
	It is zero, if it does not work as server.
	\see localPort()
	*/
	int nListenPort;
	/**
	The port on which is listened for UDP data, if zero the port is unknown
	*/
	unsigned int nUDPPort;

private:
	DECLSPEC bool registerSocket( TCPsocket sock );
	DECLSPEC void unregisterSocket( TCPsocket sock );

	int nPeersConnected;

	SDLNet_SocketSet socksPeers, socksServer;
	TCPsocket sockServer;
	UDPsocket sockUDP;
	// used to receive/send incomming/outgoing messages
	UDPpacket *ppackUDPrecv, *ppackUDPsend;
	// max. size of a UDP-packet's content
	unsigned int nUDPMaxSize;

	BasicObject::FunctionData cbServer, cbPeer, cbUDP;

//	char *chLocalName;
	// reference counter for usage of SDL-network functions
	static unsigned int nSDLNetRefCount;

	friend class NetSocket;
};

class NetStandardSocket;

/**
\class NetSimpleManager
\ingroup net
\brief A very simple class, derived from NetBasicManager

This is a very simple network manager, containing the most important things.
The NetSocket-class, that is compatible to this managing class is NetStandardSocket.

\ingroup singleusage
Do not initialize this class, if another class, derived from NetBasicManager has already been inited!
*/
class DECLSPEC NetSimpleManager: public NetBasicManager
{
public:
	DECLSPEC NetSimpleManager(const char *name, BasicObject *p);
	DECLSPEC NetSimpleManager(TiXmlElement *el, BasicObject *p);
	virtual DECLSPEC ~NetSimpleManager();
	virtual const char *getClassName()	{ return "NetSimpleManager"; }

	/**
	\returns a pointer to the NetSimpleManager-variable, or NULL if not initialized.
	(There is only one variable allowed in an application that is a derived class from NetBasicManager)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage, NetBasicManager::getThe(), NetPeerManager::getThe()
	*/
	static NetSimpleManager *getThe()	{ return (NetSimpleManager*)NetBasicManager::getThe(); }
protected:
	virtual DECLSPEC NetSocket *createSocket( TCPsocket socket );

private:
};


#endif
