/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETPEERMANAGER_H
#define NETPEERMANAGER_H

#include "netmanager.h"

/**
\ingroup net
\file netpeermanager.h
\brief Defines the class NetPeerManager, PEER_MAX_NAME and PEER_MAX_REASON
*/
/**
Defines the maximum length allowed for a name in the network.
*/
#define PEER_MAX_NAME		63
/**
Defines the maximum length a reason for rejecting can have.
\see NetPeerManager::hasPermission
*/
#define PEER_MAX_REASON		255

class NetPeerSocket;
class TiXmlNode;

/**
\class NetPeerManager
\ingroup net
\brief A peer to peer network managing class, derived from NetBasicManager

This class contains many functions for peer to peer communication. It uses
client and server functions to connect to others and let others connect.
As socket class, is used NetPeerSocket.

As a special feature, this class is able to work with XML Nodes.

\warning Only NetPeerSocket (or derived) objects may be children of this class.

\ingroup singleusage
Do not initialize this class, if another class, derived from NetBasicManager has
already been inited!
*/
class DECLSPEC NetPeerManager : public NetBasicManager
{
public:
	/**
	Constructor \n
	\param maxpeers limits the maximum number of allowed peers to this value,
	if it is below MAX_PEERS, otherwise the limit equals MAX_PEERS
	*/
	DECLSPEC NetPeerManager(const char *name, BasicObject *p, int maxpeers = 255 );
	/**
	Constructs the manager by using a XML node.
	\param el The node can contain the attribute "Username". If a subnode "Permissions"
	exists, this subnode will be parsed with the loadPermissions( XMLNode* ) function,
	if not and the attribute "PermissionFile" is specified, the file will be parsed
	and if no permission data is given, every host is allowed to connect to the manager.
	If the attribte "Port" exists, this port will be opened, otherwise the default
	NETWORK_PORT will be used.
	Here an example:
	\code
	<NetPeerManager Name="NetPeerManager"
			Port="1230"
			Username="my name"
			PermissionFile="user/permissions.pml"
	/>
	\endcode
	\returns true, if initialisation was successful, false if not
	\see loadPermissions(const char*), setName, init( int, const char * )
	*/
	DECLSPEC NetPeerManager(TiXmlElement *el, BasicObject *p );
	virtual DECLSPEC ~NetPeerManager();
	virtual const char *getClassName()	{ return "NetPeerManager"; }

	/**
	Initializes the manager with the given params. Only call this, if you
	do not use the constructor with the XML-element.
	\param port the port to listen for incomming connection requests
	\param permissionfn the filename of a XML file, containing data for
	permission rules, that say whom to accept and whom to reject.
	\returns true, if initialisation was successful, false if not
	\see loadPermissions(const char*)
	 */
	virtual DECLSPEC bool init( unsigned int port = NETWORK_PORT, const char *permissionfn = 0 );

	/**
	Sets the name of the local machine, which is used to connect to other machines
	in the network.
	\param name the name as NUL-terminated string
	\note the network-name should not change, as long as a NetPeerSocket is connected
	*/
	DECLSPEC void setNetName( const char *name );
	/**
	\returns the name of the local machine, as set by setNetName()
	*/
	DECLSPEC const char *getNetName() const	{return chName;}

	/**
	\param name the name to look for
	\returns a pointer to the socket, which is connected to the host with the given name.
	\see getNetName(), setNetName().
	*/
	DECLSPEC NetPeerSocket *getSocketByName( const char *name );

	/**
	Sends a ping request to all hosts. (This does not use ICMP, but TCP)
	*/
	DECLSPEC void requestPings();
	/**
	Use this function to get ping times. To get current times, call requestPings()
	before getPings and wait a bit between these two calls.
	\param min if not NULL, the time of the fastest response of a host will be assigned here
	\param max if not NULL, the time of the slowest response of a host will be assigned here
	\returns the average response time of all hosts
	*/
	DECLSPEC Uint32 getPings( Uint32 *min, Uint32 *max );
	/**
	Parses the node to store the allowed hosts in a privat struct, which is used by
	hasPermission to allow or deny an incomming connection.
	Example:
	\code
	    <Allow>
	      <Player IP="192.168.2.3" />
	      <Player IP="192.168.2.4" />
	      <Player IP="192.168.2.5" />
	      <Player Name="test" />
	      <Player IP="127.0.0.1" />
	    </Allow>
	    <Disallow>
	      <Player IP="192.168.2.6" />
	      <Player />
	    </Disallow>
	\endcode
	In the example the hosts with the addresses 192.168.2.3, -4, -5 and 127.0.0.1 are accepted,
	The host with the name "test" is only accepted, if it's address is not 192.168.2.6; all
	others are not accepted.
	\returns true, if no errors occured, false otherwise
	*/
	DECLSPEC bool loadPermissions( TiXmlNode *node );
	/**
	Loads the XML file with the specified filename and parses it's Permissions node
	with loadPermissions( XMLNode ).
	The file has to be structured as follows (minimal version):
	\code
	<PML><Permissions>
	[<Allow> (allowed Player-tags) </Allow>]
	[<Disallow> (disallowed Player-tags) </Disallow>]
	</Permissions></PML>
	\endcode
	\returns true, if no errors occured, false otherwise
	*/
	DECLSPEC bool loadPermissions( const char *fn );
	/**
	Prooves, whether the specified host has permission to connect,
	as specified in the permission-tags.
	\param name the name of the host
	\param host the host's IP address (in OSBO)
	\param reason NULL, or a pointer to reserved space, filled with the reason, why
	the host has no permission, if a reason is availible
	\returns true, if the host is accepted, false otherwise
	*/
	DECLSPEC bool hasPermission( const char *name, Uint32 host, char *reason = 0 );
	/**
	If - what ever the reason is - no further connections should be accepted
	then set this to false. It is true, when NetPeerManager is created
	*/
	bool bAllowNewConnections;

	/**
	\returns a pointer to the NetPeerManager-variable, or NULL if not initialized.
	(There is only one variable allowed in an application that is a derived class from NetBasicManager)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage, NetBasicManager::getThe(), NetSimpleManager::getThe()
	*/
	static NetPeerManager *getThe()	{ return (NetPeerManager*)NetBasicManager::getThe(); }
protected:
	virtual DECLSPEC void onInit();
	virtual DECLSPEC NetSocket *createSocket( TCPsocket socket );

private:
	struct HOST_INFO
	{
		char chName[PEER_MAX_NAME+1];
		Uint32 ip;
		bool bAllowed;
	};
	typedef list<HOST_INFO>::iterator PermissionIter;
	list<HOST_INFO> lPermissionInfos;

	int nMaxPeers;
	char chName[PEER_MAX_NAME+1];
};

#endif
