/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETPEERSOCKET_H
#define NETPEERSOCKET_H

#include "netsocket.h"
#include <list>
#include "netpeermanager.h"

/**
\ingroup net
\file netpeersocket.h
\brief Defines the class NetPeerSocket and MAX_PING
*/
/**
highest ping, where a connection is surely possible, using NetPeerSocket
*/
#define MAX_PING	5000

using namespace std;
struct NetBasicData;
class NetPeerManager;

/**
\class NetPeerSocket
\ingroup net
\brief A peer to peer network socket class, derived from NetStandardSocket.
\todo update onRecieve()!!!

This class implements socket functions for peer to peer communication.
It works closely together with NetPeerManager.

\see NetPeerManager
*/
class DECLSPEC NetPeerSocket : public NetStandardSocket
{
public:
	/**
	Constructor. Usually the 3rd param is only used internally in the
	Netrinjo library.
	\param connection the SDL TCPsocket to use, or NULL, if no connection
	is used.
	*/
	DECLSPEC NetPeerSocket( const char *name, NetPeerManager *parent, TCPsocket connection = NULL );
	/**
	Constructor. Can be used to establish a connection to a host.
	\param ip the host's IP address, that should be connected.
	\warning this construction can take some time, if the host
	is not reachable immediatelly.
	*/
	DECLSPEC NetPeerSocket( const char *name, NetPeerManager *parent, IPaddress ip );
	/**
	Constructor with XML-data.
	\param el the XML-data,
	 */
	DECLSPEC NetPeerSocket( TiXmlElement *el, NetPeerManager *parent );
	virtual DECLSPEC ~NetPeerSocket();

	virtual const char *getClassName()	{ return "NetPeerSocket"; }
	
	/// \returns the name of the machine in the network
	const char *getNetName()	{ return chName; }

	/**
	\todo use UDP!
	Gets the ping to the host immediatelly. This function waits until
	the host responded the ping request.
	\returns the time in msecs, that went over until the response came.
	\warning using this function while other network-exchange on the same port
	with the host will cause errors!
	\see requestPing, lastPing
	*/
	DECLSPEC Uint32 fastPing();
	/**
	Requests a ping from the host and returns immediatelly.
	When the host responded, the duration of the ping can be asked with
	lastPing().\n
	In comparison to fastPing() this function is safe, but usually returns a higher result.
	*/
	DECLSPEC void requestPing();
	/**
	\returns the duration of the last ping to the host
	\see requestPing()
	*/
	Uint32 lastPing() const	{ return tPing; }
	
	/// \returns the port on which this machine can be connected
	int getListenPort()	{ return nListenPort; }

//	virtual DECLSPEC int readData( char *data, int *err = NULL );
//	virtual DECLSPEC char *readData( int *size );

	// just for usage via BasicSlot
	static void newsocket(void *presult, BasicObject *o, void *xmldata)
	{
		NetSocket *r = new NetPeerSocket((TiXmlElement*)xmldata, (NetPeerManager*)o);
		*(NetSocket**)presult = r;
	}
	
protected:
	/// the name of the host machine in the network
	char chName[PEER_MAX_NAME+1];
	/// port on which one can connect to the socket's host
	int nListenPort;
	
	/**
	Handles incomming data. This needs not to be used by the app.
	It is used internally in the Netrinjo engine.
	\param d the data, that arrived
	*/
	virtual DECLSPEC void handleData( NetBasicData *d );
	virtual DECLSPEC bool onReceive();
	
	virtual DECLSPEC void onConnect();

	/**
	Called when a ping answer arrived, so tPing has just been refreshed.
	*/
	virtual DECLSPEC void onPing()	{}
private:
	void init();
	Uint32 tPing, tPingSent;
	// ob Verbindung aktiv aufgebaut wurde, oder passiv
	bool bActiveConnect;
};

#endif
