/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETSOCKET_H
#define NETSOCKET_H

#include "SDL/SDL_net.h"
#include "basicobject.h"

/**
\ingroup net
\file netsocket.h
\brief Defines the classes NetSocket and NetStandardSocket, FILTER_SERVER, MAX_DATALEN and error codes
*/

/// filter for the socket to the main, unique server in a network
#define FILTER_SERVER	1
/// max. lenght of a TCP-message; size of "data" to be allocated when calling NetSocket::readdata(data)
#define MAX_DATALEN		0xFFFE

/**
No error occured, all fine.
*/
#define E_NET_NONE		0

/**
No host is specified (socket is not initialized)
*/
#define E_NET_NO_HOST		-16

/**
The host disconnected the connection
*/
#define E_NET_DISCONNECTED	-17

/**
The message's data is not compatible to the format of the message handler.
*/
#define E_NET_RAW_DATA		-32

/**
The packet is larger than the buffer, where it should be stored.
 */
#define E_NET_BUFTOOSMALL	-33

/**
Another error occured.
 */
#define E_NET_OTHER			-64

class NetBasicManager;

/**
\class NetSocket
\ingroup net
\brief The basic network socket class

This class implements basic socket functions network communication.
It works closely together with NetBasicManager.

Used error codes: E_NET_NONE, E_NET_NO_HOST, E_NET_DISCONNECTED, E_NET_RAW_DATA

\see NetBasicManager
*/
class DECLSPEC NetSocket: public BasicObject
{
public:
	/**
	Constructor.
	\param connection the SDL TCPsocket to use, or NULL, if no connection
	is used.
	*/
	DECLSPEC NetSocket( const char *name, NetBasicManager *p, TCPsocket connection = 0 );
	/**
	Constructor. Connects to a host.
	\param ip the host's IP address, that should be connected.
	\warning this construction can take some time, if the host
	is not reachable immediatelly.
	*/
	DECLSPEC NetSocket( const char *name, NetBasicManager *p, IPaddress ip );
	/**
	Constructor. Uses XML-data.
	\param el the XML-data, without attributes (except the name) it behaves like
	          NetSocket(const char*, NetBasicManager*).
	          If the attribute "IP" is specified it behaves similar to
	          NetSocket(const char*, NetBasicManager*, IPaddress), the form of the address
	          must be like "192.168.0.1:1234".
	\test this function
	*/
	DECLSPEC NetSocket( TiXmlElement *el, NetBasicManager *p );
	virtual DECLSPEC ~NetSocket();

	virtual bool isNetSocket() const	{ return true; }

	/// \returns true, if this socket is in the manager's socket list, false if not
	bool isValid() const	{ return bValid; }

	/**
	Connects to a host. This may take some time, especially if the host is unreachable.
	\param host the hostname or IP address to connect to (without port)
	\param port the port to connect on (in OSBO)
	\returns false, if no connection could be established or an error occured,
	true if all went perfect
	*/
	DECLSPEC bool connect( const char *host, int port );
	/**
	Connects to a host. This may take some time, especially if the host is unreachable.
	\param ip the IP address to connect to
	\returns false, if no connection could be established or an error occured,
	true if all went perfect
	*/
	DECLSPEC bool connect( IPaddress &ip );
	/**
	Disconnects an existing connection.
	*/
	DECLSPEC void disconnect();
	/// \returns true, if the socket is connected, otherwise false
	virtual bool connected() const	{ return sock != 0; }
	/**
	\returns true, if it will be deleted automatically, when disconnected;
	false, if it can be reused and has finally to be deleted by the application.
	*/
	virtual bool deleteOnDisconnected() const	{ return bDelOnDisconn; }

	/**
	\param time the time in msecs, how long to wait on data, if no data can be read
	\returns true, if data can be read, otherwise false
	*/
	virtual DECLSPEC bool wait4data( Uint32 time );

	/**
	Sends raw data via TCP to the host. This method should be overridden in derived
	classes that implement a special structure for transmitted data.
	\param data the data to send
	\param size the amount of bytes to send
	\returns true on success, false on error
	*/
	virtual bool sendData( char *data, unsigned int size );
	/**
	Sends raw data via UDP to the host.
	\note Using this makes only sense, if all clients, communicating via UDP, have their
	UDP port number one above their TCP port number.
	\param data the data to send
	\param size the amount of bytes to send
	\returns true on success, false on error
	 */
	virtual bool sendUDP( char *data, unsigned int size );
	/**
	Receives raw data from the host. This method does not block, but is probably not
	very efficient, especially when reading large packets.
	\param buffer the buffer where received data is stored
	\param maxsize the size of the buffer, that can be filled
	\returns the size of the received data; 0 if there is no data to read;
	a negative value (an error-code) on error
	*/
	virtual int readData( char *buffer, int maxsize );
	/**
	\param size pointer to an integer, where either the size (in bytes) of the returned data
	will be stored or the error code, if something failed.
	\returns a pointer to space with the recieved data or NULL, if an error occured.
	\warning the result has to be deleted after usage!
	*/
	virtual char *readData( int *size ) = 0;

	/**
	While the socket is active, the netmanager ignores the incomming-message-status
	\returns true, if data is currently received or however the socket is active,
	otherwise false.
	*/
	virtual bool isActive() const	{ return bActive; }
	/// \returns the SDL TCP-socket that is used for communication
	TCPsocket getSocket()	{ return sock; }
	/**
	\returns a pointer to the IP address of the connected host or NULL if not connected
	*/
	const IPaddress *getAddress() const	{ if(!sock) return NULL; return SDLNet_TCP_GetPeerAddress( sock ); }
	/**
	This method may block for a short time, due to reverse DNS lookup.
	\returns the hostname of the connected host; NULL, if not connected
	\note this is often simply the IP address of the host
	*/
	const char *getHostName() const	{ if(!sock) return NULL; return SDLNet_ResolveIP( SDLNet_TCP_GetPeerAddress( sock ) ); }

protected:
	virtual DECLSPEC void onInit();
	/**
	Called from the manager, when data arrived for this socket.
	In this base class, it calls the default callback function NetBasicManager::cbPeer.
	\returns false, if an error occured and the socket has to be closed, otherwise true
	*/
	virtual DECLSPEC bool onReceive();
	/**
	Called from the manager, when UDP data arrived for this socket.
	In this base class, it calls the default callback function NetBasicManager::cbUDP.
	\param udp the recieved packet. Its content is only valid until the next packet is
	received!
	 */
	virtual DECLSPEC void onReceiveUDP(UDPpacket &udp);
	/**
	Called, after a connection has been established successfully.
	If you want to overwrite this method, do not forget to call the parent class's
	onConnect();
	*/
	virtual void onConnect()	{}
	/**
	Called, before a connection is terminated.
	If you want to overwrite this method, do not forget to call the parent class's
	onConnect();
	 */
	virtual void onDisconnect()	{}
	/// the SDL TCP socket that is used for communication
	TCPsocket sock;
	/// see isValid() for description
	bool bValid;
	/// see isActive() for description
	bool bActive;
	/// see deleteOnDisconnected() for description
	bool bDelOnDisconn;
private:
	bool init();
	friend class NetBasicManager;
};


class NetSimpleManager;

/**
\class NetStandardSocket
\ingroup net
\brief The basic network socket class

This class implements basic socket functions network communication.
It works closely together with NetSimpleManager.

message-structure of a NetStandardSocket-message:
 - size of message-body (real data) [2 bytes]
 - body (content of message) ["size" bytes (max 65535)]
*/
class DECLSPEC NetStandardSocket : public NetSocket
{
public:
	/**
	Constructor. Usually used only internally in the netrinjo engine.
	\param connection the SDL TCPsocket to use, or NULL, if no connection
	is used.
	*/
	DECLSPEC NetStandardSocket( const char *name, NetBasicManager *p, TCPsocket connection = 0 );
	/**
	Constructor. Connects to a host.
	\param ip the host's IP address, that should be connected.
	\param filter the filter to use, when a message is sent, using a condition
	if (filter & condition) != 0 or condition == 0 then message will be sent
	\warning this construction can take some time, if the host
	is not reachable immediatelly.
	*/
	DECLSPEC NetStandardSocket( const char *name, NetBasicManager *p, IPaddress ip, int filter = 0 );
	/**
	Constructor. Uses XML-data.
	\param el the XML-data, the attribute "Filter" can be used to specify the
	value for filter
	\test this function
	*/
	DECLSPEC NetStandardSocket( TiXmlElement *el, NetBasicManager *p );

	virtual DECLSPEC ~NetStandardSocket();

	virtual const char *getClassName()	{ return "NetStandardSocket"; }
	
	/**
	Sends data, preceded by a word (2 bytes) containing the size, to the host.
	\param data the data to send
	\param size the size of data
	\returns true on success, false if disconnected.
	*/
	virtual bool sendData( char *data, unsigned int size );
	/**
	Sends data to the host, if a condition is true. It automatically disconnects,
	if an error occurs when sending the data.
	\param data the whole content of the message to send
	\param size the content's size in bytes
	\param condition a condition number, if zero, the data will always be sent
	\param buffer can be either reserved space for creating the message, or NULL
	if buffer is specified, sending data will be faster, especially, if size is big
	\returns true, if either the message was sent successfully or the condition was
	not fulfilled; false if disconnected
	*/
	virtual DECLSPEC bool sendData( char *data, unsigned int size, Uint32 condition, void *buffer = NULL );
	/**
	Receives data from the host. Using this is faster than NetSocket::readData(),
	but uses its own data-structure (2 Bytes specifying the size are prepended to
	each packet). So also the other host has to use this class's methods to send
	and receive data.
	\see NetSocket::readData(void*, int)
	*/
	virtual DECLSPEC int readData( char *data, int maxsize );
	/**
	Similar to readData(void*, int), but allocates memory to store the received
	data (this has to be freed after usage!).
	\param size if not NULL, the number of bytes received or an error-code is
	stored here
	\returns a pointer to the received data, that has to be freed after usage;
	or NULL on error
	*/
	virtual DECLSPEC char *readData( int *size );

	/**
	Sets the callback function.
	\param cb the function pointer
	\param o the recieving object
	*/
	void setReceiveCallback(BasicObject::Callback cb, BasicObject *o)	{cbRecieve.func = cb; cbRecieve.obj = o;}
	
	/**
	Used, when sending conditioned messages
	if (filter & condition) != 0 or condition == 0 then message will be sent
	*/
	Uint32 filter;

protected:
	virtual DECLSPEC void onInit();
	/**
	Calls the callback function, as set by setReceiveCallback(). The 3rd param
	of the callback-function is used as a pointer to a boolean value. If this
	is set false, the socket will close.
	\returns true by default, false if the value of the pointer in the 3rd param
	is set to false.
	*/
	virtual DECLSPEC bool onReceive();
	virtual DECLSPEC void onConnect();

private:
	BasicObject::FunctionData cbRecieve;
	// number of bytes to read before starting to read the next packet's size
	unsigned int nBytesUnread;
};


#endif
