/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETSTRUCTS_H
#define NETSTRUCTS_H

#include "netpeermanager.h"

/**
\ingroup net
\file netstructs.h
\brief Several structs for communication via netrinjo's network classes

Required header file:
@code #include <netstructs.h> @endcode

Using data of this file is especially neccessary when working with the
NetPeerManager class.\n
In this file, several structs are deklared, that should help making
network communication easier. Some are used by the NetPeerSocket class.
The integer values of the structs have NBO, but support reading them
in OSBO.

The file also defines more or less important values for peer to peer
communication. The definitiones used for indicating a special type of
message begin with TYP_.
The TYP_ADMIN_... definitiones are used for a game's administration,
which is only allowed for the admin(s), who have the password.
*/

// admin-ids: 0x0**
/// Packet sets time for one turn in the game
#define TYP_ADMIN_SPEED		0x010
/// Packet closes the server application
#define TYP_ADMIN_CLOSE		0x020
/// Packet rejects a player from the game
#define TYP_ADMIN_REJECT	0x031
//#define TYP_ADMIN_REQUEST	0x080	// last 7 Bits indicate, which request is meant
//#define TYP_ADMIN_ANSWER	0x080	// you can distinguish between request and answer
// by finding out whether or not this app is the server-app
/// Packet requests the longest ping to a client
#define TYP_ADMIN_MAXPING	0x081
/// max length of admin-password for accessing the server
#define MAX_ADMIN_PASSWORD	31

// connection-ids: 0x1**
/// Packet used to connect to another machine (usually directly sent after the connection request)
#define TYP_CONNECT			0x101
/// Packet used to disconnect from a host (usually sent directly befor disconnecting)
#define TYP_DISCONNECT		0x102
/// Packet contains a list with the connections, it has to others
#define TYP_CONNECTIONLIST	0x108
/// Packet requests a ping answer, used to calculate the ping time
#define TYP_PING_REQUEST	0x110
/// Packet is the answer on a ping request
#define TYP_PING_ANSWER		0x111

// chat-ids: 0x2**
/// Indicates a chat message
#define TYP_CHAT_MESSAGE	0x202

// game-ids: 0x3**
/// indicates the start of the next turn
#define TYP_NEXT_TURN		0x301

// command-ids: 0x4**
/// basic id of commands
#define TYP_BASIC_COMMAND	0x400

/**
The basic struct, used for high-level network communication.
*/
struct NetBasicData
{
	/**
	\returns the packet's type (in OSBO, so you need not convert it anymore)
	*/
	Uint32 type()				{ return SDLNet_Read32( &nType ); }
	/**
	Sets the packet's type.
	\param t the type-id (in OSBO), it will be converted to NBO
	*/
	void setType( Uint32 t )	{ SDLNet_Write32( t, &nType ); }
protected:
	/// The packet's type in NBO
	Uint32 nType;
};

/**
The basic struct for administraton via Netrinjo-network.
It uses a password to avoid non-admins executing commands
on the server.
\warning Currently the password is not encrypted, so one
might get the password by viewing the network transfer.
\todo en-/decrypt the password
*/
struct NetBasicAdminData : NetBasicData
{
	NetBasicAdminData()
	{
		chPassword[0] = '\0';
	}
	/// The password to send with the packet
	char chPassword[MAX_ADMIN_PASSWORD+1];
};

/**
Used to change the game's speed.
The value to set() or get() is the time in msecs of one turn.
*/
struct NetAdminSpeed : NetBasicAdminData
{
	NetAdminSpeed()
	{
		setType( TYP_ADMIN_SPEED );
	}
	/// \returns the value (in OSBO)
	Uint32 get()			{ return SDLNet_Read32( &nTime );	}
	/// \param t the value to set (in OSBO)
	void set( Uint32 t )	{ SDLNet_Write32( t, &nTime );		}
protected:
	/// the time in msecs of one turn.
	Uint32 nTime;
};

/**
Used to reject a player from the game, by giving it's IP address with port.
*/
struct NetAdminReject : NetBasicAdminData
{
	NetAdminReject()
	{
		setType( TYP_ADMIN_REJECT );
		ip.host = 0;
		ip.port = 0;
	}
	/// The player's IP address and port (both in NBO), which should be rejected
	IPaddress ip;
};

/**
Used when connecting, using NetPeerManager/NetPeerSocket.
*/
struct NetConnectData : NetBasicData
{
	NetConnectData()
	{
		setType( TYP_CONNECT );
		chName[PEER_MAX_NAME] = '\0';
	}
	/// \returns the listening port of the connecting machine, in OSBO
	int getPort()	{ return SDLNet_Read16( &nListenPort ); }
	/// sets the port \param p the port in OSBO
	void setPort( int p )	{ SDLNet_Write16( p, &nListenPort ); }
	/// the name of the player, who wants to connect
	char chName[PEER_MAX_NAME+1];
	/// for any further information
	char chInfo[64];
protected:
	/**
	the port on which the connectig machine listens for connections
	useful to tell others, how to connect this host
	*/
	Uint16 nListenPort;
};

/**
Used to disconnect in a friendly way, by telling the host,
why he becomes disconnected.
*/
struct NetDisconnectData : NetBasicData
{
	NetDisconnectData()
	{
		setType( TYP_DISCONNECT );
		chReason[PEER_MAX_REASON] = '\0';
	}
	/// the reason for the disconnection
	char chReason[PEER_MAX_REASON+1];
};

/**
Used to transcieve data with a list of connections, in order to
tell others, who is here on the network. This struct is compatible to
NetBasicData, to get the type.
\warning do not send this struct as it is! use getData(), when sending the list; and the constructor (void *data) when recieving
*/
struct NetConnectionList
{
	/**
	Constructor.
	\param max specifies the maximum number of addresses to store (space will be reserved accordingly)
	*/
	NetConnectionList( int max = 253 )
	{
		pData = new char[sizeof(NetBasicData) + sizeof(Uint32) + max*sizeof(IPaddress)];
		pIPs = (IPaddress*)(pData + sizeof(NetBasicData) + sizeof(Uint32));
		pType = (NetBasicData*)pData;
		pCount = (Uint32*)(pData + sizeof(NetBasicData));
		*pCount = 0;
		bAllocated = true;
		pType->setType( TYP_CONNECTIONLIST );
	}
	/**
	Constructor to use, when data is availible and has to be read/modified.
	\param data pointer to the data, in the format of NetConnectionList
	*/
	NetConnectionList( void *data )
	{
		pData = (char*)data;
		pIPs = (IPaddress*)(pData + sizeof(NetBasicData) + sizeof(Uint32));
		pType = (NetBasicData*)pData;
		pCount = (Uint32*)(pData + sizeof(NetBasicData));
		bAllocated = false;
	}
	~NetConnectionList()
	{
		if( bAllocated && pData )
			delete[] pData;
		pData = NULL;
	}
	/// \returns a pointer to the whole data, that is transfered over the network
	void *getData()	{ return pData; }
	/**
	Use this together with size() when transmitting the struct's content over the network.
	\param i the number of the address in the list to read
	\returns a pointer to the IP address
	*/
	IPaddress *get( int i )
	{
		if( i >= SDLNet_Read32( pCount ) )
			return NULL;
		return &pIPs[i];
	}
	/**
	Changes a list entry to the given address
	\param a the address to store (in NBO)
	\param i the number of the address in the list to write to (in OSBO)
	\warning be sure to have enough memory allocated when creating the struct,
	there is no memory check here.
	*/
	void set( IPaddress a, int i )
	{
		if( SDLNet_Read32( pCount ) <= i )
			SDLNet_Write32( i+1, pCount );
		pIPs[i] = a;
	}
	/**
	Use this together with get(int) when transmitting the struct's content over the network.
	\returns the size of the data that is used (in bytes).
	*/
	int size()
	{
		return SDLNet_Read32( pCount )*sizeof(IPaddress) + sizeof(NetBasicData) + sizeof(Uint32);
	}
	/// \returns the number of list entries
	Uint32 count()
	{
		return SDLNet_Read32( pCount );
	}
private:
	NetBasicData *pType;
	IPaddress *pIPs;
	char *pData;
	Uint32 *pCount;
	bool bAllocated;
};

/**
A simple struct, containing one 32-Bit value (and the type of course).
*/
struct NetValue32Data : NetBasicData
{
	/**
	\param typ the type of the package
	*/
	NetValue32Data( Uint32 typ = 0 )
	{
		setType( typ );
		nValue = 0;
	}
	/// \returns the value in OSBO
	Uint32 get()		{ return SDLNet_Read32( &nValue );	}
	/// sets the value \param v the value in OSBO
	void set( Uint32 v )	{ SDLNet_Write32( v, &nValue );		}

protected:
	/// the 32-Bit value in NBO
	Uint32 nValue;
};

#endif
