/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef NETTHREAD_H
#define NETTHREAD_H

#include "basicthread.h"
#include "basicobject.h"
#include <list>
#include "SDL/SDL_net.h"

/**
\ingroup net
\file netthread.h
\brief Defines the class NetThread.
*/

using namespace std;
class NetSocket;

/**
\class NetThread
\ingroup net
\brief A class for polling network interfaces regularly

This class handles network-events parallely.\n
Define "WEB_CLIENT", if this class need/must not poll the WebServer.\n
Use lock/unlock, when you want another thread/process to
access the network.

If NetThread is not created with XML-data then it is automatically started
when onInit() is called and stopped with onDestroy()!

\ingroup singleusage
This class must not be used in two variables at a time.
\see singleusage
*/
class DECLSPEC NetThread : public BasicThread
{
public:
#ifdef WEB_CLIENT
	/**
	Constructor, if WEB_CLIENT defined.
	\param debug if true, a log file is used for debugging, if false
	nothing will be logged.
	*/
	DECLSPEC NetThread( const char *name, BasicObject *p, bool debug = true );
#else
	/**
	Constructor, if WEB_CLIENT undefined.
	\param server if true, the webserver will be polled regularly
	if false not (equals WEB_CLIENT defined)
	\param debug if true, a log file is used for debugging, if false
	nothing will be logged.
	*/
	DECLSPEC NetThread( const char *name, BasicObject *p, bool server = true, bool debug = true );
#endif
	/**
	Constructor, for XML data.
	\param el the XML data, if the attribute "StartOnInit" is set true or does not
	exist, then the thread is started, when onInit() is called and stopped in onDestroy().
	*/
	DECLSPEC NetThread( TiXmlElement *el, BasicObject *p );
	DECLSPEC ~NetThread();
	
	virtual const char *getClassName()	{ return "NetThread"; }
	
	/**
	Tries to connect the socket to the given IP address. This function may block
	the thread for some time.
	\param s the socket to connect
	\param ip the IP address to connect to
	\param c the function to call, when the task was done; NULL, if no function
	should be called
	\param obj the object, that recieves the function-call
	*/
	DECLSPEC void connectSocket( NetSocket *s, const IPaddress &ip, BasicObject::Callback c, BasicObject *obj = NULL );

#ifndef WEB_CLIENT
	/**
	(De-)activates checking the webserver for requests.
	\param active must be true to activate the HTTP-server-functionality.
	*/
	void setServerCheck(bool active)	{bServer = active;}
#endif
	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static NetThread *the;
	/**
	\returns a pointer to the NetThread-variable, or NULL if not initialized.
	(There is only one variable of the type NetThread allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	DECLSPEC static NetThread *getThe();
protected:
	virtual DECLSPEC int onRun();
	virtual DECLSPEC void onCycle();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC void onDestroy();

private:
#ifndef WEB_CLIENT
	bool bServer;// must be true to activate the HTTP-server-functionality
#endif
	struct DATA
	{
		DATA(){}
		DATA( NetSocket *sock, const IPaddress &addr, BasicObject::FunctionData c )
		{
			s = sock;
			ip = addr;
			cb = c;// called, when connecting done (successfully or not)
		}
		NetSocket * s;
		IPaddress ip;
		BasicObject::FunctionData cb;
	};
	list<DATA> lToConnect;
};

#endif
