/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef OGLFUNCTIONS_H
#define OGLFUNCTIONS_H

#ifndef DOXYGEN
#include <GL/gl.h>
#include <SDL.h>
#include "definitions.h"
#endif

/**
 * @ingroup noclass
 * @file oglfunctions.h
 * @brief Contains functions for setting/drawing simple things with OpenGL.
 * @author Tobias Nadler
 * 
 * Required header file:
 * @code #include <oglfunctions.h> @endcode
 */

/**
 * @namespace ogl
 * @brief Contains functions for setting/drawing simple things with OpenGL.
 */
namespace ogl
{
	/**
	 * Sets the currently active color, including the alpha value.
	 * After you are finished with drawing, you should reset the OpneGL color
	 * state with resetColor().
	 * @param c the ARGB color value
	 */
	inline void setColor4( ARGB_COLOR c )
	{
		glColor4ub( c >> 16, c >> 8, c, ~(c >> 24) );
	}
	/**
	 * Sets the currently active color, with the alpha value 255.
	 * After you are finished with drawing, you should reset the OpneGL color
	 * state with resetColor().
	 * @param c the RGB color value
	 */
	inline void setColor3( ARGB_COLOR c )
	{
		glColor3ub( c >> 16, c >> 8, c );
	}
	/**
	 * Sets the color to the default color (currently this is always white).
	 * This function shall be called after drawing a GUI object, if the color
	 * has changed.
	 */
	inline void resetColor()
	{
		glColor3ub( 255, 255, 255 );
	}
	/**
	 * Draws a line loop.
	 * @param x,y pointers to arrays of x- and y-coordinates
	 * @param num the number of coordinates in the arrays
	 * @param width the lines' width
	 * @param factor,pattern see OpenGL documentation of glLineStipple()
	 */
	DECLSPEC void drawLineLoop( GLint *x, GLint *y, unsigned int num, GLfloat width, GLint factor=1, GLushort pattern=0xFFFF );
	/**
	 * Draws a line loop without pattern.
	 * See drawLines() for explanation of the first four params.
	 * @param c the lines' color
	 */
	inline void drawPlainLineLoop( GLint *x, GLint *y, unsigned int num, GLfloat width, ARGB_COLOR c )
	{
		setColor4( c );
		drawLineLoop( x, y, num, width );
		resetColor();
	}
	/**
	 * Simply draws a rect as GL_QUADS with its four vertices.
	 * (Color... depends on the currently set OpenGL states)
	 * @param r the rect to draw
	 */
	inline void drawRect( SDL_Rect r )
	{
		glBegin( GL_QUADS );
			glVertex2s( r.x, r.y );
			glVertex2s( r.x+r.w, r.y );
			glVertex2s( r.x+r.w, r.y+r.h );
			glVertex2s( r.x, r.y+r.h );
		glEnd();
	}
	/**
	 * Fills a rect with a color and resets the color afterwards.
	 * @param r the rect to fill
	 * @param c the color to use
	 */
	inline void fillRect( SDL_Rect r, ARGB_COLOR c )
	{
		setColor4( c );
		glDisable( GL_TEXTURE_2D );
		drawRect( r );
		glEnable( GL_TEXTURE_2D );
		resetColor();
	}
	/**
	 * Enables the first "count" clipping planes (plane zero to count-1).
	 * @param count the number of clipping planes to enable
	 */
	inline void enableClippingPlanes( int count )
	{
		for( int i = 0; i < count; ++i )
			glEnable( GL_CLIP_PLANE0 + i );
	}
	/**
	 * Disables the first "count" clipping planes (plane zero to count-1).
	 * @param count the number of clipping planes to disable
	 */
	inline void disableClippingPlanes( int count )
	{
		for( int i = 0; i < count; ++i )
			glDisable( GL_CLIP_PLANE0 + i );
	}
	/**
	 * Enables the first 4 clipping planes and sets them so, that you can only
	 * draw inside of the specified rect. The values are applied on the current
	 * modelview-matrix.
	 * @param r the rect in which you want to draw
	 */
	DECLSPEC void set2dClippingPlanes( SDL_Rect r );
};

#endif
