/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLAUDIO_H
#define SDLAUDIO_H

#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include <list>

/**
\ingroup sdl
\file sdlaudio.h
\brief Defines the class SdlAudio.
*/

using namespace std;

/**
\ingroup sdl
\class SdlAudio
\brief Base class for an audio resource

This class is used to handle an audio resource (sound, music...).
*/
class DECLSPEC SdlAudio  
{
public:
	/**
	Function prototype, called on finish playing an audio resource
	\param SdlAudio* pointer to the resource, that just finished playing
	\param void* pointer to any data, as specified in the second param
	of setCallback()
	*/
	typedef void (*CallBack)( SdlAudio *, void * );

	/**
	Constructor. Registers itself in the SdlManager variable.
	\param name the resource's name
	\see SdlManager::getAudioByName()
	*/
	DECLSPEC SdlAudio( const char *name = 0 );
	virtual DECLSPEC ~SdlAudio();

	/**
	Changes the name of the audio resource.
	\param newname the new name of the resource
	*/
	DECLSPEC void rename( const char *newname );
	/// \returns the name of the resource
	const char *getName() const	{ return chName; }

	/**
	Plays the resource.
	\param loops the amount, how often to repeat the sound; to repeat it
	infinitely it has to be -1
	\returns the channel, it is played in (zero, if music)
	*/
	virtual int play( int loops = 0 ) = 0;
	/**
	Starts playing the resource silently and turns louder
	\param ms the time in msecs of the fade, after that, it has the
	final volume
	\param loops the amount, how often to repeat the sound; to repeat it
	infinitely it has to be -1
	\returns the channel, it is played in (zero, if music)
	*/
	virtual int fadeIn( int ms, int loops = 0 ) = 0;
	/// \returns true, if the resource is currently being played; false otherwise
	virtual bool isPlaying() = 0;
	/**
	Breaks playing the resource to be continued later.
	*/
	virtual void pause() = 0;
	/// \returns true, if the resource is currently paused; false otherwise
	virtual bool isPaused() = 0;
	/**
	Resumes playing the resource, if paused
	*/
	virtual void resume() = 0;
	/**
	Stopps the resource being played.
	*/
	virtual void stop() = 0;
	/**
	Turns the volume silentlier and then stopps playing.
	\param ms the time in msecs of the fade
	\returns 1, if it was playing; 0, if not
	*/
	virtual int fadeOut( int ms ) = 0;
	/**
	\returns MIX_NO_FADE, if the sound is currently not fading \n
	MIX_FADING_IN, if the sound is fading in \n
	MIX_FADING_OUT, if the sound is fading out
	*/
	virtual Mix_Fading fadeStatus() = 0;

	/**
	Sets the volume of the resource, when being played. The value is relative
	to the global volume.
	\param v the volume, the range is from 0 to 128; if negative, the volume is
	just recalculated (you usually do not need recalculation, because it makes
	only sense internally, when changing the global volume)
	*/
	virtual void setVolume( int v ) = 0;// range of 0..128
	/// \returns the resource's volume (relative to the global volume), ranges from 0 to 128
	virtual int getVolume() = 0;

	/**
	\returns the channel, the sound is playing in and -1, if it is currently not playing
	*/
	virtual int getChannel() = 0;

	/**
	Sets the callback function, to be called, when the resource halts playing
	\param c is called, when audio-play was halted
	\param pData pointer to anywhere, this value will be given to the callback
	as second param
	\warning do not call any audio methods from the callback function
	*/
	DECLSPEC void setCallBack( CallBack c, void *pData );
	/**
	\returns true, if the class is derived from SdlMusic;
	false, if it is derived from SdlSound
	*/
	virtual bool isMusic() = 0;

	/**
	\returns false, if the resource was loaded successfully; true, if an error
	occured.
	*/
	bool isFailed()	{ return bFailed; }

protected:
	/**
	Sets the failed state, call this, if an error occured and the resource
	could not be initialized.
	*/
	DECLSPEC virtual void failed();
	/// the function to call, when playing finished
	CallBack cb;
	/// the value to use for the second param of the callback function
	void *cbData;

private:
	bool bFailed;
	char *chName;
};

#endif
