/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILEMUSIC_H
#define SDLFILEMUSIC_H

#include "SDL/SDL_mixer.h"
#include "sdlaudio.h"

/**
\ingroup sdl
\file sdlfilemusic.h
\brief Defines the classes SdlMusic and SdlFileMusic.
*/

/**
\ingroup sdl
\class SdlMusic
\brief Base class for music (long audio data)

This class is the base class for all music resources. They are used
to play audio data, that takes much memory.
*/
class DECLSPEC SdlMusic : public SdlAudio
{
public:
	/**
	Constructor. Registers itself in the SdlManager.
	\param name the resource's name
	\see SdlAudio::SdlAudio( const char *name )
	*/
	DECLSPEC SdlMusic( const char *name );
	virtual DECLSPEC ~SdlMusic();

	virtual DECLSPEC int play( int loops = 0 );// returns 0, if any music is played, otherwise -1
	virtual DECLSPEC int fadeIn( int ms, int loops = 0 );
	virtual DECLSPEC bool isPlaying();
	virtual DECLSPEC void pause();
	virtual DECLSPEC bool isPaused();
	virtual DECLSPEC void resume();
	virtual DECLSPEC void stop();
	virtual DECLSPEC int fadeOut( int ms );
	virtual DECLSPEC Mix_Fading fadeStatus();

	virtual DECLSPEC void setVolume( int v );
	virtual DECLSPEC int getVolume()	{ return nVolume; }

	/**
	\returns the channel, the sound is playing in and -1, if it is currently not playing
	\note music resources are always played in channel 0, so this will either return
	zero or minus one.
	*/
	virtual int getChannel()	{ return isPlaying() ? 0 : -1; }

	virtual bool isMusic()	{ return true; }

	/**
	\returns the currently playing music resource or NULL, if no music is being played
	*/
	static SdlMusic *getPlayingStream()	{ return musPlaying; }
	/**
	Changes the global volume, which is used as reference in all
	SdlMusic derived classes.
	\param v the volume to set (range 0 (quiet) to 128 (loudest))
	\note all playing songs' volumes are updated automatically
	*/
	static DECLSPEC void setGlbVolume( int v );
protected:
	/**
	Uninitalizes the music resource. This is automatically called,
	when deleting the object. It halts playing, if necessary and
	frees the resource's data (the mixMusic variable).
	*/
	DECLSPEC void destroy();

	/// the source data of the music
	Mix_Music *mixMusic;
	/// the volume of this resource, relative to the global
	int nVolume;
	/// playing status of the resource (true if playing)
	bool bPlaying;

	/// the global volume (range: 0..128)
	static int g_nVolume;
	/// pointer to the currently playing music or 0, if none is being played
	static SdlMusic *musPlaying;
	/**
	This method should not be called from your application.
	It's a callback function, called, when the music-channel finishes playing
	*/
	static void onMusicFinished()
	{
		if( musPlaying != 0 )	{// just to be on the safe side
			musPlaying->bPlaying = false;
			if( musPlaying->cb )
				(*musPlaying->cb)( (SdlAudio*)musPlaying, musPlaying->cbData );
		}
	}
};


/**
\ingroup sdl
\class SdlFileMusic
\brief Simple class for songs, loaded from a WAV/OGG file.

This is a simple class for playing music resources, loaded from a
wave- or ogg-vorbis-file.

\see SdlMusicList, SdlFileMusicList
*/
class DECLSPEC SdlFileMusic : public SdlMusic
{
public:
	/**
	Constructor. Registers itself in the SdlManager and loads
	the resource data from fn. If loading failed, the fail-state
	will be set.
	\param fn the filename of the wave- or ogg-file to load
	\param name the name of the resource
	\see SdlAudio::SdlAudio( const char *name )
	*/
	DECLSPEC SdlFileMusic( const char *fn, const char *name = 0 );
	virtual DECLSPEC ~SdlFileMusic();

	/**
	Loads wave or ogg data from fn. It is halted, if currently playing
	and the data, inited previously is freed. If loading failed,
	the fail-state will be set.
	\param fn the filename of the wave- or ogg-file to load
	*/
	DECLSPEC bool load( const char *fn );
};

#endif
