/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILESOUND_H
#define SDLFILESOUND_H

#include <list>
#include "sdlaudio.h"
#include "SDL/SDL_mixer.h"

using namespace std;

/**
\ingroup sdl
\file sdlfilesound.h
\brief Defines the classes SdlSound and SdlFileSound.
*/

/**
\ingroup sdl
\class SdlSound
\brief Base class for sounds (short audio data)

This class is the base class for all sound resources. They are used
to play audio data, that takes not much memory.
*/
class DECLSPEC SdlSound : public SdlAudio
{
public:
	/**
	Constructor. Registers itself in the SdlManager.
	\param name the resource's name
	\see SdlAudio::SdlAudio( const char *name )
	*/
	DECLSPEC SdlSound( const char *name );
	virtual DECLSPEC ~SdlSound();

	virtual DECLSPEC int play( int loops = 0 );
	virtual DECLSPEC int fadeIn( int ms, int loops = 0 );
	virtual DECLSPEC bool isPlaying();
	virtual DECLSPEC void pause();
	virtual DECLSPEC bool isPaused();
	virtual DECLSPEC void resume();
	virtual DECLSPEC void stop();
	virtual DECLSPEC int fadeOut( int ms );
	virtual DECLSPEC Mix_Fading fadeStatus();

	virtual DECLSPEC void setVolume( int v );
	virtual int getVolume()	{ return nVolume; }

	virtual DECLSPEC int getChannel();

	virtual bool isMusic()	{ return false; }

	/**
	Changes the global volume, which is used as reference in all
	SdlSound derived classes.
	\param v the volume to set (range 0 (quiet) to 128 (loudest))
	\note all playing sounds' volumes are updated automatically
	*/
	static DECLSPEC void setGlbVolume( int v );
protected:
	/**
	Uninitalizes the sound resource. This is automatically called,
	when deleting the object. It halts playing, if necessary and
	frees the sound data (the mixSound variable).
	*/
	DECLSPEC void destroy();

	/// the source data of the sound
	Mix_Chunk *mixSound;
	/// the channel, where it is playing or -1, if not playing
	int nChannel;
	/// the volume of this resource, relative to the global
	int nVolume;

	/// the global volume (range: 0..128)
	static int g_nVolume;
	/// list containing all playing sound resources
	static list<SdlSound*> lPlayingSounds;
	/**
	This method should not be called from your application.
	It's a callback function, called, when a channel finishes playing
	\param channel the channel, that just finished playing
	*/
	static void onChannelFinished( int channel )
	{
		for( list<SdlSound*>::iterator i = lPlayingSounds.begin(); i != lPlayingSounds.end(); ++i )
			if( !(*i)->nChannel == channel )	{
				(*i)->nChannel = -1;
				if( (*i)->cb )
					(*(*i)->cb)( *i, (*i)->cbData );

				lPlayingSounds.erase( i );
				break;
			}
	}
};


/**
\ingroup sdl
\class SdlFileSound
\brief Simple class for sounds, loaded from a wave-file

This is a simple class for playing sound resources, loaded from a
wave-file.
*/
class DECLSPEC SdlFileSound : public SdlSound
{
public:
	/**
	Constructor. Registers itself in the SdlManager and loads
	the resource data from fn. If loading failed, the fail-state
	will be set.
	\param fn the filename of the wave-file to load
	\param name the name of the resource
	\see SdlAudio::SdlAudio( const char *name )
	*/
	DECLSPEC SdlFileSound( const char *fn, const char *name = 0 );
	virtual DECLSPEC ~SdlFileSound();

	/**
	Loads wave-data from fn. It is halted, if currently playing
	and the data, inited previously is freed. If loading failed,
	the fail-state will be set.
	\param fn the filename of the wave-file to load
	*/
	DECLSPEC bool load( const char *fn );
};

#endif
