/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILESURFACES_H
#define SDLFILESURFACES_H

#include "sdlsurfacecontainer.h"
#include <list>
#include <string>
#include "definitions.h"

using namespace std;

/**
 * \ingroup sdl
 * \file sdlfilesurfaces.h
 * \brief Defines the classes SdlFileSurfacesInfo and SdlFileSurfaces.
 *
 */

/**
\ingroup sdl
\class SdlFileSurfacesInfo
\brief Loads initialisation data for file-surfaces from a file

This class supports loading initialisation data from a text-based
file and methods to get the data in the necessary type.

\todo improve, so that crashs are no more possible, also if the
content of the file is really bad.

\see SdlFileSurfaces
*/
class DECLSPEC SdlFileSurfacesInfo
{
public:
	DECLSPEC SdlFileSurfacesInfo();
	/**
	 * Loads the data from a text based file. If some data
	 * could not be read, it is not availible, though true
	 * is returned.
	 * \param fn the filename, where to load from
	 * \returns true, if file could be opened; false, if not
	 */
	DECLSPEC bool load( const char *fn );
	/**
	 * Save data in non text based format, which cannot be
	 * used for loading. So you should not need it.
	 * \param fn the filename to save to
	 * \returns true, if saving successful
	 * \deprecated
	 */
	DECLSPEC bool save( const char *fn );

	/// \returns true, if a file has been loaded, false otherwise
	bool isLoaded() const	{ return chInfoFile[0] != 0; }

	/**
	 * \returns the filename information, where the image can
	 * be loaded from
	 */
	DECLSPEC const char *getBitmapFN() const;// relative path to >chInfoFile<
	/**
	 * \returns information of the color, which is fully
	 * transparent (the color key)
	 */
	Uint32 getTransparentColor() const	{ return cTransparent; }

	/**
	 * \returns the first rect from the list of rects, that
	 * were read from the file. The iterator will be set
	 * to the first, to be able to get the next rect by
	 * calling getNextRect(). If no rect data could be read,
	 * NULL is returned.
	 */
	DECLSPEC const SDL_Rect *getFirstRect();
	/**
	 * \returns the next rect from the list of rects, that
	 * were read from the file. The iterator will be set
	 * to the next, to be able to get the next rect by
	 * calling the function again. If the iterator is at
	 * the end, NULL is returned.
	 */
	DECLSPEC const SDL_Rect *getNextRect();
	/**
	 * \returns the number of rects that are in the list.
	 */
	int numRects()	{ return lRects.size(); }
private:
	char chBMPFile[256], chInfoFile[256];
	ARGB_COLOR cTransparent;
	list<SDL_Rect> lRects;
	list<SDL_Rect>::iterator rIter;
};

/**
 * \ingroup sdl
 * \class SdlFileSurfaces
 * \brief Manages some graphical resources, loaded from one file.
 * 
 * This class is used to handle surfaces that contain data, loaded
 * from one file. The surfaces' sizes and the contents' position in
 * the image file have to be defined in another, text based file; or
 * if all surfaces have an equal size and the rects in the image-file
 * are arranged one after the other, loading can be done without an
 * initialisation file.
 * 
 * \see SdlFileSurfacesInfo
 */
class SdlFileSurfaces : public SdlSurfaceContainer
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given
	 * name and initializes the containing surfaces by using a
	 * text based file.
	 * \param fn the filename of the text based file
	 * \param name the container's name; if NULL, fn will be used
	 * as name
	 * \param flags the flags of the surfaces, for details, have a look
	 * at the SDL documentation; usually SDL_HWSURFACE is the best value.
	 * \see SdlManager::getSurfacesByName()
	 */
	DECLSPEC SdlFileSurfaces( const char *fn, const char *name = 0, Uint32 flags = SDL_HWSURFACE );
	/**
	 * Constructor. Registers itself in SdlManager with the given
	 * name and initializes the containing surfaces by using the
	 * given params.
	 * \param fn the filename of the image file, containing the data
	 * for the surfaces
	 * \param rBegin the top-left most rect in the image file, which
	 * is the source for the first surface
	 * \param num the number of surfaces/rects to create/use
	 * \param NumPerLine the number of rects to take from every line
	 * \param byrow if true, the rects are taken line by line, if
	 * false column by column (1st row: top to bottom; 2nd row top to
	 * bottom; 3rd row...)
	 * \param name the container's name; if NULL, fn will be used
	 * as name
	 * \param colorkey the color, that is fully transparent when blitting
	 * \param type the type of the image file as string with 3 letters
	 * (BMP, JPG, PNG, TIF, PCX, TGA, PNM, XPM, XCF or LBM);
	 * if NULL, the ending of the filename is used to detect the type
	 * \param flags the flags of the surfaces, for details, have a look
	 * at the SDL documentation; usually SDL_HWSURFACE is the best value.
	 * \see SdlManager::getSurfacesByName()
	 */
	DECLSPEC SdlFileSurfaces( const char *fn, SDL_Rect rBegin, Uint32 num, Uint32 NumPerLine, bool byrow = true,
							  const char *name = 0, ARGB_COLOR colorkey = CLR_INVALID, const char *type = 0, Uint32 flags = SDL_HWSURFACE );
	virtual DECLSPEC ~SdlFileSurfaces();

protected:
	/**
	 * Refreshes all surfaces, by loading the buffer from the computer's RAM
	 * to the graphical RAM, if SDL_HWSURFACE used.
	 * This is neccessary, when restoring an application in MS Windows
	 * \test The refreshing was not tested yet.
	 */
	virtual DECLSPEC void onRefresh();
	
private:
	static SDL_Surface *SdlDisplayFormatRect( SDL_Surface *src, const SDL_Rect &r, bool alpha );
	// deletes all surfaces and clears the list
	void free();
	// if true, the data.pFileInfo is used; otherwise data.pRects is used
	bool createdByInfoFile()	{ return strFilename.empty(); }
	// data of params specified on creation
	string strFilename;
	Uint32 ulFlags;
	struct RectData	{
		SDL_Rect rBegin;
		Uint32 nNum;
		Uint32 nNumPerLine;
		bool bByRow;
		ARGB_COLOR ulColorKey;
		char chType[4];
	};
	union	{
		RectData *pRects;
		SdlFileSurfacesInfo *pFileInfo;
	} data;
};

#endif
