/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILETEXTURES_H
#define SDLFILETEXTURES_H

#include "sdltexturecontainer.h"
#include "sdlfilesurfaces.h" // for SdlFileSurfacesInfo
#include <list>
#include <string>
#include "definitions.h"

using namespace std;

/**
 * \ingroup sdl
 * \file sdlfiletextures.h
 * \brief Defines the class SdlFileTextures.
 *
 */

/**
 * \ingroup sdl
 * \class SdlFileTextures
 * \brief Manages some OpenGL resources, loaded from one file.
 * 
 * This class is used to handle textures that contain data, loaded
 * from one file. The textures' sizes and the contents' position in
 * the image file have to be defined in another, text based file; or
 * if all textures have an equal size and the rects in the image-file
 * are arranged one after the other, loading can be done without an
 * initialisation file.
 * 
 * \see SdlFileSurfacesInfo
 */
class SdlFileTextures : public SdlTextureContainer
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given
	 * name and initializes the containing textures by using a
	 * text based file.
	 * \param fn the filename of the text based file
	 * \param name the container's name; if NULL, fn will be used
	 * as name
	 * \param mirror true to mirror the textures along the x-achsis
	 * \see SdlManager::getImagesByName()
	 */
	DECLSPEC SdlFileTextures( const char *fn, const char *name = 0, bool mirror = false );
	/**
	 * Constructor. Registers itself in SdlManager with the given
	 * name and initializes the containing textures by using the
	 * given params.
	 * \param fn the filename of the image file, containing the data
	 * for the textures
	 * \param rBegin the top-left most rect in the image file, which
	 * is the source for the first texture
	 * \param num the number of textures/rects to create/use
	 * \param NumPerLine the number of rects to take from every line
	 * \param byrow if true, the rects are taken line by line, if
	 * false column by column (1st row: top to bottom; 2nd row top to
	 * bottom; 3rd row...)
	 * \param name the container's name; if NULL, fn will be used
	 * as name
	 * \param colorkey the color, that is fully transparent when blitting
	 * \param type the type of the image file as string with 3 letters
	 * (BMP, JPG, PNG, TIF, PCX, TGA, PNM, XPM, XCF or LBM);
	 * if NULL, the ending of the filename is used to detect the type
	 * \param mirror true to mirror the textures along the x-achsis
	 * \see SdlManager::getSurfacesByName()
	 */
	DECLSPEC SdlFileTextures( const char *fn, SDL_Rect rBegin, Uint32 num, Uint32 NumPerLine, bool byrow = true,
							  const char *name = 0, ARGB_COLOR colorkey = CLR_INVALID, const char *type = 0, bool mirror = false );
	virtual DECLSPEC ~SdlFileTextures();

protected:
	/**
	 * Refreshes all surfaces, by loading the buffer from the computer's RAM
	 * to the graphical RAM.
	 * This is neccessary, when restoring an application in MS Windows
	 * \test The refreshing was not tested yet.
	 */
	virtual DECLSPEC void onRefresh();
	
private:
	// deletes all textures and clears the list
	void free();
	// if true, the data.pFileInfo is used; otherwise data.pRects is used
	bool createdByInfoFile()	{ return strFilename.empty(); }
	// data of params specified on creation
	string strFilename;
	bool bYMirror;
	struct RectData	{
		SDL_Rect rBegin;
		Uint32 nNum;
		Uint32 nNumPerLine;
		bool bByRow;
		ARGB_COLOR ulColorKey;
		char chType[4];
	};
	union	{
		RectData *pRects;
		SdlFileSurfacesInfo *pFileInfo;
	} data;
};

#endif
