/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFONT_H
#define SDLFONT_H

#include "sdlsurfacecontainer.h"
#include "sdlmanager.h"

/**
\ingroup sdl
\file sdlfont.h
\brief Defines the class SdlFont.
*/

/**
\ingroup sdl
\class SdlFont
\brief Used to write text with a specific font

This class is used to write text on SdlSurface objects by using the font
that was used to init it.\n
Currently only fixed letter fonts are supported (every character has the
same width).

\see SdlFontSurfaces
*/
class DECLSPEC SdlFont  
{
public:
	DECLSPEC SdlFont();
	virtual DECLSPEC ~SdlFont();
	/**
	Constructor. Initializes the font with the params.
	\param n the font's identifier (usually bigger, if font-size is bigger);
	it must not be above or equal to SdlFont::nFonts (=16)
	\param light true, if the font is better used on dark background; false
	if it is better visible on light background
	\param bmps the surfaces with the letters
	\param l a string of the letters as they are arranged in bmps
	\see create
	*/
	DECLSPEC SdlFont( int n, bool light, SdlImageContainer *bmps, const char *l );

	/**
	Initializes the font.
	\param n the font's identifier (usually bigger, if font-size is bigger);
	it must not be above or equal to SdlFont::nFonts (=16)
	\param light true, if the font is better used on dark background; false
	if it is better visible on light background
	\param bmps the surfaces with the letters
	\param l a string of the letters as they are arranged in bmps
	\param color the color, the font's letter have
	\param upper true, if only uppercase letters are availible; false, if both
	(upper- and lowercase) are availible
	\returns true, if all went good; false, if an error occured
	*/
	DECLSPEC bool create( int n, bool light, SdlImageContainer *bmps, const char *l, Uint32 color = RGB(128,128,128), bool upper = false );
	/**
	Uninitializes the font.
	*/
	DECLSPEC void destroy();
	/// \returns the surfaces of the letters
	SdlImageContainer *getBmps() const	{ return bmps; }

	/**
	Vertical alignment of a text to draw \n
	\a top the text is drawn at the top of a rect \n
	\a baseline the text is drawn at the vertical center of a rect \n
	\a bottom the text is drawn at the bottom of a rect
	*/
	enum VAlign {top, baseline, bottom};
	/**
	Horizontal alignment of a text to draw \n
	\a left the text's first letter is drawn at the left border of a rect \n
	\a center the text is drawn horizontally centred in a rect \n
	\a right the text's right end is drawn at the right border of a rect
	*/
	enum HAlign {left, center, right};

	/**
	Calculates the position of a text-cursor, that is nearest to a coordinate.
	\param text the text string, can contain newline-characters
	\param dst the rect, in which the string is located
	\param xCur the x coordinate
	\param yCur the y coordinate
	\param va the vertical alignment of the text in the rect
	\param ha the horizontal alignment of the text in the rect
	*/
	DECLSPEC int getCurpos( const char *text, const SDL_Rect &dst, Sint32 xCur, Sint32 yCur, VAlign va = baseline, HAlign ha = center );
	/**
	Calculates the position of a text-cursor, that is nearest to a coordinate.
	\param text the text string, can contain newline-characters
	\param dstx the x coordinate, that sets the string's x-location
	\param dsty the y coordinate, that sets the string's y-location
	\param xCur the x coordinate
	\param yCur the y coordinate
	\param va the vertical alignment of the text in the rect
	\param ha the horizontal alignment of the text in the rect
	*/
	DECLSPEC int getCurpos( const char *text, Sint32 dstx, Sint32 dsty, Sint32 xCur, Sint32 yCur, VAlign va = baseline, HAlign ha = center );

	/**
	Draws the text onto a surface, using a reference position.
	\param surf the destination surface where the text is drawn on
	\param text the string to write
	\param dstx the reference x-position, where to draw the text
	\param dsty the reference y-position, where to draw the text
	\param va the vertical alignment of the text
	\param ha the horizontal alignment of the text
	\param clip the clipping rect
	\param curpos the text-cursor's position (zero, if before first character; strlen(text), if after the last character)
	*/
	DECLSPEC Uint32 write( SdlSurface *surf, const char *text, Sint32 dstx, Sint32 dsty, VAlign va = baseline, HAlign ha = center, const SDL_Rect * clip = 0, int curpos = -1 );
	DECLSPEC Uint32 writeGL( const char *text, Sint32 dstx, Sint32 dsty, VAlign va = baseline, HAlign ha = center, const SDL_Rect * clip = 0, int curpos = -1 );
	/**
	Draws the text onto a surface, using a reference rect.
	\param surf the destination surface where the text is drawn on
	\param text the string to write
	\param dst the reference rect, where to draw the text
	\param va the vertical alignment of the text
	\param ha the horizontal alignment of the text
	\param clip the clipping rect
	\param curpos the text-cursor's position (zero, if before first character; strlen(text), if after the last character)
	*/
	DECLSPEC Uint32 write( SdlSurface *surf, const char *text, const SDL_Rect &dst, VAlign va = baseline, HAlign ha = center, const SDL_Rect * clip = 0, int curpos = -1 );
	/// \todo implement
	DECLSPEC Uint32 writeGL( const char *text, const SDL_Rect &dst, VAlign va = baseline, HAlign ha = center, const SDL_Rect * clip = 0, int curpos = -1 );

	/// the font's color, as set in create()
	Uint32 color;

	/**
	This method can be used to get a pointer to a font.
	\param n the identifyer of the font
	\param light the light-property of the wanted font
	\returns a pointer to a font with the given id and given light-property
	or NULL, if the font does not exist
	*/
	static DECLSPEC SdlFont *get( int n, bool light );

	/**
	Similar to strchr, but returns NULL, if the first occurance of c is
	behind count characters in string.
	\param string the string to scan
	\param c the character to look for
	\param count the number of characters of string to check
	\returns a pointer to the first occurance of c in string, or NULL, if
	it does not occur up to the count'th character
	*/
	static DECLSPEC const char *strnchr( const char *string, int c, size_t count );
	/**
	Similar to strrchr, but only seeks in the first count characters of string.
	\param string the string to scan
	\param c the character to look for
	\param count the number of characters of string to check
	\returns a pointer to the last occurance of c in the first count characters
	of string, or NULL, if it does not occur up to the count'th character
	*/
	static DECLSPEC const char *strnrchr( const char *string, int c, size_t count );
	/**
	Converts a string to a kind of vertical alignment. Possible strings are:
	top; baseline (or center); bottom.
	\param s the string to convert
	\returns the corresponding value or top, if the expression is unknown
	*/
	static DECLSPEC VAlign strToVAlign( const char *s );
	/**
	Converts a string to a kind of horizontal alignment. Possible strings are:
	left; center (or middle); right.
	\param s the string to convert
	\returns the corresponding value or left, if the expression is unknown
	*/
	static DECLSPEC HAlign strToHAlign( const char *s );
protected:
	/// whether to use uppercase letters only, or mixed
	bool bUpperCase;
	/// the images of the letters
	SdlImageContainer *bmps;
	/// the characters, as arranged in the surface container
	char *chLetters;

private:
	struct FONT
	{
		SdlFont *f[2];
	};
	static FONT *lFonts;
	// the maximum number of light fonts or dark fonts allowed
	static const int nFonts;

	void init();
	char * calcRectToPoint( const char *text, const SDL_Rect &dst, int &xPos, int &yPos, VAlign va, HAlign ha, int curpos );

	int num;
	bool bLight;
};

#endif
