/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLIMAGE_H
#define SDLIMAGE_H

#include "SDL/SDL.h"
#include <list>
#include "definitions.h"

using namespace std;

/**
 * \ingroup sdl
 * \file sdlimage.h
 * \brief Defines the class SdlImage and SdlImageContainer.
 */

/**
 * \ingroup sdl
 * \class SdlImage
 * \brief Base class for an image resource to use for drawing.
 * 
 * This class is the base for all image resources, OpenGL-based as well as
 * 2D-blitting-based. It supports very common methods like specifying the
 * resource's name and (un)registering itself in the SdlManager.
 */
class DECLSPEC SdlImage
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given name.
	 * After the construction, it cannot be used for drawing, because
	 * there is no texture buffer created or assigned.
	 * \param name the resource's name
	 */
	DECLSPEC SdlImage( const char *name );
	virtual DECLSPEC ~SdlImage();

	/**
	 * Changes the resource's name.
	 * \param newname the new name of the resource
	 * \see SdlManager::getImageByName()
	 */
	void DECLSPEC rename( const char *newname );
	/// \returns the resource's name
	const char *getName() const			{ return chName; }
	
	/// \returns true if the creation of the surface failed, otherwise false
	bool isFailed()	{ return bFailed; }

	/**
	 * \param w the image's width will be stored here
	 * \param h the image's height will be stored here
	 */
	void getSize( int &w, int &h )	{ w = (int)nWidth;	h = (int)nHeight; }
	/**
	 * \param w the image's width will be stored here
	 * \param h the image's height will be stored here
	 */
	void getSize( Uint16 &w, Uint16 &h )	{ w = nWidth;	h = nHeight; }

	/// \returns the texture's width
	Uint32 getWidth() const		{ return nWidth; }
	/// \returns the texture's height
	Uint32 getHeight() const	{ return nHeight; }

protected:
	/**
	 * Should refresh the texture in a derived class.
	 * In this base class, nothing is done.
	 */
	virtual DECLSPEC void onRefresh()	{}
	/// sets the fail-state of the resource
	DECLSPEC void failed();
	
	unsigned int nWidth/** image's width */, nHeight/** image's height */;

private:
	void initName( const char *name );
	char *chName;
	bool bFailed;
};


/**
 * \ingroup sdl
 * \class SdlImageContainer
 * \brief Base class for managing image containers.
 * 
 * This class is the base for all image resource containers, OpenGL-based as
 * well as 2D-blitting-based. It supports very common methods like specifying
 * the resource's name and (un)registering itself in the SdlManager.
 */
class DECLSPEC SdlImageContainer
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given name.
	 * \param name the container's name
	 * \see SdlManager::getImagesByName()
	 */
	DECLSPEC SdlImageContainer( const char *name );
	virtual DECLSPEC ~SdlImageContainer();

	/// \returns the container's name
	const char *getName() const	{ return chName; }
	/// \returns the fail-state (true, if failed; false, if not)
	bool isFailed()	{ return bFailed; }
	
	/**
	 * Gets an image, which is stored in the container.
	 * \param i the number of the image
	 * \returns a pointer to the selected image or NULL, if the image is not
	 * availible
	 */
	virtual SdlImage *getImage( unsigned int i ) = 0;
	/// \returns the number of images, stored in the container
	virtual unsigned int numImages() = 0;

protected:
	/**
	 * Should refresh the texture in a derived class.
	 * In this base class, nothing is done.
	 */
	virtual DECLSPEC void onRefresh()	{}
	/// sets the fail-state of the resource
	DECLSPEC void failed();

private:
	void initName( const char *name );
	char *chName;
	bool bFailed;
};

#endif
