/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLMANAGER_H
#define SDLMANAGER_H

#include "SDL/SDL.h"
#include "tinyxml.h"
#include <list>
#include "sdlsurface.h"

/**
 * Makes a color-value for the display-surface out of the specified red, green
 * and blue values.
 * \note usually ARGB color values are used, which can simply be specified as
 * integer in the form of 0xAARRGGBB and thus needs not to be calculated at
 * run-time. So better avoid using this declaration.
 */
#define RGB( r, g, b )		SDL_MapRGB( SdlManager::the->getDisplayFormat(), r, g, b )
/// Just an abbreviation, see SdlSurface::convertColor()
#define convertDisplayARGB( c )	SdlManager::the->getDisplaySimple()->convertColor( c )
/// Just an abbreviation, see SdlSurface::isColorVisible()
#define isDisplayColorVisible( c )	SdlManager::the->getDisplaySimple()->isColorVisible( c )

using namespace std;

class SdlImageContainer;
class SdlAudio;
class SdlMusicList;

/**
 * \defgroup sdl SDL based classes
 * \version 0.8
 *
 * \author Tobias Nadler (nadlerto at gmx dot de)
 * 
 * The Sdl* classes are the interface between the SDL library and the netrinjo
 * UI classes (user interface). Networking is not part of these classes.
 * 
 * \section video Video Output
 * For video output the most important class is SdlSurface as the base for
 * drawing 2D-graphics onto the screen or other surfaces. For convenient
 * loading of data, the classes SdlFileSurface and SdlFileSurfaces (for
 * multiple surfaces) are provided.
 * 
 * Because the color-mapping may be different between environments, Netrinjo
 * offers an uniform mapping in ARGB-byte-order. This is used everywhere you
 * see ARGB_COLOR as type. Especially in text-based files that are read during
 * run-time, this byte-order is always used when reading color-values.
 * 
 * \see ui
 */

/**
 * \ingroup sdl
 * \file sdlmanager.h
 * \brief Defines the class SdlManager.
 */

/**
 * \ingroup sdl
 * \class SdlManager
 * \brief The central class, that manages Sdl*-classes
 * 
 * This class provides functions to initialize and access audio and graphic
 * resources. It also contains methods to get video and audio settings and a
 * function to make screenshots.
 * 
 * \ingroup singleusage
 * This class can only be used in one variable at a time. So do not initialize
 * it more than once, but it is required once, if you need a user interface.
 */
class DECLSPEC SdlManager
{
public:
	DECLSPEC SdlManager();
	virtual DECLSPEC ~SdlManager();

	/**
	 * Initializes video and audio, using a screen resolution of
	 * 1024*768*32Bit.
	 * \returns true, if video and audio could be initialized, false if at
	 * least one failed
	 */
	DECLSPEC bool init();
	/**
	Initializes video and audio, using a XML node to set the screen resolution.
	\param el a XML element, that contains attributes, telling the resolution:
	- Width: width of the resolution (in pixels, x direction); if it is not
	  specified, resolution will be set to 1024.
	- Height: height of the resolution (in pixels, y direction); if it is not
	  specified, resolution will be set to 768.
	- Depth: color depth of the resolution in bits; if it is not
	  specified, resolution will be set to 32.
	- Fullscreen: (a boolean value) whether to use fullscreen-mode or not;
	  if not specified, fullscreen-mode will be activated;
	  if true, input is grabbed, otherwise not
	Furthermore by putting a subnode "WaitOnStart", containing the attribute
	"value", the function will delay for the given amount (in msecs).
	\returns true, if video and audio could be initialized, false if at least
	one failed
	*/
	DECLSPEC bool init( TiXmlElement *el );
	/**
	Initializes video and audio, using the params to set the screen resolution.
	\param width of the resolution (in pixels, x direction)
	\param height of the resolution (in pixels, y direction)
	\param depth color depth of the resolution in bits
	\param fullscreen whether to use fullscreen-mode or not; true by default;
	       if true, input is grabbed, otherwise not
	\param opengl whether to either use OpenGL or do 2D-blitting
	\returns true, if video and audio could be initialized, false if at least
	one failed
	*/
	DECLSPEC bool init( Uint32 width, Uint32 height, Uint32 &depth, bool fullscreen = true, bool opengl = false );
	/**
	 * Uninitializes the manager. All resources will be freed and the audio
	 * mixer will be closed.
	 */
	DECLSPEC void destroy();

	/// \returns true, if audio can be used, false if not
	bool isAudioInited()	{ return bAudioInited; }
	/// \returns true, if video can be used, false if not
	bool isVideoInited()	{ return bVideoInited; }
	/// \returns true, if OpenGL is used for drawing; false if blitting is used
	bool isUsingOpenGL()	{ return (nVideoFlags & SDL_OPENGL) != 0; }
	/// \returns true, if the video is used in fullscreen mode
	bool isFullscreen()	{ return (nVideoFlags & SDL_FULLSCREEN) != 0; }
	/**
	 * Refreshes the sound volumes of all sound resources.
	 * This is necessary, when changing sound-volume globally.
	 */
	DECLSPEC void refreshSoundVolumes();

	/**
	 * Use this function, if you have only the name and no resource of the
	 * sound you need.
	 * \param name the name of the audio resource to return
	 * \returns a pointer to the audio resource with the given name
	 */
	DECLSPEC SdlAudio *getAudioByName( const char *name );
	/**
	 * Use this function, if you have only the name and no resource of the
	 * surface you need.
	 * \param name the name of the image resource to return
	 * \returns a pointer to the image resource with the given name
	 */
	DECLSPEC SdlImage *getImageByName( const char *name );
	/**
	 * Use this function, if you have only the name and no resource of the list
	 * of images you need.
	 * \param name the name of the image list resource to return
	 * \returns a pointer to the image list resource with the given name
	 */
	DECLSPEC SdlImageContainer *getImagesByName( const char *name );
	/**
	 * Use this function, if you have only the name and no resource of the list
	 * of songs/audios you need.
	 * \param name the name of the audio list resource to return
	 * \returns a pointer to the audio list resource with the given name
	 */
	DECLSPEC SdlMusicList *getMusicsByName( const char *name );
	/**
	 * \returns a pointer to the surface that will be shown, when the next
	 * frame becomes active.
	 */
	SdlSurface *getDisplay()	{ SDL_SetClipRect( surfDisplay->surf(), 0 ); return surfDisplay; }
	/**
	 * \returns a pointer to the display surface in blitting-mode
	 */
	SdlSurface *getDisplaySimple()	{ return surfDisplay; }
	/**
	 * Equal to "&getDisplaySimple()->surf()->format".
	 * \returns a pointer to the pixel-format used on the display surface
	 */
	SDL_PixelFormat *getDisplayFormat()	{ return surfDisplay->surf()->format; }
	/**
	 * Use this to get the resolution of the screen and to ask, whether
	 * fullscreen-mode is active or not.
	 * \param w reference, where the width will be stored
	 * \param h reference, where the height will be stored
	 * \returns true, if fullscreen-mode is active; false if it is not
	 */
	DECLSPEC bool getScreenSize( int &w, int &h );
	/**
	 * \returns the screen's rect (top-left corner is at (0|0); width and
	 * height equal the values of getScreenSize())
	 */
	DECLSPEC const SDL_Rect &getScreenRect();
	/**
	 * \returns the screen's width
	 * \warning you must not call this, as long as the video-mode is not set up
	 * successfully!
	 */
	Uint32 getScreenWidth()	{ return surfDisplay->getWidth(); }
	/**
	 * \returns the screen's height
	 * \warning you must not call this, as long as the video-mode is not set up
	 * successfully!
	 */
	Uint32 getScreenHeight()	{ return surfDisplay->getHeight(); }

	/**
	 * Makes a screenshot of the currently visible screen and saves the
	 * picture to a file with the bitmap format.
	 * \param fn the filename, where the bitmap will be saved. Use "%d" in it
	 * to set the number of the photo there (if nooverwrite is false) or to
	 * place a number there in order not to overwrite another photo (if
	 * nooverwrite true)\n
	 * if NULL, "./screens/photo%d.bmp" will be used
	 * \param nooverwrite if false and the given filename exists, the file
	 * will be overwritten; if true, no file will be overwritten, but if
	 * "%d" does not occur in fn, and fn exists, the photo will not be saved
	 * \returns true, if the photo was saved, false if not
	 * \todo enable OpenGL screenshot
	 */
	DECLSPEC bool screenShot( const char *fn = 0, bool nooverwrite = true );
	/// Grabs the mouse+keyboard input, so that only this apps recieves events.
	static void grabInput( void )	{ if( bGrabInputEnabled ) SDL_WM_GrabInput( SDL_GRAB_ON ); }
	/// Ungrabs the mouse+keyboard input, so that other apps can recieve input events.
	static void ungrabInput( void )	{ SDL_WM_GrabInput( SDL_GRAB_OFF ); }

	/**
	 * only accessible in Linux, and equals the result of getThe()
	 */
	static SdlManager *the;
	/**
	 * \returns a pointer to the SdlManager-variable, or NULL if not
	 * initialized. (There is only one SdlManager-variable allowed in an
	 * application)
	 * 
	 * Every netrinjo-class that can only exist once at a time, has this static
	 * function.
	 * \see singleusage
	 */
	DECLSPEC static SdlManager *getThe();

	/// indicates, whether or not, input grabbing is enabled
	static bool bGrabInputEnabled;
private:
	bool initDisplay( Uint32 w, Uint32 h, Uint32 &d, bool fullscr = true, bool opengl = false );
	bool initAudio( Uint8 channels = 2/*stereo*/, int freq = 22050, Uint16 format = AUDIO_S16SYS );
	SdlSurface *surfDisplay;

	bool bVideoInited, bAudioInited;
	Uint32 nColorDepth;
	Uint32 nVideoFlags;

	list<SdlAudio*> lAudios, lFailedAudios;
	list<SdlImage*> lImages, lFailedImages;
	list<SdlImageContainer*> lImageContainers, lFailedImageContainers;
	list<SdlMusicList*> lMusicLists;
	typedef list<SdlAudio*>::iterator AudioIter;
	typedef list<SdlImage*>::iterator ImageIter;
	typedef list<SdlImageContainer*>::iterator ImagesIter;
	typedef list<SdlMusicList*>::iterator MusicsIter;

	friend class SdlAudio;
	friend class SdlImage;
	friend class SdlImageContainer;
	friend class SdlMusicList;
};

#endif
