/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLMUSICLIST_H
#define SDLMUSICLIST_H

#include <vector>
#include "SDL/SDL.h"

using namespace std;

class SdlMusic;
class SdlAudio;

/**
\ingroup sdl
\file sdlmusiclist.h
\brief Defines the classes SdlMusicInfo and SdlMusicList.
*/

/**
\ingroup sdl
\class SdlMusicInfo
\brief Base class for managing a list of music resources.

This class manages the filename, author, title and duration information
of a song.

\see SdlMusicList, SdlFileMusic
*/
class DECLSPEC SdlMusicInfo
{
public:
	/**
	Constructor. Manages to store filename, title and author to the
	concerning variables.
	\param fn the filename of the song
	\param title the title, or however you want to use it
	\param author the author, or however you want to use it
	\param secs the length of the song in seconds
	*/
	DECLSPEC SdlMusicInfo( const char *fn, const char *title, const char *author, int secs = -1 );
	virtual DECLSPEC ~SdlMusicInfo();

	/**
	Changes the filename of the song.
	\param n the new filename
	*/
	DECLSPEC void setFileName( const char *n );
	/// \returns the filename of the song
	const char * getFileName()	{ return chName; }

	/**
	Changes the title-info of the song.
	\param t the new title
	*/
	DECLSPEC void setTitle( const char *t );
	/// \returns the title of the song
	const char * getTitle()	{ return chTitle; }

	/**
	Changes the author-info.
	\param a the new author
	*/
	DECLSPEC void setAuthor( const char *a );
	/// \returns the author
	const char * getAuthor()	{ return chAuthor; }

	/**
	Changes the duration-info of the song.
	\param l the new length in seconds
	*/
	void setLength( int l )	{ nLength = l; }

	/// \returns the amount, how often the song has been played
	int numPlayed()	{ return nPlayed; }
	/**
	Call it, when the song has been played.
	Increments the played-counter.
	*/
	void played()	{ ++nPlayed; }
	/**
	Resets the played-counter to zero.
	*/
	void reset()	{ nPlayed = 0; }

private:
	char *chName;// filename
	char *chTitle;
	char *chAuthor;
	int nPlayed;// how often it was played yet
	int nLength;// in seconds
};



/**
\ingroup sdl
\class SdlMusicList
\brief Base class for managing a list of music resources.

Required header file:
\code #include <sdlmusiclist.h> \endcode

This is a base class to manage music resources. (Actually you can
use it as base to manage any list of strings, but it was designed
for music lists)

\see SdlMusicInfo, SdlFileMusic
*/
class SdlMusicList
{
public:
	/**
	Function prototype of the methods to call after playing some
	songs. Details see setCallBack()
	\param SdlMusicList* the sender of the callback-event
	\param void* pointer to any data, value equals the second param
	of setCallBack()
	*/
	typedef void ( *CallBack )( SdlMusicList *, void * );
	/**
	Some ways of playing a list
	\a ptForward play songs in strict rotation
	\a ptBackward play songs strict against rotation
	\a ptShuffled take any song to play, until all have been played once
	\a ptRandom take any song to play without regard for how often a song was played
	*/
	enum PlayTypes{ptForward, ptBackward, ptShuffled/*don't play one twice until everyone has been played*/, ptRandom};

	/**
	Constructor. Registers itself in SdlManager with the given name.
	\param name the name of the resource
	*/
	DECLSPEC SdlMusicList( const char *name );
	virtual DECLSPEC ~SdlMusicList();

	/// \returns the name of the resource
	const char *getName() const		{ return chName; }
	/**
	Get a pointer to a selected song of the list
	\param i the number of the song in the list
	\returns a pointer to the song
	*/
	DECLSPEC SdlMusic *getSong( unsigned int i );
	/**
	Get a pointer to a selected song of the list
	\param i the number of the song in the list
	\returns a pointer to the song's additional information
	*/
	DECLSPEC SdlMusicInfo *getSongInfo( unsigned int i );
	/// \returns the number of songs that are in the list
	unsigned int numSongs()			{ return lSongs.size(); }

	/**
	Starts playing the list (infinite).
	\param t the type, how to play the list
	\returns true, if playing could be started; false on error, of if
	the list is empty.
	*/
	DECLSPEC bool play( PlayTypes t = ptForward );
	/// halts playing the list
	DECLSPEC void stop();

	/**
	Changes the volume, which to use, playing the songs
	(relative to the global volume)
	\param v the new volume (possible range: 0 to 128)
	*/
	DECLSPEC void setVolume( int v );

	/**
	Sets the function to call after every song of the list hasbeen
	played (will not be called, if the type is ptRandom!)
	\param c pointer to the function to call
	\param data pointer to anywhere, it will be sent to the called
	function as second param
	*/
	void setCallBack( CallBack c, void *data )	{ cb = c; cbData = data; }

	/**
	\returns the fail-state, which is false, if the list is so
	far successfully initialized, that using it is possible
	*/
	bool isFailed()		{ return bFailed; }

protected:
	/// sets the fail-state
	virtual DECLSPEC void failed();

	/// list of all songs
	vector<SdlMusic *> lSongs;
	/// list of all additional song information
	vector<SdlMusicInfo*> lInfos;

private:
	bool playNext();

	bool bFailed;
	char *chName;

	PlayTypes ePlayType;
	bool bStopped;

	int nVolume;

	CallBack cb;
	void *cbData;

	int nCurrentSong;// ID of currently playing song
	SdlMusic *musPlaying;
	static inline void onFinished( SdlAudio *sender, void *data )
	{
		((SdlMusicList*)data)->playNext();
	}
};

#endif
