/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLSURFACE_H
#define SDLSURFACE_H

#include "SDL/SDL.h"
#include <list>
#include "definitions.h"
#include "sdlimage.h"

using namespace std;

/**
 * \ingroup sdl
 * \file sdlsurface.h
 * \brief Defines the class SdlSurface.
 */

/**
 * \ingroup sdl
 * \class SdlSurface
 * \brief Base class for a graphical surface resource to use for 2D blitting.
 * 
 * This class contains methods for painting/drawing on a surface resource
 * and to get and set information concerning the surface.
 * 
 * \note this class (and derived ones) cannot be used for drawing in OpenGL!
 */
class DECLSPEC SdlSurface : public SdlImage
{
public:
	/**
	Constructor. Registers itself in SdlManager with the given name.
	After the construction, it cannot be used for drawing, because
	there is no surface buffer created or assigned.
	\param name the resource's name
	*/
	DECLSPEC SdlSurface( const char *name );
	/**
	Constructor. Registers itself in SdlManager with the given name.
	After the construction, the given surface buffer is used.
	\param surf the surface buffer to use; if NULL, this constructor
	does the same as SdlSurface( const char *name )
	\param name the resource's name
	\see SdlManager::getSurfaceByName()
	*/
	DECLSPEC SdlSurface( SDL_Surface *surf, const char *name );
	/**
	Constructor. Registers itself in SdlManager with the given name.
	After the construction, a surface buffer with the given size and
	flags is availible for usage.
	\param w the width in pixels of the surface buffer to create
	\param h the height in pixels of the surface buffer to create
	\param name the resource's name
	\param flags the flags of the surface, for details, have a look
	at the SDL documentation; usually SDL_HWSURFACE is the best value.
	\see SdlManager::getSurfaceByName()
	*/
	DECLSPEC SdlSurface( Uint32 w, Uint32 h, const char *name = (char*)-1, Uint32 flags = SDL_HWSURFACE );
	virtual DECLSPEC ~SdlSurface();

	/**
	Draws the content of another surface onto this surface, using the
	color key and alpha value of the source surface.
	This method is (much) faster, than the other blit-functions. When
	using a transparent alpha value, it will take longer.
	\param src_surf the source surface, its content will be used to draw
	\param xDst the coordinate of the left border, where it will be drawn
	\param yDst the coordinate of the top border, where it will be drawn
	\param src the rect in src_surf, to draw; if NULL, the whole content
	of src_surf will be drawn
	\returns true, if successful; false on error
	*/
	DECLSPEC bool fastBlit( SdlSurface *src_surf, Sint16 xDst, Sint16 yDst, SDL_Rect *src );
	/**
	Fills a rect with the specified color. This method is usually similar
	fast as fastBlit() without transparency.
	\param r pointer to the rect to fill; if NULL, the whole surface will
	be filled.
	\param color the color to use for filling (surface-specific format)
	\returns true, if successful; false on error
	*/
	DECLSPEC bool fill( SDL_Rect *r, Uint32 color );
	/**
	 * Equal to fill(), but the color here is in the surface-independent
	 * ARGB-Byte-format.
	 */
	DECLSPEC bool fillARGB( SDL_Rect *r, ARGB_COLOR color );
	/**
	Draws another surface zoomed onto this surface.
	\param src_surf the source surface, its content will be used to draw
	\param xDst the coordinate of the left border, where it will be drawn
	\param yDst the coordinate of the top border, where it will be drawn
	\param xZoom stretch factor to use in x direction i.e. 2 = double size
	\param yZoom stretch factor to use in y direction i.e. 2 = double size
	\param smooth if true, anti aliasing will be used (takes much time)
	\returns true, if successful; false on error
	*/
	DECLSPEC bool zoomBlit( SdlSurface *src_surf, Sint16 xDst, Sint16 yDst, double xZoom, double yZoom, bool smooth = false );
	/**
	Draws another surface stretched into a rect of this surface.
	\param src_surf the source surface, its content will be used to draw
	\param dst the rect to draw it in
	\param smooth if true, anti aliasing will be used (takes much time)
	\returns true, if successful; false on error
	*/
	DECLSPEC bool stretchBlit( SdlSurface *src_surf, SDL_Rect *dst, bool smooth = false );
	/**
	Draws another surface zoomed and rotated onto this surface.
	\param src_surf the source surface, its content will be used to draw
	\param xDst the x-coordinate of the center, where it will be drawn
	\param yDst the y-coordinate of the center, where it will be drawn
	\param angle the angle in degrees (clockwise rotation)
	\param zoom stretch factor to use in both directions (x and y)
	\param smooth if true, anti aliasing will be used (takes much time)
	\returns true, if successful; false on error
	*/
	DECLSPEC bool rotozoomBlit( SdlSurface *src_surf, Sint16 xDst, Sint16 yDst, double angle, double zoom, bool smooth = false );
	
	/// \returns a pointer to the surface buffer
	SDL_Surface *surf()			{ return surface; }
	/**
	\param w the surface's width will be stored here
	\param h the surface's height will be stored here
	*/
	void getSize( int &w, int &h )	{ w = surface->w;	h = surface->h; }
	/**
	\param w the surface's width will be stored here
	\param h the surface's height will be stored here
	*/
	void getSize( Uint16 &w, Uint16 &h )	{ w = surface->w;	h = surface->h; }

	/// \returns the surface's width
	Uint32 getWidth() const		{ return surface->w; }
	/// \returns the surface's height
	Uint32 getHeight() const	{ return surface->h; }

	/**
	Changes the clip rect. When blitting/filling, nothing will be drawn
	beyond its borders.
	\param r the new clipping rect
	*/
	void setClip( const SDL_Rect &r )	{ SDL_SetClipRect( surface, &r ); }
	/**
	Changes the clip rect to the whole area of the surface.
	*/
	void setNoClip()	{ SDL_SetClipRect( surface, NULL ); }
	/**
	\returns the clipping rect
	*/
	const SDL_Rect &getClip() const		{ return surface->clip_rect; }
	/**
	Sets the color, that will be fully transparent when drawing.
	\param color the color to set transparent
	*/
	DECLSPEC void setColorKey( ARGB_COLOR color );
	/**
	 * Changes the transparency for blits.
	 * \param a the new alpha value
	 * \note the range goes from SDL_ALPHA_TRANSPARENT (0) to SDL_ALPHA_OPAQUE
	 * (255).
	 */
	DECLSPEC void setAlphaValue( unsigned char a );

	/**
	Locks the surface. When locked, the surface cannot be used for
	blitting/filling.
	\returns true, if locking was possible; false on error
	*/
	bool lock()	{ return SDL_LockSurface( surface ) >= 0; }
	/**
	Unlocks the surface.
	*/
	void unlock()	{ SDL_UnlockSurface( surface ); }
	/**
	 * Gets a pixel's color.
	 * \param x,y the pixel-position
	 * \returns the color-value
	 * \note The surface must usually be locked during this procedure!
	 */
	SDL_Color getPixel( unsigned int x, unsigned int y );
	/**
	 * Sets a pixel of the surface.
	 * \param x,y the pixel-position
	 * \param color the color-value in the correct bit-format
	 * \note The surface must usually be locked during this procedure!
	 */
	void setPixel( unsigned int x, unsigned int y, Uint32 color );
	/**
	 * Sets a pixel of the surface.
	 * \param x,y the pixel-position
	 * \param color the color
	 * \note The surface must usually be locked during this procedure!
	 */
	void setPixel( unsigned int x, unsigned int y, SDL_Color color );
	/**
	 * Converts a color-value in ARGB order (Alpha-value by bits 31..24, Blue
	 * value by bits 7..0) to the color-value as used by the surface-surface.
	 * The alpha-value is inverted, so that 0xFFrrggbb is an invisible color
	 * and 0x00rrggbb an opaque color.
	 * \param cARGB the ARGB-color-value
	 */
	DECLSPEC Uint32 convertColor( ARGB_COLOR cARGB );
	/**
	 * Checks, whether the color-value (in surface-format) is visible
	 * (has an alpha-value above zero)
	 * \returns true if visible, false if invisible
	 */
	DECLSPEC bool isColorVisible( Uint32 color );

	/**
	Saves the surface to a bitmap file (BMP format).
	*/
	virtual DECLSPEC bool saveToFile( const char *fn );

protected:
	/// sets a new surface and assigns the size to nWidth and nHeight
	void setSurface( SDL_Surface *s );
	/// pointer to the surface buffer
	SDL_Surface *surface;
};

#endif
