/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLTEXTURE_H
#define SDLTEXTURE_H

#include "SDL/SDL.h"
#include "GL/gl.h"
#include <list>
#include "sdlimage.h"

using namespace std;

/**
 * \ingroup sdl
 * \file sdltexture.h
 * \brief Defines the class SdlTexture.
 */

/**
 * \ingroup sdl
 * \class SdlTexture
 * \brief Base class for a graphical OpenGL-resource to use for drawing.
 * 
 * This class contains methods for painting/drawing a texture resource
 * and to get and set information concerning the texture.
 * 
 * \note the texture coordinates are (0, 0) at top-left edge, (0, 1) at
 * bottom-left edge, (1, 0) at top-right edge and (1, 1) at bottom-right edge!
 */
class DECLSPEC SdlTexture : public SdlImage
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given name.
	 * After the construction, it cannot be used for drawing, because
	 * there is no texture buffer created or assigned.
	 * \param name the resource's name
	 */
	DECLSPEC SdlTexture( const char *name );
	/* *
	 * Constructor. Registers itself in SdlManager with the given name.
	 * After the construction, a copy of the given surface is used.
	 * \param surf the surface data to copy to the texture, this may be deleted
	 * after the construction of the SdlTexture object.
	 * If NULL, this constructor does the same as SdlTexture(const char *name)
	 * \param name the resource's name
	 * \see SdlManager::getTextureByName()
	 */
// 	DECLSPEC SdlTexture( SDL_Surface *surf, const char *name );
	/**
	 * Constructor. Registers itself in SdlManager with the given name.
	 * After the construction, the specified resource is used.
	 * \param gl_id the id of the OpenGL texture resource.
	 * If zero, this constructor does the same as SdlTexture(const char *name)
	 * \param w,h the size (width and height) of the texture in pixels
	 * \param name the resource's name
	 * \see SdlManager::getTextureByName()
	 */
	DECLSPEC SdlTexture( GLuint gl_id, unsigned int w, unsigned int h, const char *name );
	virtual DECLSPEC ~SdlTexture();

	/// \returns the texture's OpenGL-name, zero if invalid
	/// (i.e. to use in glBindTexture() as 2nd param)
	GLuint getId()	{ return nId; }

	/**
	 * Draws the whole texture into the framebuffer.
	 * \param r the rect where to draw the texture
	 */
	DECLSPEC void drawSimple( const SDL_Rect &r );
	/**
	 * Draws a part of the texture into the framebuffer.
	 * \param src the rect from the texture to draw (note that in most cases,
	 * the texture is mirrored in y-direction); the ranges are from zero to the
	 * texture's width and height respectively.
	 * \param dst the rect where to draw the texture part
	 */
	DECLSPEC void draw( const SDL_Rect &src, const SDL_Rect &dst );
	/**
	 * Draws a part of the texture into the framebuffer.
	 */
	DECLSPEC void draw( const FloatRect &src, const SDL_Rect &dst );
	/**
	 * Draws the texture into the clip-rect on the framebuffer.
	 * \param clip the clipping rect
	 * \param whole the size+position of the area that matches the whole
	 * texture.
	 * \note if "clip" is (partially) outside of "whole", the texture is by
	 * default repeated in that area.
	 */
	DECLSPEC void drawClipped( SDL_Rect clip, SDL_Rect whole );
	/**
	 * Draws the texture zoomed and rotated into the framebuffer.
	 * \param src the rect from the texture to draw (note that in most cases,
	 * the texture is mirrored in y-direction); the ranges are 0 to 1 in x and
	 * y direction.
	 * \param dst the position where to draw it and the size: the center of the
	 * painting will be at (dst.x+dst.w/2; dst.y+dst.h/2); the size is dst.w
	 * and dst.h, but in the turned direction.
	 * \param angle the angle in which to draw the texture part
	 */
	DECLSPEC void rotoDraw( const FloatRect &src, const SDL_Rect &dst, float angle );
	DECLSPEC void rotoDrawClipped( const SDL_Rect &clip, int x, int y, float angle, float zoom );
	
	/**
	 * Sets a pixel of the texture.
	 * \param x,y the pixel-position
	 * \param color the color-value in the correct bit-format: on little-endian
	 * architectures it has to be a ARGB_COLOR value
	 * \note The surface must usually be locked during this procedure!
	 */
	void setPixel( unsigned int x, unsigned int y, Uint32 color );
	/**
	 * Sets a pixel on the texture.
	 * \param x,y the pixel-position
	 * \param color the RGB-color
	 * \note The surface must usually be locked during this procedure!
	 */
	void setPixel( unsigned int x, unsigned int y, SDL_Color color );

	/**
	 * Converts the pixel data from a software surface to an array of bytes,
	 * with 4 bytes per pixel (in BGRA format).
	 * \param src the source surface
	 * \param format the byte order of the result is assigned to this variable,
	 * it is either GL_RGBA, or GL_BGRA
	 * \param mirrored if true, the lines are interchanged (top to bottom ->
	 * bottom to top)
	 */
	static unsigned char *pixelsFromSurface4ub( SDL_Surface *src, GLenum &format, bool mirrored = false );
	/**
	 * As pixelsFromSurface4b(SDL_Surface, bool), but takes a rect from the
	 * source surface instead of the whole.
	 * \param src the source surface
	 * \param r the rect in the source-surface to read from.
	 * \param mirrored if true, the lines are interchanged (top to bottom ->
	 * bottom to top)
	 */
	static unsigned char *pixelsFromSurface4ub( SDL_Surface *src, const SDL_Rect &r, GLenum &format, bool mirrored = false );

protected:
	/**
	 * Should refresh the texture in a derived class.
	 * In this base class, nothing is done.
	 */
	virtual DECLSPEC void onRefresh()	{}

	/// ID of the texture buffer; zero if not assigned or invalid
	GLuint nId;
};

#endif
