/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef THEMEMANAGER_H
#define THEMEMANAGER_H

#include <list>

using namespace std;

#include "uitheme.h"

/**
\ingroup ui
\file thememanager.h
\brief Defines the class ThemeManager.
*/

/**
\ingroup ui
\class ThemeManager
\brief Handles themes (the look and feel)

Using themes makes it easier to change the look and feel of an application.
ThemeManager is the central for all used themes. Every variable of a class derved
from UiTheme will be registered here and can become active when required.

\ingroup singleusage
This class can only be used in one variable at a time. So do not initialize it
more than once, but it is required once, if you use themes.
*/
class DECLSPEC ThemeManager
{
public:
	/**
	lsOnAppStart: Theme will be loaded, when it is created (usually at app-start), so starting the app will take longer
	lsOnActivate: Theme will be loaded, directly before it becomes active, so activating a theme will take longer
	*/
	enum LoadScheme{lsOnAppStart, lsOnActivate};

	/**
	Constructor \n
	\param ls specifies, when to load the themes' data
	*/
	DECLSPEC ThemeManager( LoadScheme ls );
	virtual DECLSPEC ~ThemeManager();

	/**
	Activates a theme
	\param t The theme to activate
	\param hidden if true, there is no progress shown to the user; if false, a progressbar may be shown
	*/
	DECLSPEC bool setActiveTheme( UiTheme *t, bool hidden );
	/**
	Change the load scheme of themes
	\param ls specifies, when to load the themes' data, if ls is lsOnAppStart, then all themes will be loaded immediatelly
	 if it is lsOnActivate, there will be probably no changes, because all themes have already been loaded;
	 this makes sense only, if a theme is created afterwards
	*/
	DECLSPEC void setLoadScheme( LoadScheme ls );
	/**
	Use this function in GUI or UI classes, to get the look and feel for your element
	\returns the currently active theme
	*/
	UiTheme *getActiveTheme()	{ return thActive; }
	/**
	\param name the name of the theme as returned by UiTheme::getName()
	\returns the theme with the specified name
	*/
	DECLSPEC UiTheme *getThemeByName( const char *name );

	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static ThemeManager *the;
	/**
	\returns a pointer to the ThemeManager-variable, or NULL if not initialized.
	(There is only one ThemeManager-variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	DECLSPEC static ThemeManager *getThe();
private:
	typedef list<UiTheme*>::iterator ThemeIter;
	list<UiTheme*> lThemes;

	UiTheme *thActive;
	LoadScheme lsThemes;

	friend class UiTheme;
};

#endif
