/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UIKEYCOMBINATION_H
#define UIKEYCOMBINATION_H

#include <list>
#include "SDL/SDL.h"

#include "basicobject.h"
#include "tinyxml.h"

/**
\ingroup ui
\file uikeycombination.h
\brief Defines the class UiKeyCombination.
*/

/**
\ingroup ui
\class UiKeyCombination
\brief This class handles events on key cominations

This class handles key combinations, pressed by the user on the
keyboard and calls a function, if the keys are equal to a specified
combination. Each combination consists of a usual key and a modifier
like Ctrl, Alt, Shift... (Alt or Ctrl has to be in it)

\ingroup singleusage
This class can only be used in one variable at a time and is created
with UiManager. So do not create yet another.

\see UiManager
*/
class DECLSPEC UiKeyCombination
{
public:
	DECLSPEC UiKeyCombination();
	virtual DECLSPEC ~UiKeyCombination();

	/**
	Adds key combinations as specified in the XML element. The XML
	element's children have to look like this:
	\code
	<Combi Key="p" Mod="ctrl" OnPress="my.object#photo" OnRelease="my.object#lightning" />
	\endcode
	(You can leave away either OnPress or OnRelease, if you do not want
	to handle the event)
	\param el the XML element
	*/
	DECLSPEC void init( TiXmlElement *el );
	/**
	Adds a key combination to the list that is checked each time a
	combination is pressed and calls the specified function, if the
	keys equal k and m
	\param k the key, that is part of the combination
	\param m the modify key, that is the 2nd part of the combination
	\param aim pointer to the reciever object, this will be the param,
	that is given to the called function (see BasicObject::Callback)
	\param p pointer to the function to call, when the combination is
	pressed down
	\param r pointer to the function to call, when the combination is
	released
	\note if a combination with k and m already exists, it will be
	overwritten
	*/
	DECLSPEC void addCombination( SDLKey k, SDLMod m, BasicObject *aim_p, BasicObject::Callback p, BasicObject *aim_r = NULL, BasicObject::Callback r = NULL );
	/**
	Removes a combination from the list.
	\param k the key, that is part of the combination
	\param m the modify key, that is part of the combination
	\returns true, if the combination was found and removed; otherwise
	false
	*/
	DECLSPEC bool removeCombination( SDLKey k, SDLMod m );
	/**
	Removes all key combinations from the list.
	\see removeCombination()
	*/
	DECLSPEC void removeAll();
	/**
	Called from UiManager, when a combination of a key and a modifier
	is pressed down. So usually you do not need this, but you can
	use it to simulate a combination.
	\param k the key
	\param m the modifier
	\returns true, if the combination was in the list; otherwise false
	*/
	DECLSPEC bool keyPressed( SDLKey k, SDLMod m );
	/**
	Called from UiManager, when a combination of a key and a modifier
	is released. So usually you do not need this, but you can use it
	to simulate a combination.
	\param k the key
	\param m the modifier
	\returns true, if the combination was in the list; otherwise false
	*/
	DECLSPEC bool keyReleased( SDLKey k, SDLMod m );

	/**
	Not accessible in Windows (dynamic linking), equals the result of getThe()
	*/
	static UiKeyCombination *the;
	/**
	\returns a pointer to the UiKeyCombination-variable, or NULL if not initialized.
	(There is only one UiKeyCombination-variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	static DECLSPEC UiKeyCombination *getThe();
private:
	struct KeyComb
	{
		SDLMod mod;
		SDLKey key;
		BasicObject::FunctionData pressed, released;
	};
	std::list<KeyComb*> lKeyCombs;
	typedef std::list<KeyComb*>::iterator KeyCombIter;
};

#endif
