/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UIMANAGER_H
#define UIMANAGER_H

#include <list>
#include "SDL/SDL.h"
#include "uiobject.h"
#include "tinyxml.h"

using namespace std;

class UiObject;
class UiTheme;

/**
\defgroup ui User interface
\version 0.8
\todo improve

\author Tobias Nadler (nadlerto at gmx dot de) and Georg Schnattinger

The Ui* files, frametimer.* and thememanager.* contain classes for basic user-interaction.
*/

/**
\ingroup ui
\file uimanager.h
\brief Defines the class UiManager.
*/

/**
\ingroup ui
\class UiManager
\brief Manager of user interface objects

UiManager is neccessary, if you want to use Netrinjo-supported graphical user interaction.
It supports also keyboard and mouse input to non-graphical objects.
This class recieves events from SDL and calls the corresponding functions of the focused
objects and calls a function for display update in each cycle.

\ingroup singleusage
There is only one variable of this type allowed at a time. Do not initialize it
more than once.
*/
class UiManager : public BasicObject
{
public:
	/**
	 * Constructor.
	 * The XML data may consist of the children "GuiManager" (to initialize the
	 * GuiManager) and "UiKeyCombination" (to set events for pressed key
	 * combinations) and can have the attributes "LimitFPS" (to set a top limit
	 * for the drawn frames per second).
	 * \param el the XML element
	 * \param p the parent object
	 * \see GuiManager::init(), UiKeyCombination::init()
	 */
	DECLSPEC UiManager(TiXmlElement*el, BasicObject *p);
	virtual DECLSPEC ~UiManager();

	virtual const char *getClassName()	{return "UiManager";}
	/**
	This function does not end, until stop becomes true (stop is set to false
	directly after the function is called). In the loop handleEvents() and
	nextFrame() are called.
	\param stop reference to a variable that indicates, when to stop the loop.
	\note if you use this function, you should not call handleEvents() or
	nextFrame() parallel from another thread!
	*/
	DECLSPEC void loop(bool &stop);
	/**
	Handles events concerning the application and user input (mouse / keyboard).
	This function should be called regularly.
	*/
	DECLSPEC void handleEvents();
	/**
	Handles drawing of graphical objects... (usually called directly after/before
	handleEvents() in the main loop)
	*/
	DECLSPEC void drawFrame();
	/**
	Sets the focus of keyboard, mouse or display to the specified object.
	\param f the kind of focus to set
	\param obj the object that will be focussed (may be NULL)
	*/
	DECLSPEC void setFocus(UiObject::FocusType f, UiObject *obj);
	/* *
	Releases the focus of keyboard, mouse or display from the object,
	if the focus is on the specified object
	\param f the kind of focus to set
	\param obj the object that should be unfocused
	* /
	DECLSPEC bool releaseFocus(FocusType f, UiObject *obj);*/
	/**
	\param f the kind of focus to get
	\returns the object that currently has the focus
	*/
	DECLSPEC UiObject *getFocus(UiObject::FocusType f);
	
	/**
	\returns the duration in msecs of the last frame
	(equals the time between two drawFrame() calls)
	*/
	Uint32 getTimePerFrame() const	{ return tPerFrame; }
	/**
	\returns the number of frames that have been drawn yet
	*/
	Uint32 getFrames() const		{ return nFrames; }
	/**
	\returns the average duration of one frame since init() has been called
	*/
	DECLSPEC Uint32 getAvgTimePerFrame() const;

	/**
	Limits the framerate by waiting, if the minimum time did not elapse.
	(This has no effect on the vertical or horizontal frequency of the monitor!)
	\param n the maximum number of frames per second allowed
	\note The value is calculated to the time in milliseconds and microseconds,
	which has rounding errors as a consequence (esp. on systems that use the
	milliseconds value)
	*/
	DECLSPEC void setMaxFramerate( Uint32 n );

	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static UiManager *the;
	/**
	\returns a pointer to the UiManager-variable, or NULL if not initialized.
	(There is only one UiManager-variable allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	DECLSPEC static UiManager *getThe();
	
protected:
	/**
	Initializes the manager by setting it up, as the XML element says it.
	The XML data may consist of the children "GuiManager" (to initialize the GuiManager)
	and "UiKeyCombination" (to set events for pressed key combinations).
	\param el the XML element
	\see GuiManager::init(), UiKeyCombination::init()
	*/
	virtual DECLSPEC void onInit();
private:
#if __linux__
	// if true, the ualarm function is called to set a timer.
	// Using the kernel timer has the advantage that its precision is microseconds
	// the disadvantage is that the kernel is usualy not compiled tickless and thus
	// the precision is not at all in the range of microseconds, but in the range of
	// 1/<kernel-ticks> (usually something from 250 up to 1000)
	static bool bUseKernelTimer;
	static void timerSignal( int signal );
#endif
	void themeChanged( UiTheme *t );

	UiObject *uiKeyboardFocus;
	UiObject *uiMouseFocus;
	UiObject *uiDisplayFocus;
	bool bActive;

	Uint32 tInit, tFrame, tPerFrame;
	Uint32 nFrames;
	Uint32 tMinPerFrame_ms/*in millisecs*/, tMinPerFrame_us/* in microseconds*/;

//	friend class UiObject;
	friend class ThemeManager;
};

#endif
