/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UIOBJECT_H
#define UIOBJECT_H

#include "SDL/SDL.h"
#include "definitions.h"
#include "basicobject.h"

class UiTheme;
class SdlSurface;

/**
\ingroup ui
\file uiobject.h
\brief Defines the class UiObject.
*/

/**
\ingroup ui
\class UiObject
\brief Base class for all objects, that have a managed user interface

All classes, derived from UiObject, are managed by UiManager to handle
events that are needed for user-program interaction.

\see UiManager
*/
class DECLSPEC UiObject : public BasicObject
{
public:
	/// kinds of focus an UiObject can have
	enum FocusType {ftKeyboard = 1, ftMouse = 2, ftDisplay = 4};
	/// combinations of focustypes
	typedef Uint8 FocusTypes;
	/**
	Constructor.
	\param parent the parent object
	\param name the object's name
	*/
	DECLSPEC UiObject( BasicObject *parent, const char *name );
	/**
	Constructor.
	\param el the XML-element to use for initialization
	\param parent the parent object
	 */
	DECLSPEC UiObject( TiXmlElement *el, BasicObject *parent );
	virtual DECLSPEC ~UiObject();

	virtual const char *getClassName()	{ return "UiObject"; }
	/**
	Brings the element to top, so that sibling elements, that are (partially) drawn
	at the same place, appear behind this element
	*/
	DECLSPEC void toTop();
	/**
	Sends the element to bottom, so that sibling elements, that are (partially) drawn
	at the same place, appear in front of this element
	*/
	DECLSPEC void toBottom();
	/**
	Brings the element one step to top, so that it appeares in front of the element,
	that was one step above it before.
	*/
	DECLSPEC void oneStepToTop();
	/**
	Sends the element one step to bottom, so that it appeares behind the element,
	that was one step below it before.
	*/
	DECLSPEC void oneStepToBottom();
	
	/**
	\returns true, if the object is a class derived from GuiElement;
	false, if not
	*/
	virtual bool isGuiElement() const	{ return false; }
	virtual bool isUiObject() const	{ return true; }
	
	/**
	 * Calls onThemeChanged(t) of this and all its child objects.
	 * \param t the theme to apply
	 * \note calling this makes usually only sense after the initialisation of
	 * this object
	 * \todo !!!Make applying themes on creation automatically!!!
	 */
	void applyTheme( UiTheme *t )	{ onThemeChanged(t); }
	/**
	Translates a text-string into another language, if possible.
	It uses UiTheme::translate( text ).
	\param text the string to translate
	*/
	static DECLSPEC const char *tr( const char *text );

protected:
	/**
	This event is called, when the object has the keyboard focus and
	a key is pressed down.
	\param ev the key-down event, that occured
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onKeyDown( SDL_KeyboardEvent *ev )		{ return 0; }
	/**
	This event is called, when the object has the keyboard focus and
	a key is released.
	\param ev the key-up event, that occured
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onKeyUp( SDL_KeyboardEvent *ev )			{ return 0; }
	/**
	This event is called, when the object has the mouse focus and
	a mouse button is pressed down.
	\param ev the button-down event, that occured
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onMouseDown( SDL_MouseButtonEvent *ev )	{ return 0; }
	/**
	This event is called, when the object has the mouse focus and
	a mouse button is released.
	\param ev the button-up event, that occured
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onMouseUp( SDL_MouseButtonEvent *ev )	{ return 0; }
	/**
	This event is called, when the mouse cursor is moved over the object.
	\param x the absolute x coordinate of the cursor (distance from left)
	\param y the absolute y coordinate of the cursor (distance from top)
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onMouseMotion(SDL_MouseMotionEvent *ev)	{ return 0; }
	/**
	This event is called, when the object gains keyboard or mouse focus.
	\param gained the gained focustypes
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onFocus(FocusTypes gained)	{ return 0; }
	/**
	This event is called, when the object loses keyboard or mouse focus.
	\param lost the lost focustypes
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual Uint32 onRelease(FocusTypes lost)	{ return 0; }
	/**
	This event is called when graphics should be updated. It does not call
	the children's onFrame() function!
	\return currently the result is not used; may be zero, if no error...
	\note in classes derived from GuiElement, it is probably more useful to
	      implement onFrame(SdlSurface*)
	*/
	virtual Uint32 onFrame()	{return 0;}
	/**
	This event is called, when the theme changes. Perform here the changes
	of the object's look and feel.
	\param t pointer to the new used theme
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
	virtual DECLSPEC void onDestroy();
	
private:
	friend class UiManager;
};

#endif
