/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UITHEME_H
#define UITHEME_H

#include "SDL/SDL.h"

class ThemeManager;

/**
\ingroup ui
\file uitheme.h
\brief Defines the class UiTheme.
*/

/**
\ingroup ui
\class UiTheme
\brief Base class for all themes, used for easy change of look and feel

All classes, derived from UiTheme, are managed by ThemeManager to handle
themes.

\see ThemeManager
*/
class DECLSPEC UiTheme
{
public:
	/**
	Constructor. Registers itself in the ThemeManager with it's name.
	\param name The theme's name, that can be used to get the theme
	by using ThemeManager::getThemeByName
	*/
	DECLSPEC UiTheme( const char *name );
	virtual DECLSPEC ~UiTheme();

	/**
	Get a theme-specific string.
	\param usage a string that says what for the result is needed.
	\param length if not NULL, the length of the string will be copied there
	or zero, if the usage-string is invalid
	\returns a string with the data or NULL, if the usage-string could not be found
	*/
	virtual const char *getString( const char *usage, int *length = 0 ) = 0;
	/**
	Get a theme-specific number.
	\param usage a string that says what for the result is needed.
	\param found if not NULL, it's content will become true, if the "usage" could be
	found and false, if it could not be found
	\returns the corresponding number to "usage" and zero, if "usage" was not found
	*/
	virtual Uint32 getUint32( const char *usage, bool *found = 0 ) = 0;
	/**
	Get a theme-specific boolean value.
	\param usage a string that says what for the result is needed.
	\param found if not NULL, it's content will become true, if the "usage" could be
	found and false, if it could not be found
	\returns the corresponding value to "usage" and always false, if "usage" was not found
	*/
	virtual bool getBool( const char *usage, bool *found = 0 ) = 0;
	/**
	Get a theme-specific rect.
	\param usage a string that says what for the result is needed.
	\returns the corresponding rect to "usage", or an empty rect, if "usage" was not found
	*/
	virtual SDL_Rect getRect( const char *usage ) = 0;
	/**
	Translates a text-string to the theme's language.
	\param text the string to translate (NULL not allowed!)
	\returns the translated string or text, if it could not be translated.
	Never returns NULL!
	\see UiTranslator
	*/
	virtual const char *translate( const char *text ) = 0;

	/**
	\returns the theme's name, with which is was created
	\see UiTheme( const char * )
	*/
	const char *getName() const	{ return chName; }
protected:
	/**
	This event will be called, when the theme should load it's data.
	\param hidden if true, the screen is not used to display a progressbar/...;
	              if false the progress may be shown by the loading theme
	\return zero, if everything could be loaded successfully, and anything else, if it failed
	*/
	virtual Uint32 onLoad(bool hidden) = 0;
//	virtual Uint32 onSave() = 0;
	/**
	This event will be called, before the theme becomes active.
	\return zero, if it can be used now, and anything else, if not
	*/
	virtual Uint32 onActivate() = 0;
	/**
	This event will be called, when the theme becomes inactive.
	\return zero (recommended), but the result is currently not used
	*/
	virtual Uint32 onDeactivate() = 0;
	/**
	\return true, if the theme's data is loaded, false if not
	*/
	virtual bool isLoaded() = 0;
private:
	char *chName;

	friend class ThemeManager;
};

#endif
