/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UITRANSLATOR_H
#define UITRANSLATOR_H

#include <list>
#include "SDL/SDL.h"

using namespace std;

/**
\ingroup ui
\file uitranslator.h
\brief Defines the class UiTranslator.
*/

/**
\ingroup ui
\class UiTranslator
\brief Base class for doing translation

This class contains (partially virtual) functions for managing translation
of some text or expressions. It also is used (in UiFileTheme) to translate
the keyboard's keys.\n
It is currently only used by UiFileTheme, but is very useful when designing
a new theme class.

\see UiFileTranslator, UiFileTheme
*/
class DECLSPEC UiTranslator
{
public:
	/**
	Constructor.
	\param language the language, to which the translator can convert text;
	may be NULL, if the derived class sets the language.
	*/
	DECLSPEC UiTranslator( const char *language = 0 );
	virtual DECLSPEC ~UiTranslator();

	/**
	\returns the language to which the object can translate text
	*/
	const char *getLanguage() const	{ return chLanguage; }
	/**
	Translates an expression or text.
	\param expression the text or expression to translate
	\returns the translated expression/text
	*/
	virtual const char *translate( const char *expression ) = 0;

protected:
	/**
	Changes the language to which the translator converts text/expressions.
	(It changes only the language's name, it does not change the translated
	expressions)
	*/
	DECLSPEC void setLanguage( const char * lang );

private:
	char * chLanguage;
};

/**
\ingroup ui
\class UiFileTranslator
\brief Translation class, which reads the expressions from a text based file

Required header file:
\code #include <uitranslator.h> \endcode

This class makes translating expressions easy by loading a text based file, that
can easily be edited with a text editor.
It is currently used by UiFileTheme.

\see UiFileTheme
*/
class DECLSPEC UiFileTranslator : public UiTranslator
{
public:
	/**
	Constructor.
	\param fn the filename of the file containing the expressions
	*/
	DECLSPEC UiFileTranslator( const char *fn = 0 );
	virtual DECLSPEC ~UiFileTranslator();

	/**
	Loads a file containing the expressions.
	\param fn the filename to load the data from
	\returns true, if successful; if failed, false
	*/
	DECLSPEC bool load( const char *fn );
	virtual DECLSPEC const char *translate( const char *expression );

private:
	struct EXPRESSION
	{
		EXPRESSION()	{
			chCoded = 0;
			chTranslated = 0;
		}
		char *chCoded;
		char *chTranslated;
	};
	list<EXPRESSION> lExpressions[27];// [0..25]: alphabetical order a..z; [26]: numbers&co
};



#endif
