/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBSERVER_H
#define WEBSERVER_H

#include "SDL/WebSrv.h"
#include <list>
#include "basiclog.h"

using namespace std;

class WebPage;
class WebThread;
class TiXmlElement;
class TiXmlNode;

/**
\ingroup web
\file webserver.h
\brief Defines the class WebServer.
*/

/**
\defgroup web Webserver
\version 0.5
\todo testing, looking for errors

\author Tobias Nadler (nadlerto at gmx dot de)
\brief Some classes for setting up a simple HTTP 1.0 server

These classes can be used to build a HTTP 1.0 webserver, which can
be used i.e. to show game statistics, configure the gameserver, ...
*/

/**
\class WebServer
\ingroup web
\brief The central class of the webserver

Required header file:
\code #include <webserver.h> \endcode

This class is not dependant on any Net* classes, but if you do not init a
NetBasicManager-derived class before initing this, you have to call SDLNet_Init().

If you use NetThread for polling, be sure not to have WEB_CLIENT defined.

To use text-based configuration files, this class supports XML based files for
basic configuration.

\ingroup singleusage
This class must not be used in two variables at a time.
\see singleusage
*/
class DECLSPEC WebServer : public BasicLog
{
public:
	DECLSPEC WebServer(const char *name, BasicObject *p);
	/**
	\param el the TiXmlElement to parse for getting the initialisation data
	It can contain "Port" as an attribute, which contains the port to listen
	for requests (usually 80); if there is no attribute, the default port (80)
	is used. Furthermore the element can contain subnodes, which will be parsed by
	calling parseXMLNode(node).
	Here an example of a XML element:
	\code
	  <Web Port="8000">
	    <TimeoutIv value="30000" />
	    <Directory Path="html/SDL" URI="/help/" Indexpage="index.html" SubDirs="true" />
	    <Page URI="/favicon.ico" Filename="html/favicon.ico" ContentType="image/x-icon" />
	    <Forward URI="/" Aim="index.html" />
	  </Web>
	\endcode
	*/
	DECLSPEC WebServer(TiXmlElement *el, BasicObject *p);
	DECLSPEC ~WebServer();

	virtual const char *getClassName()	{ return "WebServer"; }
	/**
	(probably obsolete soon!)
	Initializes the web server. Use this only, if no XML data is used for
	Initialisation.\n
	Before calling init, make sure to have inited SDL-net with SDLNet_Init()
	(which is also done with initing a NetBasicManager-derived class).
	\param port The port to listen for requests (usually 80)
	\returns true, if successful, false if not
	*/
	virtual DECLSPEC bool init( Uint16 port = 80 );
	/**
	Uninitializes the web server. Call this before closing the application.
	*/
	virtual DECLSPEC void destroy();

	/**
	Waits for connecting clients. Call this regularly to be able to
	keep the response time short.
	\param tWait time to wait in msecs, if no request is pending
	\returns true, if connection-request was recieved, false if not
	*/
	virtual DECLSPEC bool poll( Uint16 tWait = 0 );
	/**
	Closes connections of inactive / stopped threads and deletes them.
	Kills threads that did not do anything for WebThread::tIdleToExit msecs
	*/
	DECLSPEC void closeOutTimedThreads();

	/**
	Parses the TiXmlNode and configures the server accordingly.
	\param node the TiXmlNode, that can contain the following subnodes:
	- TimeoutIv: the time in msecs, after that an inactive thread can be killed
	- Directory: (more than one node possible) has to contain the attributes:
	  - Path: the path (relative to the working dir or absolute) to the html
	          files to make accessible via HTTP
	  - URI: the basic URI, where the directory can be found from a browser
	         the complete URI looks like http://IP-Addr"URI"/"filename"
		 URI has to start with a '/'!
	  - Indexpage: the file to send, if there is no file specified after the '/'
	         like http://IP-addr/mydir/ then this equals http://IP-addr/mydir/"Indexpage"
	  - SubDirs: (boolean value) if true, subdirectories of Path will be published;
	         if false, only the files in Path will be published
	- Page: (more than one node possible) has to contain the attributes:
	  - URI: the URI, where a browser can find the file (like http://IP-addr"URI").
	         URI has to start with a '/'!
	  - Filename: the file's filename (relative to the cwd or absolute),
		      where the file is located
	  - ContentType: (optional) the MIME-type of the file like "text/html"
			 if it is not specified, the app looks at the ending of
			 the filename and uses the default type for this ending
	- Forward: (more than one node possible) has to contain the attributes:
	  - URI: the URI, that should be forwarded, it has to start with a '/'!
	  - Aim: the URI of the file to respond, it has to start with a '/'!
	\see addPageDirectory, WebFilePage, WebForwardPage; for an example: init(TiXmlNode)
	*/
	DECLSPEC void parseXMLNode( TiXmlNode *node );
	/**
	Makes the files in the directory availible through the web
	\param dir the directory, of which the files should be published
	(it can be relative to the app's data directory or absolute)
	\param basic_uri the basic URI, where the directory can be accessed via browser
		like http://IP-addr"basic_uri"/ \n basic_uri has to start with a '/'!
	\param fn_index if fn_index != 0 then the uri [..]/ is forwarded to [..]/"fn_index"
	\param subdirs if true, subdirectories' files will also be published, if false not
	\returns the number of files that were made availible
	*/
	DECLSPEC int addPageDirectory( const char *dir, const char *basic_uri = "/", const char *fn_index = 0, bool subdirs = true );
/*	DECLSPEC WebPage *getFirstPage();
	DECLSPEC WebPage *getNextPage();*/
	/**
	\returns a WebPage list iterator, pointing to the first page
	*/
	list<WebPage*>::iterator beginPage()	{ return lPages.begin(); }
	/**
	\returns a WebPage list iterator, pointing after the last page
	*/
	list<WebPage*>::iterator endPage()	{ return lPages.end(); }

	/**
	only accessible in Linux, and equals the result of getThe()
	*/
	static WebServer *the;
	/**
	\returns a pointer to the WebServer-variable, or NULL if not initialized.
	(There is only one variable with the type WebServer allowed in an application)

	Every netrinjo-class that can only exist once at a time, has this static function.
	\see singleusage
	*/
	DECLSPEC static WebServer *getThe();
protected:
	virtual DECLSPEC void onInit();
	virtual DECLSPEC void onDestroy();
	/// the socket, that listens for HTTP requests
	TCPsocket socket;
	
	/// the list with all published web pages/files
	list<WebPage*> lPages;
	/// the list with all threads, that respond on requests
	list<WebThread*> lThreads;
	
//	list<WebPage*>::iterator itPage;
	
private:
	void logIntro();
	WebSrv *webServer;
	
	friend class WebPage;
	friend class WebThread;
};

#endif
