/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef XMLAPP_H
#define XMLAPP_H

#include "tinyxml.h"
#include "basicapp.h"
#include "basicobject.h"

/**
\defgroup xml XML specific classes
\brief Classes that heavily use XML.

Besides the TinyXml files, the class XmlApp belongs to this group.
*/
/**
@page xmlapp-main An example of a main.cpp-file with the class XmlApp
If you want to use XmlApp for the initialisation and basic management
of your application, it is probably the best way to just copy this
code into your main.cpp. Maybe little changes make sense, but are not
required.

Here the code:
@code
#define APP_XMLFILE		"data/main.xml"

int the_main( int argc, char *argv[] );
int main( int argc, char *argv[] )
{
	return the_main( argc, argv );
}

#include "SDL/SDL.h"// "SDL.h" cannot be included before main, if compiling with mingw
#include "xmlapp.h"
#include "sdlmanager.h"
#include <iostream>

int the_main( int argc, char *argv[] )
{
	// attach SDL_Quit to the exit routine, to make sure, that (in X11 fullscreen-mode)
	// the desktop will appear when the program quits (even if the app crashes).
	// You can also call SDL_Quit when closing the app, but the X11-fullscreen-mode will
	// not be quitted, if the app crashes!
	atexit( SDL_Quit );
	// Construct the GameApp class (derived from BasicApp), using the program arguments
	XmlApp app( argc, argv );
	string s = string(BasicApp::getThe()->getExePath()) + APP_XMLFILE;
	// if initialisation failed, exit the program
	switch( app.init( s.c_str() ) )
	{
		case 0:
			break;
		case E_APP_NOXML:
			fprintf( stderr, "ERROR: XML-file could not be opened.\n" );
			return 1;
		default:
			fprintf( stderr, "ERROR: Initialization failed (common error).\n" );
			return 1;
	}
	// start the application's main loop
	return app.mainLoop();
}
@endcode

For details on how the XML-file has to look like, etc. have a look at the example at
http://netrinjo.sourceforge.net/
*/
/**
\ingroup xml
\file xmlapp.h
\brief Definition of the class XmlApp.
*/

/// Error ID, indicating that either a specified xml-file cannot be opened or none was not specified
#define E_APP_NOXML	1

/**
\class XmlApp
\brief A class to make game development very easy and fast.

The class XmlApp makes it possible to change the look of your app in a
xml-file. You do not need to recompile the application, when changing
XML-data.
\see \ref xmlapp-main
*/
class XmlApp: public BasicApp
{
public:
	/**
	 * Constructor.
	 * \param xmlfile the path to the XML file to be used for initialisation
	 * The followin events can be specified in the xml root element:
	 * DoBeforeInit, DoAfterCreation, DoAfterInit; further attributes can be:
	 * UserXML (path to user's xml-file), UserdataPath (path to user's data
	 * directory), CanGrabInput (bool), Caption (string), WaitOnStart (ulong).
	 */
	DECLSPEC XmlApp( int argc, char *argv[], const char *xmlfile = NULL);
	virtual DECLSPEC ~XmlApp();

	/**
	Initializes the application. After loading the specified XML file,
	BasicApp::init() is called with the loaded data's root element as
	param.
	\param fn filename of the XML file to use for initialisation, if NULL
	          the filename specified on creation is used
	\returns zero on success, otherwise an error id
	*/
	DECLSPEC int init(const char *fn = NULL);
	/// Call destroy to quit the application.
	virtual DECLSPEC void destroy();
	/// Never returns, but exits the application.
	virtual DECLSPEC long mainLoop();
	/**
	\returns a XML handle for application specific data. The data is loaded
	from the filename as specified in the constructor XmlApp() when init()
	is called.
	*/
	TiXmlHandle getXmlHandle()		{return TiXmlHandle(xmlData);}
	/**
	\returns a XML handle for user specific data. You can use this data for
	user settings. The data is automatically loaded when init() is called
	and saved when destroy() is called.
	*/
	TiXmlHandle getXmlHandleUser()	{return TiXmlHandle(xmlUser);}

	static void quitApplication(void*,BasicObject*,void*)
	{
		((XmlApp*)the)->destroy();
	}
protected:
	virtual DECLSPEC void onInit();
private:
	static void onExit();

	TiXmlDocument xmlDoc, xmlUserdoc;
	TiXmlElement *xmlUser;
	// must be true as long as the mainloop is not entered!
	bool bStopped;
};

#endif
