/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "basicapp.h"
#include <iostream>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include "functions.h"

using namespace std;

BasicApp * BasicApp::the = NULL;
BasicApp *BasicApp::getThe()
{
	return the;
}

// static methods first:

bool BasicApp::chDirToUser()
{
	return chdir( getUserDirectory() ) == 0;
}

bool BasicApp::setWorkingDir( const char *dn )
{
	if( chdir( dn ) == 0 )
		return true;
#ifdef __windows__
	if( mkdir( dn ) == -1 )
#else
	if( mkdir( dn, 0744 ) == -1 )// rw-r--r--
#endif
		return false;
	return chdir( dn ) == 0;
}

const char *BasicApp::getOSName()
{
#if __linux__
	return "Linux";
#elif __windows__
	return "Windows";
#else
#warning Please extend this method by inserting the name of the OS, you are porting to
	return "Unknown";
#endif
}

const char *BasicApp::getCPUArch()
{
#if __x86_64
	return "x86_64";
#elif __i386
	return "i386";/// \todo check whether __i386 defined
#else
#warning Please extend this method by inserting the name of the CPU-architecture, you compile for
	return "Unknown";
#endif
}

utsname BasicApp::osData;

const char *BasicApp::getRunningOSName()
{
#ifndef __windows__
	uname(&osData);
	return osData.sysname;
#else
	return "TODO(Windows)";
#endif
}

const char *BasicApp::getRunningOSVersion()
{
#ifndef __windows__
	uname(&osData);
	return osData.release;
#else
	return "TODO";
#endif
}

const char *BasicApp::getRunningOSArch()
{
#ifndef __windows__
	uname(&osData);
	return osData.machine;
#else
	return "TODO";
#endif
}

// then non-static methods:

BasicApp::BasicApp( int _argc, char *_argv[], const char *datapath, const char *debug ): BasicLog( NULL, NULL, debug )
{
	nInitFailStatus = 0;
	argc = _argc;
	argv = _argv;

	chExePath = NULL;
	if( argc > 0 )	{
		if(*argv[0] != '.')	{
			int l = strlen( argv[0] );
			chExePath = new char[ l+1 ];
			strcpy( chExePath, argv[0] );
			for( int i = 0; i < l; ++i )
				if( chExePath[i] == '\\' )
					chExePath[i] = '/';
			for( ; chExePath[l] != '/'; --l)
				chExePath[l] = '\0';
		}
		else	{
#ifdef __windows__
#warning This section is not ported to windows yet
#else
			const char *p1 = getenv("PWD"),
				*p2 = argv[0];
			if(p1 && p2)	{// /path1 + ./path2
				chExePath = new char[strlen(p1) + strlen(p2)];
				strcpy(chExePath, p1);
				strcat(chExePath, p2+1);
				for( int l = strlen(chExePath)-1; chExePath[l] != '/'; --l)
					chExePath[l] = '\0';
			}
#endif
		}
	}
	if( chExePath )	{
		int l = strlen( chExePath ) - 1;
		chExePath[ l ] = '\0';
		setenv("EXEPATH", chExePath, 1);
		chExePath[ l ] = '/';
	}
	
	chDataPath = NULL;
	setDataPath(datapath);
	const char *p = getUserDirectory();
	chUserdataPath = new char[strlen(p)+2];
	strcpy(chUserdataPath, p);
	strcat(chUserdataPath, "/");
	the = this;
}

BasicApp::BasicApp( int _argc, char *_argv[], TiXmlElement *el ): BasicLog( el, NULL )
{
	nInitFailStatus = 0;
	argc = _argc;
	argv = _argv;

	chExePath = NULL;
	if( argc > 0 )	{
		if(*argv[0] != '.')	{
			int l = strlen( argv[0] );
			chExePath = new char[ l+1 ];
			strcpy( chExePath, argv[0] );
			for( int i = 0; i < l; ++i )
				if( chExePath[i] == '\\' )
					chExePath[i] = '/';
			for( ; chExePath[l] != '/'; --l)
				chExePath[l] = '\0';
		}
		else	{
#ifdef __windows__
#warning This section is not ported to windows yet
#else
			string s = makeEnvStr("$(PWD)/$(_)");
			chExePath = new char[strlen(s.c_str())+1];
			strcpy(chExePath, s.c_str());
			for( int l = strlen(chExePath)-1; chExePath[l] != '/'; --l)
				chExePath[l] = '\0';
#endif
		}
	}
	if( chExePath )	{
		int l = strlen( chExePath ) - 1;
		chExePath[ l ] = '\0';
		setenv("EXEPATH", chExePath, 1);
		chExePath[ l ] = '/';
	}
	
	chDataPath = NULL;
	setDataPath(el->Attribute("DataPath"));
	const char *p = getUserDirectory();
	chUserdataPath = new char[strlen(p)+2];
	strcpy(chUserdataPath, p);
	strcat(chUserdataPath, "/");
	the = this;
}

BasicApp::~BasicApp()
{
	the = 0;

	if( chExePath )
		delete[] chExePath;
	if(chDataPath)
		delete[] chDataPath;
	if(chUserdataPath)
		delete[] chUserdataPath;
}

void BasicApp::onInit()
{
	if(xmlData)	{
		const char *str = xmlData->Attribute("DataPath");
		if( str )
			setDataPath(makeEnvPath(str).c_str());
	}
	BasicLog::onInit();
}

int BasicApp::init(TiXmlElement *el)
{
	if(el)
		xmlData = el;
	onInit();
	return nInitFailStatus;
}

void BasicApp::destroy()
{
	logLine( "--------------------------" );
	char buf[33];
	logLine( "After " );
	logString( intToStr( SDL_GetTicks(), buf ) );
	logString( "ms running:" );
	logLine( "THE END.\r\n" );
	exit( 0 );
}


void BasicApp::exitWithMessage( const char *str )
{
	logLine( "Program exiting with message: " );
	logString( str );
	std::cout << "\r\nExiting with error:\r\n" << str;

	BasicApp::destroy();
}


const char * BasicApp::getExePath()
{
	if( chExePath == NULL )
		return "./";
	return chExePath;
}

const char * BasicApp::getDataPath()
{
	if(chDataPath)
		return chDataPath;
	return getExePath();
}

void BasicApp::setDataPath(const char *path)
{
	logLevelLine( "Changing data path to: " );
	if( chDataPath )
		delete[] chDataPath;
	int len;
	if(path)	{
		len = strlen(path);
		if(path[len-1] != '/')	{
			chDataPath = new char[len+2];
			strcpy(chDataPath, path);
			chDataPath[len] = '/';
			chDataPath[len+1] = '\0';
		}
		else	{
			chDataPath = new char[len+1];
			strcpy(chDataPath, path);
		}
		logString( chDataPath );
	}
	else	{
		chDataPath = NULL;
		logString( "executing path" );
	}
	// put the new data path without the last '/' into the environment variables
	len = strlen(getDataPath());
	char *dp = new char[len];
	dp[len-1] = '\0';
	setenv("DATAPATH", dp, 1);
	delete[] dp;
}

void BasicApp::setUserdataPath(const char *path)
{
	logLevelLine( "Changing user specific data path to: " );
	if( chUserdataPath )
		delete[] chUserdataPath;
	
	int len;
	if(!path)
		path = getUserDirectory();
	if(path[0] != '~' || path[1] != '/')	{// absolute path
		len = strlen(path);
		if(path[len-1] != '/')	{
			chUserdataPath = new char[len+2];
			strcpy(chUserdataPath, path);
			chUserdataPath[len] = '/';
			chUserdataPath[len+1] = '\0';
		}
		else	{
			chUserdataPath = new char[len+1];
			strcpy(chUserdataPath, path);
		}
		logString( chUserdataPath );
	}
	else	{// relative to user's home directory
		len = strlen(path);
		int ulen = strlen(getUserDirectory());
		if(path[len-1] != '/')	{
			len += ulen-1;
			chUserdataPath = new char[len + 2];
			strcpy(chUserdataPath, getUserDirectory());
			strcat(chUserdataPath, path+1);
			chUserdataPath[len] = '/';
			chUserdataPath[len+1] = '\0';
		}
		else	{
			chUserdataPath = new char[len+ulen];
			strcpy(chUserdataPath, getUserDirectory());
			strcat(chUserdataPath, path+1);
		}
		logString( chUserdataPath );
	}
	// put the new user data path without the last '/' into the environment variables
	len = strlen(chUserdataPath);
	chUserdataPath[len-1] = '\0';
	setenv("USERPATH", chUserdataPath, 1);
	chUserdataPath[len-1] = '/';

// try to create the folder (if it already exists, it simply fails)
#ifdef __windows__
	mkdir( chUserdataPath );
#else
	mkdir( chUserdataPath, 0744 );// rw-r--r--
#endif
}

std::string BasicApp::makeDataPath(const char *path)
{
	std::string strPath = makeEnvPath(path);
	if(strPath[0] != '/')	{
		return (std::string)getDataPath() + strPath;
	}
	return strPath;
}

std::string BasicApp::makeUserPath(const char *path)
{
	std::string strPath = makeEnvPath(path);
	if(strPath[0] != '/')	{
		return ((std::string)getUserdataPath() + strPath);
	}
	return strPath;
}
