/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "basiclog.h"
#include "SDL/SDL.h"
#include "functions.h"
#include <iostream>

using namespace std;

void BasicLog::logStdOut(const char *s)
{
	cout << s;
	cout.flush();
}

BasicLog::BasicLog( const char *name, BasicObject *parent, const char *filename ): BasicObject(name, parent)
{
	bAutoLogTime = true;
	pLog = NULL;
	bDebug = false;
	if( filename == NULL )
		return;

	initLogFile( filename );
}

BasicLog::BasicLog( const char *name, BasicObject *parent, ostream *strm ): BasicObject(name, parent)
{
	bAutoLogTime = true;
	pLog = NULL;
	bDebug = false;

	initLogStream( strm );
}

BasicLog::BasicLog( TiXmlElement *el, BasicObject *p ): BasicObject(el, p)
{
	bAutoLogTime = false;
	pLog = NULL;
	bDebug = false;
}

BasicLog::~BasicLog()
{
	if(osLogFile.is_open())
		osLogFile.close();
}

void BasicLog::onInit()
{
	const char *str = xmlData->Attribute("Logfile");
	if(str)	{
		if(!initLogFile( makeEnvPath(str).c_str() ))
		{
			if( xmlData->Attribute("Logstream") != NULL )
				goto LOG_STREAM;
			else
			{
				initLogStream( &cerr );
				logLine( "ERROR: The specified log-file could not be opened: " );
				logString( makeEnvPath(str).c_str() );
			}
		}
	}
	else
	{
LOG_STREAM:
		if((str = xmlData->Attribute("Logstream")) != NULL)	{
			if( *str == 'o' || *str == 'O' )
				initLogStream( &cout );
			else if( *str == 'e' || *str == 'E' )
				initLogStream( &cerr );
		}
		else
		{
			fprintf(stderr, "Hint: No output for logging specified in the XML-data!\n");
		}
	}
	str = xmlData->Attribute("LogTimes");
	bAutoLogTime = str == NULL || strToBool(str);
}

bool BasicLog::initLogFile( const char *fn )
{
	if(osLogFile.is_open())
		osLogFile.close();

	osLogFile.open( fn, ios::out | ios::binary );

	bDebug = osLogFile.is_open();
	if(bDebug)
		pLog = &osLogFile;
	logString( "Log-file opened." );
	nLogSpaces = 0;

	return bDebug;
}

void BasicLog::initLogStream( ostream *strm )
{
	pLog = strm;
	bDebug = true;
	logString( "Using log-stream." );
	nLogSpaces = 0;
}

void BasicLog::logLine( const char *ln )
{
	if( !bDebug || !pLog )
		return;

	pLog->write( "\r\n", 2 );
	if( bAutoLogTime )	{
		char buf[35];
		char *pT = (char*)intToStr(SDL_GetTicks(), buf);
		strcat( pT, ": " );
		pLog->write( pT, strlen( pT ) );
	}
	pLog->write( ln, strlen( ln ) );
	pLog->flush();
}

void BasicLog::logString( const char *str )
{
	if( bDebug )
		pLog->write( str, strlen( str ) );
}

void BasicLog::logTime()
{
	char buf[33];
	logString( intToStr(SDL_GetTicks(), buf) );
}

void BasicLog::logBegin( const char *ln )
{
	if( !bDebug )
		return;

	pLog->write( "\r\n", 2 );
	if( bAutoLogTime )	{
		char buf[35];
		char *pT = (char*)intToStr(SDL_GetTicks(), buf);
		strcat( pT, ": " );
		pLog->write( pT, strlen( pT ) );
	}
	for( int i = 0; i < nLogSpaces; ++i )
		pLog->write( " ", 1 );

	pLog->write( "*", 1 );
	pLog->write( ln, strlen( ln ) );
	pLog->flush();

	++nLogSpaces;
}

void BasicLog::logLevel( const char *ln )
{
	if( !bDebug )
		return;

	for( int i = 0; i < nLogSpaces; ++i )
		pLog->write( " ", 1 );

	pLog->write( ">", 1 );
	pLog->write( ln, strlen( ln ) );
	pLog->flush();
}

void BasicLog::logLevelLine( const char *ln )
{
	if( !bDebug )
		return;

	pLog->write( "\r\n", 2 );
	if( bAutoLogTime )	{
		char buf[35];
		char *pT = (char*)intToStr(SDL_GetTicks(), buf);
		strcat( pT, ": " );
		pLog->write( pT, strlen( pT ) );
	}
	for( int i = 0; i < nLogSpaces; ++i )
		pLog->write( " ", 1 );

	pLog->write( ">", 1 );
	pLog->write( ln, strlen( ln ) );
	pLog->flush();
}

void BasicLog::logEnd( const char *ln )
{
	if( !bDebug ) return;

	pLog->write( "\r\n", 2 );
	if( bAutoLogTime )	{
		char buf[35];
		char *pT = (char*)intToStr(SDL_GetTicks(), buf);
		strcat( pT, ": " );
		pLog->write( pT, strlen( pT ) );
	}
	if( nLogSpaces > 0 )
		--nLogSpaces;
	for( int i = 0; i < nLogSpaces; ++i )
		pLog->write( " ", 1 );

	pLog->write( "~", 1 );
	pLog->write( ln, strlen( ln ) );
	pLog->flush();
}

bool BasicLog::setDebug( bool dodebug )
{
	if( dodebug == bDebug )
		return bDebug;

	bDebug = dodebug;
	if( bDebug && !pLog )
		if( !osLogFile.is_open() )
			initLogFile( "debug.log" );
		else
			pLog = &osLogFile;

	return bDebug;
}

