// last review: 29.03.2007

#include "basicobject.h"
#include "basicapp.h"
#include "functions.h"
#include "tinyxml.h"

BasicSlot *BasicSlot::beFirst = NULL;

BasicSlot::BasicSlot(const char *fname, const char *classname, Callback func)
{
	if( !fname || fname[0] == '\0' )	{
		BasicApp::getThe()->logLine("WARNING: BasicSlot(): no function name specified!");
		return;
	}
	strName = new char[strlen(fname)+1];
	strcpy(strName, fname);
	
	if(classname && strcmp(classname, "NULL") != 0)	{
		strClassName = new char[strlen(classname)+1];
		strcpy(strClassName, classname);
	}
	else strClassName = NULL;
	
	pFunc = func;
	
	// prepend this event to the global event list
	beNext = beFirst;
	beFirst = this;
}

BasicSlot::~BasicSlot()
{
	// remove from global event list
	if( beFirst == this )
		beFirst = beNext;
	else	{
		BasicSlot *e;
		for(e = beFirst; e && e->beNext != this; e = e->beNext);
		if(e)
			e->beNext = beNext;
	}
	
	delete[] strName;
	if(strClassName)
		delete[] strClassName;
}

BasicSlot::Callback BasicSlot::getFunction(const char *name, const char *classname)
{
	if( name == NULL || name[0] == '\0' )
		return NULL;// silently ignore searches for an empty functionname
	
	// look, whether classname specified in name
	const char *pDot = strchr(name, '.');
	char *strTmp = NULL;
	if( pDot )	{// do not take the 2nd param as classname
		if( pDot == name )// if dot at first position then set no class
			classname = NULL;
		else	{
			strTmp = new char[ pDot - name + 1 ];
			memcpy( strTmp, name, pDot - name );
			strTmp[ pDot - name ] = '\0';
			
			classname = strTmp;
			name = pDot + 1;
		}
	}
	/// \todo optimise search algorithm (O = log(n) would be nice)
	for(BasicSlot *e = beFirst; e; e = e->beNext)	{
		if(strcmp(name, e->strName) == 0)
			if((classname == NULL) == (e->strClassName == NULL))
				if(classname == NULL || strcmp(classname, e->strClassName) == 0)
				{
					if( strTmp )
						delete[] strTmp;
					return e->pFunc;
				}
	}
	BasicApp::the->logLevelLine("WARNING: Slot not found: ");
	BasicApp::the->logString(name);
	BasicApp::the->logString(" in class ");
	BasicApp::the->logString(classname? classname : "(none)");
	if( strTmp )
		delete[] strTmp;
	return NULL;
}

///////////////////////////////////////////////////////////////////////////////////
// ---------------- BasicObject -------------------
///////////////////////////////////////////////////////////////////////////////////

BasicObject *BasicObject::pBase = NULL;

BasicObject::FunctionData BasicObject::getFunction(const char *name)
{
	if(pBase)
		return pBase->getFunctionByName(name);
	BasicApp::the->logLine("WARNING: BasicObject(): base-object is NULL, so resolving absolute function not possible!");
	FunctionData d;
	return d;
}

BasicObject::BasicObject(const char *name, BasicObject *p)
{
	xmlData = NULL;
	parent = NULL;
	strName = NULL;
	if(!name && p)	{
		BasicApp::the->logLine("FATAL ERROR: BasicObject(): the name is NULL, but the parent isn't!");
		return;
	}
	if(p)	{
		if(strchr(name, '.'))	{
			BasicApp::the->logLine("WARNING: BasicObject(): the name contains a dot ('.') !");
		}
		parent = p;
		parentitem = p->addChild(this, name);
	}
	else if( !pBase )	{
		pBase = this;
	}
}

BasicObject::BasicObject( TiXmlElement *el, BasicObject *p )
{
	xmlData = el;
	parent = NULL;
	strName = NULL;
	const char *name = el->Attribute("Name");
	if(!name && p)	{
		BasicApp::the->logLine("FATAL ERROR: BasicObject(): the name is NULL, but the parent isn't!");
		return;
	}
	if(p)	{
		if(strchr(name, '.'))	{
			BasicApp::the->logLine("WARNING: BasicObject(): the name contains a dot ('.') !");
		}
		parent = p;
		parentitem = p->addChild(this, name);
	}
	else if( !pBase )	{
		pBase = this;
	}
}

BasicObject::~BasicObject()
{
	if(parent)
		parent->removeChild(this);
	else if(this == pBase)
		pBase = NULL;
	
	while(!children.empty())
		delete children.front();
}

void BasicObject::onInit()
{
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		(*i)->onInit();
	}
}

void BasicObject::onDestroy()
{
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		(*i)->onDestroy();
	}
}

bool BasicObject::matchName(const char *name)
{
	const char *pEnd = strchr(name, '.'),
		*pSharp = strchr(name, '#');
	if( pEnd > pSharp )	{
		if( pSharp )
			pEnd = pSharp;
	}
	else if( !pEnd )	{
		if( pSharp )
			pEnd = pSharp;
		else
			return strcmp(name, strName) == 0;
	}
	if((int)strlen(strName) != pEnd-name)
		return false;
	return memcmp(name, strName, pEnd-name) == 0;
}

BasicObject *BasicObject::getObjectByName(const char *name)
{
	switch(*name)	{
	default:	// absolute path to function
		if(pBase != this)
			return pBase->getObjectByName(name);
		
		--name;
	case '.':	// a child's function
		if(*++name == '.')	{// ".." => let parent look for it
			if(parent)
				return parent->getObjectByName(name);
			// this cannot occure
			return NULL;
		}
		else if( *name == '#' )	{// ".#"
			return this;
		}
		// "." => this has to look for it
		for(child_iter i = children.begin(); i != children.end(); ++i)	{
			if( (*i)->matchName(name) )	{
				while( *++name != '.' )
					if( *name == '\0' || *name == '#' )// no sub-object specified, so return this object
						return (*i);
				
				return (*i)->getObjectByName(name);
			}
		}
		BasicApp::the->logLevelLine("WARNING: Object-path not found: ");
		BasicApp::the->logString(name);
		if(parent)	{
			BasicApp::the->logString("; as child of: ");
			BasicApp::the->logString(getName());
		}
		return NULL;
	
	case '#':	// function of this object
		return this;
	
	}
}

BasicObject *BasicObject::getChildWithName( const char *name )
{
	for( child_iter i = children.begin(); i != children.end(); ++i )
		if( (*i)->matchName( name ) )
			return *i;
	
	return NULL;
}

BasicObject::Callback BasicObject::getFuncByName(const char *name)
{
	// first get the object
	BasicObject *o;
	if(*name != '#')	{
		o = getObjectByName(name);
		if(!o)
			return NULL;
		while(*++name != '#')
			if(!*name)
				return NULL;
	}
	else	{
		o = this;
	}
	name++;
	
	return BasicSlot::getFunction(name, o->getClassName());
}

BasicObject::FunctionData BasicObject::getFunctionByName(const char *name)
{
	FunctionData fd;
	if(name == NULL || *name == '\0')
		return fd;
	// first get the object
	if(*name != '#')	{
		if(strchr(name+1,'#') == NULL)	{// no '#' in name, then look for function without classname
			fd.obj = NULL;
			fd.func = BasicSlot::getFunction(name, NULL);
			return fd;
		}
		fd.obj = getObjectByName(name);
		if(!fd.obj)	{
			return fd;
		}
		while(*++name != '#')
			if(!*name)	{
				BasicApp::the->logLevelLine("WARNING: invalid name (missing '#'): ");
				BasicApp::the->logString(name);
				return fd;
			}
	}
	else	{
		fd.obj = this;
	}
	name++;
	// then the object's function
	fd.func = BasicSlot::getFunction(name, fd.obj->getClassName());
	return fd;
}

BasicObject::child_iter BasicObject::addChild(BasicObject *obj, const char *name, bool top)
{
	if(*name == 0)	{
		int n = 1;
		char tmp[40] = "1", *pNum = tmp;
FOR:
		for(child_iter i = children.begin(); i != children.end(); ++i )	{
			if( strcmp((*i)->getName(), pNum) == 0 )	{
				pNum = intToStr(++n, tmp, 10);
				goto FOR;
			}
		}
		obj->strName = new char[strlen(tmp)+1];
		strcpy(obj->strName, tmp);
	}
	else	{
		for(child_iter i = children.begin(); i != children.end(); ++i )	{
			if( strcmp((*i)->getName(), name) == 0 )	{
				BasicApp::the->logLine("WARNING: BasicObject->addChild(): equal name already in list: ");
				BasicApp::the->logString(name);
			}
		}
		obj->strName = new char[strlen(name)+1];
		strcpy(obj->strName, name);
	}
	if( top )
		return children.insert( children.begin(), obj );
	else
		return children.insert( children.end(), obj );
}

void BasicObject::removeChild( BasicObject *obj )
{
	children.erase( obj->parentitem );
	delete[] obj->strName;
}

// ---------------- SignalMultiplier -------------------
DECLARE_SLOT(event, SignalMultiplier, SignalMultiplier::event)

SignalMultiplier::SignalMultiplier(const char *name, BasicObject *parent)
	: BasicObject(name, parent)
{
}

SignalMultiplier::SignalMultiplier(TiXmlElement *el, BasicObject *parent)
	: BasicObject(el, parent)
{
}

SignalMultiplier::~SignalMultiplier()
{
}

void SignalMultiplier::onInit()
{
	if( xmlData )	{
		for(TiXmlElement *slot = xmlData->FirstChildElement("Slot"); slot; slot = slot->NextSiblingElement("Slot"))
		{
			const char *str = slot->Attribute("Aim");
			if(str)	{
				BasicObject::FunctionData d = getFunctionByName(str);
				if(d.func)
					addSlot(d.func, d.obj);
			}
		}
	}
}

void SignalMultiplier::addSlot(Callback func, BasicObject *aim)
{
	BasicObject::FunctionData d;
	d.obj = aim;
	d.func = func;
	aims.push_back(d);
}

void SignalMultiplier::removeSlot(Callback func, BasicObject *aim)
{
	BasicObject::FunctionData d;
	d.obj = aim;
	d.func = func;
	for(cblist::iterator i = aims.begin(); i != aims.end(); ++i)
		if((*i).obj == aim && (*i).func == func)	{
			cblist::iterator tmp = i;
			--i;
			aims.erase(tmp);
		}
}

void SignalMultiplier::onevent(void *sender, void *data)
{
	for(cblist::iterator i = aims.begin(); i != aims.end(); ++i)
		(*(*i).func)(sender, (*i).obj, data);
}
