/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "basicthread.h"
#include "basicapp.h"

BasicThread::BasicThread( TiXmlElement *el, BasicObject *p )
	: BasicLog( el, p )
{
	pThread = NULL;
	ulThreadId = 0;
	mutex = NULL;
	bStopping = false;
	nCycles = 0;
	tPerCycle = ~0;
}

BasicThread::BasicThread( const char *name, BasicObject *p, const char * debug )
	: BasicLog( name, p, debug )
{
	pThread = NULL;
	ulThreadId = 0;
	mutex = NULL;
	bStopping = false;
	nCycles = 0;
	tPerCycle = ~0;
}

BasicThread::BasicThread( const char *name, BasicObject *p, std::ostream *strm )
	: BasicLog( name, p, strm )
{
	pThread = NULL;
	ulThreadId = 0;
	mutex = NULL;
	bStopping = false;
	nCycles = 0;
	tPerCycle = ~0;
}

BasicThread::~BasicThread()
{
	stop( 10 );
}

bool BasicThread::start( bool paused )
{
	bStopping = false;
	bPaused = paused;
	pThread = SDL_CreateThread( startThread, this );
	return pThread != NULL;
}

void BasicThread::kill()
{
	if( pThread )	{
		SDL_KillThread( pThread );
		ulThreadId = 0;
		pThread = NULL;
		SDL_DestroyMutex( mutex );
		mutex = NULL;
	}
}

void BasicThread::restart( Uint32 wait )
{
	stop( wait );
	start();
}

bool BasicThread::setPause( bool pause )
{
	if( bStopping || !running() )
		return false;
	if( bPaused == pause )
		return true;
	bPaused = pause;
	if( bPaused )
		tPaused = SDL_GetTicks();
	else tStart += SDL_GetTicks() - tPaused;
	return true;
}

void BasicThread::stop( Uint32 wait )
{
	if( ulThreadId != 0 )	{
		if( wait == (Uint32)~0UL )	{
			logLevelLine("Stopping thread...");
			bStopping = true;
			return;
		}
		logBegin("Stopping thread...");
		bStopping = true;
		// wait to stop
		for( Uint32 i = SDL_GetTicks() + wait; i > SDL_GetTicks() && ulThreadId != 0; SDL_Delay( 1 ));
		if( ulThreadId != 0 )	{
			logLevelLine("It has to be killed.");
			kill();
		}
		bStopping = false;
		logEnd("done.");
	}
}

bool BasicThread::lock( Uint32 wait )
{
	if( mutex == NULL )
		return false;
	if( wait )	{
		Uint32 t = SDL_GetTicks() + wait;
		while( t >= SDL_GetTicks() )	{
			if( SDL_mutexP( mutex ) != -1 )
				return true;
		}
		return false;
	}
	return SDL_mutexP( mutex ) != -1;
}

bool BasicThread::unlock()
{
	if( mutex == NULL )
		return false;
	return SDL_mutexV( mutex ) != -1;
}

unsigned int BasicThread::getAvgTimePerCycle()
{
	if( nCycles == 0 )
		return ~0;
	return (tLastCycle - tStart) / nCycles;
}


int BasicThread::onRun()
{
	logBegin("Thread started.");
	nCycles = 0;
	tLastCycle = tStart = SDL_GetTicks();
	tPerCycle = ~0;
	while( !bStopping )
	{
		if( bPaused )	{
			SDL_Delay( 1 );
			continue;
		}
		onCycle();
		++nCycles;
		tPerCycle = SDL_GetTicks() - tLastCycle;
		tLastCycle += tPerCycle;
	}
	tPerCycle = ~0;
	logEnd("Thread stopped.");
	return 0;
}

int BasicThread::startThread( void * pvData )
{
	BasicThread * pThread = ( BasicThread * )pvData;
	pThread->mutex = SDL_CreateMutex();
	pThread->ulThreadId = SDL_ThreadID();
	int r = pThread->onRun();
	pThread->ulThreadId = 0;
	pThread->pThread = NULL;
	SDL_DestroyMutex( pThread->mutex );
	pThread->mutex = NULL;
	return r;
}

