/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guibackground.h"
#include <cmath>
#include <string>
#include "rectfunctions.h"
#include "sdlmanager.h"
#include "sdlsurface.h"
#include "sdltexture.h"
#include "sdlsurfacecontainer.h"
#include "sdltexturecontainer.h"
#include "uimanager.h"
#include "guidynamicbutton.h"
#include "guitext.h"
#include "thememanager.h"
#include "basicapp.h"
#include "functions.h"
#include "oglfunctions.h"

DECLARE_SLOT(new, GuiBackground, GuiBackground::newguiobject)
DECLARE_SLOT(new, GuiFrame, GuiFrame::newguiobject)
DECLARE_SLOT(new, GuiBorder, GuiBorder::newguiobject)
DECLARE_SLOT(new, GuiRadioContainer, GuiRadioContainer::newguiobject)
DECLARE_SLOT(new, GuiPictureBg, GuiPictureBg::newguiobject)
DECLARE_SLOT(new, GuiFlowWindow, GuiFlowWindow::newguiobject)
DECLARE_SLOT(new, GuiPageControl, GuiPageControl::newguiobject)

//////////////////////////////////////////////////////////////////////
// GuiBackground

GuiBackground::GuiBackground( const SDL_Rect &r, GuiElement *p, const char *name, ARGB_COLOR c )
	: GuiElement( r, p, name )
{
	bThemeColor = c == CLR_STANDARD;
	if( bThemeColor )	{
		GuiBackground::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
	}
	else bgcolor = c;
}

GuiBackground::GuiBackground( TiXmlElement *el, GuiElement *p )
	: GuiElement( el, p )
{
	bThemeColor = true;
	bgcolor = CLR_INVALID;
}

GuiBackground::~GuiBackground()
{
}

void GuiBackground::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		ARGB_COLOR c;
		const char *str = el->Attribute( "BgColor" );
		if( str )
			c = xStrToUL( str );
		else
			c = CLR_STANDARD;
		
		bThemeColor = c == CLR_STANDARD;
		if( !bThemeColor )
			bgcolor = c;
	}
	GuiElement::onInit();
}

Uint32 GuiBackground::onFrame( SdlSurface *dst )
{
	paintBg( dst );
	return GuiElement::onFrame( dst );
}

Uint32 GuiBackground::onGLFrame()
{
	paintBgGL();
	return GuiElement::onGLFrame();
}

void GuiBackground::paintBg( SdlSurface *dst )
{
	if( bgcolor != CLR_INVALID )	{
		dst->setNoClip();
		SDL_Rect clip = getClip();
		dst->fillARGB( &clip, bgcolor );
	}
}

void GuiBackground::paintBgGL()
{
	if( bgcolor != CLR_INVALID )
		ogl::fillRect( getClip(), bgcolor );
}

Uint32 GuiBackground::onThemeChanged( UiTheme *t )
{
	if( !bThemeColor )
		return GuiElement::onThemeChanged( t );

	if( t )
		bgcolor = t->getUint32( ( string(getClassName()) + "->background-color" ).c_str() );
	else
		bgcolor = 0x000000;// black

	return GuiElement::onThemeChanged( t );
}

//////////////////////////////////////////////////////////////////////
// GuiFrame

GuiFrame::GuiFrame( const SDL_Rect &r, GuiElement *p, const char *name, Uint32 c1, Uint32 c2 )
	: GuiBackground( r, p, name, c1 )
{
	color1 = c1;
	color2 = c2;
	bgcolor = color1;
}

GuiFrame::GuiFrame( TiXmlElement *el, GuiElement *p ) : GuiBackground( el, p )
{
	color1 = color2 = CLR_INVALID;
}

GuiFrame::~GuiFrame()
{

}

void GuiFrame::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		unsigned long c;
		if( el->QueryULongAttribute( "ColorOut", &c ) != TIXML_SUCCESS )
			color1 = CLR_INVALID;
		else color1 = c;
		if( el->QueryULongAttribute( "ColorOver", &c ) != TIXML_SUCCESS )
			color2 = CLR_INVALID;
		else color2 = c;
		bgcolor = color1;
	}
	GuiBackground::onInit();
}

Uint32 GuiFrame::onMouseOver()
{
	bgcolor = color2;
	return 0;
}

Uint32 GuiFrame::onMouseOut()
{
	bgcolor = color1;
	return 0;
}

//////////////////////////////////////////////////////////////////////
// GuiBorder

GuiBorder::GuiBorder( const SDL_Rect &r, GuiElement *p, const char *name, SdlImageContainer * bmp, bool themestyle )
	: GuiElement( r, p, name )
{
	invert = false;
	left = right = top = bottom = bShowBorder = true;
	bmps = bmp;
	if( bmps )	{
		bThemeBmps = false;
		bCanNeutral = neutral = bmps->numImages() >= 18;
		bmps->getImage( 6 )->getSize( nCornerWidth, nCornerHeight );// size of corners
		bmps->getImage( 0 )->getSize( nBmpWidth, brd_height );
		bmps->getImage( 3 )->getSize( brd_width, nBmpHeight );
	}
	else bThemeBmps = true;

	bThemeStyle = themestyle;

	if( bThemeBmps || bThemeStyle )
		GuiBorder::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiBorder::GuiBorder( TiXmlElement *el, GuiElement *p )
	: GuiElement( el, p )
{
	invert = false;
	left = right = top = bottom = true;
	bShowBorder = false;
	bmps = NULL;
	bThemeStyle = true;
	bThemeBmps = true;
}

GuiBorder::~GuiBorder()
{
}

void GuiBorder::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		const char *str = el->Attribute( "InvertBorder" );
		invert = str != NULL && strToBool( str );
		str = el->Attribute( "BorderEdges" );
		if( str )	{
			left = strchr( str, 'l' ) != NULL;
			right= strchr( str, 'r' ) != NULL;
			top  = strchr( str, 't' ) != NULL;
			bottom=strchr( str, 'b' ) != NULL;
		}
		else
			left = right = top = bottom = true;
		bShowBorder = el->Attribute( "HideBorder" ) == NULL;
		str = el->Attribute( "BorderBmps" );
		if( str )
			bmps = SdlManager::the->getImagesByName( str );
		else bmps = NULL;
		if( bmps )
		{
			bThemeBmps = false;
			bCanNeutral = neutral = bmps->numImages() >= 18;
			
			bmps->getImage( 6 )->getSize( nCornerWidth, nCornerHeight );// size of corners
			bmps->getImage( 0 )->getSize( nBmpWidth, brd_height );
			bmps->getImage( 3 )->getSize( brd_width, nBmpHeight );
		}
		else bThemeBmps = true;
		str = el->Attribute( "NeutralBorder" );
		if( str && bCanNeutral )
			neutral = strToBool( str );
		bThemeStyle = ( str = el->Attribute( "BorderTheme" ) ) == NULL || strToBool( str );
	}
	
	if( bThemeBmps || bThemeStyle )
		changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );
	
	GuiElement::onInit();
}

Uint32 GuiBorder::onFrame( SdlSurface *bk )
{
	if( bmps && bShowBorder )
	{
		SdlSurfaceContainer *sc = (SdlSurfaceContainer*)bmps;
		
		int rl, lt, rt, lb, rb;
		if( bCanNeutral )	{
			rl = 3;
			lt = 6;
			rt = 9;
			lb = 12;
			rb = 15;
		}
		else	{
			rl = 2;
			lt = 4;
			rt = 6;
			lb = 8;
			rb = 10;
		}
		SDL_Rect r = getDrawWindow(), c = getClip();
		r.h -= brd_height;
		r.w -= brd_width;
		Sint32 r_right = r.x + r.w;
		Sint32 r_bottom = r.y + r.h;
		bk->setClip( c );
		if( top && bottom )
			if( neutral )	{
				for( int x = nCornerWidth; x < r_right - nBmpWidth; x += nBmpWidth )
				{
					bk->fastBlit( sc->getSurface( 2 ), x, r.y, NULL );
					bk->fastBlit( sc->getSurface( 2 ), x, r_bottom, NULL );
				}
				bk->fastBlit( sc->getSurface( 2 ), r_right - nBmpWidth, r.y, NULL );
				bk->fastBlit( sc->getSurface( 2 ), r_right - nBmpWidth, r_bottom, NULL );
			}
			else	{
				for( int x = nCornerWidth; x < r_right - nBmpWidth; x += nBmpWidth )
				{
					bk->fastBlit( sc->getSurface( invert ), x, r.y, NULL );
					bk->fastBlit( sc->getSurface( !invert ), x, r_bottom, NULL );
				}
				bk->fastBlit( sc->getSurface( invert ), r_right - nBmpWidth, r.y, NULL );
				bk->fastBlit( sc->getSurface( !invert ), r_right - nBmpWidth, r_bottom, NULL );
			}
		else	{
			if( top )
				if( neutral )	{
					for( int x = nCornerWidth; x < r_right - nBmpWidth; x += nBmpWidth )
						bk->fastBlit( sc->getSurface( 2 ), x, r.y ,NULL);
					bk->fastBlit( sc->getSurface( 2 ), r_right - nBmpWidth, r.y ,NULL );
				}
				else	{
					for( int x = nCornerHeight; x < r_right - nBmpWidth; x += nBmpWidth )
						bk->fastBlit( sc->getSurface( invert ), x, r.y ,NULL );
					bk->fastBlit( sc->getSurface( invert ), r_right - nBmpWidth, r.y ,NULL );
				}
			else if( bottom )
				if( neutral )	{
					for( int x = nCornerHeight; x < r_right - nBmpWidth; x += nBmpWidth )
						bk->fastBlit( sc->getSurface( 2 ), x, r_bottom, NULL );
					bk->fastBlit( sc->getSurface( 2 ), r_right - nBmpWidth, r_bottom, NULL );
				}
				else	{
					for( int x = nCornerHeight; x < r_right - nBmpWidth; x += nBmpWidth )
						bk->fastBlit( sc->getSurface( !invert ), x, r_bottom, NULL );
					bk->fastBlit( sc->getSurface( !invert ), r_right - nBmpWidth, r_bottom, NULL );
				}
		}
		if( left && right )
			if( neutral )	{
				for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
				{
					bk->fastBlit( sc->getSurface( 5 ), r.x, y, NULL );
					bk->fastBlit( sc->getSurface( 5 ), r_right, y, NULL );
				}
				bk->fastBlit( sc->getSurface( 5 ), r.x, r_bottom - nBmpHeight, NULL );
				bk->fastBlit( sc->getSurface( 5 ), r_right, r_bottom - nBmpHeight, NULL );
			}
			else	{
				for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
				{
					bk->fastBlit( sc->getSurface( invert + rl ), r.x,y, NULL );
					bk->fastBlit( sc->getSurface( !invert + rl ), r_right,y, NULL );
				}
				bk->fastBlit( sc->getSurface( invert + rl ), r.x, r_bottom - nBmpHeight, NULL );
				bk->fastBlit( sc->getSurface( !invert + rl ), r_right, r_bottom - nBmpHeight, NULL );
			}
		else	{
			if( left )	{
				if( neutral )	{
					for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
						bk->fastBlit( sc->getSurface( 5 ), r.x, y,NULL);
					bk->fastBlit( sc->getSurface( 5 ), r.x, r_bottom - nBmpHeight, NULL);
				}
				else {
					for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
						bk->fastBlit( sc->getSurface( invert + rl ), r.x, y, NULL);
					bk->fastBlit( sc->getSurface( invert + rl ), r.x, r_bottom - nBmpHeight, NULL);
				}
			}
		else
			if( right )	{
				if( neutral )	{
					for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
						bk->fastBlit( sc->getSurface( 5 ), r_right, y, NULL );
					bk->fastBlit( sc->getSurface( 5 ), r_right, r_bottom - nBmpHeight, NULL );
				}
				else {
					for( int y = nCornerHeight; y < r_bottom - nBmpHeight; y += nBmpHeight )
						bk->fastBlit( sc->getSurface( !invert + rl ), r_right, y,NULL );
					bk->fastBlit( sc->getSurface( !invert + rl ), r_right, r_bottom - nBmpHeight, NULL );
				}
			}
		}
		if( top && bottom && left && right )	{//corners
			if( neutral )	{
				bk->fastBlit( sc->getSurface( 8 ), r.x, r.y, NULL );
				bk->fastBlit( sc->getSurface( 11 ), r_right + brd_width - nCornerWidth, r.y, NULL );
				bk->fastBlit( sc->getSurface( 14 ), r.x, r_bottom + brd_height - nCornerHeight, NULL );
				bk->fastBlit( sc->getSurface( 17 ), r_right + brd_width - nCornerWidth, r_bottom + brd_height - nCornerHeight,NULL);
			}
			else	{
				bk->fastBlit( sc->getSurface( invert+lt ), r.x, r.y, NULL );
				bk->fastBlit( sc->getSurface( invert+rt ), r_right + brd_width - nCornerWidth, r.y, NULL );
				bk->fastBlit( sc->getSurface( invert+lb ), r.x,r_bottom + brd_height - nCornerHeight, NULL );
				bk->fastBlit( sc->getSurface( invert+rb ), r_right + brd_width - nCornerWidth, r_bottom + brd_height - nCornerHeight, NULL );
			}
		}
		else	{
			if( left && top )
				bk->fastBlit( sc->getSurface( neutral ? 8 : invert + lt ), r.x, r.y, NULL );
			if( right && top )
				bk->fastBlit( sc->getSurface( neutral ? 11 : invert + rt ), r_right + brd_width - nCornerWidth, r.y, NULL );
			if( left && bottom )
				bk->fastBlit( sc->getSurface( neutral ? 14 : invert + lb ), r.x, r_bottom + brd_height - nCornerHeight, NULL );
			if( right && bottom )
				bk->fastBlit( sc->getSurface( neutral ? 17 : invert + rb ), r_right + brd_width - nCornerWidth, r_bottom + brd_height - nCornerHeight, NULL );

			if( !top )	{
				if( left )
					bk->fastBlit( sc->getSurface( neutral ? 5 : invert + rl ), r.x, r.y, NULL );
				else if( right )
					bk->fastBlit( sc->getSurface( neutral ? 5 : !invert + rl ), r_right, r.y, NULL );
			}
			if( !bottom )	{
				if( left )
					bk->fastBlit( sc->getSurface( neutral ? 5 : invert + rl ), r.x, r_bottom, NULL );
				else if( right )
					bk->fastBlit( sc->getSurface( neutral ? 5 : !invert + rl ), r_right, r_bottom, NULL );
			}
			if( !left )	{
				if( top )
					bk->fastBlit( sc->getSurface( neutral ? 2 : invert ), r.x, r.y, NULL );
				else if( bottom )
					bk->fastBlit( sc->getSurface( neutral ? 2 : !invert ), r.x, r_bottom, NULL );
			}
			if( !right )	{
				if( top )
					bk->fastBlit( sc->getSurface( neutral ? 2 : invert ), r_right, r.y, NULL );
				else if( bottom )
					bk->fastBlit( sc->getSurface( neutral ? 2 : !invert ), r_right, r_bottom, NULL );
			}
		}
	}
	return GuiElement::onFrame( bk );
}

Uint32 GuiBorder::onGLFrame()
{
	if( bmps && bShowBorder )
	{
		SdlTextureContainer *tc = (SdlTextureContainer*)bmps;
		
		int rl, lt, rt, lb, rb;
		if( bCanNeutral )	{
			rl = 3;
			lt = 6;
			rt = 9;
			lb = 12;
			rb = 15;
		}
		else	{
			rl = 2;
			lt = 4;
			rt = 6;
			lb = 8;
			rb = 10;
		}
		SDL_Rect rDraw = getDrawWindow(), clip = getClip();
		intersectRect(clip, rDraw);
		//rDraw.w -= brd_width;
		//rDraw.h -= brd_height;
		Sint32 r_right = rDraw.x + rDraw.w - brd_width;
		Sint32 r_bottom = rDraw.y + rDraw.h - brd_height;
		SDL_Rect r;
		if( top )
		{
			r.w = rDraw.w;
			if( left )	{
				r.x = rDraw.x + nCornerWidth;
				r.w -= nCornerWidth;
			}
			else
				r.x = rDraw.x;
			if( right )
				r.w -= nCornerWidth;
			
			r.y = rDraw.y;
			r.h = brd_height;
			intersectRect(r, clip);
			FloatRect fr = {0.f, (float)(r.y-rDraw.y)/(float)brd_height, 1.f, (float)(r.h)/(float)brd_height};
			if( neutral )
				tc->getTexture( 2 )->draw(fr, r);
			else
				tc->getTexture( invert )->draw(fr, r);
		}
		if( bottom )
		{
			r.w = rDraw.w;
			if( left )	{
				r.x = rDraw.x + nCornerWidth;
				r.w -= nCornerWidth;
			}
			else
				r.x = rDraw.x;
			if( right )
				r.w -= nCornerWidth;
			
			r.y = r_bottom;
			r.h = brd_height;
			intersectRect(r, clip);
			FloatRect fr = {0.f, (float)(r.y-r_bottom)/(float)brd_height, 1.f, (float)(r.h)/(float)brd_height};
			if( neutral )
				tc->getTexture( 2 )->draw(fr, r);
			else
				tc->getTexture( !invert )->draw(fr, r);
		}
		if( left )	{
			r.x = rDraw.x;
			r.w = brd_width;
			r.h = rDraw.h;
			if( top )	{
				r.y = rDraw.y + nCornerHeight;
				r.h -= nCornerHeight;
			}
			else
				r.y = rDraw.y;
			if( bottom )
				r.h -= nCornerHeight;
			
			intersectRect(r, clip);
			FloatRect fr = {(float)(r.x-rDraw.x)/(float)brd_width, 0.f, (float)(r.w)/(float)brd_width, 1.f};
			if( neutral )
				tc->getTexture( 5 )->draw(fr, r);
			else
				tc->getTexture( invert + rl )->draw(fr, r);
		}
		if( right )	{
			r.x = r_right;
			r.w = brd_width;
			r.h = rDraw.h;
			if( top )	{
				r.y = rDraw.y + nCornerHeight;
				r.h -= nCornerHeight;
			}
			else
				r.y = rDraw.y;
			if( bottom )
				r.h -= nCornerHeight;
			
			intersectRect(r, clip);
			FloatRect fr = {(float)(r.x-r_right)/(float)brd_width, 0.f, (float)(r.w)/(float)brd_width, 1.f};
			if( neutral )
				tc->getTexture( 5 )->draw(fr, r);
			else
				tc->getTexture( !invert + rl )->draw(fr, r);
		}
		// draw corners
		{
			SDL_Rect src;
/*			rDraw.w += brd_width;
			rDraw.h += brd_height;*/
			if( left && top && clip.x < rDraw.x+nCornerWidth && clip.y < rDraw.y+nCornerHeight )
			{
				r.x = rDraw.x;
				r.y = rDraw.y;
				r.w = nCornerWidth;
				r.h = nCornerHeight;
				src = clip;
				intersectRect(src, r);
				src.x -= r.x;
				r.x += src.x;
				src.y -= r.y;
				r.y += src.y;
				tc->getTexture( neutral ? 8 : invert + lt )->draw(src, r);
			}
			if( right && top && clip.x+clip.w > rDraw.x+rDraw.w-nCornerWidth && clip.y < rDraw.y+nCornerHeight )
			{
				r.x = rDraw.x + rDraw.w - nCornerWidth;
				r.y = rDraw.y;
				r.w = nCornerWidth;
				r.h = nCornerHeight;
				src = clip;
				intersectRect(src, r);
				src.x -= r.x;
				r.x += src.x;
				src.y -= r.y;
				r.y += src.y;
				tc->getTexture( neutral ? 11 : invert + rt )->draw(src, r);
			}
			if( left && bottom && clip.x < rDraw.x+nCornerWidth && clip.y+clip.h > rDraw.y+rDraw.h-nCornerHeight)
			{
				r.x = rDraw.x;
				r.y = rDraw.y + rDraw.h - nCornerHeight;
				r.w = nCornerWidth;
				r.h = nCornerHeight;
				src = clip;
				intersectRect(src, r);
				src.x -= r.x;
				r.x += src.x;
				src.y -= r.y;
				r.y += src.y;
				tc->getTexture( neutral ? 14 : invert + lb )->draw(src, r);
			}
			if( right && bottom && clip.x+clip.w > rDraw.x+rDraw.w-nCornerWidth && clip.y+clip.h > rDraw.y+rDraw.h-nCornerHeight)
			{
				r.x = rDraw.x + rDraw.w - nCornerWidth;
				r.y = rDraw.y + rDraw.h - nCornerHeight;
				r.w = nCornerWidth;
				r.h = nCornerHeight;
				src = clip;
				intersectRect(src, r);
				src.x -= r.x;
				r.x += src.x;
				src.y -= r.y;
				r.y += src.y;
				tc->getTexture( neutral ? 17 : invert + rb )->draw(src, r);
			}
		}
	}
	return GuiElement::onGLFrame();
}

void GuiBorder::setBmps( SdlImageContainer * ic )
{
	bmps = ic;
	if( bmps != 0 )
	{
		bCanNeutral = bmps->numImages() >= 18;
		bmps->getImage( 6 )->getSize( nCornerWidth, nCornerHeight );
		bmps->getImage( 0 )->getSize( nBmpWidth, brd_height );
		bmps->getImage( 3 )->getSize( brd_width, nBmpHeight );
	}
	bThemeBmps = false;
}

Uint32 GuiBorder::onResize()
{
	return 0;
}

Uint32 GuiBorder::onThemeChanged( UiTheme *t )
{
	if( t == 0 )
		return GuiElement::onThemeChanged( t );

	changeTheme( t, getClassName() );

	return GuiElement::onThemeChanged( t );
}

Uint32 GuiBorder::changeTheme( UiTheme *t, const char *classname )
{
	if( bThemeBmps )	{
		bmps = 0;
		const char *chName = t->getString( ( string(classname) + "->border-bmps" ).c_str() );
		if( chName )	{
			SdlImageContainer * b = SdlManager::the->getImagesByName( chName );
			if( b != 0 )
			{
				bmps = b;
				bmps->getImage( 6 )->getSize( nCornerWidth, nCornerHeight );// size of corners
				bmps->getImage( 0 )->getSize( nBmpWidth, brd_height );
				bmps->getImage( 3 )->getSize( brd_width, nBmpHeight );
				
				bCanNeutral = neutral = bmps->numImages() >= 16;
			}
		}
		if( bmps == 0 )	{
			nCornerWidth = nCornerHeight = 
					nBmpWidth = nBmpHeight = 
					brd_width = brd_height = 0;
		}
	}
	if( bThemeStyle )	{
		neutral = t->getBool( ( string(classname) + "->neutral" ).c_str() );
		invert = t->getBool( ( string(classname) + "->invert" ).c_str() );
		bCanNeutral = bmps && bmps->numImages() >= 16;
		neutral &= bCanNeutral;
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////
// GuiRadioContainer

GuiRadioContainer::GuiRadioContainer( const SDL_Rect &r, GuiElement *p, const char *name, Uint32 c )
	: GuiBackground( r, p, name, c )
{
	bChanging = false;
	allownocheck = true;
	checked = buttons.end();
	index = -1;
}

GuiRadioContainer::GuiRadioContainer( TiXmlElement *el, GuiElement *p )
	: GuiBackground( el, p )
{
	bChanging = false;
	allownocheck = true;
	checked = buttons.end();
	index = -1;
}

GuiRadioContainer::~GuiRadioContainer()
{
	// to avoid erase of invalid iterators in removeButton(), called later, when children are cleaning up
	buttons.clear();
}

void GuiRadioContainer::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		const char *str = el->Attribute( "OnCheck" );
		if(str)	{
			cb = getFunctionByName(str);
		}
		setAllowNoCheck( ( str = el->Attribute( "AllowNoCheck" ) ) == NULL || strToBool( str ) );
		str = el->Attribute( "Index" );
		if( str )
			setIndex( xStrToUL( str ) );
	}
	GuiBackground::onInit();
}


GuiRadioContainer::button_iter GuiRadioContainer::addButton( GuiRadioBtn *b )
{
	button_iter iter = buttons.insert( buttons.end(), b );
	if( b->getCheck() )	{
		onCheck( iter );
	}
	else if( !allownocheck )	{
		if( checked == buttons.end() )	{
			bChanging = true;
			b->setCheck( true );// setCheck will call onCheck with invalid iter
			bChanging = false;
			// call onCheck here with correct param
			onCheck( iter );
		}
	}
	return iter;
}

void GuiRadioContainer::removeButton( button_iter iter )
{
	if( buttons.empty() )
		return;
	buttons.erase( iter );
	
	if( checked == iter )	{
		index = -1;
		checked = buttons.end();
		if( allownocheck )	{
			onSelect( -1 );
		}
		else if( !buttons.empty() )	{
			buttons.front()->setCheck( true );
		}
	}
}

void GuiRadioContainer::onCheck( button_iter iter )
{
	if( bChanging )
		return;
	
	bChanging = true;
	if( (*iter)->getCheck() )	{
		if( checked != buttons.end() )
			(*checked)->setCheck( false );
		
		int n = 0;
		for( button_iter i = buttons.begin(); i != buttons.end(); n++,++i )	{
			if( i == iter )
			{
				index = n;
				onSelect( n );
			}
		}
		checked = iter;
	}
	else	{
		if( allownocheck )	{
			checked = buttons.end();
			index = -1;
			onSelect( -1 );
		}
		else
			(*iter)->setCheck( true );
	}
	bChanging = false;
}

void GuiRadioContainer::setAllowNoCheck( bool allow )
{
	if( allownocheck == allow )
		return;
	allownocheck = allow;
	
	if( !allownocheck && !buttons.empty() && index < 0 )	{
		setIndex( 0 );
	}
}

void GuiRadioContainer::setIndex( int id )
{
	if( id < 0 )	{
		if( !allownocheck )
			return;
		else
			(*checked)->setCheck( false );
	}
	
	int n = 0;
	for( button_iter i = buttons.begin(); i != buttons.end(); n++, ++i )
		if(id == n)
			(*i)->setCheck( true );
}

int GuiRadioContainer::getIndex()
{
	return index;
}

GuiRadioBtn * GuiRadioContainer::getSelectedItem()
{
	return getItem( getIndex() );
}

GuiRadioBtn * GuiRadioContainer::getItem( int i )
{
	int n = 0;
	for( button_iter b = buttons.begin(); b != buttons.end(); ++b, n++ )
		if( i == n )
			return (*b);

	return 0;
}

void GuiRadioContainer::onSelect( int i )
{
	if( cb.func )
		(*cb.func)( this, cb.obj, &i );
}

//////////////////////////////////////////////////////////////////////
// GuiPictureBg

GuiPictureBg::GuiPictureBg( const SDL_Rect &r, GuiElement * el, const char *name, Uint32 color, SdlImage * backbmp )
	: GuiBackground( r, el, name, color )
{
	bmp = backbmp;
	bThemeBmp = bmp == 0;

//	fastblt = dofastblt;
	if( bmp != 0 )
	{
		bmp->getSize( nBmpWidth, nBmpHeight );
	}
	GuiPictureBg::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiPictureBg::GuiPictureBg(TiXmlElement *el, GuiElement *p) : GuiBackground( el,p )
{
}

GuiPictureBg::~GuiPictureBg()
{
}

void GuiPictureBg::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		const char *str = el->Attribute( "BgBmp" );
		if(str)
			bmp = SdlManager::the->getImageByName( str );
		else bmp = NULL;
		bThemeBmp = bmp == 0;
	
		if( bmp != 0 )	{
			bThemeBmp = (str = el->Attribute( "BgTheme" )) != NULL && strToBool( str );
			bmp->getSize( nBmpWidth, nBmpHeight );
		}
	}
	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );
	GuiBackground::onInit();
}

void GuiPictureBg::setBmp( SdlImage * backbmp )
{
	bmp = backbmp;
	if( bmp != 0 )	{
		bmp->getSize( nBmpWidth, nBmpHeight );
		bThemeBmp = false;
	}
}

/*void GuiPictureBg::setFastBlt( bool dofast )
{
	fastblt = dofast;
}*/

void GuiPictureBg::paintBg( SdlSurface *s )
{
	if( bmp != 0 )
	{
		s->setClip( getClip() );
		SDL_Rect dst = getDrawWindow();
//		if( fastblt )	{
			for( int x = dst.x; x < dst.x + dst.w; x += nBmpWidth )
				for( int y = dst.y; y < dst.y + dst.h; y += nBmpHeight )
					s->fastBlit( (SdlSurface*)bmp, x, y, NULL );
/*		}
		else	{
			int x = dst.x + (( dst.w - nBmpWidth ) >> 1);
			int y = dst.y + (( dst.h - nBmpHeight ) >> 1);
			s->fastBlit( bmp, x, y, NULL );
		}*/
	}
	else GuiBackground::paintBg( s );
}

void GuiPictureBg::paintBgGL()
{
	if( bmp != 0 )
	{
		((SdlTexture*)bmp)->drawClipped( getClip(), getDrawWindow() );
	}
	else GuiBackground::paintBgGL();
}

Uint32 GuiPictureBg::onFrame( SdlSurface *dst )
{
	if( bmp == 0 )
		return GuiBackground::onFrame( dst );
	paintBg( dst );
	return GuiElement::onFrame( dst );
}

Uint32 GuiPictureBg::onGLFrame()
{
	paintBgGL();
	return GuiElement::onGLFrame();
}

Uint32 GuiPictureBg::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiBackground::onThemeChanged( t );
	if( t == 0 )
		return r;

	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );

	return r;
}

Uint32 GuiPictureBg::changeTheme( UiTheme *t, const char *classname )
{
	if( bThemeBmp )	{
		bmp = 0;
		const char *chName = t->getString( ( string(classname) + "->background-bmp" ).c_str() );
		if( chName )	{
			SdlImage * b = SdlManager::the->getImageByName( chName );
			if( b != 0 )	{
				bmp = b;
				bmp->getSize( nBmpWidth, nBmpHeight );
			}
		}
	}
/*
	if( bThemeBlit )	{
		fastblt = ...;
	}
*/
	return 0;
}

//////////////////////////////////////////////////////////////////////
// guiflywindow

GuiFlowWindow::GuiFlowWindow( const SDL_Rect &r, GuiElement *el, const char *name, Uint32 color, SdlImage * backbmp, FlyStyles style )
	: GuiPictureBg( r, el, name, color, backbmp )
{
	eFlyStyle = style;
	bThemeStyle = eFlyStyle == theme_std;
	src = r;
	bFly = false;
	nTime = 500;
	elapsed = 0;
	setRectEmpty( rDestination );

	if( bThemeStyle )
		GuiFlowWindow::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiFlowWindow::GuiFlowWindow(TiXmlElement *el, GuiElement *p) : GuiPictureBg( el,p )
{
	eFlyStyle = theme_std;
	bThemeStyle = true;
	src = getRelWindow();
	bFly = false;
	nTime = 500;
	elapsed = 0;
	setRectEmpty( rDestination );
}

GuiFlowWindow::~GuiFlowWindow()
{
}

void GuiFlowWindow::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		const char *str = el->Attribute( "FlyStyle" );
		if( str )	{
			if( strcmp( str, "sinus" ) == 0 )
				eFlyStyle = sinus;
			else if( strcmp( str, "cosinus" ) == 0 )
				eFlyStyle = cosinus;
			else if( strcmp( str, "constant" ) == 0 )
				eFlyStyle = constant;
			else if( strcmp( str, "powsin" ) == 0 )
				eFlyStyle = powsin;
			else if( strcmp( str, "sin_2" ) == 0 )
				eFlyStyle = sin_2;
			else
				eFlyStyle = theme_std;
		}
		else
			eFlyStyle = theme_std;
		bThemeStyle = eFlyStyle == theme_std;
		str = el->Attribute( "DestRect" );
		if( str )	{
			rDestination = strToRect( str );
		}
		str = el->Attribute( "Time" );
		if( str )	{
			nTime = atoi( str );
		}
	}
	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );
	
	GuiPictureBg::onInit();
}

void GuiFlowWindow::flyTo( const SDL_Rect &r, Uint32 t )
{
	if( bFly )	{
		src = rDestination;
		setRelWindow( rDestination );
		elapsed = 0;
	}
	else src = getRelWindow();
	if(t != 0)
		nTime = t;
	rDestination = r;
	start();
}

SDL_Rect GuiFlowWindow::keyFrame( const SDL_Rect &s, const SDL_Rect &d, float t )
{
	SDL_Rect n;
	n.y = s.y + (Sint16)((d.y - s.y) * t);
	n.h = s.h + (Sint16)((d.h - s.h) * t);
	n.x = s.x + (Sint16)((d.x - s.x) * t);
	n.w = s.w + (Sint16)((d.w - s.w) * t);
	return n;
}

Uint32 GuiFlowWindow::onFrame( SdlSurface *drawdst )
{
	calcCurrentPos();
	return GuiPictureBg::onFrame( drawdst );
}

Uint32 GuiFlowWindow::onGLFrame()
{
	calcCurrentPos();
	return GuiPictureBg::onGLFrame();
}

void GuiFlowWindow::calcCurrentPos()
{
	if( !bFly )
		return;
	
	elapsed += UiManager::the->getTimePerFrame();
	if( elapsed >= nTime )
	{
		src = rDestination;
		setRelWindow( rDestination );
		stop();
		elapsed = 0;
		//nTime = 0;
		onPosReached();
	}
	else
	{
		float l = (float)elapsed / (float)nTime;
		float t;
		switch( eFlyStyle )
		{
		case sinus:// fast at first, then slower to the end
			t = sinf( 1.57 * l );
			break;
		case cosinus:// like sinus but backwards
			t = cosf( 3.14159265f + 1.57f * l ) + 1;
			break;
		case constant:// constant speed
			t = l;
			break;
		case theme_std:// as defined in theme
			if( ThemeManager::the && ThemeManager::the->getActiveTheme() )
			{
				eFlyStyle = (FlyStyles)ThemeManager::the->getActiveTheme()->getUint32(
								( (string)getClassName() + "->speedstyle" ).c_str()
							);
				t = 0.f;
				break;
			}
		default:
		case powsin:// accelerating at first and then slowing down
			t = powf( sinf( 1.571 * l ), 4.f );
			break;
		case sin_2:// very similar to powsin
			t = ( sinf( M_PI * ( l - 0.5f ) ) + 1 ) / 2;
			break;
		}
		SDL_Rect act = keyFrame( src, rDestination, t );
		setRelWindow( act );
	}
}

Uint32 GuiFlowWindow::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiPictureBg::onThemeChanged( t );
	changeTheme( t, getClassName() );
	return r;
}

Uint32 GuiFlowWindow::changeTheme( UiTheme *t, const char *classname )
{
	if( bThemeStyle || eFlyStyle == theme_std )
		if( t )	{
			eFlyStyle = (FlyStyles)ThemeManager::the->getActiveTheme()->getUint32( ( string(getClassName()) + "->flystyle" ).c_str() );
		}
		else eFlyStyle = powsin;

	return 0;
}

Uint32 GuiFlowWindow::onPosReached()
{
	if( cb.func )
		(*cb.func)( this, cb.obj, NULL );
	return 0;
}

void GuiFlowWindow::start()
{
	if( bFly )
		return;
	elapsed = 0;
	bFly = true;
	incRequireOnFrame();
}

void GuiFlowWindow::stop()
{
	if( !bFly )
		return;
	bFly = false;
	decRequireOnFrame();
	// draw final position onto surface
	setRedraw();
}

//////////////////////////////////////////////////////////////////////
// GuiPageControl Klasse
//////////////////////////////////////////////////////////////////////
#define BTNHEIGHT	30
#define BTNWIDTH	150

GuiPageControl::GuiPageControl( const SDL_Rect &r, GuiElement *el, const char *name, Uint32 c, SdlImageContainer *res )
	: GuiPictureBg( r,el,name,c )
{
	SDL_Rect rr = r;
	nBtnWidth = 0;
	btnBmps = res;
	bUseTheme = res == NULL;
	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );
	if( nBtnWidth == 0 )	{
		nBtnWidth = BTNWIDTH;
		nBtnHeight = BTNHEIGHT;
	}
	rr.x = rr.y = 0;
	side = bottom;
	pagerect = rr;
	pagerect.h -= nBtnHeight;
	rr.y = pagerect.y + pagerect.h;
	radios = new GuiRadioContainer( rr, this, 0 );
	radios->setAllowNoCheck( false );
	FunctionData fd(onRadio, this);
	radios->setCallback( fd );
	count = 0;
}

GuiPageControl::GuiPageControl( TiXmlElement *el, GuiElement *p )
	: GuiPictureBg( el, p )
{
	bUseTheme = true;
	btnBmps = NULL;
	
	nBtnWidth = BTNWIDTH;
	nBtnHeight = BTNHEIGHT;
	
	side = bottom;
	count = 0;
}

GuiPageControl::~GuiPageControl()
{

}

void GuiPageControl::onInit()
{
	TiXmlElement *el = getXmlData();
	if( el )	{
		const char *str = el->Attribute( "ButtonBmps" );
		if( str )
			btnBmps = SdlManager::the->getImagesByName( str );
		SDL_Rect rr = getRelWindow();
		rr.x = rr.y = 0;
		// TODO read side from xml-attrib
		pagerect = rr;
		pagerect.h -= nBtnHeight;
		rr.y = pagerect.y + pagerect.h;
		radios = new GuiRadioContainer( rr, this, 0 );
		radios->setAllowNoCheck( false );
		FunctionData fd(onRadio, this);
		radios->setCallback( fd );
	}
	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0, getClassName() );
	GuiPictureBg::onInit();
}

GuiPageControl::page_iter GuiPageControl::addPage( GuiElement *pg, const char *t, SdlImageContainer *res, Uint32 w )
{
	if( res == 0 )
		res = btnBmps;
	SDL_Rect r = pagerect;
	pg->setRelWindow( pagerect );
	pg->setVisible( count == 0 );
	r.y = 0;
	r.h = nBtnHeight;
	r.w = w != 0 ? w : nBtnWidth;
	SDL_Rect r_last;
	if( radios->getLast() )
		r_last = radios->getLast()->getRelWindow();
	else
		r_last.x = r_last.w = 0;
	r.x = r_last.x + r_last.w;
	++count;

	new GuiRadioBtn( r, radios, "", t, res );
	page_iter iter = pages.insert( pages.end(), pg );
	if( count == 1 )
		radios->setIndex( 0 );
	radios->toTop();
	return iter;
}

bool GuiPageControl::removePage( page_iter i )
{
	int n = 0;
	for( page_iter p = pages.begin(); p != pages.end(); p++, n++ )
		if( p == i )
			break;

	GuiRadioBtn *el = radios->getItem( n );
	if( el == 0 )
		return false;

	delete el;
	return true;
}

bool GuiPageControl::removePage( GuiElement *el )
{
	int n = 0;
	bool bAfterEl = false;
	int w = 0;
	for( page_iter p = pages.begin(); p != pages.end(); p++, n++ )	{
		if( *p == el )	{
			GuiRadioBtn *btn = radios->getItem( n );
			w = btn->getRelWindow().w;
			bAfterEl = true;

			page_iter pi = p;
			(*pi)->setVisible( false );
			++p;

			pages.erase( pi );
			delete btn;
			
			if( p == pages.end() )
				return true;
		}
		if( bAfterEl )	{
			GuiRadioBtn *btn = radios->getItem( n );
			SDL_Rect r = btn->getRelWindow();
			r.x -= w;
			btn->setRelWindow( r );
		}
	}

	return bAfterEl;
}

void GuiPageControl::onChange( GuiRadioContainer *sender )
{
	int i = 0;
	for( page_iter p = pages.begin(); p != pages.end(); p++, i++ )
		(*p)->setVisible( i == sender->getIndex() );
}

Uint32 GuiPageControl::onThemeChanged( UiTheme *t )
{
	if( t == 0 )	{
		return GuiPictureBg::onThemeChanged( t );
	}

	return GuiPictureBg::onThemeChanged( t );
}

Uint32 GuiPageControl::changeTheme( UiTheme *t, const char *classname )
{
	if( bUseTheme )	{
		nBtnWidth = t->getUint32( ( string(classname) + "->controlbutton-width" ).c_str() );
		nBtnHeight = t->getUint32( ( string(classname) + "->controlbutton-height" ).c_str() );
		const char *name = t->getString( ( string(classname) + "->controlbutton-bmps" ).c_str() );
		if( name != 0 )
			btnBmps = SdlManager::the->getImagesByName( name );
	}

	return 0;
}

void GuiPageControl::setSide( SIDE s )
{
	if( s == side )
		return;
	side = s;
	arrange();
}

void GuiPageControl::arrange()
{
	SDL_Rect r = getRelWindow();
	r.x = r.y = 0;
	pagerect = r;
	switch( side )
	{
	case top:
		pagerect.y += nBtnHeight;
		r.w = pagerect.y;
		break;
	case bottom:
		pagerect.h -= nBtnHeight;
		r.y = pagerect.h;
		break;
	case left:
		pagerect.x += nBtnHeight;
		r.w = pagerect.x;
		break;
	case right:
		pagerect.w -= nBtnHeight;
		r.x = pagerect.w;
		break;
	default:
		return;
	}
	for( page_iter p = pages.begin(); p != pages.end(); p++ )
		(*p)->setRelWindow( pagerect );

	switch( side )	{
	case top:
	case bottom:	{
		r.y = 0;
		r.h = nBtnHeight;
		for( int i = 0; i < count; i++ )	{
			r.x = nBtnWidth * i;
			r.w = r.x + nBtnWidth;
			radios->getItem( i )->setRelWindow( r );
		}
		break;
	}
	case left:
	case right:
		r.x = 0;
		r.w = nBtnWidth;
		r.h = nBtnHeight;
		for( int i = 0; i < count; i++ )	{
			r.y = nBtnHeight * i;
			radios->getItem( i )->setRelWindow( r );
		}
		break;
	}
}

Uint32 GuiPageControl::onResize()
{
	arrange();
	return 0;
}

void GuiPageControl::setButton( Uint32 w, Uint32 h, SdlImageContainer *res )
{
	bUseTheme = false;
	nBtnWidth = w;
	nBtnHeight = h;
	if( res )
		btnBmps = res;
	arrange();
}
