/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guidynamicbutton.h"
#include <string>
#include "rectfunctions.h"
#include "guimanager.h"
#include "thememanager.h"
#include "sdlmanager.h"
#include "sdlsurface.h"
#include "sdlsurfacecontainer.h"
#include "sdltexture.h"
#include "sdltexturecontainer.h"
#include "sdlaudio.h"
#include "sdlfont.h"
#include "guitext.h"
#include "basicapp.h"
#include "functions.h"

DECLARE_SLOT(new, GuiDynamicButton, GuiDynamicButton::newguiobject)
DECLARE_SLOT(new, GuiClickableBtn, GuiClickableBtn::newguiobject)
DECLARE_SLOT(new, GuiCheckableBtn, GuiCheckableBtn::newguiobject)
DECLARE_SLOT(new, GuiRadioBtn, GuiRadioBtn::newguiobject)
DECLARE_SLOT(new, GuiMoveableBtn, GuiMoveableBtn::newguiobject)


////////////////////////////////////////////////////////////////////////////////////
// GuiDynamicButton

GuiDynamicButton::GuiDynamicButton( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
	: GuiBorder( r, p, name, 0, false )
{
	eBtnState = normal;

	fNormal = fHover = 0;
	text = 0;
	bmps = b;
	bThemeFonts = bThemeBmps = bmps == 0;

	eDynamic = ds_fast;
	if( bmps )
		if( bmps->numImages() > 2 )
			eDynamic = ds_none;
	up = u;
	down = d;
	bThemeAudios = up == 0 && down == 0;

//	nFirstSurface = 0;
	setText( t );

	GuiDynamicButton::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiDynamicButton::GuiDynamicButton( TiXmlElement *el, GuiElement *p ) : GuiBorder( el, p )
{
	eBtnState = normal;

	fNormal = fHover = 0;
	text = 0;
	bmps = NULL;
	up = down = NULL;
	eDynamic = ds_fast;
	bThemeBmps = true;
	bThemeAudios = true;
	bThemeFonts = true;

//	nFirstSurface = 0;
}

GuiDynamicButton::~GuiDynamicButton()
{
	if( text )	{
		delete[] text;
		text = 0;
	}
}

void GuiDynamicButton::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		setText( el->Attribute("Caption") );
		const char *str = el->Attribute("ButtonBmps");
		if(str)
			bmps = SdlManager::the->getImagesByName(str);
		else
			bmps = NULL;
		bThemeBmps = bmps == 0;
		
		str = el->Attribute("ButtonFontTheme");
		bThemeFonts = str == NULL || strToBool( str );
	
		if( bmps )
			if( bmps->numImages() > 2 )
				eDynamic = ds_none;
		str = el->Attribute("ButtonSoundUp");
		if(str)
			up = SdlManager::the->getAudioByName(str);
		else
			up = NULL;
		str = el->Attribute("ButtonSoundDown");
		if(str)
			down = SdlManager::the->getAudioByName(str);
		else
			down = NULL;
		bThemeAudios = up == 0 && down == 0;
	}
	changeTheme( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0,
				 getClassName() );
	
	GuiBorder::onInit();
}

void GuiDynamicButton::setSounds( SdlAudio *u, SdlAudio *d )
{
	bThemeAudios = false;
	up = u;
	down = d;
}

void GuiDynamicButton::setButtonBmps( SdlImageContainer *b )
{
	bThemeBmps = false;
	bmps = b;
}

void GuiDynamicButton::setText(const char *t)
{
	if( text )	{
		delete[] text;
		text = NULL;
	}
	if( t )	{
		text = new char[strlen(t) + 1];
		strcpy( text, t );
	}
}

Uint32 GuiDynamicButton::onFrame( SdlSurface *drawdst )
{
	SDL_Rect clip = getClip();
	SDL_Rect dst = getDrawWindow();
	if( bCanNeutral && bShowBorder )
		inflateRect( clip, -nCornerWidth, -nCornerHeight );

	if( bmps )	{
		drawdst->setClip( clip );
		int bmp_w, bmp_h;
		bmps->getImage( 0/*(eBtnState != normal)/* + nFirstSurface*/ )->getSize( bmp_w, bmp_h );
		switch( eDynamic )
		{
		case ds_none:
			drawdst->fastBlit( ((SdlSurfaceContainer*)bmps)->getSurface( (int)eBtnState/* + nFirstSurface*/ ), dst.x + (dst.w - bmp_w)/2, dst.y + (dst.h - bmp_h)/2, NULL);
			break;
//		case ds_dynamic:
//			drawdst->fastBlit( bmps->getImage( (eBtnState != normal) + firstsurf ), dst.x + (dst.w - bmp_w)/2, dst.y + (dst.h - bmp_h)/2, NULL);
//			break;
		case ds_fast:	{
			for( int x = dst.x; x < dst.x+dst.w; x += bmp_w )
				for( int y = dst.y; y < dst.y+dst.h; y += bmp_h )
					drawdst->fastBlit( ((SdlSurfaceContainer*)bmps)->getSurface( (eBtnState != normal)/* + nFirstSurface*/ ), x, dst.y, NULL );
			break;
		}
		}
	}

	if( text )	{
		SDL_Rect dst = getDrawWindow();
//		dst.top += 10;
		inflateRect( dst, -10, -10 );
		switch( eBtnState )	{
		case normal:
			break;
		case pressed:
			offsetRect( dst, 2, 2 );
			break;
		case mouseover:
			offsetRect( dst, -2, -2 );
			break;
		}
		SdlFont *f;
		if( eBtnState == normal )
			f = fNormal;
		else
			f = fHover;
		if( f )	{
			SDL_Rect clip = getClip();
			f->write( drawdst, text, dst, SdlFont::baseline, SdlFont::center, &clip );
		}
	}

	invert = eBtnState == pressed;
	if( bCanNeutral )
		neutral = eBtnState == normal;

	if( ( bCanNeutral || eBtnState != normal )&&( bShowBorder ) )
		return GuiBorder::onFrame( drawdst );
	else
		return GuiElement::onFrame( drawdst );
}

Uint32 GuiDynamicButton::onGLFrame()
{
	SDL_Rect clip = getClip();
	SDL_Rect dst = getDrawWindow();
	if( bCanNeutral && bShowBorder )
		inflateRect( clip, -nCornerWidth, -nCornerHeight );

	if( bmps )
	{
		//drawdst->setClip( clip );
		int bmp_w, bmp_h;
		bmps->getImage( 0/*(eBtnState != normal)/* + nFirstSurface*/ )->getSize( bmp_w, bmp_h );
		switch( eDynamic )
		{
//			case ds_dynamic:
//				break;
			case ds_none:
				((SdlTextureContainer*)bmps)->getTexture( (int)eBtnState )->drawSimple( dst );
				break;
			case ds_fast:
				((SdlTextureContainer*)bmps)->getTexture( eBtnState != normal )->drawSimple( dst );
				//((SdlTextureContainer*)bmps)->getTexture( eBtnState != normal )->drawClipped( clip, dst );
				break;
		}
	}

	if( text )
	{
		SDL_Rect dst = getDrawWindow();
//		dst.top += 10;
		inflateRect( dst, -10, -10 );
		switch( eBtnState )	{
			case normal:
				break;
			case pressed:
				offsetRect( dst, 2, 2 );
				break;
			case mouseover:
				offsetRect( dst, -2, -2 );
				break;
		}
		SdlFont *f;
		if( eBtnState == normal )
			f = fNormal;
		else
			f = fHover;
		if( f )
		{
			SDL_Rect clip = getClip();
			f->writeGL( text, dst, SdlFont::baseline, SdlFont::center, &clip );
		}
	}

	invert = eBtnState == pressed;
	if( bCanNeutral )
		neutral = eBtnState == normal;

	if( ( bCanNeutral || eBtnState != normal ) && ( bShowBorder ) )
		return GuiBorder::onGLFrame();
	else
		return GuiElement::onGLFrame();
}

Uint32 GuiDynamicButton::onButtonPress( Sint32 x, Sint32 y )
{
	if( down )
		down->play();
	return 0;
}

Uint32 GuiDynamicButton::onButtonRelease( bool bInClient )
{
	if( up && bInClient )
		up->play();
	return 0;
}

Uint32 GuiDynamicButton::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiBorder::onThemeChanged( t );

	if( t == 0 )
		return r;

	r |= changeTheme( t, getClassName() );
	return r;
}

Uint32 GuiDynamicButton::changeTheme( UiTheme *t, const char *classname )
{
	const char *name;
	if( bThemeBmps )	{
		name = t->getString( ( string(classname) + "->button-bmps" ).c_str() );
		if( name )	{
			bmps = SdlManager::the->getImagesByName( name );
			if( bmps )
				eDynamic = bmps->numImages() > 2? ds_none : ds_fast;
		}
	}
	else	{
		if( bmps == 0 )
			bmps = SdlManager::the->getImagesByName( "button" );
		if( bmps )
			if( bmps->numImages() > 2 )
				eDynamic = ds_none;
	}
	if( bThemeFonts )	{
		Uint32 s = t->getUint32( ( string(classname) + "->button-normal-font-size" ).c_str() );
		bool l = t->getBool( ( string(classname) + "->button-normal-font-light" ).c_str() );
		fNormal = GuiText::fontSizeToFont( (GuiText::FontSize)s, l, t );
		s = t->getUint32( ( string(classname) + "->button-hover-font-size" ).c_str() );
		l = t->getBool( ( string(classname) + "->button-hover-font-light" ).c_str() );
		fHover = GuiText::fontSizeToFont( (GuiText::FontSize)s, l, t );
	}
	else	{
		fNormal = SdlFont::get( GuiText::fs_button, false );
		fHover = SdlFont::get( GuiText::fs_button, true );
	}
	if( bThemeAudios && !up && !down )	{
		name = t->getString( ( string(classname) + "->button-sound-up" ).c_str() );
		if( name )
			up = SdlManager::the->getAudioByName( name );
		name = t->getString( ( string(classname) + "->button-sound-down" ).c_str() );
		if( name )
			down = SdlManager::the->getAudioByName( name );
	}
//	nFirstSurface = t->..."FirstSurface";

	return 0;
}

void GuiDynamicButton::setBorderVisible( bool show )
{
	bShowBorder = show;
/*	if( show )	{
		bShowBorder = true;
		if( dynamic == ds_none )
			dynamic = ds_dynamic;
	}
	else	{
		bShowBorder = false;
		dynamic = ds_none;
	}*/
}

////////////////////////////////////////////////////////////////////////////////////
// GuiClickableBtn

GuiClickableBtn::GuiClickableBtn( const SDL_Rect &r, GuiElement *p, const char *name,const char *t, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
: GuiDynamicButton( r,p,name,t,b,u,d )
{
	cbClicked.func = 0;
	GuiClickableBtn::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiClickableBtn::GuiClickableBtn( TiXmlElement *el, GuiElement *p )
: GuiDynamicButton( el, p )
{
	cbClicked.func = 0;
}

GuiClickableBtn::~GuiClickableBtn()
{
}

void GuiClickableBtn::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("OnClick");
		if(str)
			cbClicked = getFunctionByName(str);
	}
	GuiDynamicButton::onInit();
}


Uint32 GuiClickableBtn::onMouseUp( SDL_MouseButtonEvent *ev )
{
	if( ( ev->button == 1 ) && ( GuiManager::the->getFocus() == this ) )	{
		SDL_Rect window = getAbsWindow();
		if( pointInRect( window, ev->x, ev->y ) && isEnabled() )	{
			setBtnState(mouseover);
			onButtonRelease( true );
			if( cbClicked.func )	{
				(*cbClicked.func)( this, cbClicked.obj, NULL );
			}
		}
		else	{
			setBtnState(normal);
			onButtonRelease( false );
		}

		GuiManager::the->release( this );
	}

	return 0;
}

Uint32 GuiClickableBtn::onMouseOver()
{
	if( !isEnabled() ) return 0;
	if( GuiManager::the->getFocus() != this )
		setBtnState(mouseover);
	else
		setBtnState(pressed);
	return 0;
}

Uint32 GuiClickableBtn::onMouseOut()
{
	setBtnState(normal);
	return 0;
}

Uint32 GuiClickableBtn::onMouseDown( SDL_MouseButtonEvent *ev )
{
	if( ( ev->button == 1 ) && isEnabled() )	{
		GuiManager::the->focus( this );
		setBtnState(pressed);
		onButtonPress( ev->x, ev->y );
	}

	return 0;
}

Uint32 GuiClickableBtn::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiDynamicButton::onThemeChanged( t );
	if( t == 0 )
		return r;

	return r;
}

/// \todo if enabled then setBtnState(correct state), currently stays normal even if mouse is over element
void GuiClickableBtn::onEnabledChanged( bool enabled )
{
	if( !enabled )
		setBtnState(normal);
	
	GuiElement::onEnabledChanged( enabled );
}

////////////////////////////////////////////////////////////////////////////////////
// GuiCheckablebutton

GuiCheckableBtn::GuiCheckableBtn( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
: GuiDynamicButton( r,p,name,t,b,u,d )
{
	eDynamic = ds_none;
	checked = false;
	cbChecked.func = NULL;
	cbChecked.obj = NULL;

	GuiCheckableBtn::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiCheckableBtn::GuiCheckableBtn( TiXmlElement *el, GuiElement *p ) : GuiDynamicButton( el,p )
{
	eDynamic = ds_none;
	checked = false;
	cbChecked.func = NULL;
}

GuiCheckableBtn::~GuiCheckableBtn()
{
}

void GuiCheckableBtn::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("OnCheck");
		if(str)	{
			cbChecked = getFunctionByName(str);
		}
	}
	GuiDynamicButton::onInit();
}

void GuiCheckableBtn::setCheck( bool b )
{
	if( GuiManager::the->getFocus() == this )	{
		GuiManager::the->release( this );
	}
	setBtnState( b ? pressed : normal );
	if( b != checked )	{
		checked = b;
		onCheck( checked );
	}
}

Uint32 GuiCheckableBtn::onMouseUp( SDL_MouseButtonEvent *ev )
{
	if( ( ev->button == 1 ) && ( GuiManager::the->getFocus() == this ) )	{
		SDL_Rect window = getAbsWindow();
		bool inrect = false;
		if( pointInRect( window, ev->x, ev->y ) && isEnabled() )	{
			checked = !checked;
			onCheck( checked );
			inrect = true;
		}
		
		setBtnState( checked ? pressed : normal );
		
		onButtonRelease( inrect );

		GuiManager::the->release( this );
	}

	return 0;
}

Uint32 GuiCheckableBtn::onMouseDown( SDL_MouseButtonEvent *ev )
{
	if( ( ev->button == 1 ) && isEnabled() )	{
		GuiManager::the->focus( this );
		setBtnState(mouseover);
		onButtonPress( ev->x, ev->y );
	}

	return 0;
}

Uint32 GuiCheckableBtn::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiDynamicButton::onThemeChanged( t );
	if( t == 0 )
		return r;

	return r;
}

void GuiCheckableBtn::onCheck( bool checked )
{
	if( cbChecked.func )	{
		(*cbChecked.func)( this, cbChecked.obj, &checked );
	}
}

////////////////////////////////////////////////////////////////////////////////////
// GuiRadioBtn

GuiRadioBtn::GuiRadioBtn( const SDL_Rect &r, GuiRadioContainer *p, const char *name, const char *t, SdlImageContainer *b, SdlAudio *u,SdlAudio *d)
: GuiCheckableBtn( r,p,name,t,b,u,d )
{
	iter = ((GuiRadioContainer*)getParent())->addButton( this );

	GuiRadioBtn::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiRadioBtn::GuiRadioBtn( TiXmlElement *el, GuiRadioContainer *p): GuiCheckableBtn( el, p )
{
	iter = ((GuiRadioContainer*)getParent())->addButton( this );
}

GuiRadioBtn::~GuiRadioBtn()
{
	((GuiRadioContainer*)getParent())->removeButton( iter );
}

void GuiRadioBtn::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiCheckableBtn::onInit();
}

void GuiRadioBtn::onCheck( bool checked )
{
	( (GuiRadioContainer*)getParent() )->onCheck( iter );
	//GuiCheckableBtn::onCheck( checked ); no callback allowed in radiobtn
}

Uint32 GuiRadioBtn::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiCheckableBtn::onThemeChanged( t );
	if( t == 0 )
		return r;

	return r;
}

////////////////////////////////////////////////////////////////////////////////////
// guimoveablebutton

GuiMoveableBtn::GuiMoveableBtn( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
: GuiClickableBtn( r,p,name,t,b,u,d )
{
	cbPoschanged.func = 0;
	cbPoschanged.obj = 0;
	tracking = false;
	dir = horizontal;
	getRelCenterPoint( xStart, yStart );
	xEnd = xStart;
	yEnd = yStart;

	GuiMoveableBtn::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiMoveableBtn::GuiMoveableBtn( TiXmlElement *el, GuiElement *p): GuiClickableBtn( el, p )
{
	cbPoschanged.func = 0;
	tracking = false;
	dir = horizontal;
	getRelCenterPoint( xStart, yStart );
	xEnd = xStart;
	yEnd = yStart;
}

GuiMoveableBtn::~GuiMoveableBtn()
{

}

void GuiMoveableBtn::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("OnPosChange");
		if(str)
			cbPoschanged = getFunctionByName(str);
	}
	GuiClickableBtn::onInit();
}

Uint32 GuiMoveableBtn::onResize()
{
	return 0;
}

Uint32 GuiMoveableBtn::onButtonPress( Sint32 x, Sint32 y )
{
	tracking = true;
	SDL_Rect r = getRelWindow();
	xTrackingPoint = x - r.x;
	yTrackingPoint = y - r.y;
	return GuiClickableBtn::onButtonPress( x, y );
}

Uint32 GuiMoveableBtn::onButtonRelease( bool bInClient )
{
	tracking = false;
	return GuiClickableBtn::onButtonRelease( bInClient );
}

Uint32 GuiMoveableBtn::onMouseMove( Uint32 x, Uint32 y )
{
	long newposx, newposy;

	newposx = x - xTrackingPoint;
	newposy = y - yTrackingPoint;

	if( tracking )	{
		unsigned int sx, sy;
		getWndSize( sx, sy );
		long xTrackingcenter, yTrackingcenter;
		xTrackingcenter = sx / 2 + newposx;
		yTrackingcenter = sy / 2 + newposy;
		if( dir == vertikal )	{
			xTrackingcenter = xStart;
			if( yTrackingcenter > yEnd )
				yTrackingcenter = yEnd;
			if( yTrackingcenter < yStart )
				yTrackingcenter = yStart;
		}
		else	{
			yTrackingcenter = yStart;
			if( xTrackingcenter > xEnd )
				xTrackingcenter = xEnd;
			if( xTrackingcenter < xStart )
				xTrackingcenter = xStart;
		}
		moveRelCenterTo( xTrackingcenter, yTrackingcenter );
		if( cbPoschanged.func )
			(*cbPoschanged.func)( this, cbPoschanged.obj, NULL );
	}
	return 0;
}

Uint32 GuiMoveableBtn::onMouseUp( SDL_MouseButtonEvent *ev )
{
	if( ( ev->button == 1 ) && ( GuiManager::the->getFocus() == this ) )	{
		SDL_Rect window = getAbsWindow();
		if( pointInRect( window, ev->x, ev->y ) && isEnabled() )	{
			setBtnState(mouseover);
		}
		else
			setBtnState(normal);

		if( cbClicked.func )
			(*cbClicked.func)( this, cbClicked.obj, NULL );

		onButtonRelease( getBtnState() == mouseover );

		GuiManager::the->release( this );
	}

	return 0;
}

Uint32 GuiMoveableBtn::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiClickableBtn::onThemeChanged( t );
	if( t == 0 )
		return r;

	return r;
}

void GuiMoveableBtn::setPosToRange()
{
	getRelCenterPoint( xEnd, yEnd );
	xStart = xEnd;
	yStart = yEnd;
}

bool GuiMoveableBtn::setRange( Sint32 sx, Sint32 sy, Sint32 ex, Sint32 ey )
{
	if( dir == vertikal )	{
		if( sx != ex )
			return false;
		xStart = sx;
		yStart = sy;
		xEnd = ex;
		yEnd = ey;
		if( yEnd < yStart )
			yEnd = yStart;
	}
	else	{
		if( sy != ey )
			return false;
		xEnd = ex;
		yEnd = ey;
		xStart = sx;
		yStart = sy;
		if( xEnd < xStart )
			xEnd = xStart;
	}
	return true;
}

void GuiMoveableBtn::setDirection( direction d )
{
	dir = d;
	setPosToRange();
}
