/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guielement.h"
#include "guimanager.h"
#include "rectfunctions.h"
#include "basicapp.h"
#include "functions.h"
#include "tinyxml.h"
// for GuiElement::create():
#include "guibackground.h"
#include "guicombination.h"
#include "guidropdown.h"
#include "guidynamicbutton.h"
#include "guifilelist.h"
#include "guiform.h"
#include "guihint.h"
#include "guilistbox.h"
#include "guilist.h"
#include "guimessagebox.h"
#include "guiprogressbar.h"
#include "guirotator.h"
#include "guiscroller.h"
#include "guitextfield.h"
#include "guitext.h"
#include "guiupdown.h"

DECLARE_SLOT(new,GuiElement,GuiElement::newelement);
DECLARE_SLOT(ToggleVisibility,GuiElement,GuiElement::togglevisibility);

GuiElement::GuiElement( const SDL_Rect &r, GuiElement *p, const char *name )
	: UiObject( p? (UiObject*)p : GuiManager::the, name )
{
	xmlData = NULL;
	wnd = r;
	geParent = p;
	bVisible = true;
	bRedraw = true;
	bEnabled = true;
	bClipdirty = bAbsdirty = bDrawdirty = true;
}

GuiElement::GuiElement( TiXmlElement *el, GuiElement *p )
	: UiObject( el, p? (BasicObject*)p : (BasicObject*)GuiManager::the )
{
	xmlData = el;
	const char *str = el->Attribute("Rect");
	if(!str)	{
		BasicApp::the->logLine("ERROR: GuiElement(XML): no rect attribute specified.");
		setRectEmpty(wnd);
	}
	else
		wnd = strToRect(str);
	
	bRedraw = true;
	bClipdirty = bAbsdirty = bDrawdirty = true;
	geParent = p;
}

GuiElement::~GuiElement()
{
/* now done in onDestroy():
	if( GuiManager::the )	{
		GuiManager::the->release( this );
		GuiManager::the->cleanupElement( this );
	}
*/
}

void GuiElement::onInit()
{
	if(xmlData)	{
		const char *str = xmlData->Attribute("Rect");
		if(str)
			setRelWindow( strToRect(str) );
		
		str = xmlData->Attribute("Visible");
		bVisible = str == NULL || strToBool(str);
		
		str = xmlData->Attribute("Enabled");
		bEnabled = str == NULL || strToBool(str);
		
		bRedraw = true;
	}
	UiObject::onInit();
}

void GuiElement::onDestroy()
{
	if( GuiManager::the )	{
		GuiManager::the->release( this );
		GuiManager::the->cleanupElement( this );
	}
	UiObject::onDestroy();
}

void GuiElement::resizeTo( Uint32 w, Uint32 h )
{
	wnd.w = w;
	wnd.h = h;
	setRedraw();
	setWndDirty();
}

void GuiElement::resizeBy( Sint32 dw, Sint32 dh )
{
	wnd.w += dw;
	wnd.h += dh;
	setRedraw();
	setWndDirty();
}

void GuiElement::moveTo( Sint32 x, Sint32 y )
{
	wnd.x = x;
	wnd.y = y;
	setWndDirty();
}

void GuiElement::moveRelCenterTo( Sint32 x, Sint32 y )
{
	Sint32 oldx, oldy;
	getRelCenterPoint( oldx, oldy );
	wnd.x += x - oldx;
	wnd.y += y - oldy;
	setWndDirty();
}

void GuiElement::moveBy( Sint32 x, Sint32 y )
{
	wnd.x += x;
	wnd.y += y;
	setWndDirty();
}

void GuiElement::getWndSize( Uint32 &w, Uint32 &h ) const
{
	w = wnd.w; 
	h = wnd.h;
}

SDL_Rect GuiElement::getRelWindow() const
{
	return wnd;
}

void GuiElement::setRelWindow(const SDL_Rect &r)
{
	wnd = r;
	setRedraw();
	setWndDirty();
}

SDL_Rect GuiElement::getAbsWindow()
{
	if( bAbsdirty )	{
		abswnd = wnd;
		if( geParent )	{
			SDL_Rect par = geParent->getAbsWindow();
			abswnd.x += par.x;
			abswnd.y += par.y;
		}
		bAbsdirty = false;
	}
	return abswnd;
}

void GuiElement::setAbsWindow(const SDL_Rect &r)
{
	SDL_Rect abs = getAbsWindow();
	wnd.x += r.x - abs.x;
	wnd.y += r.y - abs.y;
	setRedraw();
	setWndDirty();
}

SDL_Rect GuiElement::getClientWindow() const
{
	SDL_Rect rect = wnd;
	rect.x = 0;
	rect.y = 0;
	return rect;
}

SDL_Rect GuiElement::getClip()
{
	if( bClipdirty )	{
		if(geParent)	{
			if(geParent->hasSurface())	{
				clip = geParent->getClientWindow();
				intersectRect( clip, getRelWindow() );
			}
			else	{
				clip = getDrawWindow();
				intersectRect( clip, geParent->getClip() );
			}
		}
		else	{
			clip = getDrawWindow();
			intersectRect( clip, SdlManager::the->getScreenRect() );
		}
		bClipdirty = false;
	}
	return clip;
}

SDL_Rect GuiElement::getDrawWindow()
{
	if(bDrawdirty)	{
		if(geParent)	{
			if(geParent->hasSurface())
				drawwnd = getRelWindow();
			else	{
				drawwnd = getRelWindow();
				SDL_Rect r = geParent->getDrawWindow();
				offsetRect(drawwnd, r.x, r.y);
			}
		}
		else	{
			drawwnd = getAbsWindow();
		}
		bDrawdirty = false;
	}
	return drawwnd;
}

void GuiElement::getRelCenterPoint( Sint32 &x, Sint32 &y )
{
	SDL_Rect r = getRelWindow();
	x = r.x + r.w / 2;
	y = r.y + r.h / 2;
}

void GuiElement::getAbsCenterPoint( Sint32 &x, Sint32 &y )
{
	SDL_Rect r = getAbsWindow();
	x = r.x + r.w / 2;
	y = r.y + r.h / 2;
}

void GuiElement::getClientCenterPoint( Sint32 &x, Sint32 &y )
{
	x = wnd.w / 2;
	y = wnd.h / 2;
}

GuiElement *GuiElement::getParent() const
{
	return geParent;
}

GuiElement *GuiElement::getToplevel()
{
	if(geParent)	{
		GuiElement *toplevel = geParent->getToplevel();
		return toplevel;
	}
	return this;
}

void GuiElement::setVisible(bool b)
{
	bVisible = b;
	setRedraw();
}

bool GuiElement::isVisible() const
{
	return bVisible;
}

void GuiElement::setRedraw()
{
	bRedraw = true;
	if( geParent )
		geParent->setRedraw();
}

void GuiElement::setEnabled(bool b)
{
	if( bEnabled == b )
		return;
	bool old = isEnabled();
	bEnabled = b;
	if( isEnabled() != old )	{
		onEnabledChanged( !old );
	}
}

bool GuiElement::isEnabled() const
{
	if( geParent )
		return bEnabled && geParent->isEnabled();
	else
		return bEnabled;
}

// event-handling

Uint32 GuiElement::onFrame(SdlSurface *dest)
{
	if( bRedraw || !hasSurface() )	{
		// must be done before calling the children's onFrame()!
		bRedraw = false;
		
		for( child_reverse_iter i = children.rbegin(); i != children.rend(); ++i )	{
			Uint32 r;
			if( ((GuiElement*)(*i))->bVisible )	{
				if( ( r = ((GuiElement*)(*i))->onFrame(dest) ) != 0 )
					return r;
			}
		}
	}
	return 0;
}

Uint32 GuiElement::onGLFrame()
{
	if( bRedraw || !hasSurface() )	{
		bRedraw = false;
		
		for( child_reverse_iter i = children.rbegin(); i != children.rend(); ++i )	{
			Uint32 r;
			if( ((GuiElement*)(*i))->bVisible )	{
				if( ( r = ((GuiElement*)(*i))->onGLFrame() ) != 0 )
					return r;
			}
		}
	}
	return 0;
}

Uint32 GuiElement::onMouseOver()
{
	return 0;
}

Uint32 GuiElement::onMouseOut()
{
	return 0;
}

Uint32 GuiElement::onThemeChanged( UiTheme *t )
{
	bRedraw = true;
	return UiObject::onThemeChanged(t);
}

void GuiElement::onEnabledChanged( bool enabled )
{
	if( !enabled )
		GuiManager::the->release( this );

	for( child_iter i = children.begin(); i != children.end(); ++i )
		if( ((GuiElement*)(*i))->bEnabled )
			((GuiElement*)(*i))->onEnabledChanged( enabled );
}

GuiElement *GuiElement::elementFromPos( Sint32 x, Sint32 y )
{
	if( !pointInRect( wnd, x, y ) )
		return 0;

	x -= wnd.x;
	y -= wnd.y;

	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		GuiElement *g;
		if( ((GuiElement*)(*i))->bVisible )	{
			if( ( g = ((GuiElement *)(*i))->elementFromPos( x, y ) ) != NULL )
				return g;
		}
	}

	return this;
}

void GuiElement::setDrawRectDirty()
{
	bDrawdirty = true;
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		((GuiElement *)(*i))->setDrawRectDirty();
	}
}

void GuiElement::setWndDirty()
{
	if( geParent )
		geParent->setRedraw();
	bAbsdirty = bClipdirty = bDrawdirty = true;
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		((GuiElement *)(*i))->setWndDirty();
	}
	onResize();
}

GuiElement *GuiElement::create(TiXmlElement *el, GuiElement *parent)
{
	parent = createWithoutChildren(el, parent);
	if(parent)	{
		for(TiXmlElement *child = el->FirstChildElement(); child; child = child->NextSiblingElement())
			create(child, parent);
	}
	return parent;
}

GuiElement *GuiElement::createWithoutChildren(TiXmlElement *el, GuiElement *parent)
{
	if(!el->Attribute("Name"))
		return NULL;
	const char *str = el->Attribute("Parent");
	if(str)	{
		BasicObject *o = BasicObject::getObject(str);
		if(parent != (GuiElement *)GuiManager::the)	{
			parent = (GuiElement *)o;
			if(!parent)	{
				BasicApp::the->logLine("ERROR: GuiElement::create(): cannot find object: ");
				BasicApp::the->logString(str);
				return NULL;
			}
			if(!parent->isUiObject() || !parent->isGuiElement())	{
				BasicApp::the->logLine("ERROR: GuiElement::create(): parent is not derived from GuiElement: ");
				BasicApp::the->logString(str);
				return NULL;
			}
		}
		else parent = NULL;
	}
	Callback func = BasicSlot::getFunction("new", el->Value());
	if(!func)	{
		BasicApp::the->logLine("ERROR: GuiElement::create(): no function to create object: ");
		BasicApp::the->logString(el->Value());
		return NULL;
	}
	GuiElement *r;
	(*func)(&r, parent, el);
	return r;
}
