/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guifader.h"
#include "guimanager.h"
#include "sdlmanager.h"
#include "basicapp.h"
#include "definitions.h"
#include "functions.h"

GuiFader *GuiFader::the = 0;
GuiFader *GuiFader::getThe()
{
	return the;
}

GuiFader::GuiFader(BasicObject *p, const char *name, Uint32 normal ): UiObject( p? (UiObject*)p : GuiManager::the, name )
{
	bFading = false;
	nNormal = normal;
	nNow = normal;
	pFunction = 0;
	setGamma( nNow );
	the = this;
}

GuiFader::GuiFader(TiXmlElement *el, BasicObject *p ): UiObject( p? (UiObject*)p : GuiManager::the, el->Attribute("Name") )
{
	bFading = false;
	unsigned long n;
	if(el->QueryULongAttribute("Normal", &n) != TIXML_SUCCESS)
		nNormal = 0xFFFF;
	else nNormal = n;
	nNow = nNormal;
	pFunction = 0;
	setGamma( nNow );
	the = this;
}

GuiFader::~GuiFader()
{
	setToNormal();
	the = 0;
}


Uint32 GuiFader::onFrame()
{
	if(!bFading)
		return UiObject::onFrame();
	printf("GuiFader::onFrame()\n");
	if( pFunction )	{
		Uint32 r, b, g;
		r = b = g = nNormal;
		if( !(*pFunction)( SDL_GetTicks() - tDuration, &r, &g, &b ) )
		{
			pFunction = 0;
			bFading = false;
			setGamma( r, g, b );
		}
		else
			setGamma( r, g, b );
		return UiObject::onFrame();
	}

	if( tEnd <= SDL_GetTicks() )	{
		nNow = nEnd;
		bFading = false;
	}
	else
		if( nEnd > nStart)
			nNow = nEnd - (nEnd - nStart) * ( tEnd - SDL_GetTicks() ) / tDuration;
		else
			nNow = nEnd + (nStart - nEnd) * ( tEnd - SDL_GetTicks() ) / tDuration;

	setGamma( nNow );

	return UiObject::onFrame();
}

void GuiFader::fadeOut( Uint32 ms )
{
	fade( ms, 0 );
}

void GuiFader::fadeIn( Uint32 ms )
{
	fade( ms, nNormal );
}

void GuiFader::fade( Uint32 ms, Uint32 end )
{
	nStart = nNow;
	tDuration = ms;
	tEnd = SDL_GetTicks() + ms;
	nEnd = end;
	bFading = true;
}

void GuiFader::fadeByFunction( FadeFunction func )
{
	bFading = true;
	pFunction = func;
	tDuration = SDL_GetTicks();
}

bool GuiFader::setGamma( Uint32 intensity )
{
	if( intensity & 0x80000000 )
		return setGammaInverted( intensity & 0x7FFFFFFF );

	int i;
	if( intensity <= MAX_UINT16 )	{
		for( i = 0; i < 256; i++ )	{
			r[i] = intensity * i / 256;
			g[i] = intensity * i / 256;
			b[i] = intensity * i / 256;
		}
	}
	else	{
		Uint16 intens = intensity & 0xFFFF,
			delta = MAX_UINT16 - intens;
		for( i = 0; i < 256; i++ )	{
			r[i] = intens + delta * i / 256;
			g[i] = intens + delta * i / 256;
			b[i] = intens + delta * i / 256;
		}
	}
	nNow = intensity;

	return SDL_SetGammaRamp( r, g, b ) == 0;
}

bool GuiFader::setGamma( Uint32 red, Uint32 green, Uint32 blue )
{
	int i;
	if( red <= MAX_UINT16 )
		for( i = 0; i < 256; i++ )
			r[i] = red * i / 256;
	else	{
		Uint16 intens = red & 0xFFFF,
			delta = MAX_UINT16 - intens;
		for( i = 0; i < 256; i++ )
			r[i] = intens + delta * i / 256;
	}
	if( green <= MAX_UINT16 )
		for( i = 0; i < 256; i++ )
			g[i] = green * i / 256;
	else	{
		Uint16 intens = green & 0xFFFF,
			delta = MAX_UINT16 - intens;
		for( i = 0; i < 256; i++ )
			g[i] = intens + delta * i / 256;
	}
	if( blue <= MAX_UINT16 )
		for( i = 0; i < 256; i++ )
			b[i] = blue * i / 256;
	else	{
		Uint16 intens = blue & 0xFFFF,
			delta = MAX_UINT16 - intens;
		for( i = 0; i < 256; i++ )
			b[i] = intens + delta * i / 256;
	}

	return SDL_SetGammaRamp( r, g, b ) == 0;
}

bool GuiFader::setGammaInverted( Uint32 intensity )
{
	int i;
	if( intensity == GAMMA_WHITE )
		for( i = 0; i < 256; i++ )	{
			r[i] = MAX_UINT16;
			g[i] = MAX_UINT16;
			b[i] = MAX_UINT16;
		}
	else
		for( i = 0; i < 256; i++ )	{
			r[255-i] = MIN( MAX_UINT16, intensity * i );
			g[255-i] = MIN( MAX_UINT16, intensity * i );
			b[255-i] = MIN( MAX_UINT16, intensity * i );
		}

	return SDL_SetGammaRamp( r, g, b ) == 0;
}

