/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/
// last review: 26.03.2007

#ifdef __windows__
#include <direct.h>
#else
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <fcntl.h>
#endif

#include "guifilelist.h"
#include "functions.h"
#include "basicapp.h"

DECLARE_SLOT(new, GuiFileList, GuiFileList::newguiobject)

GuiFileList::GuiFileList( const SDL_Rect &r, GuiElement * el, const char *name ): GuiList( r, el, name )
{
	showtypes = MAX_UINT32;
	hideext = false;
	seekstr = new char[MAX_PATH];
	strcpy( seekstr, "*" );
}

GuiFileList::GuiFileList( TiXmlElement *el, GuiElement *p ): GuiList( el, p )
{
	showtypes = MAX_UINT32;
	hideext = false;
	seekstr = new char[MAX_PATH];
	strcpy( seekstr, "*" );
}

GuiFileList::~GuiFileList()
{
	attributes.clear();
	delete[] seekstr;
}

void GuiFileList::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("Path");
		if( str )
			strPath = BasicApp::the->makeUserPath( str );
		str = el->Attribute("Pattern");
		if( str )
			strcpy( seekstr, str );
		bool files, dirs, others;
		str = el->Attribute("ShowFiles");
		files = str == NULL || strToBool(str);
		str = el->Attribute("ShowDirectories");
		dirs = str != NULL && strToBool(str);
		str = el->Attribute("ShowOthers");
		others = str != NULL && strToBool(str);
#ifdef __windows__
		showtypes = (files ? 0x27:0) | (dirs ? 0x10:0) | (others ? 8:0);
#else
		showtypes = (files ? DT_REG:0) | (dirs ? DT_DIR:0) | (others ? ~(DT_DIR|DT_REG):0);
#endif
		str = el->Attribute("HideExtensions");
		hideext = str == NULL || strToBool(str);
		
		refresh();
	}
	GuiList::onInit();
}

void GuiFileList::setPath( const char * path )
{
	strPath = BasicApp::the->makeUserPath( path );
	refresh();
}

void GuiFileList::setSeekstr( const char * str )
{
	strcpy( seekstr, str );
	refresh();
}

void GuiFileList::setFiletypes( bool files, bool dirs, bool others )
{
#ifdef __windows__
	showtypes = files ? 0x27:0 | dirs ? 0x10:0 | others ? 8:0;
#else
	showtypes = files ? DT_REG:0 | dirs ? DT_DIR:0 | others ? ~(DT_DIR|DT_REG):0;
#endif
	refresh();
}

void GuiFileList::hideExtension( bool hide )
{
	hideext = hide;
	refresh();
}


void GuiFileList::refresh()
{
	clearAll();
	attributes.clear();
	
	if( strPath.empty() )
		 return;
	
#ifdef __windows__
	// activate search directory
	if( chdir( strPath.c_str() ) == -1 )	{
		BasicApp::the->logLine( "GuiFileList::refresh(): Invalid directory:" );
		BasicApp::the->logString( path );
		return;
	}
	_finddata_t attr;
	long hseek = _findfirst( seekstr, &attr );
	if( hseek == -1L )
	{
		BasicApp::the->logLine( "GuiFileList::refresh(): Invalid search parameters!" );
		return;
	}
#else
	struct dirent *pDir;
	DIR *hseek = opendir( strPath.c_str() );
	if( hseek == NULL )
	{
		BasicApp::the->logLine( "GuiFileList::refresh(): Invalid directory!" );
		return;
	}
#endif
	FIND_DATA data;
#ifdef __windows__
#define pDIR_FILENAME	attr.name
#define nDIR_FILETYPE	attr.attrib
	do
#else
#define pDIR_FILENAME	pDir->d_name
#define nDIR_FILETYPE	pDir->d_type
	while( ( pDir = readdir( hseek ) ) != NULL )
#endif
	{
		if( nDIR_FILETYPE & showtypes )
#ifndef __windows__
					if( patMatch( seekstr, pDIR_FILENAME ) )
#endif
		{
			if( strcmp( pDIR_FILENAME, ".." ) == 0 || strcmp( pDIR_FILENAME, "." ) == 0 )
				continue;
			if( hideext )	{
				char *fn = new char[strlen(pDIR_FILENAME) + 1];
				strcpy( fn, pDIR_FILENAME );
				if( strrchr( fn, '.' ) )	{
					int n = strrchr( fn, '.' ) - fn;
					fn[n] = 0;
				}
				insertStr( fn );
				delete[] fn;
			}
			else
				insertStr( pDIR_FILENAME );
			strcpy( data.chFileName, pDIR_FILENAME );
			data.ulType = nDIR_FILETYPE;
			attributes.push_back( data );
		}
	}
#ifdef __windows__
	while( _findnext( hseek, &attr ) == 0 );
	_findclose( hseek );
#else
	closedir( hseek );
#endif

	setRedraw();
}

char * GuiFileList::getFilename( unsigned int i ) const
{
	if( strPath.empty() || ( i >= attributes.size() ) )
		 return 0;
	
	char *fn = new char[strPath.length() + strlen( getAttribute( i ).chFileName ) + 2];
	strcpy( fn, strPath.c_str() );
	strcat( fn, "/" );
	strcat( fn, attributes.at( i ).chFileName );
	return fn;
}

char * GuiFileList::getSelFilename() const
{
	return getFilename( getSelId() );
}

GuiFileList::FIND_DATA GuiFileList::getAttribute( unsigned int i ) const
{
	return attributes.at( i );
}

GuiFileList::FIND_DATA GuiFileList::getSelAttr() const
{
	return attributes.at( getSelId() );
}

/*
ulFileAttributes-meanings (Win):
Hex-Nr.	|	meaning
   01	| write protected
   02	| hidden
   04	| system file
   08	| disc-ID
   10	| folder
   20	| archiv file
(  3F	| all	)

File types (Linux)
DT_UNKNOWN      0
DT_FIFO         1
DT_CHR          2
DT_DIR          4
DT_BLK          6
DT_REG          8
DT_LNK          10
DT_SOCK         12
DT_WHT          14

/**/
