/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2006-2007 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/
// last review: 29.03.2007

#include "guiflowmenu.h"
#include "basicapp.h"
#include "functions.h"
#include <stdlib.h>

DECLARE_SLOT(new, GuiFlowMenu, GuiFlowMenu::newguiobject)
DECLARE_SLOT(back, GuiFlowMenu, GuiFlowMenu::flyout_back)
DECLARE_SLOT(next, GuiFlowMenu, GuiFlowMenu::flyout_next)
DECLARE_SLOT(disappear, GuiFlowMenu, GuiFlowMenu::flyout_disappear)
DECLARE_SLOT(flyin, GuiFlowMenu, GuiFlowMenu::flyin)

DECLARE_SLOT(new, GuiFlowMenuItem, GuiFlowMenuItem::newguiobject)

const long GuiFlowMenu::ID_INVALID = 0x80000000;

GuiFlowMenu::GuiFlowMenu(TiXmlElement *el, GuiElement *p): GuiFlowWindow(el, p)
{
	nID = 0;
	pCurr = NULL;
	strXmlFilename = NULL;
}

GuiFlowMenu::~GuiFlowMenu()
{
	mapMenues.clear();
	if( strXmlFilename )
		delete[] strXmlFilename;
}

bool GuiFlowMenu::registerItem( long id, GuiFlowMenuItem *el )
{
	return mapMenues.insert( pair<long,GuiFlowMenuItem*>(id,el) ).second;
}

void GuiFlowMenu::unregisterItem( long id )
{
	if( !mapMenues.empty() )
		mapMenues.erase(id);
}

void GuiFlowMenu::onInit()
{
	TiXmlElement *xmlData = getXmlData();
	if( xmlData )	{
		const char *str;
		str = xmlData->Attribute("XmlFile");
		if( (strXmlFilename == NULL) != (str == NULL) ||
				(str != NULL && strcmp(strXmlFilename, str) != 0) )
		{
			while( !mapMenues.empty() )
				delete (*mapMenues.begin()).second;
			
			if( str )	{
				if(xmlFile.LoadFile(BasicApp::the->makeDataPath(str).c_str()))
				{
					if(strcmp(xmlFile.RootElement()->Value(), "GuiFlowMenu") != 0)
						// log a warning, but ignore the misconfiguration
						BasicApp::the->logLevelLine("WARNING: GuiFlowMenu is not the root element of an xml-file for GuiFlowMenu.");
					
					createItemsByFile();
				}
				else	{
					BasicApp::the->logLevelLine("WARNING: GuiFlowMenu::onInit(): cannot read xml-file.");
				}
			}
			if( strXmlFilename )
				delete[] strXmlFilename;
			strXmlFilename = new char[ strlen( str ) + 1 ];
			strcpy( strXmlFilename, str );
		}
		
		str = xmlData->Attribute("StartMenuID");
		if(str)
			nID = strtol(str, NULL, 10);
		str = xmlData->Attribute("ShowRect");
		if(str)
			rShow = strToRect(str);
		str = xmlData->Attribute("DisappearRect");
		if(str)
			rDisappear = strToRect(str);
		else rDisappear = getRelWindow();
		// signals:
		str = xmlData->Attribute("OnChooseNext");
		if(str)
			cbChooseNext = getFunctionByName(str);
		str = xmlData->Attribute("OnFlewIn");
		if(str)
			cbFlewIn = getFunctionByName(str);
		str = xmlData->Attribute("OnNoNext");
		if(str)
			cbNoNext = getFunctionByName(str);
		str = xmlData->Attribute("OnDisappeared");
		if(str)
			cbDisappear = getFunctionByName(str);
		str = xmlData->Attribute("OnBack");
		if(str)
			cbBack = getFunctionByName(str);
	}
	GuiFlowWindow::onInit();
	
	showItem( nID, false );
}

void GuiFlowMenu::createItemsByFile()
{
	BasicApp::the->logLevelLine("GuiFlowMenu: creating menu items");
	for(TiXmlElement *el = xmlFile.RootElement()->FirstChildElement(); el; el = el->NextSiblingElement())
	{
		if(strcmp(el->Value(), "GuiFlowMenuItem") == 0)
		{
			GuiElement *gel = GuiElement::create(el, this);
			if( !gel )	{
				BasicApp::the->logLevelLine("WARNING: GuiFlowMenu: menu element could not be created");
				continue;
			}
		}
		else
			BasicApp::the->logLevelLine("WARNING: GuiFlowMenu: Element's value is not 'GuiFlowMenuItem'.");
	}
}

bool GuiFlowMenu::showItem(long id, bool savehistory)
{
	GuiFlowMenuItem *p = mapMenues[id];
	if( p )	{
		if( pCurr )	{
			pCurr->setVisible(false);
			if(savehistory)	{
				lLastItemIDs.push_back(nID);
			}
		}
		nID = id;
		pCurr = p;
		pCurr->setVisible(true);
		return true;
	}
	else	{
		BasicApp::the->logLevelLine((string("WARNING: Item ID not found in this flow-menu: ") + intToStr(id, 10)).c_str());
		return false;
	}
}

Uint32 GuiFlowMenu::onPosReached()
{
	switch(state)
	{
		case sShow:
			setEnabled( true );
			if( cbFlewIn.func )
				cbFlewIn.func( this, cbFlewIn.obj, NULL );
			break;
			
		case sBack:
			if( lLastItemIDs.empty() )	{
				showItem( 0, false );
			}
			else	{
				showItem( lLastItemIDs.back(), false );
				lLastItemIDs.pop_back();
				setRelWindow( pCurr->getRectBack() );
				if( cbBack.func )
					cbBack.func( this, cbBack.obj, NULL );
			}
			eFlyStyle = sinus;
			flyTo( rShow, 0 );
			break;
			
		case sNext:
		{
			long n = nID;
			if( cbChooseNext.func )
				cbChooseNext.func(this, cbChooseNext.obj, &n);
			if( n == nID || n == ID_INVALID )
			{// if unchanged in the callback, take the default
				if( nIDnext != ID_INVALID && n != ID_INVALID )
					n = nIDnext;
				else	{// if callback made n=ID_INVALID, or n was unchanged and nIDnext is invalid
					if( cbNoNext.func )
						cbNoNext.func(this, cbNoNext.obj, &n);
					if( !cbNoNext.func || n == ID_INVALID )
					{
						eFlyStyle = sinus;
						flyTo( rShow, 0 );// fly back again
					}
					else setVisible( false );// otherwise hide
					break;
				}
			}
			if( showItem( n, true ) )
				setRelWindow( pCurr->getRectNext() );
			
			eFlyStyle = sinus;
			flyTo( rShow, 0 );
			break;
		}
		
		case sDisappeared:
			setVisible( false );
			if( cbDisappear.func )
				cbDisappear.func(this, cbDisappear.obj, NULL);
			break;
			
		default:
			setEnabled( true );
	}
	// do not fly again automatically after the next onPosReached
	state = (MenuState)'-';
	
	return GuiFlowWindow::onPosReached();
}

void GuiFlowMenu::appear()
{
	showItem( nID, false );
	setRelWindow( rDisappear );
	fly('s');
}

void GuiFlowMenu::disappear()
{
	fly('d');
}

void GuiFlowMenu::fly(char where_to, void *sender)
{
	state = (MenuState)where_to;
	if( !isVisible() )	{
		showItem( nID, false );
		setVisible( true );
	}
	setEnabled( false );
	switch( state )
	{
		case sShow:
			eFlyStyle = sinus;
			flyTo(rShow, 0);
			break;
			
		case sBack:
			eFlyStyle = cosinus;
			flyTo(pCurr->getRectNext(), 0);
			break;
			
		case sNext:
			eFlyStyle = cosinus;
			flyTo(pCurr->getRectBack(), 0);
			if( !sender || ( nIDnext = getIDbySender( (BasicObject*)sender ) ) == ID_INVALID )
				nIDnext = ID_INVALID;
			break;
			
		case sDisappeared:
			eFlyStyle = cosinus;
			flyTo(rDisappear, 0);
			break;
	}
}

long GuiFlowMenu::getIDbySender( BasicObject *sender )
{
	for( map<long, GuiFlowMenuItem*>::iterator i = mapMenues.begin(); i != mapMenues.end(); ++i )
	{
		if( (*i).second->activatorCompare( sender ) )
			return (*i).first;
	}
	return ID_INVALID;
}

//-----------------------------------------------------------------------------
//  GuiFlowMenuItem
//-----------------------------------------------------------------------------

GuiFlowMenuItem::GuiFlowMenuItem(TiXmlElement *el, GuiFlowMenu *prnt)
	: GuiElement(el, prnt)
{
	const char *str = el->Attribute("ID");
	nID = str ? strtol(str, NULL, 10) : 0;
	bFailed = !prnt->registerItem( nID, this );
	setVisible(false);
	
	strActivator = NULL;
}

GuiFlowMenuItem::GuiFlowMenuItem(const SDL_Rect &r, GuiFlowMenu *prnt, const char *name, long id)
	: GuiElement(r, prnt, name)
{
	bFailed = !prnt->registerItem( id, this );
	nID = id;
	strActivator = NULL;
	setVisible(false);
}

GuiFlowMenuItem::~GuiFlowMenuItem()
{
	if( strActivator )
		delete[] strActivator;
	strActivator = NULL;
	
	if( !bFailed )
		( (GuiFlowMenu *)getParent() )->unregisterItem( nID );
}

void GuiFlowMenuItem::onInit()
{
	// if ID in xml-data differs from current id, then renew it
	TiXmlElement *xmlData = getXmlData();
	if( xmlData )	{
		const char *str;
		long id;
		str = xmlData->Attribute("ID");
		if(str)	{
			id = strtol(str, NULL, 10);
			if( id != nID )	{
				GuiFlowMenu *menu = (GuiFlowMenu *)getParent();
				if( !bFailed )
					menu->unregisterItem( nID );
				bFailed = !menu->registerItem( id, this );
				nID = id;
			}
		}
		str = xmlData->Attribute("Activator");
		if( str )	{
			if( strActivator )
				delete[] strActivator;
			strActivator = new char[ strlen( str ) + 1 ];
			strcpy( strActivator, str );
		}
		str = xmlData->Attribute("NextRect");
		if( str )	{
			rNext = strToRect( str );
		}
		str = xmlData->Attribute("BackRect");
		if( str )	{
			rBack = strToRect( str );
		}
	}
	GuiElement::onInit();
	setVisible(false);
}


bool GuiFlowMenuItem::activatorCompare( BasicObject *obj )
{
	if( strActivator == NULL)
		return false;
	return getObjectByName( strActivator ) == obj;
}

