/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <math.h>
#include <string>
#include "guilist.h"
#include "sdlmanager.h"
#include "sdlsurface.h"
#include "uimanager.h"
#include "uitheme.h"
#include "sdlfont.h"
#include "rectfunctions.h"
#include "oglfunctions.h"

DECLARE_SLOT(new, GuiList, GuiList::newguiobject)

//////////////////////////////////////////////////////////////////////
// GuiList

GuiList::GuiList(const SDL_Rect &r, GuiElement *p, const char *name) : GuiPictureBg( r,p, name, CLR_INVALID )
{
	SDL_Rect tmp = {0,0,0,0};
	scroller = new GuiScroller( tmp, this, "" );
	scroller->setDirection( GuiScroller::vertikal );
	scroller->setCallbackData( this );
	scroller->setChangeCallback( &ontopitemchanged );
	scroller->bgcolor = CLR_INVALID;
	selected = -1;
	topitem = -1;
	cbSelchanged.func = 0;
	strings.clear();
	font = SdlFont::get( 0, false );
	fontSel = SdlFont::get( 0, true );

	arrange();
	update();
}

GuiList::GuiList(TiXmlElement *el, GuiElement *p) : GuiPictureBg( el,p )
{
	SDL_Rect tmp = {0,0,0,0};
	scroller = new GuiScroller( tmp, this, "" );
	scroller->setDirection( GuiScroller::vertikal );
	scroller->setCallbackData( this );
	scroller->setChangeCallback( &ontopitemchanged );
	scroller->bgcolor = CLR_INVALID;
	selected = -1;
	topitem = -1;
	strings.clear();
	font = SdlFont::get( 0, false );
	fontSel = SdlFont::get( 0, true );

	cbSelchanged.func = 0;
	arrange();
	update();
}

GuiList::~GuiList()
{

}

void GuiList::onInit()
{
	arrange();
	update();
	TiXmlElement *el = getXmlData();
	if( el )
	{
		const char *str = el->Attribute("OnSelect");
		if( str )
			cbSelchanged = getFunctionByName( str );
		char strAttrib[16];
		for( int i = 1; ; i++ )
		{
			sprintf( strAttrib, "Entry%d", i );
			str = el->Attribute(strAttrib);
			if( str == NULL )
				break;
			if( i <= strings.size() )
				changeStr( str, i-1 );
			else
				insertStr( str );
		}
	}
	GuiPictureBg::onInit();
}

Uint32 GuiList::onResize()
{
	arrange();
	return 0;
}

void GuiList::arrange()
{
	Uint32 width, height;

	SDL_Rect r;
	getWndSize( width, height );

	r.x = width - 20;
	r.y = 0;
	r.w = 20;
	r.h = height;
	scroller->setRelWindow( r );
}

Uint32 GuiList::onFrame( SdlSurface *dst )
{
	paintBg( dst );

	if( topitem >= 0 )	{
		SDL_Rect clip = getClip();
		SDL_Rect abs = getDrawWindow();
		Uint32 width, height;
		getWndSize( width, height );
		int fontheight = font->getBmps()->getImage( 0 )->getHeight();

		for( int i = topitem; i < (int)strings.size(); i++ )	{
			int xP, yP;
			char *str = strings.at( i );
			xP = 0;
			yP = fontheight * ( i - topitem );
			if( yP >= (int)height )
				break;
			xP += abs.x;
			yP += abs.y;
			if( selected == i )	{
				SDL_Rect dstr;
				dstr.y = yP;
				dstr.x = abs.x;
				dstr.w = abs.w;
				dstr.h = fontheight;
				intersectRect( dstr, clip );
				dst->fill( &dstr, ~bgcolor );
				SdlFont::get( 0, true )->write( dst, str, xP, yP, SdlFont::top, SdlFont::left, &clip );
			}
			else font->write( dst, str, xP, yP, SdlFont::top, SdlFont::left, &clip );
		}
	}
	return GuiElement::onFrame( dst );
}

Uint32 GuiList::onGLFrame()
{
	paintBgGL();

	if( topitem >= 0 )
	{
		SDL_Rect clip = getClip();
		SDL_Rect abs = getDrawWindow();
		Uint32 width, height;
		getWndSize( width, height );
		int fontheight = font->getBmps()->getImage( 0 )->getHeight();

		for( int i = topitem; i < (int)strings.size(); i++ )	{
			int xP, yP;
			char *str = strings.at( i );
			xP = 0;
			yP = fontheight * ( i - topitem );
			if( yP >= (int)height )
				break;
			xP += abs.x;
			yP += abs.y;
			if( selected == i )	{
				SDL_Rect dstr;
				dstr.y = yP;
				dstr.x = abs.x;
				dstr.w = abs.w;
				dstr.h = fontheight;
				intersectRect( dstr, clip );
				ogl::fillRect( dstr, ~bgcolor & 0xFFFFFF );
				SdlFont::get( 0, true )->writeGL( str, xP, yP, SdlFont::top, SdlFont::left, &clip );
			}
			else font->writeGL( str, xP, yP, SdlFont::top, SdlFont::left, &clip );
		}
	}
	return GuiElement::onGLFrame();
}

Uint32 GuiList::onMouseDown( SDL_MouseButtonEvent *ev )
{
	if( ev->button == 1 )	{
		int fontheight = SdlFont::get( 0, true )->getBmps()->getImage(0)->getHeight();
		int sel = ( ( ev->y - getAbsWindow().y ) / fontheight ) + topitem;
		if( sel < (int)strings.size() )	{
			selected = sel;
			if( cbSelchanged.func )
				(*cbSelchanged.func)( this, cbSelchanged.obj, NULL );
		}
		setRedraw();
	}
	return 0;
}

Uint32 GuiList::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiPictureBg::onThemeChanged( t );
	if( t == 0 )
		return r;

	int n = t->getUint32( ((string)getClassName() + "->font-size" ).c_str() );
	bool l = t->getBool( ((string)getClassName() + "->font-light" ).c_str() );
	font = SdlFont::get( n, l );
	fontSel = SdlFont::get( n, !l );

	return r;
}

int GuiList::insertStr(const char *str, int pos)
{
	char *copy = new char[strlen(str) + 1];
	strcpy(copy, str);
	int actpos = -1;
	if(pos < 0)	{
		strings.insert(strings.end(), copy);
		actpos = strings.size();
	}
	else if(pos < (int)strings.size())	{
		if(topitem == pos)
			topitem++;
		strings.insert(iterfrompos(pos),copy);
		actpos = pos;
	}
	else	{
		delete[] copy;
		actpos = -1;
	}
	update();

	return actpos;
}

bool GuiList::changeStr(const char *str, int pos)
{
	if((pos >= 0) && (pos < (int)strings.size()))	{
		char *tmp = strings.at(pos);
		delete[] tmp;
		char *copy = new char[strlen(str) + 1];
		strcpy(copy, str);
		tmp = copy;
		return true;
	}
	return false;
}


bool GuiList::clearStr(int pos)
{
	if(!strings.empty())	{
		if(pos < 0)	{
			if(topitem == ((int)strings.size() - 1))
				topitem = -1;
			delete[] (strings.back());
			strings.pop_back();
		}
		else if(pos < (int)strings.size())	{
			if(topitem == pos)
				topitem = -1;
			else if(topitem > pos)
				topitem--;
			strlist_iter item = iterfrompos(pos);
			delete[] (*item);
			strings.erase(item);
			
		}
		update();
		return true;
	}
	return false;
}



void GuiList::clearAll()
{
	for(strlist_iter i = strings.begin(); i != strings.end(); i++)
		delete[] (*i);
	strings.clear();
	selected = -1;
	topitem = 0;
	update();
}

int GuiList::getCount() const
{
	return strings.size();
}


const char *GuiList::getStr(int pos) const
{
	if((pos >= 0) && (pos < (int)strings.size()))	{
		return strings.at(pos);
	}
	return 0;
}

const char *GuiList::getSelStr() const
{
	if((selected >= 0) && (selected < (int)strings.size()))	{
		return strings.at(selected);
	}
	return 0;
}

int GuiList::getSelId() const
{
	return selected;
}

GuiList::strlist_iter 
GuiList::iterfrompos(int pos)
{
	strlist_iter iter = strings.begin();
	for(int i = 0; i < pos; i++)	{
		if(iter == strings.end())
			return iter;
		iter++;
	}
	return iter;
}

void GuiList::update()
{
	if(topitem < -1)
		topitem = -1;
	if(topitem >= (int)strings.size())
		topitem = strings.size() - 1;
	if(topitem == -1 && strings.size() > 0)
		topitem = 0;

	int fontheight = font->getBmps()->getImage( 0 )->getHeight();
	Uint32 wndheight, unused;
	getWndSize( unused, wndheight);
	nMaxScroll = strings.size() > wndheight/fontheight? strings.size() - wndheight/fontheight : 0;
	scroller->setMinMax( 0, nMaxScroll );
	scroller->setScrollPos( strings.size()-1-topitem );
	scroller->setVisible( nMaxScroll != 0 );
}

void GuiList::updatefromscroller()
{
	topitem = nMaxScroll - scroller->getScrollPos();
}
