/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "GL/glu.h"
#include "guimanager.h"
#include "uimanager.h"
#include "sdlmanager.h"
#include "thememanager.h"
#include "guielement.h"
#include "guifader.h"
#include "rectfunctions.h"
#include "functions.h"
#include "basicapp.h"

GuiManager *GuiManager::the = NULL;

GuiManager *GuiManager::getThe()
{
	return the;
}


GuiManager::GuiManager( UiManager *parent, const char *name ): UiObject( parent, name )
{
	the = this;
	nDisplayWidth = 0;
	nDisplayHeight = 0;
	ftCaptured = 0;
	geMain = geFront = geCursor = geMouseFocus = geMouseOver = NULL;
	bfClearBuffers = 0;
}

GuiManager::~GuiManager()
{
	the = NULL;
	destroy();
}

bool GuiManager::init(TiXmlElement *el)
{
	BasicApp::the->logBegin( "Initializing GuiManager" );
	SDL_ShowCursor( 0 );

	bOGL = SdlManager::the->isUsingOpenGL();
	
	nDisplayWidth = SdlManager::the->getScreenWidth();
	nDisplayHeight = SdlManager::the->getScreenHeight();
	
	if(el)	{
		GuiElement *front = NULL, *main = NULL;
		TiXmlElement *el2 = el->FirstChildElement("Front");
		if( el2 && el2->FirstChildElement())	{
			BasicApp::the->logLevelLine( "Creating front elements" );
			front = GuiElement::create(el2->FirstChildElement(), NULL);
			if( el2->FirstChildElement()->NextSiblingElement() )	{
				BasicApp::the->logLevelLine("WARNING: only one top-level front element may exist, ignoring others.");
			}
		}
		el2 = el->FirstChildElement("Main");
		if( el2 && el2->FirstChildElement())	{
			BasicApp::the->logLevelLine( "Creating main elements" );
			main = GuiElement::create(el2->FirstChildElement(), NULL);
			if( el2->FirstChildElement()->NextSiblingElement() )	{
				BasicApp::the->logLevelLine("WARNING: only one top-level main element may exist, ignoring others.");
			}
		}
		if( !main )
			BasicApp::the->logLevelLine( "ERROR: GuiManager::init: no main element availible!" );
		setBaseElements(front, main);
		el2 = el->FirstChildElement("GuiFader");
		if( el2 )	{
			BasicApp::the->logLevelLine( "Creating GuiFader" );
			new GuiFader(el2, this);
		}
		UiTheme *t;
		if( ThemeManager::the )
			t = ThemeManager::the->getActiveTheme();
		else
			t = NULL;
		if( front )	{
			BasicApp::the->logLevelLine( "Initializing front elements" );
			front->onInit();
			front->onThemeChanged(t);
		}
		if( main )	{
			BasicApp::the->logLevelLine( "Initializing main elements" );
			main->onInit();
			main->onThemeChanged(t);
		}
		// for OpenGL:
		const char *str = el->Attribute("GLClearDepth");
		if( str )
		{
			if( strToBool(str) )
				bfClearBuffers |= GL_DEPTH_BUFFER_BIT;
			else
				bfClearBuffers &= ~GL_DEPTH_BUFFER_BIT;
		}
		str = el->Attribute("GLClearColor");
		if( str )
		{
			Uint32 c = xStrToUL(str);
			if( c != CLR_INVALID )
			{
				bfClearBuffers |= GL_COLOR_BUFFER_BIT;
				GLclampf r, g, b;
				r = ((c >>16) & 255) / 255.f;
				g = ((c >> 8) & 255) / 255.f;
				b = ((c     ) & 255) / 255.f;
				glClearColor( r, g, b, 0.f );
			}
			else
				bfClearBuffers &= ~GL_COLOR_BUFFER_BIT;
		}
	}
	BasicApp::the->logEnd( "done." );
	return true;
}

void GuiManager::onDestroy()
{
	BasicApp::the->logBegin( "Destroying GuiManager" );
	BasicApp::the->logLevelLine( "Deleting main elements" );
	if(geMain)	{
		delete geMain;
	}
	BasicApp::the->logLevelLine( "Deleting front elements" );
	if(geFront)
		delete geFront;
	BasicApp::the->logEnd( "done." );
	// calling UiObject::onDestroy() here not necessary
}

void GuiManager::setBaseElements(GuiElement *front, GuiElement *main)
{
	if(geFront)
		geFront->onRelease(ftDisplay);
	geFront = front;
	if(geFront)
		geFront->onFocus(ftDisplay);
	
	if(geMain)
		geMain->onRelease(ftDisplay);
	geMain = main;
	if(geMain)
		geMain->onFocus(ftDisplay);
}

void GuiManager::focus(GuiElement *el)
{
	if(geMouseFocus)
		geMouseFocus->onRelease(ftMouse);
	geMouseFocus = el;
	if(el)
		el->onFocus(ftMouse);
}

bool GuiManager::release(GuiElement *el)
{
	if(geMouseFocus == el)	{
		geMouseFocus = NULL;
		el->onRelease(ftMouse);
		return true;
	}
	return false;
}

void GuiManager::setCursor(GuiElement *cur)
{
	if(geCursor)	{
		geCursor->onRelease(ftMouse);
	}
	geCursor = cur;
	if(geCursor)	{
		geCursor->onFocus(ftMouse);
	}
}

void GuiManager::cleanupElement(GuiElement *ge)
{
	if( geMouseFocus == ge )
		geMouseFocus = NULL;
	if( geMouseOver == ge )
		geMouseOver = NULL;
	if( geCursor == ge )
		geCursor = NULL;
	if( geMain == ge )
		geMain = NULL;
	if( geFront == ge )
		geFront = NULL;
}

void GuiManager::resetGLstates()
{
	resetGLstates( nDisplayWidth, nDisplayHeight );
}

void GuiManager::resetGLstates( Uint16 w, Uint16 h )
{
	glViewport( 0, 0, w, h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	// set h to bottom and zero to top in order to have same coordinates as in blitting mode
	gluOrtho2D( 0., (GLdouble)w, (GLdouble)h, .0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glEnable( GL_TEXTURE_2D );
	glColor3f( 1.f, 1.f, 1.f );
}

Uint32 GuiManager::onMouseDown( SDL_MouseButtonEvent *ev )
{
	if( geCursor )
		geCursor->onMouseDown(ev);
	if( geMouseFocus )
		return geMouseFocus->onMouseDown(ev);
	else if( geMouseOver )
		return geMouseOver->onMouseDown(ev);
	return 0;
}

Uint32 GuiManager::onMouseUp( SDL_MouseButtonEvent *ev )
{
	if( geCursor )
		geCursor->onMouseUp(ev);
	if( geMouseFocus )
		return geMouseFocus->onMouseUp(ev);
	else if( geMouseOver )
		return geMouseOver->onMouseUp(ev);
	return 0;
}

Uint32 GuiManager::onMouseMotion( SDL_MouseMotionEvent *ev )
{
	if( geCursor )
		geCursor->onMouseMove( ev->x, ev->y );

	if( geMouseFocus && geMouseFocus->isEnabled() )	{
		geMouseFocus->onMouseMove( ev->x, ev->y );
		SDL_Rect r = geMouseFocus->getAbsWindow();
		
		if( pointInRect( r, ev->x, ev->y ) )	{
			if( geMouseOver != geMouseFocus )	{
				if( geMouseOver )	{
					geMouseOver->onMouseOut();
				}
				geMouseOver = geMouseFocus;
				geMouseOver->onMouseOver();
			}
		}
		else	{
			if(geMouseOver)	{
				geMouseOver->onMouseOut();
				geMouseOver = NULL;
			}
		}
	}
	else	{
		GuiElement *el = geMain->elementFromPos( ev->x, ev->y );
		if( ( el != NULL ) && el->isEnabled() )	{
			el->onMouseMove( ev->x, ev->y );

			if( geMouseOver != el )	{
				if(geMouseOver)	{
					geMouseOver->onMouseOut();
	 			}
				geMouseOver = el;
				el->onMouseOver();
			}
		}
		else	{
			if( geMouseOver )	{
				geMouseOver->onMouseOut();
				geMouseOver = NULL;
			}
		}
	}
	return 0;
}

Uint32 GuiManager::onFocus(FocusTypes gained)
{
	if( gained & ftDisplay )
	{
		// TODO: refresh off-screen surfaces, if we are in fullscreen mode
		if( bOGL )
		{
			glEnable( GL_BLEND );
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
			glEnable( GL_TEXTURE_2D );
			resetGLstates();
		}
	}
	ftCaptured |= gained;
	return 0;
}

Uint32 GuiManager::onRelease( FocusTypes lost )
{
	if(lost & ftDisplay)	{
		// TODO
	}
	ftCaptured &= ~lost;
	return 0;
}

Uint32 GuiManager::onFrame()
{
	if( bOGL )
		return glFrame();
	else
		return surfFrame( SdlManager::the->getDisplay() );
}

Uint32 GuiManager::glFrame()
{
	//resetGLstates();
	
	if( bfClearBuffers != 0 )
		glClear( bfClearBuffers );
	
	if( geMain )
		geMain->onGLFrame();
	if( geFront )
		geFront->onGLFrame();
	if( GuiFader::the )
		GuiFader::the->onFrame();
	
	SDL_GL_SwapBuffers();
	return 0;
}

Uint32 GuiManager::surfFrame( SdlSurface *dest )
{
	if( geMain )
		geMain->onFrame( dest );
	if( geFront )
		geFront->onFrame( dest );
	if( GuiFader::the )
		GuiFader::the->onFrame();
	
	SDL_Flip( SdlManager::the->getDisplay()->surf() );
	return 0;
}

