/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guimenu.h"
#include "guimanager.h"
#include "sdlmanager.h"
#include "uitheme.h"

GuiMenuBar::GuiMenuBar( const SDL_Rect &r, GuiElement *parent, const char *name, bool horizontal )
	: GuiElement( r, parent, name )
{
	if( !parent )
		BasicApp::the->logLevelLine( "ERROR: GuiMenuBar: parent must not be NULL" );
	bHorizontal = horizontal;
	pActivator = NULL;
}

GuiMenuBar::~GuiMenuBar()
{
	while( !lItems.empty() )
		delete lItems.back();
}

void GuiMenuBar::onSelect( GuiMenuItem *selected )
{
	//setVisible( false );
	if( pActivator )	{
		pActivator->onSubClose( selected );
	}
	if( fdSelect.func )
		fdSelect.func( selected, fdSelect.obj, selected->getItemData() );
}

void GuiMenuBar::hideSubmenus()
{
	for( list<GuiMenuItem*>::iterator i = lItems.begin(); i != lItems.end(); ++i )
	{
		if( (*i)->pSubmenu )	{
			(*i)->pSubmenu->hideSubmenus();
			(*i)->pSubmenu->setVisible( false );
		}
		else if( (*i)->gmbSubmenu )	{
			(*i)->gmbSubmenu->hideSubmenus();
			(*i)->gmbSubmenu->setVisible( false );
		}
	}
}

void GuiMenuBar::showAt( const SDL_Rect &r, bool parent_horizontal )
{
	SDL_Rect rAbs = getRelWindow();
	SDL_Rect rSpace = getParent()->getRelWindow();
	
	if( rSpace.w/2 < r.x + r.w/2 )	{// put to left hand side of r
		if( parent_horizontal )
			rAbs.x = r.x - rAbs.w + r.w;
		else
			rAbs.x = r.x - rAbs.w - 2/*let a little space*/;
	}
	else	{
		if( parent_horizontal )
			rAbs.x = r.x;
		else
			rAbs.x = r.x + r.w + 2;
	}
	if( rSpace.h/2 < r.y + r.h/2 )	{// put above r
		if( !parent_horizontal )
			rAbs.y = r.y - rAbs.h + r.h;
		else
			rAbs.y = r.y - rAbs.h - 2/*let a little space*/;
	}
	else	{
		if( !parent_horizontal )
			rAbs.y = r.y;
		else
			rAbs.y = r.y + r.h + 2;
	}
	
	setRelWindow( rAbs );
	setVisible( true );
	toTop();
}

void GuiMenuBar::refresh()
{
	int offset = 0;
	int size = 0;
	SDL_Rect r;
	if( bHorizontal )	{
		for( list<GuiMenuItem*>::iterator i = lItems.begin(); i != lItems.end(); ++i )
		{
			if( (*i)->isVisible() )	{
				(*i)->moveTo( offset, 0 );
				r = (*i)->getRelWindow();
				offset += r.w;
				if( r.h > size )
					size = r.h;
			}
		}
	}
	else	{
		for( list<GuiMenuItem*>::iterator i = lItems.begin(); i != lItems.end(); ++i )
		{
			if( (*i)->isVisible() )	{
				(*i)->moveTo( 0, offset );
				r = (*i)->getRelWindow();
				offset += r.h;
				if( r.w > size )
					size = r.w;
			}
		}
	}
	// if no visible items then make associated item unselectable
	if( offset == 0 )	{
		if( pActivator )
			pActivator->setSelectable( false );
	}
	else	{
		r = getRelWindow();
		if( bHorizontal )	{
			r.w = offset;
			r.h = size;
		}
		else	{
			r.w = size;
			r.h = offset;
		}
		setRelWindow( r );
	}
}

void GuiMenuBar::add( GuiMenuItem *item )
{
	lItems.push_back( item );
	refresh();
}

//-----------------------------------------------------------------------------
//  class GuiMenuItem
//-----------------------------------------------------------------------------

GuiMenuItem::GuiMenuItem( const SDL_Rect &r, GuiMenuBar *parent, const char *name, bool checkable,
						  const char *txt, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
	: GuiElement( r, parent, name )
{
	type = checkable ? tCheckable : tClickable;
	bCheckChanging = false;
	pSubmenu = NULL;
	gmbSubmenu = NULL;
	
	parent->add( this );
	
	if( checkable )	{
		SDL_Rect rMain = r;
		rMain.x = rMain.y = 0;
		GuiCheckableBtn *btn = new GuiCheckableBtn( rMain, this, "main", txt, b, u, d );
		btn->setCheckCallback( OnClicked );
		btn->setCallbackData( this );
		gdbMain = btn;
	}
	else	{
		SDL_Rect rMain = r;
		rMain.x = rMain.y = 0;
		GuiClickableBtn *btn = new GuiClickableBtn( rMain, this, "main", txt, b, u, d );
		btn->setClickCallback( OnClicked );
		btn->setCallbackData( this );
		gdbMain = btn;
	}
}

GuiMenuItem::GuiMenuItem( const SDL_Rect &r, GuiMenuBar *parent, const char *name, GuiMenuBar *submenu,
						  const char *txt, SdlImageContainer *b, SdlAudio *u, SdlAudio *d )
	: GuiElement( r, parent, name )
{
	bCheckChanging = false;
	type = tSubmenu;
	pSubmenu = submenu;
	if( !pSubmenu )	{
		SDL_Rect r = {0, 0, 0, 0};
		gmbSubmenu = new GuiMenuBar( r, GuiManager::the->getMain(), "" );
		gmbSubmenu->setVisible( false );
	}

	parent->add( this );

	SDL_Rect rMain = r;
	rMain.x = rMain.y = 0;
	GuiClickableBtn *btn = new GuiClickableBtn( rMain, this, "main", txt, b, u, d );
	btn->setClickCallback( OnClicked );
	btn->setCallbackData( this );
	gdbMain = btn;
}

GuiMenuItem::~GuiMenuItem()
{
	GuiMenuBar *parent = (GuiMenuBar *)getParent();
	parent->lItems.remove( this );
}

void GuiMenuItem::onDestroy()
{
	if( gmbSubmenu )	{
		delete gmbSubmenu;
		gmbSubmenu = NULL;
	}
}

Uint32 GuiMenuItem::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiElement::onThemeChanged( t );

	if( t == NULL )
		return r;

	const char *name;
	name = t->getString( ( string(getClassName()) + "->border-bmps" ).c_str() );
	if( name )	{
		SdlImageContainer * bmps = SdlManager::the->getImagesByName( name );
		if( bmps )
			gdbMain->setBmps( bmps );
	}
	return r;
}

void GuiMenuItem::setSelectable( bool cansel )
{
	if( cansel == isVisible() )
		return;
	
	setVisible( cansel );
	GuiMenuBar *parent = (GuiMenuBar *)getParent();
	parent->refresh();
}

void GuiMenuItem::setChecked( bool checked )
{
	if( type == tCheckable )	{
		bCheckChanging = true;
		GuiCheckableBtn *b = (GuiCheckableBtn *)gdbMain;
		b->setCheck( checked );
		bCheckChanging = false;
	}
}

void GuiMenuItem::setButtonBmps( SdlImageContainer *b )
{
	if( gdbMain )
		gdbMain->setButtonBmps( b );
}

void GuiMenuItem::onClicked()
{
	GuiMenuBar *menubar;
	if( pSubmenu )
		menubar = pSubmenu;
	else if( gmbSubmenu )
		menubar = gmbSubmenu;
	else	{// if no submenu is associated to this item, tell parent
		if( bCheckChanging )// ignore the callback if it is caused by setCheck()
			return;
		GuiMenuBar *parent = (GuiMenuBar *)getParent();
		parent->onSelect( this );
		return;
	}
	// if there is a submenu associated, then toggle its visibility
	if( !menubar->isVisible() )	{
		// first, hide all sibling menus
		GuiMenuBar *parent = (GuiMenuBar *)getParent();
		parent->hideSubmenus();
		// show this one's submenu
		menubar->showAt( getAbsWindow(), parent->isHorizontalAligned() );
		menubar->pActivator = this;
	}
	else	{
		menubar->hideSubmenus();
		menubar->setVisible( false );
	}
}

void GuiMenuItem::onSubClose( GuiMenuItem *selected )
{
	GuiMenuBar *parent = (GuiMenuBar *)getParent();
	parent->onSelect( selected );
}


