/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guimessagebox.h"
#include "uimanager.h"
#include "rectfunctions.h"
#include "definitions.h"
#include "basicapp.h"

DECLARE_SLOT(new, GuiMessageBox, GuiMessageBox::newguiobject)

GuiMessageBox::GuiMessageBox( const SDL_Rect &r, GuiElement * el, const char *name, const char * t, int buttons )
: GuiPictureBg( r, el, name )
{
	SDL_Rect rr = r;
	cb.func = 0;
	index = 0;

	rr.x = rr.y = 0;
	border = new GuiBorder( rr, this, "" );
		inflateRect( rr, -10, -10 );
		rr.h -= 40;
		text = new GuiText( rr, border, "", t );
		text->chooseFont( GuiText::fs_heading, true );
		text->setAlign( SdlFont::baseline, SdlFont::center );

	gcbOk = gcbYes = gcbCancel = gcbNo = 0;

	setButtons( buttons );
}

GuiMessageBox::GuiMessageBox( TiXmlElement *el, GuiElement *p ): GuiPictureBg( el, p )
{
	SDL_Rect rr = getRelWindow();
	cb.func = 0;
	index = 0;

	rr.x = rr.y = 0;
	border = new GuiBorder( rr, this, "" );
		inflateRect( rr, -10, -10 );
		rr.h -= 40;
		text = new GuiText( rr, border, "", el->Attribute("Text") );
		text->chooseFont( GuiText::fs_heading, true );
		text->setAlign( SdlFont::baseline, SdlFont::center );

	gcbOk = gcbYes = gcbCancel = gcbNo = 0;

	unsigned long buttons;
	if(el->QueryULongAttribute("Buttons",&buttons) == TIXML_SUCCESS)
		setButtons( buttons );
	else setButtons( -1 );
}

GuiMessageBox::~GuiMessageBox()
{
	free();
}

void GuiMessageBox::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("OnButton");
		if(str)	{
			cb = getFunctionByName(str);
		}
		unsigned long buttons;
		if(el->QueryULongAttribute("Buttons",&buttons) == TIXML_SUCCESS)
			setButtons( buttons );
	}
	GuiPictureBg::onInit();
}

int GuiMessageBox::getAnswer( bool hide )
{
	if( hide )
		setVisible( false );
	return index;
}

void GuiMessageBox::arrange()
{
	SDL_Rect rr = getRelWindow();
	rr.x = rr.y = 0;
	border->setRelWindow( rr );

	inflateRect( rr, -10, -10 );
	rr.h -= 40;
	text->setRelWindow( rr );

	rr.y += rr.h + 10;
	rr.h = 30;
	inflateRect( rr, -rr.w/10, 0 );
	rr.w /= 3;
	int w = rr.w;
	inflateRect( rr, -rr.w/10, 0 );
	int x = rr.x;

	switch( nButtons )	{
	case SMB_OK:
		rr.x = x + w;
		gcbOk->setRelWindow( rr );
		break;

	case SMB_YESNOCANCEL:
		rr.x = x + w;
		gcbCancel->setRelWindow( rr );
	case SMB_YESNO:
		rr.x = x;
		gcbNo->setRelWindow( rr );
		rr.x = x + 2*w;
		gcbYes->setRelWindow( rr );
		break;

	default:
		BasicApp::the->logLevelLine( "ERROR: invalid button-combination in GuiMessageBox()!" );
	}
}

Uint32 GuiMessageBox::onResize()
{
	arrange();
	return 0;
}

void GuiMessageBox::onButton(GuiClickableBtn *sender)
{
	if( sender == gcbOk )	{
		index = 0;
		if( cb.func )
			(*cb.func)(this, cb.obj, &index);
	}
	else if( sender == gcbCancel )	{
		index = 1;
		if( cb.func )
			(*cb.func)(this, cb.obj, &index);
	}
	else if( sender == gcbNo )	{
		index = -1;
		if( cb.func )
			(*cb.func)(this, cb.obj, &index);
	}
	else if( sender == gcbYes )	{
		index = -1;
		if( cb.func )
			(*cb.func)(this, cb.obj, &index);
	}
	else	{
		index = 0;
		if( cb.func )
			(*cb.func)(this, cb.obj, &index);

		BasicApp::the->logLevelLine( "ERROR: unresolved sender in GuiMessageBox::onButton()" );
	}
}

#define DELOBJ(x)	if(x != NULL)	{ delete x; x = NULL; }

void GuiMessageBox::free()
{
	DELOBJ(gcbOk)
	DELOBJ(gcbCancel)
	DELOBJ(gcbNo)
	DELOBJ(gcbYes)
	DELOBJ(text)
	DELOBJ(border)
}

void GuiMessageBox::setButtons( long buttons )
{
	DELOBJ(gcbOk)
	DELOBJ(gcbCancel)
	DELOBJ(gcbNo)
	DELOBJ(gcbYes)

	SDL_Rect rr = getRelWindow();
	rr.x = rr.y = 0;
	inflateRect( rr, -10, -10 );
	rr.h -= 40;

	rr.y += rr.h + 10;
	rr.h = 30;
	inflateRect( rr, -rr.w/10, 0 );
	rr.w /= 3;
	int w = rr.w;
	inflateRect( rr, -rr.w/10, 0 );
	int x = rr.x;

	nButtons = buttons;
	switch( buttons )	{
	case SMB_OK:
		rr.x = x + w;
		gcbOk = new GuiClickableBtn( rr, border, "", tr("OK") );
		gcbOk->setCallbackData( this );
		gcbOk->setClickCallback( onButtonClick );
		break;

	case SMB_YESNOCANCEL:
		rr.x = x + w;
		gcbCancel = new GuiClickableBtn( rr, border, "", tr("Cancel") );
		gcbCancel->setCallbackData( this );
		gcbCancel->setClickCallback( onButtonClick );
	case SMB_YESNO:
		rr.x = x;
		gcbNo = new GuiClickableBtn( rr, border, "", tr("Yes") );
		gcbNo->setCallbackData( this );
		gcbNo->setClickCallback( onButtonClick );
		rr.x = x + 2*w;
		gcbYes = new GuiClickableBtn( rr, border, "", tr("No") );
		gcbYes->setCallbackData( this );
		gcbYes->setClickCallback( onButtonClick );
		break;

	default:
		BasicApp::the->logLevelLine( "ERROR: invalid button-combination in GuiMessageBox()!" );
	}
	
	setVisible( true );
}

void GuiMessageBox::showMessage( const char *m, long buttons )
{
	text->setText( m );
	if( buttons != -1 )
		setButtons( buttons );
	else setVisible( true );
}
