/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guitext.h"
#include <string>
#include "uimanager.h"
#include "thememanager.h"
#include "sdlsurface.h"
#include "sdlmanager.h"
#include "sdlfont.h"
#include "basicapp.h"
#include "rectfunctions.h"
#include "definitions.h"
#include "functions.h"
#include "oglfunctions.h"

Uint32 GuiTextInput::nBlinkIv = 400;

DECLARE_SLOT(new, GuiText, GuiText::newguiobject)
DECLARE_SLOT(SetText, GuiText, GuiText::settext)
DECLARE_SLOT(new, GuiTextInput, GuiTextInput::newguiobject)
DECLARE_SLOT(new, GuiKeyControl, GuiKeyControl::newguiobject)
DECLARE_SLOT(new, GuiFrameCounter, GuiFrameCounter::newguiobject)

//////////////////////////////////////////////////////////////////////
// GuiText

GuiText::FontSize GuiText::strToSize( const char *s )
{
	if(strcmp(s, "text") == 0)	{
		return fs_text;
	}
	if(strcmp(s, "button") == 0)	{
		return fs_button;
	}
	if(strcmp(s, "heading") == 0)	{
		return fs_heading;
	}
	if(strcmp(s, "theme") == 0)	{
		return fs_theme;
	}
	if(*s >= '0' && *s <= '0' + (int)fs_heading)
		return (FontSize)(*s - '0');
	return fs_standard;
}

GuiText::GuiText( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, FontSize font, bool light, bool stdAlign )
	: GuiElement( r, p, name )
{
	nBufferSize = 16;
	chBuffer = new char[nBufferSize];
	pText = 0;
	bThemeAlign = stdAlign;
	bThemeFont = font == fs_theme;
	bLightFont = light;

	eVAlign = SdlFont::baseline;
	eHAlign = SdlFont::left;
	eFontSize = font;

	curpos = -1;
	showcursor = false;
	bWordWrap = true;
	setText( t );
	if( bThemeFont )	{
		this->font = 0;
		SdlFont *f = SdlFont::get( 0, true );
		if( f )
			f->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );
	}
	else	{
		this->font = SdlFont::get( eFontSize, bLightFont );
		if( this->font )
			this->font->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );
	}

	GuiText::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiText::GuiText(TiXmlElement *el, GuiElement *p): GuiElement( el, p )
{
	font = 0;
	nBufferSize = 16;
	chBuffer = new char[nBufferSize];
	pText = 0;
	bThemeAlign = false;
	eFontSize = fs_theme;
	bThemeFont = true;
	bLightFont = false;

	eVAlign = SdlFont::baseline;
	eHAlign = SdlFont::left;

	curpos = -1;
	showcursor = false;
	bWordWrap = true;
	SdlFont *f = SdlFont::get( 0, true );
	if( f )	{
		f->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );
	}
}

GuiText::~GuiText()
{
	if( chBuffer )	{
		pText = NULL;
		delete[] chBuffer;
		chBuffer = NULL;
		nBufferSize = 0;
	}
}

void GuiText::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str;
		bThemeAlign = true;
		str = el->Attribute("VAlign");
		if(str)	{
			eVAlign = SdlFont::strToVAlign(str);
			bThemeAlign = false;
		}
		str = el->Attribute("HAlign");
		if(str)	{
			eHAlign = SdlFont::strToHAlign(str);
			bThemeAlign = false;
		}
		str = el->Attribute("ThemeAlign");
		if( str )
			bThemeAlign = strToBool(str);
		str = el->Attribute("FontSize");
		if(str)	{
			eFontSize = strToSize(str);
		}
		bThemeFont = eFontSize == fs_theme;
		str = el->Attribute("LightFont");
		bLightFont = str != NULL && strToBool(str);
		if(eFontSize != fs_theme)
			chooseFont(eFontSize, bLightFont);
		str = el->Attribute("Text");
		if(str)
			setText( str );
	}
	GuiElement::onInit();
}

void GuiText::setText(const char *t)
{
	if( t )	{
		int len = (int)strlen( t );
		if( len >= 0xFFFF )	{
			BasicApp::the->logLevelLine("WARNING: GuiText::setText: the length of the specified param is above 0xFFFE.");
			return;
		}
		if( len >= nBufferSize )	{
			if( chBuffer )
				delete[] chBuffer;
			nBufferSize = (strlen( t ) & 0xFF00) + 0x100;
			chBuffer = new char[nBufferSize];
		}
		strcpy( chBuffer, t );
		pText = chBuffer;
	}
	else pText = NULL;
	setRedraw();
}

void GuiText::addText( const char *t )
{
	if( t == NULL )
		return;
	if( pText == NULL )	{
		setText( t );
		setRedraw();
		return;
	}
	if( (int)(strlen( t ) + strlen( chBuffer )) >= nBufferSize )	{
		if( (int)(strlen( t ) + strlen( pText )) >= nBufferSize )	{
			nBufferSize = ((strlen( t ) + strlen( pText )) & 0xFF00) + 0x100;
			char *chTmp = new char[nBufferSize];
			strcpy( chTmp, pText );
			delete[] chBuffer;
			chBuffer = chTmp;
		}
		else
			strcpy( chBuffer, pText );
		pText = chBuffer;
	}
	strcat( pText, t );
	setRedraw();
}

void GuiText::addLine( const char *t )
{
	if( t == NULL )
		t = "";
	char *tmp = new char[strlen(t)+2];
	*tmp = '\n';
	strcpy( tmp+1, t );
	addText( tmp );
	delete[] tmp;
	setRedraw();
}

bool GuiText::clearLine( int l )
{
	if( pText == NULL )
		return false;
	if( l <= 0 )	{
		pText = strchr( pText, '\n' );
		if( pText )	{
			++pText;
			setRedraw();
		}
		return true;
	}
	char *p = pText;
	int i;
	for( i = 1; (p = strchr(p, '\n')) != NULL; ++i )	{
		if( i == l )	{
			char *pLineEnd = strchr(p+1, '\n');
			if( pLineEnd )
				strcpy( p, pLineEnd );
			else
				*p = '\0';
			setRedraw();
			return true;
		}
		++p;
	}
	return false;
}

int GuiText::numLines()
{
	if( pText == 0 )
		return 0;
	char *p = pText;
	int i;
	for( i = 1; (p = strchr(p, '\n')) != NULL; ++i ) ++p;
	return i;
}

void GuiText::chooseFont( FontSize size, bool light )
{
	bLightFont = light;
	eFontSize = size;
	font = SdlFont::get( eFontSize, bLightFont );
	if( font )
		font->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );

/*	if(stdfont)
		charsize = gui->getfont()->getbmp()->surf()->getSize();
	else
		switch(fontsz)	{
		case fs_text:
			charsize = gui->gettextfont(lightfont)->getbmp()->surf()->getSize();
			break;
		case fs_button:
			charsize = gui->getbuttonfont(lightfont)->getbmp()->surf()->getSize();
			break;
		case fs_heading:
			charsize = gui->getheadingfont(lightfont)->getbmp()->surf()->getSize();
			break;
		}*/
	setRedraw();
}

void GuiText::setFont( SdlFont *f )
{
	if( font == f )
		return;
	font = f;
	if( font )
		font->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );
	setRedraw();
}

Uint32 GuiText::onFrame( SdlSurface *dst )
{
	if( font )	{
		SDL_Rect abs = getDrawWindow();
		SDL_Rect clip = getClip();
		inflateRect( abs, -3, -3 );
		dst->setClip( clip );
		if( pText )
			if( bWordWrap )
				font->write( dst, pText, abs, eVAlign, eHAlign, &clip, showcursor ? curpos : -1 );
			else	{
				switch( eVAlign )	{
				case SdlFont::baseline:
					abs.y += abs.h/2;
					break;
				case SdlFont::bottom:
					abs.y += abs.h;
					break;
				default:;
				}
				switch( eHAlign )	{
				case SdlFont::center:
					abs.x += abs.w/2;
					break;
				case SdlFont::right:
					abs.x += abs.w;
					break;
				default:;
				}
				font->write( dst, pText, abs.x, abs.y, eVAlign, eHAlign, &clip, showcursor ? curpos : -1 );
			}
		else
			font->write( dst, "", abs, eVAlign, eHAlign, &clip, showcursor ? 0 : -1 );
	}
	return GuiElement::onFrame( dst );
}

Uint32 GuiText::onGLFrame()
{
	if( font )
	{
		SDL_Rect abs = getDrawWindow();
		SDL_Rect clip = getClip();
		inflateRect( abs, -3, -3 );
		if( pText )
			if( bWordWrap )
				font->writeGL( pText, abs, eVAlign, eHAlign, &clip, showcursor ? curpos : -1 );
			else
			{
				switch( eVAlign )
				{
				case SdlFont::baseline:
					abs.y += abs.h/2;
					break;
				case SdlFont::bottom:
					abs.y += abs.h;
					break;
				default:;
				}
				switch( eHAlign )
				{
				case SdlFont::center:
					abs.x += abs.w/2;
					break;
				case SdlFont::right:
					abs.x += abs.w;
					break;
				default:;
				}
				font->writeGL( pText, abs.x, abs.y, eVAlign, eHAlign, &clip, showcursor ? curpos : -1 );
			}
		else
			font->writeGL( "", abs, eVAlign, eHAlign, &clip, showcursor ? 0 : -1 );
	}
	return GuiElement::onGLFrame();
}

Uint32 GuiText::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiElement::onThemeChanged( t );
	if( t == 0 )
		return r;

	if( bThemeAlign )	{
		eVAlign = (VAlign)t->getUint32( ( string(getClassName()) + "->valign" ).c_str() );
		eHAlign = (HAlign)t->getUint32( ( string(getClassName()) + "->halign" ).c_str() );
	}
	if( bThemeFont )	{
		bLightFont = t->getBool( ( string(getClassName()) + "->font-light" ).c_str() );
		eFontSize = (FontSize)t->getUint32( ( string(getClassName()) + "->font-size" ).c_str() );
		font = fontSizeToFont( eFontSize, bLightFont, t );
		if( font )
			font->getBmps()->getImage( 0 )->getSize( nCharWidth, nCharHeight );
	}

	return r;
}

SdlFont *GuiText::fontSizeToFont( FontSize s, bool l, UiTheme *t )
{
	if( t == 0 )
		return 0;

	switch( s )	{
	case fs_text:
		return SdlFont::get( t->getUint32("font-text"), l );

	case fs_button:
		return SdlFont::get( t->getUint32("font-button"), l );

	case fs_heading:
		return SdlFont::get( t->getUint32("font-heading"), l );

	case fs_standard:
		return SdlFont::get( t->getUint32("font-standard"), l );

	default:
		return 0;
	}
}

//////////////////////////////////////////////////////////////////////
// GuiTextInput

const unsigned int GuiTextInput::nKeys = 128;//58
// German key-code:
char GuiTextInput::alpha[nKeys+1]	=	"AA1234567890ߴAAqwertzuiop+AAasdfghjkl�^A#yxcvbnm,.-AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA<AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
char GuiTextInput::ALPHA[nKeys+1]	=	"  !\"$%&/()=?`  QWERTZUIOP�  ASDFGHJKL� 'YXCVBNM;:_                                >AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
char GuiTextInput::altAlpha[nKeys+1]=	"      {[]}\\   @         ~                                                         |                                         ";
char GuiTextInput::MOD_LETTER = 'A';
/*const SDLKey GuiTextInput::code[count] = {SDLK_a,SDLK_b,SDLK_c,SDLK_d,SDLK_e,SDLK_f,SDLK_g,SDLK_h,SDLK_i,SDLK_j,SDLK_k,
	SDLK_l,SDLK_m,SDLK_n,SDLK_o,SDLK_p,SDLK_q,SDLK_r,SDLK_s,SDLK_t,SDLK_u,SDLK_v,SDLK_w,SDLK_x,SDLK_y,SDLK_z,
	SDLK_SPACE, SDLK_1,SDLK_2,SDLK_3,SDLK_4,SDLK_5,SDLK_6,SDLK_7,SDLK_8,SDLK_9,SDLK_0,
	SDLK_MINUS,SDLK_PLUS,SDLK_EQUALS,SDLK_SEMICOLON,SDLK_QUOTE,SDLK_CARET,SDLK_COMMA,SDLK_PERIOD,SDLK_SLASH,SDLK_BACKSLASH,SDLK_RETURN};
*/

GuiTextInput::GuiTextInput( const SDL_Rect &r, GuiElement *p, const char *name, const char *t, bool singleline )
	: GuiText( r,p, name,t )
{
	if( !t )
		setText( "" );
	bSingleLine = singleline;
	hasfocus = false;
	showcursor = false;
	curpos = 0;
	maxlen = -1;
	cb.func = NULL;

	cActive = 0x80000000;
	cInactive = 0x80000000;
	bwActive = 0x80000000;
	bwInactive = 0x80000000;
	GuiTextInput::onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiTextInput::GuiTextInput( TiXmlElement *el, GuiElement *p )
	: GuiText( el, p )
{
	if( !el->Attribute("Text") )
		setText( "" );
	bSingleLine = true;
	hasfocus = false;
	showcursor = false;
	curpos = 0;
	maxlen = -1;
	cb.func = NULL;

	cActive = 0x80000000;
	cInactive = 0x80000000;
	bwActive = 0x80000000;
	bwInactive = 0x80000000;
	blinkTimer = NULL;
}

GuiTextInput::~GuiTextInput()
{
	if( hasfocus )	{
		FrameTimer::the->removeCallback(blinkTimer);
		BasicApp::the->logLevelLine("WARNING: ~GuiTextInput: focus shall be released before by a destroy()-call");
	}
}

void GuiTextInput::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("SingleLine");
		if(str)
			bSingleLine = strToBool(str);
		str = el->Attribute("MaxLen");
		if(str)
			maxlen = atoi(str);
		str = el->Attribute("OnReturn");
		if(str)
			cb = getFunctionByName(str);
	}
	GuiText::onInit();
}

Uint32 GuiTextInput::onMouseDown( SDL_MouseButtonEvent *ev )
{
	UiManager::the->setFocus( ftKeyboard, this );
/*	if(button & 1)	{
		curpos = relpos...;
	}*/
	return 0;
}

Uint32 GuiTextInput::onKeyDown( SDL_KeyboardEvent *ev )
{
	int len = getText()? strlen( getText() ) : 0;
	if( curpos > len )
		curpos = len;
	int len1 = curpos,
		lPostCur = len - curpos;

	char *buf = new char[len + 2];
	if( getText() )
		strcpy( buf, getText() );
	else
		buf[ 0 ] = '\0';
	buf[ len + 1 ] = '\0';

	switch( ev->keysym.sym )	{
	case SDLK_BACKSPACE:
		if( len1 > 0 )	{
			--curpos;
			strcpy( buf + curpos, buf + curpos + 1 );
		}
		break;
	case SDLK_DELETE:
		if( lPostCur > 0 )
			strcpy( buf + curpos, buf + curpos + 1 );
		break;
	case SDLK_LEFT:
		if( curpos > 0 )
			--curpos;
		break;
	case SDLK_RIGHT:
		if( curpos < len )
			++curpos;
		break;
	case SDLK_UP:
		if( curpos <= 0 || bSingleLine )
			break;
		{
			char *pLn2 = NULL, *pnew;
			for( pnew = buf + curpos-1; pnew >= buf; --pnew )
				if( *pnew == '\n' )
					if( pLn2 == NULL )
						pLn2 = pnew+1;
					else
						break;
			if( pLn2 )	{
				curpos -= pLn2 - pnew - 1;
				if( curpos >= pLn2 - buf )
					curpos = pLn2 - buf - 1;
			}
		}
		break;
	case SDLK_DOWN:// todo: debug
		if( curpos >= len || bSingleLine )
			break;
		{
			char *pLn2, *pLn1;
			for( pLn1 = buf + curpos; pLn1 > buf; --pLn1 )
				if( *pLn1 == '\n' )	{
					if( pLn1 != buf + curpos )
						++pLn1;
					else if( curpos != 0 )
						continue;
					break;
				}
			for( pLn2 = buf + curpos; *pLn2 != '\0'; ++pLn2 )
				if( *pLn2 == '\n' )	{
					++pLn2;
					break;
				}
			if( *pLn2 != '\0' )	{
				curpos += pLn2 - pLn1;
				if( buf + curpos > pLn2 + strlen( pLn2 ) )
					curpos = pLn2 - buf + strlen( pLn2 );
			}
		}
		break;
	case SDLK_HOME:
		if( bSingleLine )
			curpos = 0;
		else
			while( curpos > 0 && buf[ curpos - 1 ] != '\n' ) --curpos;
		break;
	case SDLK_END:
		if( bSingleLine )
			curpos = len;
		else
			while( buf[ curpos ] != '\0' && buf[ curpos ] != '\n' ) ++curpos;
		break;
	case SDLK_RETURN:
		if( bSingleLine )	{
			if( cb.func )
				(*cb.func)( this, cb.obj, NULL );
			delete[] buf;
			return 0;// avoid setText(buf), because in (*cb) the text could have been changed
		}
		if( maxlen == len )
			break;
		buf[ curpos ] = '\n';
		if( curpos < len )
			strcpy( buf + curpos + 1, getText() + curpos );
		++curpos;
		break;
	default:
		{
			if( maxlen == len || ev->keysym.scancode >= nKeys || alpha[ev->keysym.scancode] == MOD_LETTER )
				break;

			const char *pAlpha;
			if( (ev->keysym.mod & KMOD_MODE) != 0 )	{
				if ( altAlpha[ev->keysym.scancode] == ' ' )// break, if it's no valid alt-key
					break;
				else
					pAlpha = altAlpha;
			}
			else
				pAlpha = ( ( ev->keysym.mod & KMOD_SHIFT ) != 0 ) ? ALPHA : alpha;
			buf[ curpos ] = pAlpha[ev->keysym.scancode];
			if( curpos < len )
				strcpy( buf + curpos + 1, getText() + curpos );
			++curpos;
		}
	}
	
	setText( buf );
	delete[] buf;
	
	setRedraw();
	return 0;
}

Uint32 GuiTextInput::onKeyUp( SDL_KeyboardEvent *ev )
{
	return 0;
}

Uint32 GuiTextInput::onFocus( FocusTypes gained )
{
	if(gained != ftKeyboard)
		return 0;
	hasfocus = true;
	showcursor = true;
	setRedraw();
	blinkTimer = FrameTimer::the->addCallback(refresh, this, nBlinkIv);
	return 0;
}

Uint32 GuiTextInput::onRelease( FocusTypes lost )
{
	if(lost != ftKeyboard)
		return 0;
	FrameTimer::the->removeCallback(blinkTimer);
	hasfocus = false;
	showcursor = false;
	setRedraw();
	return 0;
}

Uint32 GuiTextInput::onFrame( SdlSurface *dst )
{
	Uint32 hr = GuiText::onFrame( dst );
	int width;
	Uint32 color;
	if( hasfocus )	{
		width = bwActive;
		color = cActive & ~0x80000000;
		//showcursor = ( SDL_GetTicks() % nBlinkIv ) > ( nBlinkIv / 2 );
	}
	else	{
		width = bwInactive;
		color = cInactive & ~0x80000000;
	}
	dst->setClip( getClip() );
	SDL_Rect r = getDrawWindow(), clear;
	clear = r;
	clear.h = width;
	dst->fillARGB( &clear, color );
	clear = r;
	clear.y = r.y + r.h - width;
	dst->fillARGB( &clear, color );
	clear = r;
	clear.w = width;
	dst->fillARGB( &clear, color );
	clear = r;
	clear.x = r.x + r.w - width;
	dst->fillARGB( &clear, color );
	return hr;
}

Uint32 GuiTextInput::onGLFrame()
{
	Uint32 hr = GuiText::onGLFrame();
	int width;
	Uint32 color;
	if( hasfocus )	{
		width = bwActive;
		color = cActive & ~0x80000000;
		//showcursor = ( SDL_GetTicks() % nBlinkIv ) > ( nBlinkIv / 2 );
	}
	else	{
		width = bwInactive;
		color = cInactive & ~0x80000000;
	}
	
	ogl::set2dClippingPlanes( getClip() );
	SDL_Rect r = getDrawWindow(), clear;
	GLint x[4] = {r.x+1, r.x+r.w, r.x+r.w, r.x+1},
		  y[4] = {r.y, r.y, r.y+r.h, r.y+r.h};
	ogl::drawPlainLineLoop( x, y, 4, 2.f, color );
	ogl::disableClippingPlanes( 4 );
	
	return hr;
}

Uint32 GuiTextInput::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiText::onThemeChanged( t );
	if( t == 0 )
		return r;
		
	char *chCode = "[KEYBOARD-CODE-MOD_LETTER]";
	if( t->translate( chCode ) != chCode )
		MOD_LETTER = t->translate( chCode )[1];

	chCode = "[KEYBOARD-CODE-SMALL]";
	if( t->translate( chCode ) != chCode )	{
		memcpy( alpha, t->translate( chCode )+1, nKeys - 1 );
		for( int i = strlen( alpha ); i < (int)nKeys; ++i )
			alpha[i] = MOD_LETTER;
	}

	chCode = "[KEYBOARD-CODE-BIG]";
	if( t->translate( chCode ) != chCode )
		memcpy( ALPHA, t->translate( chCode )+1, nKeys - 1 );

	chCode = "[KEYBOARD-CODE-ALT]";
	if( t->translate( chCode ) != chCode )
		memcpy( altAlpha, t->translate( chCode )+1, nKeys - 1 );

//	BasicApp::the->logLine(alpha);
//	BasicApp::the->logLine(ALPHA);
//	BasicApp::the->logLine(altAlpha);

	if( ( bwInactive & 0x80000000 ) != 0 )
		bwInactive = t->getUint32( ( string(getClassName()) + "->borderwidth-inactive" ).c_str() ) | 0x80000000;
	if( ( bwActive & 0x80000000 ) != 0 )
		bwActive = t->getUint32( ( string(getClassName()) + "->borderwidth-active" ).c_str() ) | 0x80000000;
	if( ( cInactive & 0x80000000 ) != 0 )
		cInactive = t->getUint32( ( string(getClassName()) + "->bordercolor-inactive" ).c_str() ) | 0x80000000;
	if( ( cActive & 0x80000000 ) != 0 )
		cActive = t->getUint32( ( string(getClassName()) + "->bordercolor-active" ).c_str() ) | 0x80000000;

	nBlinkIv = t->getUint32( "textcursor-blinkinterval" );

	return r;
}

void GuiTextInput::setMaxlen(int l)
{
	if( maxlen == l )
		return;
	if( l < 0 )
		maxlen = MAX_SINT32;
	else
		if( maxlen < l )
			maxlen = l;
	else	{
		maxlen = l;
		char * buf = new char[l + 1];
		strncpy( buf, getText(), l );
		setText( buf );
		if( curpos > l )
			curpos = l;
	}
}

//////////////////////////////////////////////////////////////////////
// GuiKeyControl

const unsigned int GuiKeyControl::count = 50;

const char *GuiKeyControl::alpha[count] = 
{
	"0","1","2","3","4","5","6","7","8","9",
	"A","B","C","D","E","F","G","H","I","J","K","L","M",
	"N","O","P","Q","R","S","T","U","V","W","X","Y","Z",
	"Numpad 0","Numpad 1","Numpad 2","Numpad 3","Numpad 4","Numpad 5","Numpad 6","Numpad 7","Numpad 8","Numpad 9",
	"Up", "Down", "Right", "Left"
};
const SDLKey GuiKeyControl::code[count] = 
{
	SDLK_0,SDLK_1,SDLK_2,SDLK_3,SDLK_4,SDLK_5,SDLK_6,SDLK_7,SDLK_8,SDLK_9,
	SDLK_a,SDLK_b,SDLK_c,SDLK_d,SDLK_e,SDLK_f,SDLK_g,SDLK_h,SDLK_i,SDLK_j,SDLK_k,SDLK_l,SDLK_m,
	SDLK_n,SDLK_o,SDLK_p,SDLK_q,SDLK_r,SDLK_s,SDLK_t,SDLK_u,SDLK_v,SDLK_w,SDLK_x,SDLK_y,SDLK_z,
	SDLK_KP0,SDLK_KP1,SDLK_KP2,SDLK_KP3,SDLK_KP4,SDLK_KP5,SDLK_KP6,SDLK_KP7,SDLK_KP8,SDLK_KP9,
	SDLK_UP, SDLK_DOWN, SDLK_RIGHT, SDLK_LEFT
};


GuiKeyControl::GuiKeyControl( const SDL_Rect &r, GuiElement *p, const char *name ) : GuiText( r, p, name )
{
	hasfocus = false;
}

GuiKeyControl::GuiKeyControl( TiXmlElement *el, GuiElement *p ) : GuiText( el, p )
{
	hasfocus = false;
}

GuiKeyControl::~GuiKeyControl()
{
}

void GuiKeyControl::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiText::onInit();
}

Uint32 GuiKeyControl::onMouseDown( SDL_MouseButtonEvent *ev )
{
	UiManager::the->setFocus( ftKeyboard, this );
	return 0;
}

Uint32 GuiKeyControl::onKeyDown( SDL_KeyboardEvent *ev )
{
	key = ev->keysym;
	update();
	
	return 0;
}

void GuiKeyControl::setKey( const SDL_keysym &k )
{
	key = k;
	update();
}

void GuiKeyControl::update()
{
	for( int i = 0; i < (int)count; ++i )
		if( code[i] == key.sym )	{
			setText( alpha[i] );
			return;
		}

	setText("Unknown");
}

Uint32 GuiKeyControl::onFocus()
{
	hasfocus = true;
	return 0;
}

Uint32 GuiKeyControl::onRelease()
{
	hasfocus = false;
	return 0;
}

Uint32 GuiKeyControl::onFrame( SdlSurface *dst )
{
	Uint32 hr = GuiText::onFrame( dst );
/*	if( hasfocus )	{
		SDL_Rect &clip = getClip();
		int width = 2;
		Uint32 color = RGB(0,0,255);
		SDL_Rect r = getDrawWindow(), clear;
		clear = r;
		clear.bottom = r.top + width;
		dd->getbk()->clear(&clear,color,&clip);
		clear = r;
		clear.top = r.bottom - width;
		dd->getbk()->clear(&clear,color,&clip);
		clear = r;
		clear.right = r.left + width;
		dd->getbk()->clear(&clear,color,&clip);
		clear = r;
		clear.left = r.right - width;
		dd->getbk()->clear(&clear,color,&clip);
	}*/
	return hr;
}

Uint32 GuiKeyControl::onGLFrame()
{
	return GuiText::onGLFrame();
}

//////////////////////////////////////////////////////////////////////
// GuiFrameCounter


GuiFrameCounter::GuiFrameCounter( const SDL_Rect &r, GuiElement *p, const char *name ) : GuiText( r, p, name )
{
	frame = 0;
	time = 0;
	onThemeChanged( ThemeManager::the? ThemeManager::the->getActiveTheme() : 0 );
}

GuiFrameCounter::GuiFrameCounter( TiXmlElement *el, GuiElement *p ) : GuiText( el, p )
{
	frame = 0;
	time = 0;
}

GuiFrameCounter::~GuiFrameCounter()
{
}

void GuiFrameCounter::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiText::onInit();
}

Uint32 GuiFrameCounter::onFrame( SdlSurface *dst )
{
	time += UiManager::the->getTimePerFrame();
	++frame;

	if( frame >= 30 )	{
		int f = frame * 1000 / time;
		setText( intToStr( f, 10 ) );
		time = 0;
		frame = 0;
	}
	return GuiText::onFrame( dst );
}

Uint32 GuiFrameCounter::onGLFrame()
{
	time += UiManager::the->getTimePerFrame();
	++frame;

	if( frame >= 30 )	{
		int f = frame * 1000 / time;
		setText( intToStr( f, 10 ) );
		time = 0;
		frame = 0;
	}
	return GuiText::onGLFrame();
}
