/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "netmanager.h"
#include <string>
#include "netsocket.h"
#include "basicapp.h"
#include "functions.h"
#ifndef __windows__
#include <netdb.h>
#include <unistd.h>
#endif

unsigned int NetBasicManager::nSDLNetRefCount = 0;

NetBasicManager *NetBasicManager::the = 0;
NetBasicManager *NetBasicManager::getThe()
{
	return the;
}

bool NetBasicManager::initSDL()
{
	if( nSDLNetRefCount > 0 )	{
		nSDLNetRefCount++;
		return true;
	}
	if( SDLNet_Init() != -1 )	{
		nSDLNetRefCount++;
		return true;
	}
	return false;
}

void NetBasicManager::quitSDL()
{
	if( nSDLNetRefCount != 0 )	{
		if( --nSDLNetRefCount == 0 )
			SDLNet_Quit();
	}
}


NetBasicManager::NetBasicManager(const char *name, BasicObject *p): BasicObject(name, p)
{
	the = this;
	bInited = false;
	sockServer = 0;
	socksServer = 0;
	socksPeers = 0;
	sockUDP = NULL;
	cbServer.func = NULL;
	nPeersConnected = 0;
//	chLocalName = NULL;
	nUDPMaxSize = 1024;
}

NetBasicManager::NetBasicManager(TiXmlElement *el, BasicObject *p): BasicObject(el, p)
{
	the = this;
	bInited = false;
	sockServer = 0;
	socksServer = 0;
	socksPeers = 0;
	sockUDP = NULL;
	cbServer.func = NULL;
	nPeersConnected = 0;
//	chLocalName = NULL;
	nUDPMaxSize = 1024;
}

NetBasicManager::~NetBasicManager()
{
	destroy();
	the = 0;
}

void NetBasicManager::onInit()
{
	unsigned int port = 0, udpport = NETWORK_UDPPORT;
	if(xmlData)	{
		const char *str = NULL;
		str = xmlData->Attribute("Port");
		if(str)	{
			port = xStrToUL(str);
			udpport = port+1;
		}
		str = xmlData->Attribute("UDPPort");
		if(str)
			udpport = xStrToUL(str);
		
		if(children.size() == 0)	{
			for(TiXmlElement *el = xmlData->FirstChildElement(); el; el = el->NextSiblingElement())	{
				Callback func = BasicSlot::getFunction("new", el->Value());
				if(!func)	{
					BasicApp::the->logLine("Error: NetBasicManager::onInit(): no function to create socket: ");
					BasicApp::the->logString(el->Value());
					continue;
				}
				NetSocket *r;
				(*func)(&r, this, el);
			}
		}
		str = xmlData->Attribute("UDPMaxSize");
		if(str)
			nUDPMaxSize = xStrToUL(str);
	}
	if( port == 0 )
		port = NETWORK_PORT;
	
	init(port, udpport, nUDPMaxSize);
	
	BasicObject::onInit();
}

bool NetBasicManager::init(unsigned int port, unsigned int udpport, int udpmaxsize)
{
	BasicApp::the->logBegin( "Initializing basic network" );
	if( bInited )	{
		if( (unsigned int)nListenPort != port )	{
			destroy();
			// SDL_Delay(1000);
		}
		else if( !sockUDP || (nUDPPort != udpport && udpport != 0) )	{
			if( sockUDP )
				SDLNet_UDP_Close( sockUDP );
			sockUDP = SDLNet_UDP_Open( udpport );
			nUDPPort = udpport;
			if( (int)nUDPPort != nListenPort+1 )	{
				BasicApp::the->logLevelLine("Warning: UDP port number is not <TCP port number> + 1");
			}
			return true;// always return true, because TCP is still initialized
		}
		else	{
			BasicApp::the->logEnd( "not neccesary." );
			return true;// don't init again, if no changes
		}
	}

	bInited = initSDL();
	if( !bInited )	{
		BasicApp::the->logEnd( "ERROR: " );
		BasicApp::the->logString( SDL_GetError() );
		return false;
	}
	localIP();// call it once to make it thread-safe

	if( port )	{
		IPaddress ip;
		SDLNet_ResolveHost( &ip, NULL, port );
		
#ifdef __windows__
		BasicApp::the->logLevelLine( "Using IP: " );
		char buf[33];
		BasicApp::the->logString( IPToStr( ip, buf ) );
#endif
		socksServer = SDLNet_AllocSocketSet( 1 );
		if( !socksServer )	{
			BasicApp::the->logEnd( (string("ERROR: ") + SDL_GetError()).c_str() );
			destroy();
			return false;
		}
		sockServer = SDLNet_TCP_Open( &ip );
		if( sockServer == 0 )	{
			BasicApp::the->logEnd( (string("ERROR: open server socket:") + SDL_GetError()).c_str() );
			destroy();
			return false;
		}
		if( SDLNet_TCP_AddSocket( socksServer, sockServer ) == -1 )	{
			BasicApp::the->logEnd( (string("ERROR: ") + SDL_GetError()).c_str() );
			destroy();
			return false;
		}
	}
	socksPeers = SDLNet_AllocSocketSet( MAX_PEERS );
	if( !socksPeers )	{
		BasicApp::the->logEnd( (string("ERROR: ") + SDL_GetError()).c_str() );
		destroy();
		return false;
	}
	nListenPort = port;
	
	// setup UDP
	nUDPMaxSize = udpmaxsize;
	ppackUDPrecv = SDLNet_AllocPacket( nUDPMaxSize );
	ppackUDPsend = SDLNet_AllocPacket( nUDPMaxSize );
	sockUDP = SDLNet_UDP_Open( udpport );
	nUDPPort = udpport;
	if( sockUDP == NULL )	{
		BasicApp::the->logLevelLine( "Warning: UDP-socket could not be initialized!" );
		BasicApp::the->logLevelLine( (string("Error: ") + SDL_GetError()).c_str() );
	}
	else if( (int)nUDPPort != nListenPort+1 )	{
		BasicApp::the->logLevelLine("Warning: UDP port number is not <TCP port number> + 1");
	}
	
	BasicApp::the->logEnd( "done (NetManager)." );
	return true;
}

void NetBasicManager::onDestroy()
{
	BasicObject::onDestroy();
	
	if( !bInited )
		return;
	BasicApp::the->logBegin( "Destroying network..." );

	nPeersConnected = 0;
	SDLNet_FreePacket(ppackUDPsend);
	SDLNet_FreePacket(ppackUDPrecv);
	if( sockUDP )	{
		SDLNet_UDP_Close( sockUDP );
		sockUDP = NULL;
	}
	if( sockServer )	{
		SDLNet_TCP_Close( sockServer );
		sockServer = 0;
	}
	if( socksPeers )	{
		SDLNet_FreeSocketSet( socksPeers );
		socksPeers = 0;
	}
	if( socksServer )	{
		SDLNet_FreeSocketSet( socksServer );
		socksServer = 0;
	}
/*	if( chLocalName )	{
		delete[] chLocalName;
		chLocalName = NULL;
	}*/
	quitSDL();
	bInited = false;

	BasicApp::the->logEnd( "done." );
}

unsigned int NetBasicManager::send( char *data, int size )
{
	int nRecievers = 0;
	for( child_iter i = children.begin(); i != children.end(); ++i )
		if( ((NetSocket*)*i)->sendData( data, size ) )
			++nRecievers;
	
	return nRecievers;
}

bool NetBasicManager::sendUDP( char *data, int size, const IPaddress &ip )
{
	if(size > (int)nUDPMaxSize)	{
		BasicApp::the->logLevelLine("Warning: NetBasicManager::sendUDP called with too big size");
		return false;
	}
	memcpy(&ppackUDPsend->address, &ip, sizeof(IPaddress));
	memcpy(ppackUDPsend->data, data, size);
	return SDLNet_UDP_Send(sockUDP, -1, ppackUDPsend) > 0;
}

bool NetBasicManager::checkPeers()
{
	if( !bInited )
		return false;
	// check UDP
	if( sockUDP )	{
		if(SDLNet_UDP_Recv(sockUDP, ppackUDPrecv) == 1)	{
			onReceiveUDP(ppackUDPrecv);
		}
	}
	// check TCP
	if( nPeersConnected == 0 )
		return false;
	int n = SDLNet_CheckSockets( socksPeers, 0 );
	if( n == 0 )
		return false;
	if( n == -1 )	{
		BasicApp::the->logLevelLine( (string("ERROR: checkPeers(): ") + SDL_GetError()).c_str() );
		return false;
	}
	for( child_iter i = children.begin(); n != 0 && i != children.end(); ++i )	{
		NetSocket *s = ((NetSocket*)*i);
		if( SDLNet_SocketReady( s->sock ) == 0 )
			continue;
		--n;
		if( s->isActive() )// ignore message, if already active (usually in another thread)
			continue;
		if(!s->onReceive())
			s->disconnect();
	}

	return true;
}

bool NetBasicManager::checkServer()
{
	if( !bInited || !sockServer )
		return false;

	switch( SDLNet_CheckSockets( socksServer, 0 ) )
	{
	case -1:
		BasicApp::the->logLevelLine( (string("ERROR: checkServer(): ") + SDL_GetError()).c_str() );
	case 0:
		return false;
	}

	TCPsocket s = SDLNet_TCP_Accept( sockServer );
	if( s == 0 )
		return false;
	char buf[33];
	BasicApp::the->logLevelLine( (string("Accept: ") + IPToStr(*SDLNet_TCP_GetPeerAddress( s ), buf )).c_str() );

	NetSocket *ns = createSocket( s );
	if( !ns->isValid() )	{
		ns->disconnect();
		return false;
	}
	
	if( cbServer.func )	{
		bool allow = true;
		(*cbServer.func)( ns, cbServer.obj, &allow );
		if(!allow)	{
			ns->disconnect();
			return false;
		}
	}

	return true;
}

void NetBasicManager::onReceiveUDP(UDPpacket *udp)
{
	udp->address.port--;// UDP port number must be the one after the TCP port number
	const IPaddress *ip;
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		ip = ((NetSocket*)*i)->getAddress();
		if( !ip )
			continue;
		if( memcmp(ip, &udp->address, sizeof(IPaddress)) == 0 )	{
			((NetSocket*)*i)->onReceiveUDP(*udp);
		}
	}
	if( cbUDP.func )	{
		(*cbUDP.func)( this, cbUDP.obj, udp );
	}
}


NetSocket *NetBasicManager::getSocket( const IPaddress &ip )
{
	if( ip.port == 0 )	{
		for( child_iter i = children.begin(); i != children.end(); ++i )	{
			const IPaddress *addr = ((NetSocket*)*i)->getAddress();
			if(addr && addr->host == ip.host)
				return (NetSocket*)*i;
		}
	}
	else	{
		for( child_iter i = children.begin(); i != children.end(); ++i )	{
			if( memcmp( ((NetSocket *)*i)->getAddress(), &ip, sizeof(IPaddress) ) == 0 )
				return (NetSocket*)*i;
		}
	}
	return 0;
}

void NetBasicManager::deleteDisconnected()
{
	child_iter i = children.begin();
	while( i != children.end() )	{
		NetSocket *s = (NetSocket*)*i;
		if( !s->connected() && s->deleteOnDisconnected() )	{
			++i;
			delete s;
		}
		else
			++i;
	}
}

bool NetBasicManager::registerSocket( TCPsocket sock )
{
	int i = SDLNet_TCP_AddSocket( socksPeers, sock );
	if( i == -1 )
		return false;

	nPeersConnected = i;
	return true;
}

void NetBasicManager::unregisterSocket( TCPsocket sock )
{
	int i = SDLNet_TCP_DelSocket( socksPeers, sock );
	if( i == -1 )	{
		BasicApp::the->logLevelLine( "ERROR: Could not unregister a NetSocket." );
		return;
	}

	nPeersConnected = i;
}

/// \note this method is only IPv4 compatible
const char *NetBasicManager::localIP()
// thread-safe after calling it once
{/// \test verify result
static char buf[64] = {'\0'};
	if( buf[0] == 0 )	{
#ifdef __windows__
		IPaddress ip = { INADDR_ANY, localPort() };
		SDLNet_ResolveHost( &ip, SDLNet_ResolveIP( &ip ), localPort() );
		IPToStr( ip.host, buf );
#else
		char *strIP = getLocalIP();
		if( strIP )	{
			strcpy( buf, strIP );
			delete[] strIP;
		}
/*		char strHostname[128];
		if(gethostname(strHostname, 128) < 0)	{
			BasicApp::the->logLevelLine("ERROR: hostname too long, local IP cannot be resolved.");
			strcpy( buf, "?" );
		}
		hostent *pHostEntry = gethostbyname2(strHostname, AF_INET);
		if(pHostEntry && pHostEntry->h_length)	{
			unsigned long ip = SDLNet_Read32(pHostEntry->h_addr);// h_addr is in NBO
			IPToStr(ip, buf);
		}
		else	{
			BasicApp::the->logLevelLine("ERROR: file \"/etc/hosts\" could not opened for reading.");
			strcpy( buf, "?" );
		}*/
#endif
	}
	return buf;
}




NetSimpleManager::NetSimpleManager(const char *name, BasicObject *p): NetBasicManager(name, p)
{
}

NetSimpleManager::NetSimpleManager(TiXmlElement *el, BasicObject *p): NetBasicManager(el, p)
{
}

NetSimpleManager::~NetSimpleManager()
{
}

NetSocket *NetSimpleManager::createSocket( TCPsocket socket )
{
	return new NetStandardSocket( "", this, socket );
}
