/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "netpeermanager.h"
#include <fstream>
#include "tinyxml.h"
#include "netpeersocket.h"
#include "basicapp.h"
#include "functions.h"

using namespace std;

NetPeerManager::NetPeerManager( const char *name, BasicObject *p, int maxpeers ): NetBasicManager( name, p )
{
	if( maxpeers < MAX_PEERS )
		nMaxPeers = maxpeers;
	else nMaxPeers = MAX_PEERS;
	chName[PEER_MAX_NAME] = '\0';
	chName[0] = '\0';
	bAllowNewConnections = true;
	BasicApp::the->logLevelLine("Created NetPeerManager");
}

NetPeerManager::NetPeerManager( TiXmlElement *el, BasicObject *p ): NetBasicManager( el, p )
{
	nMaxPeers = MAX_PEERS;
	chName[PEER_MAX_NAME] = '\0';
	bAllowNewConnections = true;
	BasicApp::the->logLevelLine("Created NetPeerManager");
}

NetPeerManager::~NetPeerManager()
{

}

void NetPeerManager::onInit()
{
	BasicApp::the->logBegin( "Initializing peer-network..." );
	
	if( xmlData )	{
		if( xmlData->FirstChild("Permissions") )
			loadPermissions( xmlData->FirstChild("Permissions") );
		else if( xmlData && xmlData->Attribute("PermissionFile") )
			loadPermissions( xmlData->Attribute("PermissionFile") );
		else	{
			BasicApp::the->logLevelLine( "Error: no permission settings found." );
		}
		if( xmlData->Attribute("MaxPeers") )	{
			nMaxPeers = xStrToUL(xmlData->Attribute("MaxPeers"));
			if( nMaxPeers > MAX_PEERS )
				nMaxPeers = MAX_PEERS;
		}
		if( xmlData->Attribute("Username") )
			setNetName( xmlData->Attribute("Username") );
		
		const char *str = xmlData->Attribute("OnPeerEvent");
		if(str)	{
			
				///\todo here
		}
	}
	NetBasicManager::onInit();
	BasicApp::the->logEnd( "done (NetPeerManager)." );
}

bool NetPeerManager::init( unsigned int port, const char *permissionfn )
{
	BasicApp::the->logBegin( "Initializing peer-network..." );
	loadPermissions( permissionfn );
	if( NetBasicManager::init( port ) )	{
		BasicApp::the->logEnd( "peer-network done." );
		return true;
	}
	BasicApp::the->logEnd( "failed." );
	return false;
}


void NetPeerManager::setNetName( const char *name )
{
	strncpy( chName, name, PEER_MAX_NAME );
}

NetPeerSocket *NetPeerManager::getSocketByName( const char *name )
{
	for( child_iter i = children.begin(); i != children.end(); ++i )
		if( strcmp( ((NetPeerSocket*)*i)->getNetName(), name ) == 0 )
			return (NetPeerSocket*)*i;

	return 0;
}

void NetPeerManager::requestPings()
{
	for( child_iter i = children.begin(); i != children.end(); ++i )
		((NetPeerSocket*)*i)->requestPing();
}

Uint32 NetPeerManager::getPings( Uint32 *min, Uint32 *max )
{
	Uint32 sum = 0;
	if( min )
		*min = ~0;
	if( max )
		*max = 0;
	int nUnconnected = 0;
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		NetPeerSocket *s = (NetPeerSocket*)*i;
		if( !s->connected() )	{
			++nUnconnected;
			continue;
		}
		sum += s->lastPing();
		if( min && *min > s->lastPing() )
			*min = s->lastPing();
		if( max && *max < s->lastPing() )
			*max = s->lastPing();
	}
	if( children.size() - nUnconnected == 0 )
		return 0;
	return sum / (children.size() - nUnconnected);
}

bool NetPeerManager::loadPermissions( TiXmlNode *node )
{
	BasicApp::the->logBegin( "reading permissions..." );
	TiXmlNode *n_allow;
	TiXmlElement *n_curr;
	HOST_INFO info;
	for( n_allow = node->FirstChild( "Allow" ); n_allow; n_allow = node->IterateChildren( "Allow", n_allow ) )	{
		info.bAllowed = true;
		for( n_curr = n_allow->FirstChildElement( "Player" ); n_curr; n_curr = n_curr->NextSiblingElement( "Player" ) )	{
			if( n_curr->Attribute( "IP" ) )	{
				info.ip = strToIP( n_curr->Attribute( "IP" ) );
			}
			else info.ip = 0;
			if( n_curr->Attribute( "Name" ) )	{
				strcpy( info.chName, n_curr->Attribute( "Name" ) );
			}
			else info.chName[0] = '\0';
			lPermissionInfos.push_back( info );
		}
	}
	for( n_allow = node->FirstChild( "Disallow" ); n_allow; n_allow = node->IterateChildren( "Disallow", n_allow ) )	{
		info.bAllowed = false;
		for( n_curr = n_allow->FirstChildElement( "Player" ); n_curr; n_curr = n_curr->NextSiblingElement( "Player" ) )	{
			if( n_curr->Attribute( "IP" ) )	{
				info.ip = strToIP( n_curr->Attribute( "IP" ) );
			}
			else info.ip = 0;
			if( n_curr->Attribute( "Name" ) )	{
				strcpy( info.chName, n_curr->Attribute( "Name" ) );
			}
			else info.chName[0] = '\0';
			lPermissionInfos.push_back( info );
		}
	}
	BasicApp::the->logEnd( "done." );
	return lPermissionInfos.size() > 0;
}

bool NetPeerManager::loadPermissions( const char *fn )
{
	if( fn == NULL )
		return false;
	BasicApp::the->logBegin( "Loading permissions..." );
	
	TiXmlDocument xmlDoc(fn);
	if(!xmlDoc.LoadFile())	{
		BasicApp::the->logEnd( "failed" );
		return false;
	}
	TiXmlElement *root = xmlDoc.RootElement();
	if(!root)	{
		BasicApp::the->logEnd( "failed (no root element found)" );
		return false;
	}
//	root = root->FirstChildElement( "PML" );
	if(strcmp(root->Value(),"PML") != 0)	{
		BasicApp::the->logEnd( "failed (no \"PML\" element found)" );
		return false;
	}
	if( root->Attribute("version") && strcmp( root->Attribute("version"), "0.5" ) != 0 )
		BasicApp::the->logLevelLine( "WARNING: unknown version-id." );
	TiXmlNode *n = root->FirstChild("Permissions");
	if( n )	{
		if( n->NextSibling("Permissions") )
			BasicApp::the->logLevelLine( "WARNING: only first permission-node is parsed, others are ignored." );
		bool r = loadPermissions( n );
		BasicApp::the->logEnd( "done." );
		return r;
	}
	BasicApp::the->logEnd( "failed, ERROR: >tag \"Permissions\" not found<" );
	return false;
}

bool NetPeerManager::hasPermission( const char *name, Uint32 host, char *reason )
{
	if( !bAllowNewConnections )	{
		if( reason )
			strcpy( reason, "We do not accept any further connections." );
		return false;
	}
	if( numConnections() >= nMaxPeers )	{
		if( reason )
			strcpy( reason, "Our connection-limit has been reached." );
		return false;
	}
	if( name[0] == '\0' )	{
		if( reason )
			strcpy( reason, "We did not recieve your name." );
		return false;
	}
	if( strcmp( name, chName ) == 0 )	{
		if( reason )
			strcpy( reason, "Your name equals mine." );
		return false;
	}

	bool bAllow = true;
	int nAllowPrio = -1;// the more common a permission-info is, the less is nAllowPrio
	// if an item exactly equals the name and host, nAllowPrio is highest
	for( PermissionIter i = lPermissionInfos.begin(); i != lPermissionInfos.end(); ++i )
	{
		if( *(*i).chName == '\0' )	{
			if( (*i).ip == 0 )	{
				if( nAllowPrio <= 0 )	{
					bAllow = (*i).bAllowed;
					nAllowPrio = 0;
				}
				continue;
			}
			if( (*i).ip == host )	{
				if( nAllowPrio <= 1 )	{
					bAllow = (*i).bAllowed;
					nAllowPrio = 1;
				}
				continue;
			}
		}
		else if( strcmp( name, (*i).chName ) == 0 )	{
			if( (*i).ip == 0 )	{
				if( nAllowPrio <= 1 )	{
					bAllow = (*i).bAllowed;
					nAllowPrio = 1;
				}
				continue;
			}
			if( (*i).ip == host )	{
				if( nAllowPrio <= 2 )	{
					bAllow = (*i).bAllowed;
					nAllowPrio = 2;
				}
				continue;
			}
		}
	}
	return bAllow;
}


NetSocket *NetPeerManager::createSocket( TCPsocket socket )
{
	BasicApp::the->logLevelLine( "createSocket(sock)" );
	return new NetPeerSocket( "", this, socket );
}
