/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <string>
#include "netpeersocket.h"
#include "netpeermanager.h"
#include "basicapp.h"
#include "netstructs.h"

DECLARE_SLOT(new,NetPeerSocket,NetPeerSocket::newsocket);

NetPeerSocket::NetPeerSocket( const char *name, NetPeerManager *parent, TCPsocket connection )
	: NetStandardSocket( name, parent, connection )
{
	init();
	if( !sock )	{
		bActiveConnect = true;
	}
	else	{
		bActiveConnect = false;
		NetPeerSocket::onConnect();
	}
	bActive = false;
}

NetPeerSocket::NetPeerSocket( const char *name, NetPeerManager *parent, IPaddress ip )
	: NetStandardSocket( name, parent, NULL )
{
	init();
	bDelOnDisconn = true;
	bActiveConnect = true;
}

NetPeerSocket::NetPeerSocket( TiXmlElement *el, NetPeerManager *parent )
	: NetStandardSocket( el, parent )
{
	init();
	bDelOnDisconn = false;
	bActiveConnect = true;
}

void NetPeerSocket::init()
{
	bActive = false;
	tPing = ~0;
	tPingSent = 0;
}

NetPeerSocket::~NetPeerSocket()
{

}


void NetPeerSocket::onConnect()
{
	bActive = true;
	NetConnectData d;
	// when connecting actively, first send own data, then receive the other's data
	if( bActiveConnect )	{
		strcpy( d.chName, ((NetPeerManager*)NetBasicManager::the)->getName() );
		if( d.chName[0] == '\0' )
			BasicApp::the->logLevelLine( "WARNING: NetPeerManager has no name." );
		d.setPort( NetBasicManager::the->localPort() );
		
		if( !NetStandardSocket::sendData( (char*)&d, sizeof(d) ) )	{
			bActive = false;
			return;
		}
	}
	if( !wait4data( MAX_PING ) )	{
		disconnect();
		bActive = false;
		return;
	}
	if( NetStandardSocket::readData( (char*)&d, sizeof(NetConnectData) ) <= 0 || d.type() != TYP_CONNECT )	{
		disconnect();
		bActive = false;
		return;
	}
	// on passive connect, send own data after having received the other's data
	if( !bActiveConnect )	{
		NetDisconnectData discondata;
		if( ((NetPeerManager*)NetBasicManager::the)->hasPermission( d.chName, getAddress()->host, discondata.chReason ) )	{
			strcpy( chName, d.chName );
			nListenPort = d.getPort();
			BasicApp::the->logLevelLine( ((string)"Accepting player: " + d.chName ).c_str() );
			strcpy( d.chName, ((NetPeerManager*)NetBasicManager::the)->getName() );
			NetStandardSocket::sendData( (char*)&d, sizeof(d) );
		}
		else	{
			NetStandardSocket::sendData( (char*)&discondata, sizeof(discondata) );
			SDL_Delay( 1 );// let it a little time to send data before disconnecting<--required?
			disconnect();
			BasicApp::the->logLevelLine( ((string)"Rejected player: " + d.chName ).c_str() );
		}
	}
	bActive = false;

	strcpy( chName, d.chName );
	BasicApp::the->logLevelLine( "NetPeerSocket successfully connected." );
}

Uint32 NetPeerSocket::fastPing()
{ // TODO: UDP
	NetBasicData d;
	d.setType( TYP_PING_REQUEST );
	Uint32 tSend = SDL_GetTicks();
	if( !sendData( (char*)&d, sizeof( NetBasicData ) ) )
		return ~0;
	if( readData( (char*)&d, sizeof( NetBasicData ) ) != sizeof( NetBasicData ) )
		return ~0;
	tSend = SDL_GetTicks() - tSend;
	if( d.type() != TYP_PING_ANSWER )
		return ~0;
	return tSend;
}

void NetPeerSocket::requestPing()
{
	NetBasicData d;
	d.setType( TYP_PING_REQUEST );
	tPingSent = SDL_GetTicks();
	if( !sendData( (char*)&d, sizeof(d) ) )
		tPing = ~0;
}

void NetPeerSocket::handleData( NetBasicData *d )
{
	switch( d->type() )
	{
		case TYP_DISCONNECT:
			onDisconnect();
			disconnect();
			break;
			
		case TYP_PING_REQUEST:	{
			NetBasicData answer;
			answer.setType( TYP_PING_ANSWER );
			sendData( (char*)&answer, sizeof(answer) );
			break;
		}
		case TYP_PING_ANSWER:
			tPing = SDL_GetTicks() - tPingSent;
			onPing();
			break;
			
		case TYP_CONNECTIONLIST:	{
			NetConnectionList ncl( d );
			for( Uint32 n = 0; n < ncl.count(); ++n )	{
				// avoid connecting twice to one host
				if( NetBasicManager::the->getSocket( *ncl.get(n) ) )
					continue;
				new NetPeerSocket( "", (NetPeerManager*)getParent(), *ncl.get(n) );
			}
			break;
		}
	}
}

bool NetPeerSocket::onReceive()
{
	int size;
	char *data = NetStandardSocket::readData(&size);
	if(!data)
		return false;
	handleData( (NetBasicData*)data );
	
	delete[] data;
	return true;
}

