/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2006 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "netsocket.h"
#include "netmanager.h"
#include "basicapp.h"
#include "functions.h"

NetSocket::NetSocket( const char *name, NetBasicManager *p, TCPsocket connection ): BasicObject(name, p)
{
	sock = connection;
	if( !init() )
		return;
	
	if( sock )	{
		bDelOnDisconn = true;
		if( !((NetBasicManager*)getParent())->registerSocket( sock ) )	{
			BasicApp::the->logLevelLine( "ERROR: NetSocket not added to SocketSet." );
			SDLNet_TCP_Close( sock );
			sock = NULL;
		}
	}
	else bDelOnDisconn = false;
}

NetSocket::NetSocket( const char *name, NetBasicManager *p, IPaddress ip ): BasicObject(name, p)
{
	sock = NULL;
	if(!init())
		return;
	connect( ip );
	bDelOnDisconn = false;
}

NetSocket::NetSocket( TiXmlElement *el, NetBasicManager *p ): BasicObject(el, p)
{
	sock = NULL;
	bDelOnDisconn = false;
}

bool NetSocket::init()
{
	bActive = false;

	if( !getParent() || !((NetBasicManager*)getParent())->bInited )	{
		bValid = false;
		return false;
	}

	if( getParent()->numChildren() >= MAX_PEERS )	{
		bValid = false;
		bDelOnDisconn = false;
		BasicApp::the->logLevelLine( "WARNING: New socket invalid, because socket-list is full." );
		if(sock)	{
			SDLNet_TCP_Close( sock );
			sock = NULL;
		}
		return false;
	}
	
	bValid = true;
	return true;
}

NetSocket::~NetSocket()
{
	if( sock )	{
		disconnect();
	}
	BasicApp::the->logLevelLine( "Deleted a NetSocket." );
}

void NetSocket::onInit()
{
	if(xmlData)	{
		if( !init() )
			return;
		
		const char *s = xmlData->Attribute("IP");
		if(s)	{
			IPaddress a = strToIPaddress(s);
			if(a.port == 0)	{
				BasicApp::the->logLevelLine( "NetSocket: no port specified in attribute 'IP' of xml-file" );
			}
			else if(!connect(a))	{
				BasicApp::the->logLevelLine( "NetSocket cannot connect on initialisation to " );
				BasicApp::the->logString(s);
			}
		}
		s = xmlData->Attribute("DeleteOnDisconnect");
		bDelOnDisconn = s && strToBool(s);
	}
	BasicObject::onInit();
}

bool NetSocket::connect( const char *host, int port )
{
	IPaddress address;
	if( SDLNet_ResolveHost( &address, host, port ) == -1 )
		return false;
	return connect( address );
}

bool NetSocket::connect( IPaddress &ip )
{
	if( sock )	{
		disconnect();
		BasicApp::the->logLevelLine( "NetSocket disconnected for reconnecting." );
	}

	sock = SDLNet_TCP_Open( &ip );
	if( !sock )	{
		BasicApp::the->logLevelLine( "ERROR: NetSocket: Connecting to >" );
		char buf[32];
		BasicApp::the->logString( IPToStr(ip, buf) );
		BasicApp::the->logString( "< failed, reason: " );
		BasicApp::the->logString( SDL_GetError() );
		return false;
	}

	if( !((NetBasicManager*)getParent())->registerSocket( sock ) )	{
		BasicApp::the->logLevelLine( "ERROR: NetSocket not added to SocketSet." );
		SDLNet_TCP_Close( sock );
		sock = 0;
		return false;
	}
	onConnect();
	return true;
}

void NetSocket::disconnect()
{
	if( sock == 0 )
		return;
	
	onDisconnect();
	
	((NetBasicManager*)getParent())->unregisterSocket( sock );
	SDLNet_TCP_Close( sock );
	BasicApp::the->logLevelLine( "Disconnected a NetSocket." );
	sock = 0;
}

bool NetSocket::wait4data( Uint32 msecs )
{
	if( sock == 0 )
		return false;
	bool bOldActive = bActive;
	bActive = true;

	bool r = false;
	SDLNet_SocketSet set = SDLNet_AllocSocketSet( 1 );
	if( SDLNet_TCP_AddSocket( set, sock ) != -1 )	{
		for( Uint32 t = SDL_GetTicks() + msecs; t > SDL_GetTicks(); SDL_Delay( 1 ) )	{
			if( SDLNet_CheckSockets( set, 0 ) == 1 )	{
				r = true;
				break;
			}
		}
		SDLNet_TCP_DelSocket( set, sock );
	}
	else return false;
	SDLNet_FreeSocketSet( set );
	bActive &= bOldActive;// bActive might have changed during this procedure, so use &=
	return true;
}

bool NetSocket::sendData( char *data, unsigned int size )
{
	if( sock == 0 )
		return false;
	if( SDLNet_TCP_Send( sock, data, size ) == (int)size )
		return true;
	
	disconnect();
	return false;
}

bool NetSocket::sendUDP( char *data, unsigned int size )
{
	const IPaddress * ip = getAddress();
	if( ip )	{
		return NetBasicManager::the->sendUDP( data, size, *ip);
	}
	else
		return false;
}

int NetSocket::readData( char *buffer, int maxsize )
{
	if( sock == 0 )
		return E_NET_NO_HOST;
	
	int nBytesRead = E_NET_OTHER;
	// check, whether there is data to recieve
	SDLNet_SocketSet set = SDLNet_AllocSocketSet( 1 );
	if( SDLNet_TCP_AddSocket( set, sock ) != -1 )	{
		for( nBytesRead = 0; SDLNet_CheckSockets( set, 0 ) == 1 && nBytesRead < (int)maxsize; nBytesRead++ )	{
			if( SDLNet_TCP_Recv(sock, buffer, 1) <= 0 )	{
				disconnect();
				nBytesRead = E_NET_DISCONNECTED;
				break;
			}
			buffer++;
		}
		SDLNet_TCP_DelSocket( set, sock );
	}
	SDLNet_FreeSocketSet( set );
	
	return nBytesRead;
}

bool NetSocket::onReceive()
{
	bool b = true;
	if(((NetBasicManager*)getParent())->cbPeer.func)
		(*((NetBasicManager*)getParent())->cbPeer.func)(this, ((NetBasicManager*)getParent())->cbPeer.obj, &b);
	return b;
}

void NetSocket::onReceiveUDP(UDPpacket &udp)
{
	if(((NetBasicManager*)getParent())->cbUDP.func)
		(*((NetBasicManager*)getParent())->cbUDP.func)(this, ((NetBasicManager*)getParent())->cbUDP.obj, &udp);
}


/********************************************************************/
/* Implementation of NetStandardSocket                              */

NetStandardSocket::NetStandardSocket( const char *name, NetBasicManager *p, TCPsocket connection )
: NetSocket( name, p, connection )
{
	filter = 0xFFFFFFFF;
}

NetStandardSocket::NetStandardSocket( const char *name, NetBasicManager *p, IPaddress ip, int f )
: NetSocket( name, p, ip )
{
	filter = f;
}

NetStandardSocket::NetStandardSocket( TiXmlElement *el, NetBasicManager *p )
	: NetSocket( el, p )
{
	filter = 0xFFFFFFFF;
	nBytesUnread = 0;
}

NetStandardSocket::~NetStandardSocket()
{
}

void NetStandardSocket::onInit()
{
	if( xmlData )	{
		const char *s = xmlData->Attribute("Filter");
		if( s )
			filter = xStrToUL(s);
	}
}

bool NetStandardSocket::sendData( char *data, unsigned int size )
{
	char *pBuf = new char[2 + size];
	
	SDLNet_Write16( size, pBuf );
	memcpy( pBuf+2, data, size );
	if( NetSocket::sendData( pBuf, size+2 ) )	{
		delete[] pBuf;
		return true;
	}
	delete[] pBuf;
	return false;
}

bool NetStandardSocket::sendData( char *data, unsigned int size, Uint32 condition, void *buffer )
{
	if( sock == 0 )
		return false;

	if( condition && ( filter & condition ) == 0 )
		return true;

	char *pBuf = (char *)buffer;
	if( pBuf == NULL )
		pBuf = new char[2 + size];
	
	SDLNet_Write16( size, (char*)pBuf );
	memcpy( pBuf+2, data, size );
	bool r = (unsigned int)SDLNet_TCP_Send( sock, pBuf, 2 + size ) == 2 + size;
	if( buffer == NULL )
		delete pBuf;
	if( r )
		return true;

	disconnect();
	return false;
}

int NetStandardSocket::readData( char *data, int maxsize )
{
	if( sock == 0 )
		return E_NET_NO_HOST;

	if( nBytesUnread > 0 )	{
		if( nBytesUnread > (unsigned int)maxsize )	{
			return E_NET_BUFTOOSMALL;
		}
	}
	else	{
		Uint16 size;
		int result = SDLNet_TCP_Recv( sock, &size, 2 );
		if( result <= 1 )	{
			disconnect();
			return E_NET_DISCONNECTED;
		}
		nBytesUnread = SDLNet_Read16( &size );
		if( nBytesUnread > (unsigned int)maxsize )	{
			return E_NET_BUFTOOSMALL;
		}
	}
	
	if( SDLNet_TCP_Recv( sock, data, nBytesUnread ) < (int)nBytesUnread )	{
		disconnect();
		return E_NET_DISCONNECTED;
	}
	int n = nBytesUnread;
	nBytesUnread = 0;
	return n;
}

char * NetStandardSocket::readData( int *pSize )
{
	if( sock == 0 )	{
		if( pSize )
			*pSize = E_NET_NO_HOST;
		return NULL;
	}

	if( nBytesUnread == 0 )	{
		Uint16 size;
		int result = SDLNet_TCP_Recv( sock, &size, 2 );
		if( result <= 1 )	{
			disconnect();
			if( pSize )
				*pSize = E_NET_DISCONNECTED;
			return NULL;
		}
		nBytesUnread = SDLNet_Read16( &size );
	}
	char *data = new char[nBytesUnread];
	if( SDLNet_TCP_Recv( sock, data, nBytesUnread ) < (int)nBytesUnread )	{
		delete[] data;
		disconnect();
		if( pSize )
			*pSize = E_NET_DISCONNECTED;
		return NULL;
	}
	if( pSize )
		*pSize = nBytesUnread;
	nBytesUnread = 0;
	return data;
}

bool NetStandardSocket::onReceive()
{
	bool b = true;
	if(cbRecieve.func)
		(*cbRecieve.func)(this, cbRecieve.obj, &b);
	return b;
}

void NetStandardSocket::onConnect()
{
	nBytesUnread = 0;
	NetSocket::onConnect();
}
