/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "webserver.h"
#ifdef __windows__
#include <direct.h>
#else
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <fcntl.h>
#endif
#include "tinyxml.h"
#include "netmanager.h"
#include "webthread.h"
#include "webpage.h"
#include "webfilepage.h"
#include "webforwardpage.h"
#include "basicapp.h"
#include "functions.h"

WebServer *WebServer::the = NULL;
WebServer *WebServer::getThe()
{
	return the;
}

WebServer::WebServer(const char *name, BasicObject *p): BasicLog( name, p, "HTTP.log" )
{
	the = this;
	webServer = NULL;
	BasicApp::the->logLevelLine("Created WebServer");
}

WebServer::WebServer(TiXmlElement *el, BasicObject *p): BasicLog( el, p )
{
	the = this;
	webServer = NULL;
	BasicApp::the->logLevelLine("Created WebServer");
}

WebServer::~WebServer()
{
	destroy();
	the = 0;
}

void WebServer::logIntro()
{
	logLine( "REQUEST-INFOS:" );
	logLine( "HOST:PORT - - [TIME (msecs since appstart)] \"METHOD QUERY-STRING\" - "
			 "RETURN CODE - BYTES SENT - TIME (in msecs)." );
}

void WebServer::onInit()
{
	BasicApp::the->logBegin("Initializing server...");
	
	if(!NetBasicManager::initSDL())
		BasicApp::the->logLevelLine("SDL-networking cannot be initialised");
	
	if(xmlData)	{
		webServer = WebSrv_Create();
		socket = WebSrv_Open( webServer, xmlData->Attribute("Port") ? xStrToUL(xmlData->Attribute("Port")) : 80 );
		if( !socket )	{
			WebSrv_Destroy( webServer );
			BasicApp::the->logEnd("failed.");
		}
		else
			parseXMLNode( xmlData );
	}
	
	BasicLog::onInit();
	logIntro();
	
	BasicApp::the->logEnd("done.");
}

bool WebServer::init( Uint16 port )
{
	if(xmlData)	{
		onInit();
		return true;
	}
	BasicApp::the->logBegin("Initializing webserver...");
	if(!NetBasicManager::initSDL())
		BasicApp::the->logLevelLine("SDL-networking cannot be initialised");
	webServer = WebSrv_Create();
	socket = WebSrv_Open( webServer, port );
	if( !socket )	{
		WebSrv_Destroy( webServer );
		BasicApp::the->logEnd("failed.");
		return false;
	}
	BasicApp::the->logEnd("done.");
	logIntro();
	return true;
}

void WebServer::onDestroy()
{
	destroy();
	
	BasicLog::onDestroy();
}

void WebServer::destroy()
{
	BasicApp::the->logBegin( "Destroying server..." );
	while( lPages.size() )	{
		delete lPages.back();
	}
	while( lThreads.size() )	{
		delete lThreads.back();
	}
	if( !socket )	{
		BasicApp::the->logEnd("done.");
		return;
	}
	if( webServer )	{
		WebSrv_Destroy( webServer );
		webServer = NULL;
	}
	NetBasicManager::quitSDL();
	
	BasicApp::the->logEnd( "done." );
}

bool WebServer::poll( Uint16 tWait )
{
	if( WebSrv_Poll( webServer, tWait ) != 1 )
		return false;
	if( !(new WebThread("", this))->start() )	{
		BasicApp::the->logLevelLine( "ERROR: Cannot create thread." );
		return false;
	}

	return true;
}

void WebServer::closeOutTimedThreads()
{
	list<WebThread*>::iterator i;
	for( i = lThreads.begin(); i != lThreads.end(); ++i )	{
		// if stopped regularly or timed out, delete thread and close the connection to the client (only done, if client!=NULL)
		if( !(*i)->running() || (*i)->tLastAction + WebThread::tIdleToExit < SDL_GetTicks() )	{
			WebSrv_CloseClient( webServer, (*i)->client );
			list<WebThread*>::iterator i2 = i;
			--i2;
			delete *i;
			i = i2;
		}
	}
}

void WebServer::parseXMLNode( TiXmlNode *node )
{
	BasicApp::the->logBegin("Parsing XML-Node...");
	if( node->FirstChildElement( "TimeoutIv" ) && node->FirstChildElement( "TimeoutIv" )->Attribute("value") )
		WebThread::tIdleToExit = xStrToUL( node->FirstChildElement( "TimeoutIv" )->Attribute("value") );
	TiXmlElement *el;
	for( el = node->FirstChildElement( "Directory" ); el; el = el->NextSiblingElement( "Directory" ) )	{
		if( el->Attribute("Path") == NULL )
			continue;
		int nFiles = addPageDirectory( el->Attribute("Path"), el->Attribute("URI"),
								el->Attribute("Indexpage"), strToBool(el->Attribute("SubDirs")) );
		if( nFiles )	{
			BasicApp::the->logLevelLine("Added webpage-directory: ");
			BasicApp::the->logString( el->Attribute("Path") );
			BasicApp::the->logString( ", number of files: " );
			BasicApp::the->logString( intToStr( nFiles ) );
		}
	}
	int nDone = 0;
	for( el = node->FirstChildElement( "Page" ); el; el = el->NextSiblingElement( "Page" ) )	{
		if( el->Attribute("URI") && el->Attribute("Filename") )	{
			new WebFilePage( el->Attribute("URI"), el->Attribute("Filename"), el->Attribute("ContentType") );
			++nDone;
		}
	}
	if( nDone )	{
		BasicApp::the->logLevelLine("Number of added file-pages: ");
		BasicApp::the->logString( intToStr( nDone ) );
		nDone = 0;
	}
	for( el = node->FirstChildElement( "Forward" ); el; el = el->NextSiblingElement( "Forward" ) )	{
		if( el->Attribute("URI") && el->Attribute("Aim") )	{
			new WebForwardPage( el->Attribute("URI"), el->Attribute("Aim") );
			++nDone;
		}
	}
	if( nDone )	{
		BasicApp::the->logLevelLine("Number of added forwarders: ");
		BasicApp::the->logString( intToStr( nDone ) );
	}
	BasicApp::the->logEnd("done.");
}

int WebServer::addPageDirectory( const char *dir, const char *basic_uri, const char *fn_index, bool subdirs )
{
	string pn = BasicApp::the->makeDataPath(dir);
	
	BasicApp::the->logLevelLine( "Parsing directory: " );
	BasicApp::the->logString( pn.c_str() );
#ifdef __windows__
#define pDIR_FILENAME	i.name
#define bDIR_ISSUBDIR	(i.attrib & _A_SUBDIR)
	char chCurrDir[1024];
	getcwd( chCurrDir, 1024 );
	chdir( pn.c_str );

	_finddata_t i;
	long f = _findfirst( "*", &i );
	if( f == -1L )
		return 0;
#else
#define pDIR_FILENAME	pDir->d_name
#define bDIR_ISSUBDIR	( (pDir->d_type & DT_DIR) != 0 )
	struct dirent *pDir;
	DIR *f = opendir( pn.c_str() );
	if( f == NULL )	{
		BasicApp::the->logLevelLine("Directory could not be opened");
		return 0;
	}
	if( subdirs )
		BasicApp::the->logLine( "sub=1" );
#endif
	if( basic_uri == NULL )
		basic_uri = "/";
	// path must not end with '/' from here on
	if(pn[pn.length()-1] == '/')
		pn[pn.length()-1] = '\0';
	char tmp[256];
	int nFiles = 0;
	strcpy( tmp, basic_uri );
	if( fn_index )
		new WebForwardPage( tmp, fn_index );
	for(;;)	{
#ifndef __windows__
		if( ( pDir = readdir( f ) ) == NULL )
			break;
#endif
		if( !(strcmp( pDIR_FILENAME, ".." ) == 0 || strcmp( pDIR_FILENAME, "." ) == 0) )	{
			if( bDIR_ISSUBDIR && subdirs )	{
				char uri[256];
				strcpy( uri, basic_uri );
				strcat( uri, pDIR_FILENAME );
				strcat( uri, "/" );
				nFiles += addPageDirectory( (pn + '/' + pDIR_FILENAME).c_str(), uri, fn_index, true );
			}
			else	{
				char uri[256];
				strcpy( uri, basic_uri );
				strcat( uri, pDIR_FILENAME );
				char *subUri = strrchr(uri,'/');
				new WebFilePage( uri, (pn + subUri).c_str() );
				++nFiles;
			}
		}
#ifdef __windows__
		if( _findnext( f, &i ) != 0 )
			break;
#endif
	}
#ifdef __windows__
	_findclose( f );
	chdir( chCurrDir );
#else
	closedir( f );
#endif
	
	return nFiles;
}
/*
WebPage * WebServer::getFirstPage()
{
	if( lPages.size() == 0 )
		return NULL;
	itPage = lPages.begin();
	return *itPage;
}

WebPage * WebServer::getNextPage()
{
	if( lPages.end() == ++itPage )
		return NULL;
	return *itPage;
}
*/

