/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfilemusic.h"
#include "basicapp.h"

SdlMusic * SdlMusic::musPlaying = 0;
int SdlMusic::g_nVolume = 128;

void SdlMusic::setGlbVolume( int v )
{
	g_nVolume = v;
	if( musPlaying != 0 )
		musPlaying->setVolume( -1 );
}




SdlMusic::SdlMusic( const char *name ): SdlAudio( name )
{
	mixMusic = 0;
	bPlaying = false;

	nVolume = 128;
}

SdlMusic::~SdlMusic()
{
	destroy();
}

void SdlMusic::destroy()
{
	stop();
	if( mixMusic != 0 )
		Mix_FreeMusic( mixMusic );
}

int SdlMusic::play( int loops )
{
	if( mixMusic == 0 )
		return -1;

	if( musPlaying != 0 )
		musPlaying->stop();

	if( Mix_PlayMusic( mixMusic, loops ) == -1 )	{
		BasicApp::the->logLine( "ERROR: Unable to play music, message: " );
		BasicApp::the->logString( Mix_GetError() );

		return -1;
	}
	Mix_VolumeMusic( nVolume * g_nVolume / 128 );

	bPlaying = true;
	musPlaying = this;
	Mix_HookMusicFinished( onMusicFinished );

	return 0;
}

int SdlMusic::fadeIn( int ms, int loops )
{
	if( mixMusic == 0 )
		return -1;

	if( musPlaying != 0 )
		musPlaying->stop();

	Mix_VolumeMusic( nVolume * g_nVolume / 128 );
	if( Mix_FadeInMusic( mixMusic, loops, ms ) == -1 )	{
		BasicApp::the->logLine( "ERROR: Unable to fade-in music, message: " );
		BasicApp::the->logString( Mix_GetError() );

		return -1;
	}

	musPlaying = this;
	Mix_HookMusicFinished( onMusicFinished );

	return 0;
}

bool SdlMusic::isPlaying()
{
	return bPlaying && (Mix_PlayingMusic() != 0);
}

void SdlMusic::pause()
{
	if( bPlaying )
		Mix_PauseMusic();
}

bool SdlMusic::isPaused()
{
	if( bPlaying )
		return Mix_PausedMusic() != 0;
	return false;
}

void SdlMusic::resume()
{
	if( bPlaying )
		Mix_ResumeMusic();
}

void SdlMusic::stop()
{
	if( !bPlaying )
		return;

	Mix_HaltMusic();
	bPlaying = false;
}

int SdlMusic::fadeOut( int ms )
{
	if( !bPlaying )
		return -1;

	return Mix_FadeOutMusic( ms );
}

Mix_Fading SdlMusic::fadeStatus()
{
	if( !bPlaying )
		return MIX_NO_FADING;

	return Mix_FadingMusic();
}

void SdlMusic::setVolume( int v )
{
	if( v >= 0 )
		nVolume = v;
	if( bPlaying )
		Mix_VolumeMusic( nVolume * g_nVolume / 128 );
}





SdlFileMusic::SdlFileMusic( const char *fn, const char *name ): SdlMusic( name ? name : fn )
{
	if( fn == 0 )	{
		return;
	}

	mixMusic = Mix_LoadMUS( fn );
	if( mixMusic == 0 )	{
		BasicApp::the->logLine( "ERROR: Cannot load music-file, message: " );
		BasicApp::the->logString( Mix_GetError() );

		failed();
	}
}

SdlFileMusic::~SdlFileMusic()
{
}

bool SdlFileMusic::load( const char *fn )
{
	destroy();

	mixMusic = Mix_LoadMUS( fn );
	if( mixMusic == 0 )	{
		BasicApp::the->logLine( "ERROR: Cannot load music-file, message: " );
		BasicApp::the->logString( Mix_GetError() );

		failed();
		return false;
	}
	return true;
}

