/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfilesound.h"
#include "basicapp.h"
#include "sdlmanager.h"

list<SdlSound*> SdlSound::lPlayingSounds;
int SdlSound::g_nVolume = 128;


void SdlSound::setGlbVolume( int v )
{
	g_nVolume = v;
	SdlManager::the->refreshSoundVolumes();
}




SdlSound::SdlSound( const char *name ): SdlAudio( name )
{
	mixSound = 0;
	nChannel = -1;
}

SdlSound::~SdlSound()
{
	destroy();
}

void SdlSound::destroy()
{
	if( nChannel != -1 )	{
		Mix_HaltChannel( nChannel );
		nChannel = -1;
	}
	if( mixSound != 0 )	{
		Mix_FreeChunk( mixSound );
		mixSound = 0;
	}
}


int SdlSound::play( int loops )
{
	if( mixSound == 0 )
		return -1;

	nChannel = Mix_PlayChannel( -1, mixSound, loops );
	if(nChannel == -1) {
		BasicApp::the->logLine( "ERROR: Cannot play sound, message: " );
		BasicApp::the->logString( Mix_GetError() );

		return -1;
	}

	lPlayingSounds.push_back( this );
	Mix_ChannelFinished( onChannelFinished );

	return nChannel;
}

int SdlSound::fadeIn( int ms, int loops )
{
	if( mixSound == 0 )
		return -1;

	nChannel = Mix_FadeInChannel( -1, mixSound, loops, ms );
	if( nChannel == -1 )	{
		BasicApp::the->logLine( "ERROR: Unable to fade-in music, message: " );
		BasicApp::the->logString( Mix_GetError() );

		return -1;
	}

	lPlayingSounds.push_back( this );
	Mix_ChannelFinished( onChannelFinished );

	return 0;
}

bool SdlSound::isPlaying()
{
	if( nChannel == -1 )
		return false;
	return Mix_Playing( nChannel ) != 0;
}

void SdlSound::pause()
{
	if( nChannel != -1 )
		Mix_Pause( nChannel );
}

bool SdlSound::isPaused()
{
	if( nChannel != -1 )
		return Mix_Paused( nChannel ) != 0;

	return false;
}

void SdlSound::resume()
{
	if( nChannel != -1 )
		Mix_Resume( nChannel );
}

void SdlSound::stop()
{
	if( nChannel == -1 )
		return;
	
	Mix_HaltChannel( nChannel );
	lPlayingSounds.remove( this );
	nChannel = -1;
}

int SdlSound::fadeOut( int ms )
{
	if( nChannel == -1 )
		return -1;

	return Mix_FadeOutChannel( nChannel, ms );
}

Mix_Fading SdlSound::fadeStatus()
{
	if( nChannel == -1 )
		return MIX_NO_FADING;

	return Mix_FadingChannel( nChannel );
}

void SdlSound::setVolume( int v )
{
	if( v >= 0 )
		nVolume = v;
	Mix_VolumeChunk( mixSound, nVolume * g_nVolume / 128 );
}

int SdlSound::getChannel()
{
	if( !isPlaying() )
		return -1;

	return nChannel;
}





SdlFileSound::SdlFileSound( const char *fn, const char *name ): SdlSound( name ? name : fn )
{
	if( fn == 0 )	{
		return;
	}
	mixSound = Mix_LoadWAV( fn );
	if( mixSound == 0 )	{
		BasicApp::the->logLine( "ERROR: Cannot load sound-file, message: " );
		BasicApp::the->logString( Mix_GetError() );
		failed();
		return;
	}
}

SdlFileSound::~SdlFileSound()
{
}

bool SdlFileSound::load( const char *fn )
{
	destroy();

	mixSound = Mix_LoadWAV( fn );
	if( mixSound == 0 )	{
		BasicApp::the->logLine( "ERROR: Cannot load sound-file, message: " );
		BasicApp::the->logString( Mix_GetError() );
		failed();
		return false;
	}
	return true;
}

