/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfilesurfaces.h"
#include <fstream>
#include "basicapp.h"
#include "sdlfilesurface.h"
#include "uiobject.h"
#include "functions.h"

// returns true, if str contains one of ';', ':', ',', '.' and '|'
bool hasSeparator( const char *str )
{
	return strchr( str, ';' ) != 0 || strchr( str, ':' ) != 0 ||
			strchr( str, ',' ) != 0 || strchr( str, '.' ) != 0 ||
			strchr( str, '|' ) != 0;
}

SdlFileSurfacesInfo::SdlFileSurfacesInfo()
{
	chBMPFile[0] = 0;
	chInfoFile[0] = 0;
	cTransparent = CLR_STANDARD;
}

bool SdlFileSurfacesInfo::load( const char *fn )
{
	ifstream file( fn, ios::in | ios::binary );
	if( !file.is_open() )
		return false;

	bool bHasFN = false;
	while( !bHasFN )	{
		char *chStr = chBMPFile;
		file.getline( chBMPFile, 256 );
SPACE_READ:
		switch( chStr[0] )	{
		case '#':
		case '\r':
		case '\0':
			break;
		case ' ':
		case '\t':
			++chStr;
			goto SPACE_READ;
		default:
			if( chStr != chBMPFile )
				strcpy( chBMPFile, chStr );
			bHasFN = true;
		}
	}
	if( strchr( chBMPFile, '#' ) )
		*strchr( chBMPFile, '#' ) = '\0';
	if( strchr( chBMPFile, '\r' ) )
		*strchr( chBMPFile, '\r' ) = '\0';

	strcpy( chInfoFile, fn );

	char chTmp[255];
	char chDst[31];
	SDL_Rect r;
	char nComment = 0;
	while( !(file.eof() || file.bad()) )	{
		file.getline( chTmp, 255 );

		chDst[0] = '\0';

		bool bEnd = false;
		int n = 0;
		for( int i = 0; !bEnd; ++i )	{
			switch( chTmp[i] )	{
			case '\r':
			case '\0':
				bEnd = true;
				break;
			case '\\':
				if( nComment == 0 )	{
					file.getline( chTmp, 256 );
					i = 0;
				}
				break;
			case '/':
			case '#':
				bEnd = true;
				break;
			case '*':
			case '\'':
			case '"':
				if( nComment == chTmp[i] )
					nComment = 0;
				else if( nComment == 0 )
					nComment = chTmp[i];
				break;
			case ' ':
			case '\t':
				break;
			case '|':
			case ':':
			case ',':
			case '.':
				chTmp[i] = ';';
			default:
				if( nComment == 0 )	{
					chDst[n] = chTmp[i];
					++n;
				}
			}
		}
		chDst[n] = '\0';
		// the first non-empty line that has no separator contains the colorkey
		if( ( strlen( chDst ) > 0 ) && !hasSeparator( chDst + 1/*ignore first character*/ ) )
		{
			if( cTransparent == CLR_STANDARD)	{
				cTransparent = xStrToUL( chDst );
			}
		}
		else if( strlen( chDst ) >= 7 )// at least 7 characters required for one rect (3*separator + 4*number)
		{
			r = strToRect( chDst );
			lRects.push_back( r );
		}
	}
	if( file.bad() )
		lRects.pop_back();// last rect may be damaged

	file.close();

	if( cTransparent == CLR_STANDARD)
		cTransparent = CLR_INVALID;

	return true;
}

bool SdlFileSurfacesInfo::save( const char *fn )
{
	ofstream file( fn, ios::out | ios::binary );
	if( !file.is_open() )
		return false;

	file.write( chBMPFile, 256 );
	for( list<SDL_Rect>::iterator i = lRects.begin(); i != lRects.end(); ++i )	{
		file.write( (char*)&(*i), sizeof(SDL_Rect) );
	}
	if( file.bad() )	{
		file.close();
		return false;
	}
	file.close();

	return true;
}

const char *SdlFileSurfacesInfo::getBitmapFN() const
{
	static char chFN[256];
	chFN[0] = '\0';
	const char *chR = strrchr( chInfoFile, '/' );
	if( chR != 0 )	{
		strncpy( chFN, chInfoFile, chR - chInfoFile + 1 );
		chFN[chR - chInfoFile + 1] = '\0';
	}
	strcat( chFN, chBMPFile );

	return chFN;
}

const SDL_Rect *SdlFileSurfacesInfo::getFirstRect()
{
	if( lRects.size() == 0 )
		return 0;
	rIter = lRects.begin();
	return &(*rIter);
}

const SDL_Rect *SdlFileSurfacesInfo::getNextRect()
{
	++rIter;
	if( rIter == lRects.end() )
		return 0;
	return &(*rIter);
}

/***************************************************************************************/

SDL_Surface * SdlFileSurfaces::SdlDisplayFormatRect( SDL_Surface *src, const SDL_Rect &r, bool alpha )
{
	SDL_PixelFormat *f = src->format;
	SDL_Surface *surf = SDL_CreateRGBSurface( src->flags, r.w, r.h, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask );
	if( SDL_MUSTLOCK( surf ) )
		SDL_LockSurface( surf );
	if( SDL_MUSTLOCK( src ) )
		SDL_LockSurface( src );
	// copy pixels to surface
	char *pLineSrc = (char *)src->pixels + src->pitch * r.y + f->BytesPerPixel * r.x;
	char *pLineDst = (char *)surf->pixels;
	int bpl = f->BytesPerPixel * r.w;
	for( Uint16 y = 0; y < r.h; ++y )	{
		memcpy( pLineDst, pLineSrc, bpl );
		pLineDst += surf->pitch;
		pLineSrc += src->pitch;
	}
	if( SDL_MUSTLOCK( src ) )
		SDL_UnlockSurface( src );
	if( SDL_MUSTLOCK( surf ) )
		SDL_UnlockSurface( surf );
	
	SDL_Surface *result;
	if( alpha )
		result = SDL_DisplayFormatAlpha( surf );
	else
		result = SDL_DisplayFormat( surf );
	
	SDL_FreeSurface( surf );
	return result;
}


SdlFileSurfaces::SdlFileSurfaces( const char *fn, const char *name, Uint32 flags )
	: SdlSurfaceContainer( name? name : fn )
{
	data.pFileInfo = new SdlFileSurfacesInfo();
	if( !data.pFileInfo->load( fn ) )	{
		BasicApp::the->logLevelLine( "ERROR: SdlFileSurfaces: file cannot be read correctly: " );
		BasicApp::the->logString( fn );
		failed();
		return;
	}
	ulFlags = flags;
	
	onRefresh();
}

SdlFileSurfaces::SdlFileSurfaces( const char *fn, SDL_Rect rBeg, Uint32 num, Uint32 NumPerLine, bool byrow,
								  const char *name, ARGB_COLOR colorkey, const char *type, Uint32 flags )
	: SdlSurfaceContainer( name ? name : fn )
{
	strFilename = fn;
	if( strFilename.empty() )	{
		failed();
		data.pRects = NULL;
		return;
	}
	data.pRects = new RectData;
	ulFlags = flags;
	data.pRects->rBegin = rBeg;
	data.pRects->nNum = num;
	data.pRects->nNumPerLine = NumPerLine;
	data.pRects->bByRow = byrow;
	data.pRects->ulColorKey = colorkey;
	if( type )	{
		strncpy( data.pRects->chType, type, 3 );
		data.pRects->chType[3] = '\0';
	}
	else
		data.pRects->chType[0] = '\0';
	
	onRefresh();
}


SdlFileSurfaces::~SdlFileSurfaces()
{
	// delete all surfaces
	free();
	if( data.pRects )
	{
		if( createdByInfoFile() )
			delete data.pFileInfo;
		else
			delete data.pRects;
	}
}

void SdlFileSurfaces::free()
{
	for( int i = 0; i < lSurfaces.size(); ++i )	{
		delete lSurfaces[i];
	}
	lSurfaces.clear();
}

void SdlFileSurfaces::onRefresh()
{
	SdlSurface *sysSurface;
	if( createdByInfoFile() )
	{// load by data.fileinfo
		sysSurface = new SdlFileSurface( data.pFileInfo->getBitmapFN(), (char*)-1, CLR_INVALID, NULL, ulFlags & ~(SDL_HWSURFACE) );
		if( sysSurface->isFailed() )	{
			failed();
			return;
		}
		// delete all surfaces
		free();
		lSurfaces.reserve( data.pFileInfo->numRects() );
		
		SDL_PixelFormat *f = sysSurface->surf()->format;
		const SDL_Rect *r = data.pFileInfo->getFirstRect();
		while( r )	{
			SDL_Surface *surf;
			surf = SdlDisplayFormatRect( sysSurface->surf(), *r, data.pFileInfo->getTransparentColor() == CLR_INVALID );
			SdlSurface *s = new SdlSurface( surf, (char*)-1 );
			s->setColorKey( data.pFileInfo->getTransparentColor() );
			lSurfaces.push_back( s );
			
			r = data.pFileInfo->getNextRect();
		}
	}
	else
	{// load by data.rects
		sysSurface = new SdlFileSurface( strFilename.c_str(), (char*)-1, CLR_INVALID, data.pRects->chType, ulFlags & ~(SDL_HWSURFACE) );
		if( sysSurface->isFailed() )	{
			failed();
			return;
		}
		// delete all surfaces
		free();
		lSurfaces.reserve( data.pRects->nNum );
		
		SDL_PixelFormat *f = sysSurface->surf()->format;
		SDL_Rect r = data.pRects->rBegin;
		for( int n = 0; n < data.pRects->nNum; ++n )	{
			SDL_Surface *surf;
			surf = SdlDisplayFormatRect( sysSurface->surf(), r, data.pRects->ulColorKey == CLR_INVALID );
			SdlSurface *s = new SdlSurface( surf, (char*)-1 );
			s->setColorKey( data.pRects->ulColorKey );
			lSurfaces.push_back( s );
			
			if( data.pRects->bByRow )	{
				r.x += r.w;
				if( r.x >= r.w * data.pRects->nNumPerLine )	{
					r.x = 0;
					r.y += r.h;
				}
			}
			else	{
				r.y += r.h;
				if( r.y >= r.h * data.pRects->nNumPerLine )	{
					r.y = 0;
					r.x += r.w;
				}
			}
		}
	}
}
