/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfiletextures.h"
#include <fstream>
#include "basicapp.h"
#include "sdlfilesurface.h"
#include "sdlfiletexture.h"
#include "uiobject.h"
#include "functions.h"

SdlFileTextures::SdlFileTextures( const char *fn, const char *name, bool mirror )
	: SdlTextureContainer( name? name : fn )
{
	data.pFileInfo = new SdlFileSurfacesInfo();
	if( !data.pFileInfo->load( fn ) )
	{
		BasicApp::the->logLevelLine( "ERROR: SdlFileTextures: file cannot be read correctly: " );
		BasicApp::the->logString( fn );
		failed();
		return;
	}
	bYMirror = mirror;
	
	onRefresh();
}

SdlFileTextures::SdlFileTextures( const char *fn, SDL_Rect rBeg, Uint32 num, Uint32 NumPerLine, bool byrow,
								  const char *name, ARGB_COLOR colorkey, const char *type, bool mirror )
	: SdlTextureContainer( name ? name : fn )
{
	strFilename = fn;
	if( strFilename.empty() )	{
		failed();
		data.pRects = NULL;
		return;
	}
	bYMirror = mirror;
	data.pRects = new RectData;
	data.pRects->rBegin = rBeg;
	data.pRects->nNum = num;
	data.pRects->nNumPerLine = NumPerLine;
	data.pRects->bByRow = byrow;
	data.pRects->ulColorKey = colorkey;
	if( type )	{
		strncpy( data.pRects->chType, type, 3 );
		data.pRects->chType[3] = '\0';
	}
	else
		data.pRects->chType[0] = '\0';
	
	onRefresh();
}


SdlFileTextures::~SdlFileTextures()
{
	// delete all surfaces
	free();
	if( data.pRects )
	{
		if( createdByInfoFile() )
			delete data.pFileInfo;
		else
			delete data.pRects;
	}
}

void SdlFileTextures::free()
{
	for( int i = 0; i < lTextures.size(); ++i )
	{
		delete lTextures[i];
	}
	lTextures.clear();
}

void SdlFileTextures::onRefresh()
{
	SdlSurface *sysSurface;
	if( createdByInfoFile() )
	{// load by data.fileinfo
		sysSurface = new SdlFileSurface( data.pFileInfo->getBitmapFN(), (char*)-1, data.pFileInfo->getTransparentColor(), NULL, SDL_SWSURFACE );
		if( sysSurface->isFailed() )	{
			failed();
			delete sysSurface;
			return;
		}
		// delete all textures
		free();
		lTextures.reserve( data.pFileInfo->numRects() );
		
		const SDL_Rect *r = data.pFileInfo->getFirstRect();
		GLenum eFormat;
		while( r )	{
			unsigned char *pPixels = SdlTexture::pixelsFromSurface4ub( sysSurface->surf(), *r, eFormat, bYMirror );
			if( !pPixels )
			{
				failed();
				BasicApp::the->logLevelLine( "ERROR: SdlFileTextures: pixels cannot be copied, file: " );
				BasicApp::the->logString( data.pFileInfo->getBitmapFN() );
				free();
				delete sysSurface;
				return;
			}
			GLuint id;
			glGenTextures( 1, &id );
			glBindTexture( GL_TEXTURE_2D, id );
			glTexImage2D( GL_TEXTURE_2D, 0, 4, r->w, r->h, 0,
						  eFormat, GL_UNSIGNED_BYTE, pPixels );
			delete[] pPixels;
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			lTextures.push_back( new SdlTexture( id, r->w, r->h, (char*)-1 ) );
			
			r = data.pFileInfo->getNextRect();
		}
	}
	else
	{// load by data.rects
		sysSurface = new SdlFileSurface( strFilename.c_str(), (char*)-1, data.pRects->ulColorKey, data.pRects->chType, SDL_SWSURFACE );
		if( sysSurface->isFailed() )	{
			failed();
			delete sysSurface;
			return;
		}
		// delete all surfaces
		free();
		lTextures.reserve( data.pRects->nNum );
		
		SDL_PixelFormat *f = sysSurface->surf()->format;
		SDL_Rect r = data.pRects->rBegin;
		GLenum eFormat;
		for( int n = 0; n < data.pRects->nNum; ++n )
		{
			unsigned char *pPixels = SdlTexture::pixelsFromSurface4ub( sysSurface->surf(), r, eFormat, bYMirror );
			if( !pPixels )
			{
				failed();
				BasicApp::the->logLevelLine( "ERROR: SdlFileTextures: pixels cannot be copied, file: " );
				BasicApp::the->logString( data.pFileInfo->getBitmapFN() );
				free();
				delete sysSurface;
				return;
			}
			GLuint id;
			glGenTextures( 1, &id );
			glBindTexture( GL_TEXTURE_2D, id );
			glTexImage2D( GL_TEXTURE_2D, 0, 4, r.w, r.h, 0,
						  eFormat, GL_UNSIGNED_BYTE, pPixels );
			delete[] pPixels;
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			lTextures.push_back( new SdlTexture( id, r.w, r.h, (char*)-1 ) );
			
			if( data.pRects->bByRow )	{
				r.x += r.w;
				if( r.x >= r.w * data.pRects->nNumPerLine )	{
					r.x = 0;
					r.y += r.h;
				}
			}
			else	{
				r.y += r.h;
				if( r.y >= r.h * data.pRects->nNumPerLine )	{
					r.y = 0;
					r.x += r.w;
				}
			}
		}
	}
	delete sysSurface;
}
