/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfont.h"
#include <list>
#include "basicapp.h"
#include "sdlsurface.h"
#include "sdltexturecontainer.h"
#include "rectfunctions.h"
#include "functions.h"
#include "oglfunctions.h"

using namespace std;

SdlFont::FONT *SdlFont::lFonts = 0;
const int SdlFont::nFonts = 16;

SdlFont *SdlFont::get( int n, bool light )
{
	if( lFonts == 0 )
		return 0;
	if( n >= nFonts )
		n = nFonts - 1;
/*	if( light )
		if( lFonts[n].light )
			return lFonts[n].light;
		else	{
			light = false;
			return lFonts[n].dark;
		}

	if( lFonts[n].dark )
		return lFonts[n].dark;

	light = true;
	return lFonts[n].light;*/

	return lFonts[n].f[light];
}

SdlFont::VAlign SdlFont::strToVAlign( const char *s )
{
	if(strcmp(s, "bottom") == 0)	{
		return bottom;
	}
	if(strcmp(s, "baseline") == 0 || strcmp(s, "center") == 0)	{
		return baseline;
	}
	return top;
}

SdlFont::HAlign SdlFont::strToHAlign( const char *s )
{
	if(strcmp(s, "right") == 0)	{
		return right;
	}
	if(strcmp(s, "middle") == 0 || strcmp(s, "center") == 0)	{
		return center;
	}
	return left;
}

/*********************************************************************/

SdlFont::SdlFont()
{
	init();
	bUpperCase = true;
	color = RGB( 128,128,128 );
}

SdlFont::SdlFont( int n, bool light, SdlImageContainer *b, const char *l )
{
	init();
	create( n, light, b, l );
}


void SdlFont::init()
{
	bmps = 0;
	chLetters = 0;
}

SdlFont::~SdlFont()
{
	destroy();
}


bool SdlFont::create( int n, bool light, SdlImageContainer *b, const char *l, Uint32 c, bool upper )
{
	if( ( b == 0 ) || ( l == 0 ) || ( n >= nFonts ) )	{
		bmps = 0;
		return false;
	}

	if( b->numImages() < strlen( l ) )	{
		BasicApp::the->logLine( "Error: Font cannot be created, due to missing surfs, we have:" );
		BasicApp::the->logString( intToStr( b->numImages(), 10 ) );
		return false;
	}

	destroy();

	bmps = b;
	num = n;
	bLight = light;
	if( lFonts == 0 )	{
		lFonts = new FONT[nFonts];
		memset( lFonts, 0, nFonts * sizeof( FONT ) );
	}

	if( lFonts[n].f[light] )
		delete lFonts[n].f[light];

	lFonts[n].f[light] = this;

	chLetters = new char[strlen( l ) + 1];
	strcpy( chLetters, l );
	bUpperCase = upper || !( strchr(l,'a') && strchr(l,'A') );
	color = c;

	return true;
}

void SdlFont::destroy()
{
	if( chLetters )	{
		delete[] chLetters;
		chLetters = 0;
	}
	if( bmps )	{
		lFonts[num].f[bLight] = 0;
		bmps = 0;
	}
}

int SdlFont::getCurpos( const char *text, Sint32 dstx, Sint32 dsty, Sint32 xCur, Sint32 yCur, VAlign va, HAlign ha )
{
	if( ( bmps == 0 ) || ( chLetters == 0 ) )
		return (Uint32)-1;

	Uint32 result = 0;
	int width, height;
	bmps->getImage( 0 )->getSize( width, height);

	list<int> lLinePos;
	const char *p;
	for( p = text; *p != '\0'; ++p )
		if( *p == '\n' )	{
			lLinePos.push_back( p - text );
		}
	lLinePos.push_back( p - text );

	xCur -= dstx;
	yCur -= dsty;

	int x, y;
	switch(va)	{
//	case top:
//		break;
	case baseline:
		yCur += height * lLinePos.size() / 2;
		break;
	case bottom:
		yCur += height * lLinePos.size();
		break;
	}
	y = yCur / height;
	if( y >= lLinePos.size() )
		return p - text;
	else if( y < 0 )
		return 0;

	list<int>::iterator iLine = lLinePos.begin();
	switch( ha )	{
	case left:
		if( y == 0 )
			return xCur / width;
		for( int i = 1; i < y; ++i )
			++iLine;
		return *iLine + xCur / width;
	case center:	{
		if( y == 0 )
			return (xCur + (*iLine-1)*width/2) / width;
		for( int i = 1; i < y; ++i )	{
			++iLine;
		}
		int lenLast = *iLine;
		++iLine;
		return lenLast + (xCur + (*iLine-lenLast-1)*width/2) / width;
	}
	case right:
		if( y == 0 )
			return xCur / width + *iLine-1;
		for( int i = 1; i < y; ++i )	{
			++iLine;
		}
		int lenLast = *iLine;
		++iLine;
		return lenLast + xCur / width + (*iLine-lenLast-1);
	}
	return 0;
}

int SdlFont::getCurpos( const char *text, const SDL_Rect &dst, Sint32 xCur, Sint32 yCur, VAlign va, HAlign ha )
{
	int width, height;
	bmps->getImage( 0 )->getSize( width, height );

	list<int> lNewLines;
	int maxlen = dst.w / width;
	if( maxlen <= 0 ) maxlen = 1;
	int currlen = 0;
	const char *lastspace = NULL;
	const char *p;
	for( p = text; *p != '\0'; ++p )	{
		switch( *p )	{
		case '\n':
			currlen = 0;
			lastspace = NULL;
			break;
		case ' ':
			lastspace = p;
			++currlen;
			break;
		default:
			++currlen;
			if( currlen > maxlen )	{
				if( lastspace )	{
					p = lastspace + 1;
					lastspace = NULL;
				}
				lNewLines.push_back( p - text );
				currlen = 0;
			}
		}
	}
	char *nltext = new char[ strlen( text ) + 1 + lNewLines.size() ];
	p = text;
	char *pNl = nltext;
	int delta, last = 0;
	for( list<int>::iterator i = lNewLines.begin(); i != lNewLines.end(); ++i )	{
		delta = *i - last;
		strncpy( pNl, p, delta );
		pNl += delta;
		p += delta;
		if( *(pNl-1) == ' ' )
			--pNl;
//		else
//			if( pNl - nltext < curpos )
//				++curpos;
		*pNl = '\n';
		++pNl;
		last = *i;
	}
	strcpy( pNl, p );

	int xPos, yPos;
	switch( va )	{
	case top:
		yPos = dst.y;
		break;
	case bottom:
		yPos = dst.y + dst.h;
		break;
	case baseline:
		yPos = dst.h / 2 + dst.y;
	}
	switch( ha )	{
	case left:
		xPos = dst.x;
		break;
	case center:
		xPos = dst.x + dst.w / 2;
		break;
	case right:
		xPos = dst.x + dst.w;
		break;
	}
	int r = getCurpos( nltext, xPos, yPos, xCur, yCur, va, ha );
	pNl = nltext;
	last = 0;
	for( list<int>::iterator i = lNewLines.begin(); i != lNewLines.end(); ++i )	{
		delta = *i - last;
		pNl += delta;
		if( *(pNl-1) == ' ' )
			--pNl;
		else
			if( pNl - nltext < r )
				--r;
			else break;
		++pNl;
		last = *i;
	}
	delete[] nltext;

	return r;
}

Uint32 SdlFont::write( SdlSurface *surf, const char *text, Sint32 dstx, Sint32 dsty, VAlign va, HAlign ha, const SDL_Rect * clip, int curpos )
{
	if( ( bmps == 0 ) || ( chLetters == 0 ) )
		return (Uint32)-1;

	SdlSurfaceContainer *sc = dynamic_cast<SdlSurfaceContainer *>(bmps);
	if( sc == NULL )
		return (Uint32)-2;

	Uint32 result = 0;
	int width, height;
	sc->getSurface( 0 )->getSize( width, height);

	list<int> lLineLengths;
	int nLines = 1;// using list<>::size() would take more time
	const char *pN = text;
	const char *p;
	for( p = text; *p != '\0'; ++p )
		if( *p == '\n' )
		{
			lLineLengths.push_back( p - pN );
			++nLines;
			pN = p+1;
		}
	lLineLengths.push_back( p - pN );

	int x, y;
	switch(va)
	{
	case top:
		y = dsty;
		break;
	case baseline:
		y = dsty - height * nLines / 2;
		break;
	case bottom:
		y = dsty - height * nLines;
		break;
	}
	list<int>::iterator iLine = lLineLengths.begin();
	switch(ha)
	{
	case left:
		x = dstx;
		break;
	case center:
		x = dstx - ( *iLine * width ) / 2;
		break;
	case right:
		x = dstx - *iLine * width;
		break;
	}

	if( clip )
		surf->setClip( *clip );
	for( const char *p = text; ; p++ )
	{
		if( clip )
		{
			if( x >= clip->x + clip->w )
			{
				pN = strchr( p, '\n' );
				if( pN == NULL )
					break;
				p = pN;
			}
		}
		SDL_Rect cursor_dst;
		if( *p == '\n' )
		{
			setRect( cursor_dst, x-1, y, 2, height );
			y += height;
			if( clip && ( y >= clip->y + clip->h ) )
				break;
			switch( ha )
			{
			case left:
				x = dstx;
				break;
			case center:
				x = dstx - ( *++iLine * width ) / 2;
				break;
			case right:
				x = dstx - *++iLine * width;
				break;
			}
		}
		else if( *p != '\0' )
		{
			char b = *p;
			if( bUpperCase )
				b = toupper( b );

			const char *s;
			if( ( s = strchr( chLetters, b ) ) != 0 )
			{
				int index = ( s - chLetters );
				surf->fastBlit( sc->getSurface( index ), x, y, 0 );
			}
			setRect( cursor_dst, x-1, y, 2, height );
			x += width;
		}
		else
			setRect( cursor_dst, x-1, y, 2, height );

		if( p - text == curpos )
		{
			if( clip )
				intersectRect( cursor_dst, *clip );
			surf->fill( &cursor_dst, color );
		}

		if( *p == '\0' )
			break;
	}

	return result;
}

Uint32 SdlFont::write( SdlSurface *surf, const char *text, const SDL_Rect &dst, VAlign va, HAlign ha, const SDL_Rect * clip, int curpos )
{
	int xPos, yPos;
	char *nltext = calcRectToPoint( text, dst, xPos, yPos, va, ha, curpos );
	write( surf, nltext, xPos, yPos, va, ha, clip, curpos );
	delete[] nltext;
	return 0;
}

// OpenGL methods for writing

Uint32 SdlFont::writeGL( const char *text, const SDL_Rect &dst, VAlign va, HAlign ha, const SDL_Rect * clip, int curpos )
{
	int xPos, yPos;
	char *nltext = calcRectToPoint( text, dst, xPos, yPos, va, ha, curpos );
	writeGL( nltext, xPos, yPos, va, ha, clip, curpos );
	delete[] nltext;
	return 0;
}

Uint32 SdlFont::writeGL( const char *text, Sint32 dstx, Sint32 dsty, VAlign va, HAlign ha, const SDL_Rect * clip, int curpos )
{
	if( ( bmps == 0 ) || ( chLetters == 0 ) )
		return (Uint32)-1;

	SdlTextureContainer *tc = dynamic_cast<SdlTextureContainer *>(bmps);
	if( tc == NULL )
		return (Uint32)-2;

	Uint32 result = 0;
	int width, height;
	tc->getTexture( 0 )->getSize( width, height);

	SDL_Rect dst = { 0, 0, width, height };
	list<int> lLineLengths;
	int nLines = 1;// using list<>::size() would take more time
	const char *pN = text;
	const char *p;
	for( p = text; *p != '\0'; ++p )
		if( *p == '\n' )
		{
			lLineLengths.push_back( p - pN );
			++nLines;
			pN = p+1;
		}
	lLineLengths.push_back( p - pN );

	int x, y;
	switch(va)
	{
	case top:
		y = dsty;
		break;
	case baseline:
		y = dsty - height * nLines / 2;
		break;
	case bottom:
		y = dsty - height * nLines;
		break;
	}
	list<int>::iterator iLine = lLineLengths.begin();
	switch(ha)
	{
	case left:
		x = dstx;
		break;
	case center:
		x = dstx - ( *iLine * width ) / 2;
		break;
	case right:
		x = dstx - *iLine * width;
		break;
	}

	for( const char *p = text; ; p++ )
	{
		if( clip )
		{
			if( x >= clip->x + clip->w )
			{
				pN = strchr( p, '\n' );
				if( pN == NULL )
					break;
				p = pN;
			}
		}
		SDL_Rect cursor_dst;
		if( *p == '\n' )
		{
			setRect( cursor_dst, x-1, y, 2, height );
			y += height;
			if( clip && ( y >= clip->y + clip->h ) )
				break;
			switch( ha )
			{
			case left:
				x = dstx;
				break;
			case center:
				x = dstx - ( *++iLine * width ) / 2;
				break;
			case right:
				x = dstx - *++iLine * width;
				break;
			}
		}
		else if( *p != '\0' )
		{
			char b = *p;
			if( bUpperCase )
				b = toupper( b );

			const char *s;
			if( ( s = strchr( chLetters, b ) ) != 0 )
			{
				int index = ( s - chLetters );
				dst.x = x;
				dst.y = y;
				if( clip )
					tc->getTexture( index )->drawClipped( *clip, dst );
				else
					tc->getTexture( index )->drawSimple( dst );
			}
			setRect( cursor_dst, x-1, y, 2, height );
			x += width;
		}
		else
			setRect( cursor_dst, x-1, y, 2, height );

		if( p - text == curpos )
		{
			if( clip )
				intersectRect( cursor_dst, *clip );
			glDisable( GL_TEXTURE_2D );
			ogl::setColor3( color );
			ogl::drawRect( cursor_dst );
			ogl::resetColor();
			glEnable( GL_TEXTURE_2D );
		}

		if( *p == '\0' )
			break;
	}

	return result;
}

char *SdlFont::calcRectToPoint( const char *text, const SDL_Rect &dst, int &xPos, int &yPos, VAlign va, HAlign ha, int curpos )
{
	int width, height;
	bmps->getImage( 0 )->getSize( width, height );

	list<int> lNewLines;
	int maxlen = dst.w / width;
	if( maxlen <= 0 ) maxlen = 1;
	int currlen = 0;
	const char *lastspace = NULL;
	const char *p;
	for( p = text; *p != '\0'; ++p )
	{
		switch( *p )
		{
		case '\n':
			currlen = 0;
			lastspace = NULL;
			break;
		case ' ':
			lastspace = p;
			++currlen;
			break;
		default:
			++currlen;
			if( currlen > maxlen )
			{
				if( lastspace )
				{
					p = lastspace + 1;
					lastspace = NULL;
				}
				lNewLines.push_back( p - text );
				currlen = 0;
			}
		}
	}
	char *nltext = new char[ strlen( text ) + 1 + lNewLines.size() ];
	p = text;
	char *pNl = nltext;
	int delta, last = 0;
	for( list<int>::iterator i = lNewLines.begin(); i != lNewLines.end(); ++i )
	{
		delta = *i - last;
		strncpy( pNl, p, delta );
		pNl += delta;
		p += delta;
		if( *(pNl-1) == ' ' )
			--pNl;
		else
			if( pNl - nltext < curpos )
				++curpos;
		*pNl = '\n';
		++pNl;
		last = *i;
	}
	strcpy( pNl, p );

	switch( va )
	{
		case top:
			yPos = dst.y;
			break;
		case bottom:
			yPos = dst.y + dst.h;
			break;
		case baseline:
			yPos = dst.h / 2 + dst.y;
	}
	switch( ha )
	{
		case left:
			xPos = dst.x;
			break;
		case center:
			xPos = dst.x + dst.w / 2;
			break;
		case right:
			xPos = dst.x + dst.w;
			break;
	}
	return nltext;
}

/*****************************************************************/

const char *SdlFont::strnchr( const char *string, int c, size_t count )
{
	const char *find = strchr( string, c );
	if( find && ( ( find - string ) < count ) )
		return find;
	return 0;
}

const char *SdlFont::strnrchr( const char *string, int c, size_t count )
{
	if( count > strlen( string ) )
		count = strlen( string );
	for( const char *pStr = string + count-1; pStr >= string; --pStr )	{
		if( c == *pStr )
			return pStr;
	}
	return NULL;
/*	const char *find = NULL, *tmp = string;
	while(((tmp = strchr(tmp, c)) != NULL) && ((tmp - string) < count))	{
		find = tmp;
		tmp++;
	}
	return find;*/
}
