/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 - 2007 by Tobias Nadler                                  */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlmanager.h"

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <GL/gl.h>
#include <stdlib.h>
#include "functions.h"
#include "rectfunctions.h"
#include "definitions.h"

#include "basicapp.h"
#include "sdlaudio.h"
#include "sdlimage.h"
#include "sdlmusiclist.h"

bool SdlManager::bGrabInputEnabled = false;

SdlManager *SdlManager::the = NULL;
SdlManager *SdlManager::getThe()
{
	return the;
}

SdlManager::SdlManager()
{
	the = this;
	surfDisplay = 0;
	bAudioInited = false;
	bVideoInited = false;
	nVideoFlags = 0;
}

SdlManager::~SdlManager()
{
	the = 0;
}

bool SdlManager::init()
{
	nColorDepth = 32;
	bVideoInited = initDisplay( 1024, 768, nColorDepth, true );
	if( !bVideoInited )	// VIDEO must be inited
		return false;

	bAudioInited &= initAudio();

	return bVideoInited && bAudioInited;
}

bool SdlManager::init( TiXmlElement *el )
{
	Uint32 w, h, d;
	const char *ch = el->Attribute("Width");
	w = ch ? xStrToUL( ch ) : 1024;
	ch = el->Attribute("Height");
	h = ch ? xStrToUL( ch ) : 768;
	ch = el->Attribute("Depth");
	d = ch ? xStrToUL( ch ) : 32;
	ch = el->Attribute("Fullscreen");
	bool full = ch ? strToBool( ch ) : true;
	ch = el->Attribute("OpenGL");
	bool ogl = ch ? strToBool( ch ) : false;
	if( full )	{
		grabInput();
	}
	bool ret = init( w, h, d, full, ogl );
	
	if( el->FirstChildElement("WaitOnStart") && (ch = el->FirstChildElement("WaitOnStart")->Attribute("value")) )	{
		SDL_Delay( xStrToUL(ch) );
	}
	return ret;
}

bool SdlManager::init( Uint32 width, Uint32 height, Uint32 &depth, bool fullscreen, bool opengl )
{
	bVideoInited = initDisplay( width, height, depth, fullscreen, opengl );
	if( !bVideoInited )	// VIDEO must be inited
		return false;

	bAudioInited = initAudio();

	return bVideoInited && bAudioInited;
}


void SdlManager::destroy()
{
	ungrabInput();

	BasicApp::the->logLevelLine("Cleaning audio data...");
	while( !lAudios.empty() )
		delete lAudios.front();
	BasicApp::the->logLevelLine("Cleaning music data...");
	while( !lMusicLists.empty() )
		delete lMusicLists.front();
	BasicApp::the->logLevelLine("Cleaning surface data...");
	while( !lImages.empty() )
		delete lImages.front();
	
	BasicApp::the->logLevelLine("Cleaning surface-container data...");
	while( !lImageContainers.empty() )
		delete lImageContainers.front();

	if( bAudioInited )	{
		Mix_CloseAudio();
		bAudioInited = false;
	}
	SDL_Quit();
	BasicApp::the->logLevelLine("Quitted SDL.");
}

bool SdlManager::initDisplay( Uint32 w, Uint32 h, Uint32 &d, bool fullscr, bool opengl )
{
	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
		return false;
	nVideoFlags = SDL_HWSURFACE;

	if( opengl )
	{
		if( fullscr )
		{// TODO: test this with fullscreen mode
			Uint32 bitspercolor = d/3;// minimal number of bits per color
			SDL_GL_SetAttribute( SDL_GL_RED_SIZE, bitspercolor );
			SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, bitspercolor );
			SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, bitspercolor );
			SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, d );
		}
		SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
		nVideoFlags |= SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWACCEL;
		BasicApp::the->logLevelLine( "Setting up video mode for OpenGL" );
	}
	else
		nVideoFlags |= SDL_DOUBLEBUF;
	
	if( fullscr )
		nVideoFlags |= SDL_FULLSCREEN;

	d = SDL_VideoModeOK( w, h, d, nVideoFlags );
	if( d == 0 )
	{
		BasicApp::the->logLevelLine( "Setting of video mode failed!" );
		return false;
	}
	nColorDepth = d;

	SDL_Surface *display = SDL_SetVideoMode( w, h, d, nVideoFlags );
	if( display == NULL )
	{
		return false;
	}
	surfDisplay = new SdlSurface( display, (char*)-1 );
	return true;
}

bool SdlManager::getScreenSize( int &w, int &h )
{
	surfDisplay->getSize( w, h );
	return isFullscreen();
}

const SDL_Rect &SdlManager::getScreenRect()
{
	static SDL_Rect result;
	setRect( result, 0, 0, surfDisplay->getWidth(), surfDisplay->getHeight() );
	return result;
}


bool SdlManager::initAudio( Uint8 channels, int freq, Uint16 format )
{
	if( SDL_Init( SDL_INIT_AUDIO ) < 0 )
		return false;
	if( Mix_OpenAudio( freq, format, channels, 1024/*4096(takes too much time)*/ ) != 0 ) {
		BasicApp::the->logLine("ERROR: Unable to initialize audio");// Mix_GetError()
		return false;
	}

	return true;
}



void SdlManager::refreshSoundVolumes()
{
	for( AudioIter i = lAudios.begin(); i != lAudios.end(); ++i)
		if( !(*i)->isMusic() )
			(*i)->setVolume( -1 );
}


SdlAudio *SdlManager::getAudioByName( const char *name )
{
	for( AudioIter i = lAudios.begin(); i != lAudios.end(); ++i)
		if( strcmp( (*i)->getName(), name ) == 0 )
			return *i;

	return 0;
}

SdlImage *SdlManager::getImageByName( const char *name )
{
	for( ImageIter i = lImages.begin(); i != lImages.end(); ++i )
		if( strcmp( (*i)->getName(), name ) == 0 )
			return *i;

	return 0;
}

SdlImageContainer *SdlManager::getImagesByName( const char *name )
{
	for( ImagesIter i = lImageContainers.begin(); i != lImageContainers.end(); ++i )
		if( strcmp( (*i)->getName(), name ) == 0 )
			return *i;

	return 0;
}

SdlMusicList *SdlManager::getMusicsByName( const char *name )
{
	for( MusicsIter i = lMusicLists.begin(); i != lMusicLists.end(); ++i )
		if( strcmp( (*i)->getName(), name ) == 0 )
			return *i;

	return 0;
}


bool SdlManager::screenShot( const char *fn, bool nooverwrite )
{
static int nShot = 0;

	if( surfDisplay == NULL )
		return false;
	
	char filename[4096];
	if( nooverwrite )	{
		ifstream file;
		if( strstr(fn, "%d") == NULL )	{
			file.open( fn, ios::in );
			if( !file.is_open() )	{
				strcpy( filename, fn );
			}
			else	{
				file.close();
				return false;
			}
		}
		else	{
			for( int nFile = 1; ; ++nFile )	{
				if( fn )
					sprintf( filename, fn, nFile );
				else
					sprintf( filename, "./screens/photo%d.bmp", nFile );
				file.open( filename, ios::in );
				if( !file.is_open() )
					break;
				file.close();
			}
		}
	}
	else	{
		++nShot;
		if( fn )
			sprintf( filename, fn, nShot );
		else
			sprintf( filename, "./screens/photo%d.bmp", nShot );
	}
	return surfDisplay->saveToFile( filename );
}

