/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 - 2007 by Tobias Nadler                                  */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlsurface.h"
#include <string.h>
#include "SDL/SDL_rotozoom.h"

#include "definitions.h"
#include "sdlmanager.h"
#include "rectfunctions.h"
#include "basicapp.h"

SdlSurface::SdlSurface( const char *name )
	: SdlImage( name )
{
	surface = NULL;
}

SdlSurface::SdlSurface( SDL_Surface *surf, const char *name )
	: SdlImage( name )
{
	surface = surf;
	if( surf == 0 )
		BasicApp::the->logLine( "WARNING: SdlSurface: no surf specified" );
	else
	{
		nWidth = surf->w;
		nHeight = surf->h;
	}
}

SdlSurface::SdlSurface( Uint32 w, Uint32 h, const char *name, Uint32 flags )
	: SdlImage( name )
{
	SDL_PixelFormat *f = SdlManager::the->getDisplay()->surf()->format;
	surface = SDL_CreateRGBSurface( flags, w, h, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask );
	if( surface == 0 )
		BasicApp::the->logLine( "ERROR: SdlSurface: could not create surface" );
	else
	{
		nWidth = surface->w;
		nHeight = surface->h;
	}
}

SdlSurface::~SdlSurface()
{
	if( surface )
		SDL_FreeSurface( surface );
}

bool SdlSurface::saveToFile( const char *fn )
{
	if( surface == 0 )
		return false;
	
	return SDL_SaveBMP( surface, fn ) == 0;
}


bool SdlSurface::fastBlit( SdlSurface *surf, Sint16 xDst, Sint16 yDst, SDL_Rect *src )
{
	static SDL_Rect r = { 0,0,0,0 };

	r.x = xDst;
	r.y = yDst;
	return SDL_BlitSurface( surf->surf(), src, surface, &r ) == 0;
}

bool SdlSurface::fill( SDL_Rect *r, Uint32 color )
{
	return SDL_FillRect( surface, r, color ) == 0;
}

bool SdlSurface::fillARGB( SDL_Rect *r, ARGB_COLOR color )
{
	return SDL_FillRect( surface, r, convertColor( color ) ) == 0;
}

bool SdlSurface::zoomBlit( SdlSurface *src_surf, Sint16 xDst, Sint16 yDst, double xZoom, double yZoom, bool smooth )
{
	static SDL_Rect r = { 0,0,0,0 };

	SDL_Surface *temp = zoomSurface( src_surf->surf(), xZoom, yZoom, (int)smooth );
	if( temp == 0 )
		return false;
	SDL_SetColorKey( temp, SDL_SRCCOLORKEY, src_surf->surf()->format->colorkey );
	SDL_SetAlpha( temp, SDL_SRCALPHA, src_surf->surf()->format->alpha );

	r.x = xDst;
	r.y = yDst;
	bool res = SDL_BlitSurface( temp, 0, surface, &r ) == 0;
	SDL_FreeSurface( temp );
	return res;
}

bool SdlSurface::stretchBlit( SdlSurface *src_surf, SDL_Rect *dst, bool smooth )
{
	double xZoom = (double)dst->w / (double)src_surf->getWidth(),
		yZoom = (double)dst->h / (double)src_surf->getHeight();
	SDL_Surface *temp = zoomSurface( src_surf->surf(), xZoom, yZoom, (int)smooth );
	if( temp == 0 )
		return false;
	SDL_SetColorKey( temp, SDL_SRCCOLORKEY | SDL_RLEACCEL, src_surf->surf()->format->colorkey );
	SDL_SetAlpha( temp, SDL_SRCALPHA | SDL_RLEACCEL, src_surf->surf()->format->alpha );

	bool res = SDL_BlitSurface( temp, 0, surface, dst ) == 0;
	SDL_FreeSurface( temp );
	return res;
}

bool SdlSurface::rotozoomBlit( SdlSurface *src_surf, Sint16 xDst, Sint16 yDst, double angle, double zoom, bool smooth )
{
	static SDL_Rect r = { 0,0,0,0 };

	SDL_Surface *temp = rotozoomSurface( src_surf->surf(), angle, zoom, (int)smooth );
	if( temp == 0 )
		return false;
	SDL_SetColorKey( temp, SDL_SRCCOLORKEY | SDL_RLEACCEL, src_surf->surf()->format->colorkey );
	SDL_SetAlpha( temp, SDL_SRCALPHA | SDL_RLEACCEL, src_surf->surf()->format->alpha );

	r.x = xDst - temp->w / 2;
	r.y = yDst - temp->h / 2;
	bool res = SDL_BlitSurface( temp, 0, surface, &r ) == 0;
	SDL_FreeSurface( temp );
	return res;
}


void SdlSurface::setColorKey( ARGB_COLOR color )
{
	if( color == CLR_INVALID )
		SDL_SetColorKey( surface, 0, 0 );
	else
		SDL_SetColorKey( surface, SDL_SRCCOLORKEY | SDL_RLEACCEL, convertColor( color ) );
}

void SdlSurface::setAlphaValue( unsigned char a )
{
	if( a == SDL_ALPHA_OPAQUE )// disable transparency
		SDL_SetAlpha( surface, 0, 0 );
	else
		SDL_SetAlpha( surface, SDL_SRCALPHA | SDL_RLEACCEL, a );
}

SDL_Color SdlSurface::getPixel( unsigned int x, unsigned int y )
{
	Uint32 value = *(Uint32*)((char*)surface->pixels + surface->pitch * y + surface->format->BytesPerPixel * x);
	SDL_Color c;
	SDL_GetRGB( value, surface->format, &c.r, &c.g, &c.b );
	return c;
}

void SdlSurface::setPixel( unsigned int x, unsigned int y, Uint32 color )
{
	char *p = (char*)surface->pixels + surface->pitch * y + surface->format->BytesPerPixel * x;
	memcpy( p, &color, surface->format->BytesPerPixel );
}

void SdlSurface::setPixel( unsigned int x, unsigned int y, SDL_Color color )
{
	Uint32 c = SDL_MapRGB( surface->format, color.r, color.g, color.b );
	char *p = (char*)surface->pixels + surface->pitch * y + surface->format->BytesPerPixel * x;
	memcpy( p, &c, surface->format->BytesPerPixel );
}

Uint32 SdlSurface::convertColor( ARGB_COLOR cARGB )
{
	// invert alpha-value, because 0xFFFFFFFF shall be the invalid color (= invisible)
	cARGB ^= 0xFF000000;
	
	SDL_PixelFormat *f = surface->format;
	if( f->Amask == 0xFF000000 && f->Rmask == 0xFF0000 && f->Gmask == 0x00FF00 && f->Bmask == 0x0000FF )
		return cARGB;// no converting required
	
	return SDL_MapRGBA( f, cARGB >> 16, cARGB >> 8, cARGB, cARGB >> 24 );
}

bool SdlSurface::isColorVisible( Uint32 color )
{
	return surface->format->Amask & color > 0;
}

void SdlSurface::setSurface( SDL_Surface *s )
{
	surface = s;
	if( s )
	{
		nWidth = s->w;
		nHeight = s->h;
	}
}
