/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdltexture.h"
#include <string.h>

#include "definitions.h"
#include "sdlmanager.h"
#include "rectfunctions.h"
#include "oglfunctions.h"
#include "basicapp.h"

unsigned char *SdlTexture::pixelsFromSurface4ub( SDL_Surface *src, GLenum &format, bool mirrored )
{
	SDL_Rect r = {0, 0, src->w, src->h};
	return pixelsFromSurface4ub( src, r, format, mirrored );
}

unsigned char *SdlTexture::pixelsFromSurface4ub( SDL_Surface *src, const SDL_Rect &r, GLenum &format, bool mirrored )
{
	SDL_PixelFormat *pFormat = src->format;
	Uint8 nBytesPerPixel = pFormat->BytesPerPixel;
	if( nBytesPerPixel < 3 || nBytesPerPixel > 4 )
	{
		BasicApp::the->logLevelLine("WARNING: SdlTexture::pixelsFromSurface4b returned NULL, due to unsupported bits-per-pixel value (only 24bpp and 32bpp are supported)");
		return NULL;
	}
#ifdef __BIG_ENDIAN__
#warning big-endian is untested and may be very bugy
	if( pFormat->Bshift == 24 && pFormat->Gshift == 16 && pFormat->Rshift == 8 )
		format = GL_BGRA;
	else if( pFormat->Bshift == 8 && pFormat->Gshift == 16 && pFormat->Rshift == 24 )
		format = GL_RGBA;
	else
	{
		BasicApp::the->logLevelLine("WARNING: SdlTexture::pixelsFromSurface4b returned NULL, due to unsupported byte order");
		return NULL;
	}
#else
	if( pFormat->Bshift == 0 && pFormat->Gshift == 8 && pFormat->Rshift == 16 )
		format = GL_BGRA;
	else if( pFormat->Bshift == 16 && pFormat->Gshift == 8 && pFormat->Rshift == 0 )
		format = GL_RGBA;
	else
	{
		BasicApp::the->logLevelLine("WARNING: SdlTexture::pixelsFromSurface4b returned NULL, due to unsupported byte order");
		return NULL;
	}
#endif

	int w4 = r.w * 4;
	unsigned char *pResult = new unsigned char[w4 * r.h];
	if( nBytesPerPixel == 4 )
	{
		unsigned char *p_bPixels = pResult;
		unsigned char *p_bSource = (unsigned char*)src->pixels + 4 * r.x + r.y * src->pitch;
		if( mirrored )
		{
			p_bPixels += w4 * r.h;
			while( p_bPixels != pResult )
			{
				p_bPixels -= w4;
				memcpy( p_bPixels, p_bSource, w4 );
				p_bSource += src->pitch;
			}
		}
		else	{
			unsigned char *p_bEnd = pResult + w4 * r.h;
			while( p_bPixels != p_bEnd )
			{
				memcpy( p_bPixels, p_bSource, w4 );
				p_bPixels += w4;
				p_bSource += src->pitch;
			}
		}
	}
	else // extend to 4 bytes per pixel and set alpha values to zero where transparent
	{
		unsigned char *p_bPixels = pResult;
		if( mirrored )
			p_bPixels += w4 * r.h;
		unsigned char *p_bSource = (unsigned char*)src->pixels + 3 * r.x + r.y * src->pitch;
		bool isColorKey;
		int i,j;
		Uint32 ulColorKey = pFormat->colorkey;
		unsigned char a_bColorKey[3];
		switch( format )
		{
			case GL_RGBA:
				a_bColorKey[0] = ulColorKey >> 16;
				a_bColorKey[1] = ulColorKey >> 8;
				a_bColorKey[2] = ulColorKey;
				break;
			case GL_BGRA:
				a_bColorKey[0] = ulColorKey;
				a_bColorKey[1] = ulColorKey >> 8;
				a_bColorKey[2] = ulColorKey >> 16;
				break;
			default:
				BasicApp::the->logLevelLine("ERROR: SdlTexture::pixelsFromSurface4ub: implementation lacks.");
		}
		unsigned char *p_bEnd;
		if( mirrored )
			p_bEnd = pResult;
		else
			p_bEnd = pResult + w4 * r.h;
		
		while( p_bPixels != p_bEnd )
		{
			if( mirrored )
				p_bPixels -= w4;
			j = 0;
			if( ulColorKey == CLR_INVALID )
			{
				for( i = 0; i < w4; i++ )
				{
					if( (i & 3) == 3 )
					{
						p_bPixels[i] = 255;
						continue;
					}
					p_bPixels[i] = p_bSource[j++];
				}
			}
			else
			{
				isColorKey = true;
				for( i = 0; i < w4; i++ )
				{
					if( (i & 3) == 3 )
					{
						p_bPixels[i] = isColorKey ? 0 : 255;
						isColorKey = true;
						continue;
					}
					p_bPixels[i] = p_bSource[j++];
					if( p_bPixels[i] != a_bColorKey[i & 3] )
						isColorKey = false;
				}
			}
			p_bSource += src->pitch;
			if( !mirrored )
				p_bPixels += w4;
		}
	}
	return pResult;
}


SdlTexture::SdlTexture( const char *name )
	: SdlImage( name )
{
	nId = 0;
}

SdlTexture::SdlTexture( GLuint gl_id, unsigned int w, unsigned int h, const char *name )
	: SdlImage( name )
{
	nId = gl_id;
	if( nId == 0 )
		BasicApp::the->logLevelLine("WARNING: SdlTexture created with gl_id = 0");
	nWidth = w;
	nHeight = h;
}

SdlTexture::~SdlTexture()
{
	if( nId != 0 )
	{
		glDeleteTextures( 1, &nId );
	}
}


void SdlTexture::drawSimple( const SDL_Rect &r )
{
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin( GL_QUADS );
	 glTexCoord2f( 0.0f, 0.0f );// bottom-left
	 glVertex2s( r.x, r.y );
	 glTexCoord2f( 1.0f, 0.0f );// bottom-right
	 glVertex2s( r.x+r.w, r.y );
	 glTexCoord2f( 1.0f, 1.0f );// top-right
	 glVertex2s( r.x+r.w, r.y+r.h );
	 glTexCoord2f( 0.0f, 1.0f );// top-left
	 glVertex2s( r.x, r.y+r.h );
	glEnd();
}

void SdlTexture::draw( const FloatRect &src, const SDL_Rect &dst )
{
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin( GL_QUADS );
	 glTexCoord2f( src.x, src.y );// bottom-left
	 glVertex2s( dst.x, dst.y );
	 glTexCoord2f( src.x+src.w, src.y );// bottom-right
	 glVertex2s( dst.x+dst.w, dst.y );
	 glTexCoord2f( src.x+src.w, src.y+src.h );// top-right
	 glVertex2s( dst.x+dst.w, dst.y+dst.h );
	 glTexCoord2f( src.x, src.y+src.h );// top-left
	 glVertex2s( dst.x, dst.y+dst.h );
	glEnd();
}

void SdlTexture::draw( const SDL_Rect &src, const SDL_Rect &dst )
{
	GLfloat x[2];
	x[0] = (GLfloat)src.x / (GLfloat)nWidth;
	x[1] = (GLfloat)(src.x + src.w) / (GLfloat)nWidth;
	GLfloat y[2];
	y[0] = (GLfloat)src.y / (GLfloat)nHeight;
	y[1] = (GLfloat)(src.y + src.h) / (GLfloat)nHeight;
	
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin(GL_QUADS);
	 glTexCoord2f( x[0], y[0] );// bottom-left
	 glVertex2s( dst.x, dst.y );
	 glTexCoord2f( x[1], y[0] );// bottom-right
	 glVertex2s( dst.x+dst.w, dst.y );
	 glTexCoord2f( x[1], y[1] );// top-right
	 glVertex2s( dst.x+dst.w, dst.y+dst.h );
	 glTexCoord2f( x[0], y[1] );// top-left
	 glVertex2s( dst.x, dst.y+dst.h );
	glEnd();
}

void SdlTexture::drawClipped( SDL_Rect clip, SDL_Rect whole )
{
	intersectRect( clip, whole );
	clip.x -= whole.x;
	whole.x += clip.x;
	clip.y -= whole.y;
	whole.y += clip.y;
	whole.w = clip.w;
	whole.h = clip.h;
	GLfloat x[2];
	x[0] = (GLfloat)clip.x / (GLfloat)nWidth;
	x[1] = (GLfloat)(clip.x + clip.w) / (GLfloat)nWidth;
	GLfloat y[2];
	y[0] = (GLfloat)clip.y / (GLfloat)nHeight;
	y[1] = (GLfloat)(clip.y + clip.h) / (GLfloat)nHeight;
	
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin(GL_QUADS);
	 glTexCoord2f( x[0], y[0] );// bottom-left
	 glVertex2s( whole.x, whole.y );
	 glTexCoord2f( x[1], y[0] );// bottom-right
	 glVertex2s( whole.x+whole.w, whole.y );
	 glTexCoord2f( x[1], y[1] );// top-right
	 glVertex2s( whole.x+whole.w, whole.y+whole.h );
	 glTexCoord2f( x[0], y[1] );// top-left
	 glVertex2s( whole.x, whole.y+whole.h );
	glEnd();
}

void SdlTexture::rotoDraw( const FloatRect &src, const SDL_Rect &dst, float angle )
{
	glPushMatrix();
	GLshort w_2 = dst.w/2, h_2 = dst.h/2;
	glTranslatef( (GLfloat)(dst.x + w_2), (GLfloat)(dst.y + h_2), 0.f );
	glRotatef( angle, 0.f, 0.f, 1.f );
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin( GL_QUADS );
	 glTexCoord2f( src.x, src.y );// bottom-left
	 glVertex2s( -w_2, -h_2 );
	 glTexCoord2f( src.x+src.w, src.y );// bottom-right
	 glVertex2s( w_2, -h_2 );
	 glTexCoord2f( src.x+src.w, src.y+src.h );// top-right
	 glVertex2s( w_2, h_2 );
	 glTexCoord2f( src.x, src.y+src.h );// top-left
	 glVertex2s( -w_2, h_2 );
	glEnd();
	glPopMatrix();
}

void SdlTexture::rotoDrawClipped( const SDL_Rect &clip, int x, int y, float angle, float zoom )
{
	ogl::set2dClippingPlanes( clip );
	glPushMatrix();
	GLfloat w_2 = (float)getWidth() * zoom / 2.f,
			h_2 = (float)getHeight()* zoom / 2.f;
	glTranslatef( (GLfloat)x, (GLfloat)y, 0.f );
	glRotatef( angle, 0.f, 0.f, 1.f );
	glBindTexture( GL_TEXTURE_2D, nId );
	glBegin( GL_QUADS );
	 glTexCoord2f( 0.f, 0.f );// bottom-left
	 glVertex2f( -w_2, -h_2 );
	 glTexCoord2f( 1.f, 0.f );// bottom-right
	 glVertex2f( w_2, -h_2 );
	 glTexCoord2f( 1.f, 1.f );// top-right
	 glVertex2f( w_2, h_2 );
	 glTexCoord2f( 0.f, 1.f );// top-left
	 glVertex2f( -w_2, h_2 );
	glEnd();
	glPopMatrix();
	ogl::disableClippingPlanes( 4 );
}

/*SDL_Color SdlTexture::getPixel( unsigned int x, unsigned int y )
{
	SDL_Color c;
	return c;
}*/

void SdlTexture::setPixel( unsigned int x, unsigned int y, Uint32 color )
{
#ifdef __BIG_ENDIAN__
	Uint32 c = (color << 8) | (color >> 24);
	glTexSubImage2D( GL_TEXTURE_2D, 0, x, y, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, &c );
#else
	glTexSubImage2D( GL_TEXTURE_2D, 0, x, y, 1, 1, GL_BGRA, GL_UNSIGNED_BYTE, &color );
#endif
}

void SdlTexture::setPixel( unsigned int x, unsigned int y, SDL_Color color )
{
	glTexSubImage2D( GL_TEXTURE_2D, 0, x, y, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, &color );
}


