/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 - 2007 by Tobias Nadler                                  */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <stdlib.h>
#include <fstream>
#include <string>
#ifdef __windows__
#include <direct.h>
#endif

#include "uifiletheme.h"

#include "sdlfilesurface.h"
#include "sdlfilesurfaces.h"
#include "sdlfontsurfaces.h"
#include "sdlfiletexture.h"
#include "sdlfiletextures.h"
#include "sdlfilemusic.h"
#include "sdlfilesound.h"
#include "sdlfilemusiclist.h"
#include "sdlfont.h"
#include "guiprogressbar.h"
#include "basicapp.h"
#include "rectfunctions.h"
#include "functions.h"
#include "uitranslator.h"


UiFileTheme::UiFileTheme( const char *fn, const char *name ): UiTheme( name? name : fn )
{
	chFileName = new char[ strlen( fn ) + 1 ];
	strcpy( chFileName, fn );
	bLoaded = false;
	translator = 0;
}

UiFileTheme::~UiFileTheme()
{
	for( infoIter i = lInfos.begin(); i != lInfos.end(); ++i )	{
		delete[] (*i).content;
		delete[] (*i).usage;
	}

	if( chFileName )
		delete[] chFileName;
}

const char *UiFileTheme::getString( const char *usage, int *lenght )
{
	for( infoIter i = lInfos.begin(); i != lInfos.end(); ++i )
		if( strcmp( (*i).usage, usage ) == 0 )	{
			if( lenght )
				*lenght = (*i).lenght;
			return (*i).content;
		}

	BasicApp::the->logLine( "WARNING: Item not in theme: " );
	BasicApp::the->logString( usage );

	if( lenght )
		*lenght = 0;
	return 0;
}

Uint32 UiFileTheme::getUint32( const char *usage, bool *found )
{
	const char *s = getString( usage );
	if( found )
		*found = s != 0;
	if( s )
		return xStrToUL( s );

	return 0;
}

bool UiFileTheme::getBool( const char *usage, bool *found )
{
	const char *s = getString( usage );
	if( found )
		*found = s != 0;
	if( s )
		return strToBool( s );

	return false;
}

SDL_Rect UiFileTheme::getRect( const char *usage )
{
	const char *s = getString( usage );
	SDL_Rect r;
	if( s )	{
		char *tmp = new char[ strlen(s) + 1 ];
		strcpy( tmp, s );
		r = strToRect( tmp );
		delete[] tmp;
		return r;
	}
	setRectEmpty( r );
	return r;
}

const char *UiFileTheme::translate( const char *text )
{
	if( translator != 0 )
		return translator->translate( text );
	else
		return text;
}


Uint32 UiFileTheme::onLoad(bool hidden)
{
	if( bLoaded )
		return 0;

	BasicApp::the->logBegin( "Loading Filetheme..." );
	int nDone = 0;
	const int nMax = 1000;

	string fn = BasicApp::the->getDataPath();
	fn += chFileName;
	ifstream file( fn.c_str(), ios::in | ios::binary );

	if( !file.is_open() )	{
		BasicApp::the->logLevelLine( "Could not open file:" );
		BasicApp::the->logString( chFileName );
		BasicApp::the->logEnd( "failed." );
		return 1;
	}

	char *cwd = new char[1024];
	getcwd( cwd, 1024 );
	chdir( BasicApp::the->getDataPath() );

	char chLine[ 1024 ];
	char *subs[16];
	unsigned int nParams;

//	list<FONTSURFS> lFS;
	list<FONTINFO> lFonts;

	BasicApp::the->logLevelLine( "Parsing file: " );
	BasicApp::the->logString( chFileName );
	for( int nLine = 1; !file.eof(); ++nLine )	{
		BasicApp::the->logString( "." );
		file.getline( chLine, 1024 );
		int len = strlen( chLine );
		if( len )
			if( chLine[ len - 1 ] == '\r' )
				chLine[ len - 1 ] = '\0';
		if( strncmp( chLine, "#?", 2 ) == 0)// progress in per-mille (1/1000)
		{
			if( chLine[2] == ':' )	{
				if(!hidden)
					GuiProgressBar::initsimple( &chLine[3] );
				continue;
			}
			nDone = atoi( &chLine[2] );
			if(!hidden)
				GuiProgressBar::showprogresssimple( (float)nDone / (float)nMax );
			BasicApp::the->logLevelLine( ((string)"Finished " + intToStr(nDone/10) + "%." ).c_str() );
			continue;
		}
		nParams = extractSubStrings( chLine, subs, 16 );
		if( nParams == 0 )
			continue;

		if( strcmp( subs[0], "s" ) == 0 || strcmp( subs[0], "surface" ) == 0 )	{
			if( nParams > 3 )	{
				char *name = *(subs[2]) == '\0' ? 0 : subs[2];
				new SdlFileSurface( subs[1], name, xStrToUL( subs[3] ) );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "ss" ) == 0 || strcmp( subs[0], "surfaces" ) == 0 )	{
			if( nParams < 3 )	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line " );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
			char *name = *(subs[2]) == '\0' ? 0 : subs[2];
			if( nParams <= 7 )	{
				new SdlFileSurfaces( subs[1], name );
			}
			else	{
				new SdlFileSurfaces( subs[1], strToRect( subs[3] ), xStrToUL( subs[4] ), xStrToUL( subs[5] ), strToBool( subs[6] ), name, xStrToUL( subs[7] ) );
			}
		}
		else if( strcmp( subs[0], "fs" ) == 0 || strcmp( subs[0], "fontsurfaces" ) == 0 )	{
			if( nParams <= 4 )	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line " );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
/*			else if( nParams > 6 )	{ unused, due to error in sdlfontsurfaces
				FONTSURFS fs;
				strcpy( fs.chBgBmp, subs[6] );
				strcpy( fs.chTTF, subs[1] );
				strcpy( fs.chChars, subs[2] );
				strcpy( fs.chName, subs[5] );
				fs.nSize = xStrToUL( subs[3] );
				fs.color = ULToColor( xStrToUL( subs[4] ) );
				lFS.push_back( fs );
			}*/
			else	{
				ifstream file( subs[2], ios::in | ios::binary );
				if( file.is_open() )	{
					char chars[256];
					file.read( chars, 255 );
					chars[ file.gcount() ] = '\0';
					char *name = *(subs[5]) == '\0' ? 0 : subs[5];
					new SdlFontSurfaces( subs[1], chars, xStrToUL( subs[3] ), ULToColor( xStrToUL( subs[4] ) ), name );
				}
				else	{
					BasicApp::the->logLine( "ERROR: cannot open file: " );
					BasicApp::the->logString( subs[2] );
				}
			}
		}
		else if( strcmp( subs[0], "t" ) == 0 || strcmp( subs[0], "texture" ) == 0 )	{
			if( nParams > 2 )	{
				char *name = *(subs[2]) == '\0' ? 0 : subs[2];
				new SdlFileTexture( subs[1], name, nParams > 3? xStrToUL( subs[3] ) : CLR_INVALID );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "ts" ) == 0 || strcmp( subs[0], "textures" ) == 0 )	{
			if( nParams < 3 )	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line " );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
			else
			{
				char *name = *(subs[2]) == '\0' ? 0 : subs[2];
				if( nParams < 7 )	{
					new SdlFileTextures( subs[1], name, nParams > 3 ? strToBool( subs[3] ) : false );
				}
				else	{
					new SdlFileTextures( subs[1], strToRect( subs[3] ), xStrToUL( subs[4] ), xStrToUL( subs[5] ), strToBool( subs[6] ), name, nParams > 7 ? xStrToUL( subs[7] ) : CLR_INVALID );
				}
			}
		}
		else if( strcmp( subs[0], "a" ) == 0 || strcmp( subs[0], "sound" ) == 0 )	{
			if( nParams > 2 )	{
				new SdlFileSound( subs[1], subs[2] );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "m" ) == 0 || strcmp( subs[0], "music" ) == 0 )	{
			if( nParams > 2 )	{
				new SdlFileMusic( subs[1], subs[2] );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "ml" ) == 0 || strcmp( subs[0], "musiclist" ) == 0 )	{
			if( nParams > 2 )	{
				new SdlFileMusicList( subs[1], subs[2] );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "l" ) == 0 || strcmp( subs[0], "language" ) == 0 )	{
			if( nParams > 1 )	{
				if( translator )
					delete translator;
				translator = new UiFileTranslator( subs[1] );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( strcmp( subs[0], "f" ) == 0 || strcmp( subs[0], "font" ) == 0 )	{
			if( nParams > 6 )	{
				FONTINFO f;
				f.n = xStrToUL( subs[1] );
				f.chLightBmps = new char[strlen(subs[2]) + 1];
				strcpy( f.chLightBmps, subs[2] );
				f.chDarkBmps = new char[strlen(subs[3]) + 1];
				strcpy( f.chDarkBmps, subs[3] );
				f.fnChars = new char[strlen(subs[4]) + 1];
				strcpy( f.fnChars, subs[4] );
				f.cLight = xStrToUL( subs[5] );
				f.cDark = xStrToUL( subs[6] );

				lFonts.push_back( f );
			}
			else	{
				BasicApp::the->logLine( "ERROR: missing param(s) in theme-file '" );
				BasicApp::the->logString( chFileName );
				BasicApp::the->logString( "', line '" );
				BasicApp::the->logString( intToStr( nLine, 10 ) );
			}
		}
		else if( nParams == 2 )	{
			INFO i;
			i.usage = new char[ strlen( subs[0] ) + 1 ];
			strcpy( i.usage, subs[0] );
			i.lenght = strlen( subs[1] );
			if( i.lenght == 0 )	{
				i.content = 0;
			}
			else	{
				++i.lenght;
				i.content = new char[ i.lenght ];
				strcpy( i.content, subs[1] );
			}

			lInfos.push_back( i );
		}
		else if( nParams != 1 )	{
			INFO i;
			i.usage = new char[ strlen( subs[0] ) + 1 ];
			strcpy( i.usage, subs[0] );
			i.lenght = 1;
			unsigned int n;
			for( n = 1; n < nParams; ++n )
				i.lenght += strlen( subs[n] ) + 1;
			i.content = new char[ i.lenght ];
			char *pContent = i.content;
			for( n = 1; n < nParams; ++n )	{
				strcpy( pContent, subs[n] );
				pContent += strlen( pContent ) + 1;
			}

			lInfos.push_back( i );
		}
		else	{
			BasicApp::the->logLine( "WARNING: no param in theme-file '" );
			BasicApp::the->logString( chFileName );
			BasicApp::the->logString( "', line '" );
			BasicApp::the->logString( intToStr( nLine, 10 ) );
		}
	}

	char chars[256];
	for( list<FONTINFO>::iterator it = lFonts.begin(); it != lFonts.end(); ++it )	{
		ifstream file( (*it).fnChars, ios::in | ios::binary );
		if( file.is_open() )	{
			file.read( chars, 255 );
			chars[ file.gcount() ] = '\0';
			(new SdlFont( (*it).n, true, SdlManager::the->getImagesByName( (*it).chLightBmps ), chars ))->color = (*it).cLight;
			(new SdlFont( (*it).n, false, SdlManager::the->getImagesByName( (*it).chDarkBmps ), chars ))->color = (*it).cDark;
		}
		else	{
			BasicApp::the->logLine( "ERROR: cannot open file: " );
			BasicApp::the->logString( (*it).fnChars );
		}
		delete[] (*it).chDarkBmps;
		delete[] (*it).chLightBmps;
		delete[] (*it).fnChars;
	}
	
	chdir( cwd );
	delete[] cwd;

	BasicApp::the->logEnd( "done." );

	bLoaded = true;

	return 0;
}

Uint32 UiFileTheme::onActivate()
{
	return 0;
}

Uint32 UiFileTheme::onDeactivate()
{
	return 0;
}

unsigned int UiFileTheme::extractSubStrings( char *line, char **subs, unsigned int max )
{/*
<max> or all parameters of <line> are stored in <subs>;
return-value is number of stored params, if line has no params, 0 is returned
params are divided either by a ':' or a '|', spaces (' ') at the beginning or end of a param
are ignored, a line ends with '\n', and everything after a '#' in a line is ignored
WARNING: the data in line will be changed!
*/
	char *pChr;
	while( line[0] == ' ' )
		++line;
	if( ( pChr = strchr( line, '#' ) ) != 0 )
		*pChr = '\0';
	else
		pChr = line + strlen( line );

	if( pChr == line )
		return 0;

	while( *(--pChr) == ' ' )// last sign must not be a space
		*pChr = '\0';

	if( max == 0 )
		return 0;

	subs[0] = line;
	if( max == 1 )
		return 1;
	unsigned int n = 1;

	for( int i = 0; line[i] != '\0'; ++i )
	{
		switch( line[i] )
		{
		case ':':
		case '|':
			line[i] = '\0';
			pChr = &line[i];
			while( *(--pChr) == ' ' )
				*pChr = '\0';
			subs[n] = &line[i+1];

			while( *subs[n] == ' ' )
				++subs[n];

			if( max == ++n )
				return n;

			break;
		}
	}
	return n;
}

