/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "uimanager.h"

#include "basicapp.h"
#include "sdlmanager.h"
#include "uikeycombination.h"
#include "uiobject.h"
#include "frametimer.h"
#include "guimanager.h"
#include "functions.h"
#if __linux__
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#endif

bool UiManager::bUseKernelTimer = false;

UiManager *UiManager::the = NULL;
UiManager *UiManager::getThe()
{
	return the;
}

UiManager::UiManager(TiXmlElement*el, BasicObject *p): BasicObject(el, p)
{
	the = this;
	tInit = SDL_GetTicks();
	nFrames = 0;
	tFrame = tInit;
	tMinPerFrame_ms = 0;
	tMinPerFrame_us = 0;
	tPerFrame = 0;
#if __linux__
	struct sigaction sa;
	sigemptyset( &sa.sa_mask );
	sa.sa_flags = 0;
	sa.sa_handler = timerSignal;
	sigaction( SIGALRM, &sa, NULL );
#endif
	
	uiMouseFocus = NULL;
	uiKeyboardFocus = NULL;
	uiDisplayFocus = NULL;
	
	if( FrameTimer::the == NULL )
		new FrameTimer;
	if( UiKeyCombination::the == NULL )
		new UiKeyCombination;
}

UiManager::~UiManager()
{
	the = NULL;
	
	if( FrameTimer::the )
		delete FrameTimer::the;
	if( UiKeyCombination::the )
		delete UiKeyCombination::the;
}

void UiManager::onInit()
{
	if(!xmlData)	{
		BasicApp::the->logLevelLine( "FATAL ERROR: UiManager has no XML data!" );
		BasicObject::onInit();
		return;
	}
	BasicApp::the->logBegin("Initializing UiManager");
	const char *str = xmlData->Attribute("LimitFPS");
	if( str )
		setMaxFramerate( xStrToUL( str ) );
	str = xmlData->Attribute("UseKernelTimer");
	bUseKernelTimer = str && strToBool( str );
	
	TiXmlElement *fce = xmlData->FirstChildElement("GuiManager");
	if( fce )	{
		if(!GuiManager::the)
			new GuiManager(this);
		GuiManager::the->init(fce);
		
		BasicApp::the->logLevelLine( "setting focus on GuiManager::the" );
		setFocus(UiObject::ftDisplay, GuiManager::the);
		setFocus(UiObject::ftKeyboard, GuiManager::the);
		setFocus(UiObject::ftMouse, GuiManager::the);
	}
	else	{
		fce = xmlData->FirstChildElement("OglManager");
		if( fce )	{
			BasicApp::the->logLevelLine( "FATAL ERROR: OglManager is not supported by this version!" );
/*			if(!OglManager::the)
				new OglManager(this);
			OglManager::the->init(fce);
			
			setFocus(UiObject::ftDisplay, OglManager::the);
			setFocus(UiObject::ftKeyboard, OglManager::the);
			setFocus(UiObject::ftMouse, OglManager::the);
*/
		}
	}
	fce = xmlData->FirstChildElement("UiKeyCombination");
	if( fce )	{
		UiKeyCombination::the->init(fce);
	}
	for(fce = xmlData->FirstChildElement("SignalMultiplier"); fce; fce = fce->NextSiblingElement("SignalMultiplier"))
		new SignalMultiplier(fce, this);

/*	str = xmlData->Attribute("BgMusic");
	if(str)	{
		SdlMusicList *m = SdlManager::the->getMusicsByName(str);
		if(m) m->play();
	}*/
	BasicObject::onInit();
	BasicApp::the->logEnd("done.");
}

void UiManager::loop(bool &stop)
{
	stop = false;
	BasicApp::the->logLevelLine( "Entered UiManager::loop." );
	while( !stop )
	{
		handleEvents();
		drawFrame();
	}
}

void UiManager::handleEvents()
{
	SDL_Event ePolled;
	while( SDL_PollEvent( &ePolled ) )
	{
		switch( ePolled.type )	{
		case SDL_QUIT:
			BasicApp::the->logLevelLine( "QuitEvent" );
			BasicApp::the->destroy();
			break;
			
		case SDL_ACTIVEEVENT:
			{
				SDL_ActiveEvent * ae = ( SDL_ActiveEvent * )&ePolled;
				bActive = ae->gain;
				if(ae->gain)	{
					BasicApp::the->logLine( "Activated" );
					SdlManager::grabInput();
					if(uiDisplayFocus)
						uiDisplayFocus->onFocus(UiObject::ftDisplay);
				}
				else	{
					BasicApp::the->logLine( "Deactivated" );
					SdlManager::ungrabInput();
					if(uiDisplayFocus)
						uiDisplayFocus->onRelease(UiObject::ftDisplay);
				}
				break;
			}
		case SDL_KEYDOWN:
			{
				SDL_KeyboardEvent * ke = ( SDL_KeyboardEvent * )&ePolled;
				if( UiKeyCombination::the && (ke->keysym.mod & (KMOD_CTRL | KMOD_LALT)) != 0 )	{
					if( UiKeyCombination::the->keyPressed( ke->keysym.sym, ke->keysym.mod ) )	{
						break;
					}
				}
				/*if( ke->keysym.sym == SDLK_ESCAPE )	{ better emergency-kill: use UiKeyCombination (ie with Ctrl+ESC)
					BasicApp::the->destroy();
					break;
				}*/
				// if not ESC and no Combination then send it to uiKeyboardFocus
				if( uiKeyboardFocus )
					uiKeyboardFocus->onKeyDown( ( SDL_KeyboardEvent* )&ePolled );
				break;
			}
		case SDL_KEYUP:
			{
				SDL_KeyboardEvent * ke = ( SDL_KeyboardEvent * )&ePolled;
				if( UiKeyCombination::the && (ke->keysym.mod & (KMOD_CTRL | KMOD_LALT)) != 0 )
					if( UiKeyCombination::the->keyReleased( ke->keysym.sym, ke->keysym.mod ) )
						break;
				// if no Combination then send it to uiKeyboardFocus
				if( uiKeyboardFocus )
					uiKeyboardFocus->onKeyUp( ( SDL_KeyboardEvent* )&ePolled );
				break;
			}
		case SDL_MOUSEMOTION:
			if( uiMouseFocus )
				uiMouseFocus->onMouseMotion( ( SDL_MouseMotionEvent* )&ePolled );
			break;
			
		case SDL_MOUSEBUTTONDOWN:
			if( uiMouseFocus )
				uiMouseFocus->onMouseDown( ( SDL_MouseButtonEvent* )&ePolled );
			break;
			
		case SDL_MOUSEBUTTONUP:
			if( uiMouseFocus )
				uiMouseFocus->onMouseUp( ( SDL_MouseButtonEvent* )&ePolled );
			break;
		}
	}
}

void UiManager::drawFrame()
{
	if( uiDisplayFocus )	{
		uiDisplayFocus->onFrame();
		
		tPerFrame = SDL_GetTicks() - tFrame;
		if( tPerFrame < tMinPerFrame_ms )	{
			Uint32 delta = tMinPerFrame_ms - tPerFrame;
			tPerFrame = tMinPerFrame_ms;
#if __linux__
			if( bUseKernelTimer )
				SDL_Delay( delta );
			else
				sleep(1);// wait for the next timer interrupt
#else
			SDL_Delay( delta );
#endif
		}
		tFrame += tPerFrame;
		++nFrames;
	}
	FrameTimer::the->doTimer();
}

void UiManager::setFocus( UiObject::FocusType t, UiObject *o )
{
	switch(t)	{
	case UiObject::ftKeyboard:
		if(uiKeyboardFocus)
			uiKeyboardFocus->onRelease(UiObject::ftKeyboard);
		uiKeyboardFocus = o;
		if(o != NULL)
			o->onFocus(UiObject::ftKeyboard);
		break;
		
	case UiObject::ftMouse:
		if(uiMouseFocus)
			uiMouseFocus->onRelease(UiObject::ftMouse);
		uiMouseFocus = o;
		if(o)
			o->onFocus(UiObject::ftMouse);
		break;
		
	case UiObject::ftDisplay:
		if(uiDisplayFocus)
			uiDisplayFocus->onRelease(UiObject::ftDisplay);
		uiDisplayFocus = o;
		if(o && bActive)
			o->onFocus(UiObject::ftDisplay);
		break;
	}
}

UiObject * UiManager::getFocus(UiObject::FocusType t)
{
	switch(t)	{
	case UiObject::ftKeyboard:
		return uiKeyboardFocus;
		
	case UiObject::ftMouse:
		return uiMouseFocus;
		
	case UiObject::ftDisplay:
		return uiDisplayFocus;
	}
	return NULL;
}

void UiManager::themeChanged( UiTheme *t )
{
	if(uiDisplayFocus)
		uiDisplayFocus->onThemeChanged( t );
}

Uint32 UiManager::getAvgTimePerFrame() const
{
	return ( tFrame - tInit + ( nFrames >> 1 ) ) / nFrames;// gerundet
}

void UiManager::setMaxFramerate( Uint32 n )
{
	if( n == 0 )
		n = 1;
	else if( n > 1000 )
		n = 1000;
	tMinPerFrame_ms = 1000 / n;
	tMinPerFrame_us = 1000000 / n;
#if __linux__
	if( bUseKernelTimer )
	{
/*	itimerval tv, otv;
	if( n == 1 )
	{
		tv.it_interval.tv_sec = 1;
		tv.it_interval.tv_usec = 0;
		tv.it_value.tv_sec = 1;
		tv.it_value.tv_usec = 0;
	}
	else
	{
		tv.it_interval.tv_sec = 0;
		tv.it_interval.tv_usec = tMinPerFrame_us;
		tv.it_value.tv_sec = 0;
		tv.it_value.tv_usec = tMinPerFrame_us;
	}
	setitimer( ITIMER_REAL, &tv, &otv );*/
		ualarm( tMinPerFrame_us, tMinPerFrame_us );
	}
#endif
}

void UiManager::timerSignal( int signal )
{// actually this is not necessary
	//bExpired = true;
}
