
#include "xmlapp.h"
#include <SDL.h>

#include "sdlmanager.h"
#include "thememanager.h"
#include "uifiletheme.h"
#include "uimanager.h"
#include "netpeermanager.h"
#include "netthread.h"
#include "webserver.h"
#include "sdlmusiclist.h"
#include "functions.h"

DECLARE_SLOT(exit, NULL, XmlApp::quitApplication)

XmlApp::XmlApp( int argc, char *argv[], const char *xmlfile): BasicApp( argc, argv ), xmlDoc(xmlfile? xmlfile : "")
{
	bStopped = true;
}

XmlApp::~XmlApp()
{
}

int XmlApp::init(const char *fn)
{
	if( fn && !xmlDoc.LoadFile(fn) || !fn && !xmlDoc.LoadFile() )	{
		nInitFailStatus = E_APP_NOXML;
		return BasicApp::init();
	}
	if( xmlDoc.RootElement()->FirstAttribute() )
	{
		return BasicApp::init( xmlDoc.RootElement() );
	}
	return BasicApp::init( xmlDoc.RootElement()->FirstChildElement() );
}

void XmlApp::onInit()
{
	// init log file, data path...
	BasicApp::onInit();

	logBegin( "Initializing app ..." );
	
	if( xmlData )	{
		const char *str = xmlData->Attribute("DoBeforeInit");
		if(str)	{
			BasicSlot::Callback f = BasicSlot::getFunction(str, NULL);
			if(f)
				(*f)(this, NULL, xmlData);
		}
		str = xmlData->Attribute("UserXML");
		if(str)
			if(!xmlUserdoc.LoadFile(makeEnvPath(str).c_str()))	{
				logLevelLine("Loading user's XML file failed: ");
				logString(makeEnvPath(str).c_str());
				str = NULL;
			}
			else	{
				str = NULL;
				xmlUser = xmlUserdoc.RootElement();
				if(!xmlUser)	{
					logLevelLine("User's XML file has no valid root element!");
				}
				else	{
					str = xmlUser->Attribute("UserdataPath");
				}
			}
		if(!str)
			str = xmlData->Attribute("UserdataPath");
		if(str)
			setUserdataPath(makeEnvPath(str).c_str());
		
		str = xmlData->Attribute("Caption");
		if(str)
			SDL_WM_SetCaption( str, 0 );
		
		str = xmlData->Attribute("CanGrabInput");
		SdlManager::bGrabInputEnabled = str != NULL && strToBool(str);
		
		TiXmlElement *xmlSdl = NULL,
			*xmlThemes = NULL, *xmlNetwork = NULL,
			*xmlWeb = NULL, *xmlUiManager = NULL;
		// initialize graphics and (if required) sound
		xmlSdl = xmlData->FirstChildElement("SdlManager");
		if(xmlSdl)	{
			new SdlManager();
			SdlManager::the->init( xmlSdl );
		}
		
		xmlThemes = xmlData->FirstChildElement("Themes");
		if(xmlThemes)	{
			new ThemeManager( ThemeManager::lsOnAppStart );
			for(TiXmlElement *thel = xmlThemes->FirstChildElement(); thel; thel = thel->NextSiblingElement())	{
				str = thel->Attribute("Filename");
				new UiFileTheme( makeEnvPath(str).c_str(), thel->Value() );
			}
		}
		
		xmlUiManager = xmlData->FirstChildElement("UiManager");
		if(xmlUiManager)
			new UiManager(xmlUiManager, this);
		else
			BasicApp::logLevelLine("ERROR: no valid tag 'UiManager' found");
		
		if(xmlThemes)	{
			str = xmlThemes->Attribute("LoadHidden");
			bool hidden = str && strToBool(str);
			str = xmlThemes->Attribute("Default");
			if(str)
				ThemeManager::the->setActiveTheme(ThemeManager::the->getThemeByName(str), hidden);
		}
		
		xmlNetwork = xmlData->FirstChildElement("Network");
		if(xmlNetwork)	{
			new NetPeerManager(xmlNetwork, this);
			new NetThread("NetThread", this, false);
		}
		xmlWeb = xmlData->FirstChildElement("Webserver");
		if(xmlWeb)	{
			new WebServer(xmlWeb, this);
			if(!xmlNetwork)
				new NetThread("NetThread", this);
			else NetThread::the->setServerCheck(true);
		}
		if(xmlUiManager)	{
			str = xmlUiManager->Attribute("BgMusic");
			if(str)	{
				SdlMusicList *m = SdlManager::the->getMusicsByName(str);
				if(m) m->play();
			}
		}
		// wait some time, if wanted (actually not necessary)
		if( (str = xmlData->Attribute("WaitOnStart")) != NULL )
			SDL_Delay( xStrToUL(str) );
		
		str = xmlData->Attribute("DoAfterCreation");
		if(str)	{
			BasicObject::FunctionData f = BasicObject::getFunction(str);
			if(f.func)
				(*f.func)(this, f.obj, xmlData);
		}
	}
	
	logEnd( "done." );
	// initialize children (initialized BasicApp already at the beginning)
	BasicObject::onInit();
	if( xmlData )
	{
		const char *str = xmlData->Attribute("DoAfterInit");
		if(str)	{
			FunctionData cb = getFunctionByName( str );
			if( cb.func )
				(*cb.func)(this, cb.obj, xmlData);
		}
	}
}

void XmlApp::destroy()
{
	if( !bStopped )	{
		bStopped = true;
		return;
	}
	BasicApp::the->logLine( "" );
	BasicApp::the->logBegin( "Shutting down app..." );

	SdlManager::ungrabInput();
	BasicApp::the->logLevelLine( "Input ungrabbed." );
	
	onExit();

	if(xmlUser)	{
		// TODO: create dir of user-xml-file, if it does not exist
		xmlUserdoc.SaveFile();
		logLevelLine( "Saved user's XML data" );
	}
	BasicApp::the->logEnd( "done." );
	BasicApp::destroy();// will exit the program
}

void XmlApp::onExit()
{
	BasicApp::the->logBegin( "Destroying components..." );
	
	if(NetThread::the)
		NetThread::the->stop( ~0 );

	if( ThemeManager::the )	{
		BasicApp::the->logLevelLine( "ThemeManager" );
		delete ThemeManager::the;
	}

	if( NetThread::the )	{
		BasicApp::the->logLevelLine( "NetThread" );
		delete NetThread::the;
	}

	if( NetPeerManager::the )	{
		BasicApp::the->logLevelLine( "NetBasicManager" );
		delete NetPeerManager::the;
	}

	if( WebServer::the )	{
		BasicApp::the->logLevelLine( "WebServer" );
		delete WebServer::the;
	}

	if( UiManager::the )	{
		BasicApp::the->logLevelLine( "UiManager" );
		delete UiManager::the;
	}

	if( SdlManager::the )	{
		BasicApp::the->logLevelLine( "SdlManager" );
		SdlManager::the->destroy();
		delete SdlManager::the;
	}

	BasicApp::the->logEnd( "Components destroyed." );
}


long XmlApp::mainLoop()
{
	logLine( "" );
	logLevelLine( "Entered XmlApp::mainLoop()." );

	if(UiManager::the)
		UiManager::the->loop(bStopped);
	else	{
		bStopped = false;
		while( !bStopped )
		{
			SDL_Delay(100);
		}
	}
	logLevelLine( "Left XmlApp::mainLoop()." );
	destroy();
	// the code after destroy() is never reached
	return 0;
}
